Package: dataset
Title: Create Data Frames for Exchange and Reuse
Version: 0.4.5
Date: 2026-06-03
Language: en-GB
Authors@R: 
    c(person(given = "Daniel", family = "Antal", 
           email = "daniel.antal@dataobservatory.eu", 
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7513-6760")
           ), 
      person(given = "Marcelo", family =  "Perlin", 
             role = c("rev"), 
             comment = c(ORCID = "0000-0002-9839-4268")
             ), 
     person(given = "Anna Márta", family =  "Mester", 
            role = c("rev"), 
            comment = c(ORCID = "0009-0008-2274-8163")
            ),
      person(given = "Mauro", family =  "Lepore", 
            role = c("rev"), 
            comment = c(ORCID = "0000-0002-1986-7988")
            )
      )
Maintainer: Daniel Antal <daniel.antal@dataobservatory.eu>
Description: The 'dataset' package extends tidy data frames with
    machine-readable metadata, semantic definitions, and provenance
    information. It supports incremental semantic stabilization,
    interoperable dataset exchange, and FAIR-oriented publication
    workflows by preserving contextual metadata directly within R
    objects. The package facilitates the creation, exchange, reuse,
    and RDF serialization of datasets in line with ISO and W3C
    standards.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://docs.ropensci.org/dataset/,
        https://github.com/ropensci/dataset,
        https://dataset.dataobservatory.eu
BugReports: https://github.com/ropensci/dataset/issues
LazyData: true
Imports: assertthat, haven, ISOcodes, labelled, pillar, stats, tibble,
        utils, vctrs
Suggests: dplyr, jsonld, knitr, rdflib, rmarkdown, spelling, tidyr,
        testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 3.5)
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-03 06:51:28 UTC; DanielAntal
Author: Daniel Antal [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7513-6760>),
  Marcelo Perlin [rev] (ORCID: <https://orcid.org/0000-0002-9839-4268>),
  Anna Márta Mester [rev] (ORCID:
    <https://orcid.org/0009-0008-2274-8163>),
  Mauro Lepore [rev] (ORCID: <https://orcid.org/0000-0002-1986-7988>)
Repository: CRAN
Date/Publication: 2026-06-03 09:10:14 UTC
Built: R 4.5.2; ; 2026-06-03 12:29:39 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",dataset_df)
S3method("[",haven_labelled_defined)
S3method("[[",haven_labelled_defined)
S3method("var_concept<-",default)
S3method("var_label<-",haven_labelled_defined)
S3method("var_namespace<-",default)
S3method("var_unit<-",default)
S3method(Ops,haven_labelled_defined)
S3method(as.Date,haven_labelled_defined)
S3method(as.POSIXct,haven_labelled_defined)
S3method(as.character,haven_labelled_defined)
S3method(as.character,prelabelled)
S3method(as.data.frame,dataset_df)
S3method(as.list,haven_labelled_defined)
S3method(as.numeric,haven_labelled_defined)
S3method(as.vector,haven_labelled_defined)
S3method(as_character,haven_labelled_defined)
S3method(as_character,prelabelled)
S3method(as_factor,haven_labelled_defined)
S3method(as_logical,haven_labelled_defined)
S3method(as_numeric,haven_labelled_defined)
S3method(as_tibble,dataset_df)
S3method(c,haven_labelled_defined)
S3method(format,haven_labelled_defined)
S3method(head,haven_labelled_defined)
S3method(is.datacite,datacite)
S3method(length,haven_labelled_defined)
S3method(names,dataset_df)
S3method(plot,dataset_df)
S3method(print,bibrecord)
S3method(print,datacite)
S3method(print,dataset_df)
S3method(print,dublincore)
S3method(print,haven_labelled_defined)
S3method(summary,dataset_df)
S3method(summary,haven_labelled_defined)
S3method(tail,haven_labelled_defined)
S3method(tbl_sum,dataset_df)
S3method(type_sum,haven_labelled_defined)
S3method(var_concept,default)
S3method(var_label,dataset_df)
S3method(var_label,defined)
S3method(var_namespace,default)
S3method(var_unit,default)
S3method(vec_cast,double.haven_labelled_defined)
S3method(xsd_convert,Date)
S3method(xsd_convert,POSIXct)
S3method(xsd_convert,character)
S3method(xsd_convert,data.frame)
S3method(xsd_convert,dataset_df)
S3method(xsd_convert,difftime)
S3method(xsd_convert,factor)
S3method(xsd_convert,haven_labelled_defined)
S3method(xsd_convert,integer)
S3method(xsd_convert,logical)
S3method(xsd_convert,numeric)
S3method(xsd_convert,tbl_df)
export("contributor<-")
export("creator<-")
export("dataset_format<-")
export("dataset_title<-")
export("description<-")
export("geolocation<-")
export("identifier<-")
export("language<-")
export("namespace_attribute<-")
export("provenance<-")
export("publication_year<-")
export("publisher<-")
export("related_item<-")
export("relation<-")
export("rights<-")
export("set_bibentry<-")
export("subject<-")
export("unit_attribute<-")
export("var_concept<-")
export("var_label<-")
export("var_labels<-")
export("var_namespace<-")
export("var_unit<-")
export(as.tibble.dataset_df)
export(as_character)
export(as_datacite)
export(as_dataset_df)
export(as_dublincore)
export(as_factor)
export(as_logical)
export(as_numeric)
export(as_tibble)
export(as_value_key)
export(bibrecord)
export(bind_defined_rows)
export(contributor)
export(creator)
export(datacite)
export(dataset_df)
export(dataset_format)
export(dataset_title)
export(dataset_to_triples)
export(defined)
export(describe)
export(description)
export(dublincore)
export(geolocation)
export(get_bibentry)
export(get_namespace_attribute)
export(get_unit_attribute)
export(get_variable_concepts)
export(get_variable_namespaces)
export(get_variable_units)
export(id_to_column)
export(identifier)
export(invert_value_key)
export(is.dataset_df)
export(is.defined)
export(is.dublincore)
export(is.prelabelled)
export(is.related)
export(is_dataset_df)
export(label_attribute)
export(language)
export(n_triple)
export(n_triples)
export(namespace_attribute)
export(prelabel)
export(provenance)
export(publication_year)
export(publisher)
export(related_create)
export(related_item)
export(relation)
export(rights)
export(set_namespace_attribute)
export(set_unit_attribute)
export(strip_defined)
export(subject)
export(subject_create)
export(unit_attribute)
export(var_concept)
export(var_label)
export(var_labels)
export(var_namespace)
export(var_unit)
export(xsd_convert)
importFrom(assertthat,assert_that)
importFrom(haven,as_factor)
importFrom(haven,labelled)
importFrom(labelled,`var_label<-`)
importFrom(labelled,is.labelled)
importFrom(labelled,label_attribute)
importFrom(labelled,var_label)
importFrom(pillar,tbl_sum)
importFrom(pillar,type_sum)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(tibble,new_tibble)
importFrom(tibble,tibble)
importFrom(utils,bibentry)
importFrom(utils,citation)
importFrom(utils,head)
importFrom(utils,person)
importFrom(utils,tail)
importFrom(vctrs,vec_cast)
importFrom(vctrs,vec_data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# The dataset R Package <a href='https://dataset.dataobservatory.eu/'><img src="man/figures/logo.png" align="right"/></a>

<!-- badges: start -->

[![rhub](https://github.com/ropensci/dataset/actions/workflows/rhub.yaml/badge.svg)](https://github.com/ropensci/dataset/actions/workflows/rhub.yaml)
[![devel-version](https://img.shields.io/badge/devel%20version-0.4.5-blue.svg)](https://github.com/ropensci/dataset)
[![Codecov test
coverage](https://codecov.io/gh/ropensci/dataset/branch/main/graph/badge.svg)](https://app.codecov.io/gh/ropensci/dataset?branch=main)
[![Project Status:
Active](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/dataset)](https://cran.r-project.org/package=dataset)
[![CRAN_time_from_release](https://www.r-pkg.org/badges/ago/dataset)](https://cran.r-project.org/package=dataset)
<!-- [![DOI](https://img.shields.io/badge/DOI-10.32614/CRAN.package.dataset-blue)](https://doi.org/10.32614/CRAN.package.dataset) --->
[![dataobservatory](https://img.shields.io/badge/ecosystem-dataobservatory.eu-3EA135.svg)](https://dataobservatory.eu/)

<!-- badges: end -->

# Overview

The `dataset` package extends tidyverse workflows with lightweight
semantic metadata, provenance tracking, and interoperable dataset
structures.

It supports gradual semantic stabilization ranging from lightweight
semantic mappings to formally defined variables and semantically
enriched datasets suitable for FAIR, machine-readable, and
standards-aligned data exchange.

The package draws inspiration from:

- **SDMX** and statistical data cube models
- **Dublin Core** and **DataCite**
- **FAIR and reproducible research workflows**

The goal is to preserve metadata when reusing statistical and repository
datasets, improve interoperability, and make it easy to turn tidy data
frames into web-ready, publishable datasets that comply with ISO and W3C
standards.

## Installation

You can install the latest released version of **`dataset`** from
[CRAN](https://cran.r-project.org/package=dataset) with:

``` r
install.packages("dataset")
```

To install the development version from GitHub with `pak` or `remotes`:

``` r
# install.packages("pak")
pak::pak("dataobservatory-eu/dataset")

# install.packages("remotes")
remotes::install_github("dataobservatory-eu/dataset")
```

## Minimal Example

Real-world datasets rarely begin with fully standardized values. Early
in a project, inconsistencies may be easy to spot, such as mixing `AD`
and `Andorra` for the same country. As datasets are combined from
multiple sources, however, additional variants often appear, for example
the ISO-3166 alpha-2 code `AD`, the country name `Andorra`, or the
ISO-3166 alpha-3 code `AND`.

The `prelabel()` constructor provides a lightweight way to stabilize
such values before committing to a formal semantic definition.

``` r
library(dataset)

x <- prelabel(
  c("AD", "Andorra", "AND", "LI", "Liechtenstein"),
  labels = c(
    Andorra = "AD",
    AND = "AD",
    Liechtenstein = "LI"
  )
)

as.character(x)
#> [1] "AD" "AD" "AD" "LI" "LI"
```

Unlike a formal semantic definition, a `prelabelled` vector records
provisional mappings that may still evolve during data integration. The
original observational values remain available alongside the current
semantic assumptions:

``` r
attr(x, "prelabel")
#>       Andorra           AND Liechtenstein            AD            LI 
#>          "AD"          "AD"          "LI"          "AD"          "LI"
```

When semantic assumptions become sufficiently stable, variables can be
formalized with `defined()` and combined into a semantically enriched
`dataset_df()` object:

``` r
library(dataset)

df <- dataset_df(
  country = defined(
    c("AD", "LI"),
    label = "Country",
    namespace = "https://www.geonames.org/countries/$1/"
  ),
  gdp = defined(
    c(3897, 7365),
    label = "GDP",
    unit = "million euros"
  ),
  dataset_bibentry = dublincore(
    title = "GDP Dataset",
    creator = person("Jane", "Doe", role = "aut"),
    publisher = "Small Repository"
  )
)

print(df)
#> Doe (2026): GDP Dataset [dataset]
#>   rowid country   gdp 
#>   <chr> <chr>   <dbl>
#> 1 obs1  AD       3897
#> 2 obs2  LI       7365
```

This illustrates the semantic lifecycle supported by the package:

``` text
raw values
    ↓
prelabelled
    ↓
defined
    ↓
dataset_df
    ↓
RDF and FAIR publication
```

Because semantic assumptions and provenance are preserved explicitly,
semantically enriched datasets can be exported as interoperable RDF
triples without manually reconstructing metadata at publication time.

Export as RDF triples:

<style type="text/css">
.smaller .table {
  font-size: 11px;
}
&#10;.smaller pre,
.smaller code {
  font-size: 11px;
  line-height: 1.2;
}
</style>

``` r
dataset_to_triples(df, format = "nt")
```

<div class="smaller">

    #> [1] "<http://example.com/dataset#obsobs1> <http://example.com/prop/country> <https://www.geonames.org/countries/AD/> ."
    #> [2] "<http://example.com/dataset#obsobs2> <http://example.com/prop/country> <https://www.geonames.org/countries/LI/> ."
    #> [3] "<http://example.com/dataset#obsobs1> <http://example.com/prop/gdp> \"3897\"^^<xsd:decimal> ."                     
    #> [4] "<http://example.com/dataset#obsobs2> <http://example.com/prop/gdp> \"7365\"^^<xsd:decimal> ."

</div>

Retain automatically recorded provenance:

``` r
provenance(df)
```

<div class="smaller">

    #> [1] "<http://example.com/dataset_prov.nt> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Bundle> ."                  
    #> [2] "<http://example.com/dataset#> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Entity> ."                         
    #> [3] "<http://example.com/dataset#> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://purl.org/linked-data/cube#DataSet> ."                 
    #> [4] "_:doejane <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Agent> ."                                              
    #> [5] "<https://doi.org/10.32614/CRAN.package.dataset> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#SoftwareAgent> ."
    #> [6] "<http://example.com/creation> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Activity> ."                       
    #> [7] "<http://example.com/creation> <http://www.w3.org/ns/prov#generatedAtTime> \"2026-06-03T06:29:26Z\"^^<xsd:dateTime> ."

</div>

## Contributing

The package does not attempt automatic ontology alignment, entity
reconciliation, or rule-based semantic inference. It focuses on
preserving semantic assumptions made by the analyst in a transparent and
reproducible form.

We welcome contributions and discussion!

- Please see our
  [CONTRIBUTING.md](https://github.com/ropensci/dataset/blob/main/CONTRIBUTING.md)
  guide.
- Ideas, bug reports, and feedback are welcome via [GitHub
  issues](https://github.com/ropensci/dataset/issues).
- The design principles and ideas for futher development are explained
  in [Design Principles & Future Work Semantically Enriched,
  Standards-Aligned Datasets in
  R](https://dataset.dataobservatory.eu/articles/design.html).

Please refer to this package as:

Daniel Antal. (2026). *dataset: Create Data Frames that are Easier to
Exchange and Reuse (0.4.4)*. The Comprehensive R Archive Network.
<https://zenodo.org/records/17621464>, DOI:
10.32614/CRAN.package.dataset

See contributors on the website and in the DESCRIPTION file.

## Code of Conduct

This project follows the [rOpenSci Code of
Conduct](https://ropensci.org/code-of-conduct/). By participating, you
are expected to uphold these guidelines.

## ----setupvignette, include = FALSE-------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----loaddata-----------------------------------------------------------------
library(dataset)
data("gdp")

## ----printgdp-----------------------------------------------------------------
print(gdp)

## ----createdataasetdf---------------------------------------------------------
small_country_dataset <- dataset_df(
  geo = defined(
    gdp$geo,
    label = "Country name",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/geo/$1"
  ),
  year = defined(
    gdp$year,
    label = "Reference Period (Year)",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refPeriod"
  ),
  gdp = defined(
    gdp$gdp,
    label = "Gross Domestic Product",
    unit = "CP_MEUR",
    concept = "http://data.europa.eu/83i/aa/GDP"
  ),
  unit = defined(
    gdp$unit,
    label = "Unit of Measure",
    concept = "http://purl.org/linked-data/sdmx/2009/attribute#unitMeasure",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/unit/$1"
  ),
  freq = defined(
    gdp$freq,
    label = "Frequency",
    concept = "http://purl.org/linked-data/sdmx/2009/code"
  ),
  dataset_bibentry = dublincore(
    title = "Small Country Dataset",
    creator = person("Jane", "Doe"),
    publisher = "Example Inc.",
    datasource = "https://doi.org/10.2908/NAIDA_10_GDP",
    rights = "CC-BY",
    coverage = "Andorra, Liechtenstein, San Marino and the Feroe Islands"
  )
)

## ----varlabel-----------------------------------------------------------------
var_label(small_country_dataset$gdp)

## ----varunit------------------------------------------------------------------
var_unit(small_country_dataset$gdp)

## ----language-----------------------------------------------------------------
language(small_country_dataset) <- "en"

## ----bibentry-----------------------------------------------------------------
print(get_bibentry(small_country_dataset), "bibtex")

## ----feroedf------------------------------------------------------------------
feroe_df <- data.frame(
  geo = rep("FO", 3),
  year = 2020:2022,
  gdp = c(2523.6, 2725.8, 3013.2),
  unit = rep("CP_MEUR", 3),
  freq = rep("A", 3)
)

## ----notevaluatedrbind, eval=FALSE--------------------------------------------
# rbind(small_country_dataset, feroe_df)

## ----fereodataset-------------------------------------------------------------
feroe_dataset <- dataset_df(
  geo = defined(
    feroe_df$geo,
    label = "Country name",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/geo/$1"
  ),
  year = defined(
    feroe_df$year,
    label = "Reference Period (Year)",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refPeriod"
  ),
  gdp = defined(
    feroe_df$gdp,
    label = "Gross Domestic Product",
    unit = "CP_MEUR",
    concept = "http://data.europa.eu/83i/aa/GDP"
  ),
  unit = defined(
    feroe_df$unit,
    label = "Unit of Measure",
    concept = "http://purl.org/linked-data/sdmx/2009/attribute#unitMeasure",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/unit/$1"
  ),
  freq = defined(
    feroe_df$freq,
    label = "Frequency",
    concept = "http://purl.org/linked-data/sdmx/2009/code"
  )
)

## ----binddefinedrows----------------------------------------------------------
joined_dataset <- bind_defined_rows(small_country_dataset, feroe_dataset)
joined_dataset

## ----backwardcompatibility----------------------------------------------------
attributes(as.data.frame(joined_dataset))

## ----coercion-----------------------------------------------------------------
as.data.frame(small_country_dataset)

## ----richdataframe------------------------------------------------------------
as.data.frame(small_country_dataset,
  strip_attributes = FALSE
)

## ----astibble-----------------------------------------------------------------
as_tibble(orange_df)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(dataset)

## ----dataframe----------------------------------------------------------------
data.frame(
  geo = c(
    "LI", "Liechtenstein", "SM", "SMR"
  ),
  year = c(2024, 2025, 2024, 2025),
  CPI = c(
    "0.8", "0.8", "0.9", "0.9"
  ),
  GNI = c(
    "8976", "8976", "9672", "9672"
  )
)

## -----------------------------------------------------------------------------
geo <- prelabel(
  c("LI", "Liechtenstein", "SM", "SMR"),
  labels = c(
    Liechtenstein = "LI", SMR = "SM"
  )
)

as.character(geo)

## ----attributes---------------------------------------------------------------
x <- 2457
attr(x, "unit") <- "CP_MEUR"
attr(x, "concept") <- "http://data.europa.eu/83i/aa/GDP"

## -----------------------------------------------------------------------------
library(dataset)

gdp <- defined(
  c(2355, 2592, 2884),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo <- defined(
  rep("AD", 3),
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

## -----------------------------------------------------------------------------
var_concept(gdp)
var_unit(gdp)
var_namespace(geo)

## ----smalldataset-------------------------------------------------------------
small_dataset <- dataset_df(
  geo = geo,
  gdp = gdp,
  identifier = c(gdp = "http://example.com/dataset#gdp"),
  dataset_bibentry = dublincore(
    title = "Small GDP Dataset",
    creator = person("Jane", "Doe", role = "aut"),
    publisher = "Small Repository",
    subject = "Gross Domestic Product"
  )
)

## ----bibrecord----------------------------------------------------------------
as_dublincore(small_dataset)
as_datacite(small_dataset)

## ----provenance---------------------------------------------------------------
provenance(small_dataset)

## ----description--------------------------------------------------------------
description_nt <- tempfile(pattern = "small_dataset", fileext = ".nt")
describe(small_dataset, description_nt)

# Only a few lines shown:
readLines(description_nt)[5:8]

## ----eurogpd------------------------------------------------------------------
euro_gdp <- defined(
  c(2355, 2592),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo_europe <- defined(
  c("AD", "LI"),
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

euros_dataset <- dataset_df(
  geo = geo_europe,
  gdp = euro_gdp,
  dataset_bibentry = dublincore(
    title = "European Microstates GDP",
    creator = person("Statistical Unit", role = "aut"),
    publisher = "Eurostat",
    subject = "Gross Domestic Product"
  )
)

## ----usdgdp-------------------------------------------------------------------
usd_gdp <- defined(
  56,
  label = "Gross Domestic Product",
  unit = "USD_MILLIONS",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo_tuvalu <- defined(
  "TV",
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

tuvalu_dataset <- dataset_df(
  geo = geo_tuvalu,
  gdp = usd_gdp,
  dataset_bibentry = dublincore(
    title = "Tuvalu GDP (USD)",
    creator = person("Island", "Bureau", role = "aut"),
    publisher = "PacificStats",
    subject = "Gross Domestic Product"
  )
)

## -----------------------------------------------------------------------------
binded <- try(bind_defined_rows(euros_dataset, tuvalu_dataset), silent = TRUE)

## ----mutate-------------------------------------------------------------------
exchange_rate <- 1.02
eur_tuv_gdp <- defined(
  56 * exchange_rate,
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

tuvalu_dataset <- dataset_df(
  geo = geo_tuvalu,
  gdp = eur_tuv_gdp,
  dataset_bibentry = dublincore(
    title = "Tuvalu GDP (USD)",
    creator = person("Island", "Bureau", role = "aut"),
    publisher = "PacificStats",
    subject = "Gross Domestic Product"
  )
)

## -----------------------------------------------------------------------------
var_unit(eur_tuv_gdp) <- "M_EUR"

## ----metadatachanges----------------------------------------------------------
global_dataset <- bind_defined_rows(euros_dataset, tuvalu_dataset)
dataset_title(global_dataset, overwrite = TRUE) <- "Global Microstates GDP"
publisher(global_dataset) <- "My Research Institute"
creator(global_dataset) <- person("Jane Doe", role = "aut")
language(global_dataset) <- "en"
description(global_dataset) <- "A dataset created from various sources about the GDP of very small states."
global_dataset

## -----------------------------------------------------------------------------
as_dublincore(global_dataset)

## -----------------------------------------------------------------------------
dataset_to_triples(global_dataset)

## -----------------------------------------------------------------------------
dataset_to_triples(global_dataset, format = "nt")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

if (!requireNamespace("rdflib",
  quietly = TRUE
)) {
  stop("Please install 'rdflib' to run this vignette.")
}

## ----predefine----------------------------------------------------------------
library(dataset)

country <- prelabel(
  c("AD", "Andorra", "AND", "LI", "Liechtenstein"),
  labels = c(
    Andorra = "AD",
    AND = "AD",
    Liechtenstein = "LI"
  )
)

country

## ----predefine-attribute------------------------------------------------------
attr(country, "prelabel")

## ----definegdpdataset---------------------------------------------------------
library(dataset)

gdp <- defined(
  c(2355, 2592, 2884),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo <- defined(
  rep("AD", 3),
  label = "Geopolitical Entity",
  concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
  namespace = "https://www.geonames.org/countries/$1/"
)

gdp
geo

## ----smalldatasetexample------------------------------------------------------
small_dataset <- dataset_df(
  geo = geo,
  gdp = gdp,
  identifier = c(gdp = "http://example.com/dataset#gdp"),
  dataset_bibentry = dublincore(
    title = "Small GDP Dataset",
    creator = person("Jane", "Doe", role = "aut"),
    publisher = "Small Repository",
    subject = "Gross Domestic Product"
  )
)

small_dataset

## ----dublincoremetadata-------------------------------------------------------
as_dublincore(small_dataset)

## ----triplesexample-----------------------------------------------------------
triples <- dataset_to_triples(small_dataset,
  format = "nt"
)
triples

## ----ntexample----------------------------------------------------------------
mycon <- tempfile("my_dataset",
  fileext = "nt"
)
my_description <- describe(
  x = small_dataset,
  con = mycon
)

# Only three statements are shown:
readLines(mycon)[c(4, 8, 12)]

## ----provenancexample---------------------------------------------------------
## Show two lines of provenance:
provenance(small_dataset)[c(6, 7)]

## ----smalldf------------------------------------------------------------------
small_df <- as.data.frame(small_dataset,
  strip_attributes = FALSE
)

attr(small_dataset, "subject")

## ----smalltbl-----------------------------------------------------------------
small_tbl <- as_tibble(
  small_dataset,
  strip_attributes = TRUE
)

small_tbl

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(dataset)
library(rdflib)
data("gdp")

## ----minimaldf----------------------------------------------------------------
small_geo <- dataset_df(
  geo = defined(
    gdp$geo[1:3],
    label = "Geopolitical entity",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
    namespace = "https://www.geonames.org/countries/$1/"
  ),
  identifier = c(
    obs = "https://dataset.dataobservatory.eu/examples/dataset.html#"
  )
)

## ----printsmallgeodf----------------------------------------------------------
print(small_geo)

## ----triplesdf, eval=FALSE----------------------------------------------------
# triples_df <- dataset_to_triples(small_geo)
# knitr::kable(triples_df)

## ----triplesdfprintsmall, echo=FALSE------------------------------------------
triples_df <- dataset_to_triples(small_geo)
knitr::kable(triples_df)

## ----createntriples-----------------------------------------------------------
ntriples <- dataset_to_triples(small_geo, format = "nt")

## ----pritriples, eval=FALSE---------------------------------------------------
# cat(ntriples, sep = "\n")

## ----printsmaller-------------------------------------------------------------
cat(ntriples, sep = "\n")

## ----ntripleexample-----------------------------------------------------------
n_triple(
  s = "https://dataset.dataobservatory.eu/examples/dataset.html#obs1",
  p = "http://purl.org/dc/terms/title",
  o = "Small Country Dataset"
)

## ----readrdf------------------------------------------------------------------
# We write to a temporary file our Ntriples created earlier
temp_file <- tempfile(fileext = ".nt")
writeLines(ntriples, con = temp_file)

rdf_graph <- rdf()
rdf_parse(rdf_graph, doc = temp_file, format = "ntriples")
rdf_graph

## ----cleanup------------------------------------------------------------------
# Clean up: delete file and clear RDF graph
unlink(temp_file)
rm(rdf_graph)
gc()

## ----scaleup------------------------------------------------------------------
small_country_dataset <- dataset_df(
  geo = defined(
    gdp$geo,
    label = "Country name",
    concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
    namespace = "https://www.geonames.org/countries/$1/"
  ),
  year = defined(
    gdp$year,
    label = "Reference Period (Year)",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refPeriod"
  ),
  gdp = defined(
    gdp$gdp,
    label = "Gross Domestic Product",
    unit = "https://dd.eionet.europa.eu/vocabularyconcept/eurostat/unit/CP_MEUR",
    concept = "http://data.europa.eu/83i/aa/GDP"
  ),
  unit = gdp$unit,
  freq = defined(
    gdp$freq,
    label = "Frequency",
    concept = "http://purl.org/linked-data/sdmx/2009/code"
  ),
  identifier = c(
    obs = "https://dataset.dataobservatory.eu/examples/dataset.html#"
  ),
  dataset_bibentry = dublincore(
    title = "Small Country Dataset",
    creator = person("Jane", "Doe"),
    publisher = "Example Inc.",
    datasource = "https://doi.org/10.2908/NAIDA_10_GDP",
    rights = "CC-BY",
    coverage = "Andorra, Lichtenstein and San Marino"
  )
)

## ----smallcountrydfnt---------------------------------------------------------
small_country_df_nt <- dataset_to_triples(
  small_country_dataset,
  format = "nt"
)

## ----smallcountrydfntsample---------------------------------------------------
## See rows 1,11,21
small_country_df_nt[c(1, 11, 21, 31, 41)]

## ----readrdf2-----------------------------------------------------------------
# We write to a temporary file our Ntriples created earlier
temp_file <- tempfile(fileext = ".nt")
writeLines(small_country_df_nt,
  con = temp_file
)

rdf_graph <- rdf()
rdf_parse(rdf_graph, doc = temp_file, format = "ntriples")

## ----readrdf2print, eval = FALSE----------------------------------------------
# rdf_graph

## ----readrdf2printsmaller-----------------------------------------------------
rdf_graph

## ----readjsonld---------------------------------------------------------------
# Create temporary JSON-LD output file
jsonld_file <- tempfile(fileext = ".jsonld")

# Serialize (export) the entire graph to JSON-LD format
rdf_serialize(rdf_graph, doc = jsonld_file, format = "jsonld")

## ----readjsonldprint----------------------------------------------------------
cat(readLines(jsonld_file)[1:30], sep = "\n")

## ----clenup2, echo=FALSE, message=FALSE---------------------------------------
unlink(temp_file)
rm(rdf_graph)
gc()

#' Convert to XML Schema Definition (XSD) Types
#'
#' Converts R vectors, data frames, and `dataset_df` objects to
#' [XML Schema Definition (XSD)](https://www.w3.org/TR/xmlschema11-2/)
#' compatible string representations such as `xsd:decimal`, `xsd:boolean`,
#' `xsd:date`, and `xsd:dateTime`.
#'
#' This is primarily used for generating RDF-compatible typed literals.
#'
#' @details
#' - For **vectors**, returns a character vector of typed literals.
#' - For **data frames** or tibbles, returns a data frame with the same
#'   structure but with all values converted to XSD strings.
#' - For `dataset_df` objects, behaves like the data frame method but
#'   preserves dataset-level attributes.
#'
#' @param x An object (vector, data frame, tibble, or `dataset_df`).
#' @param idcol Column name or position to use as row (observation) identifier.
#'   If `NULL`, row names are used.
#' @param shortform Logical. If `TRUE` (default), datatypes are abbreviated with
#'   the `xsd:` prefix (e.g. `"42"^^<xsd:integer>`). If `FALSE`, datatypes are
#'   expanded to full URIs (e.g.
#'   `"42"^^<http://www.w3.org/2001/XMLSchema#integer>`).
#' @param ... Additional arguments passed to methods.
#'
#' @return A character vector or data frame with values serialized as
#' XSD-compatible RDF literals.
#'
#' @examples
#' # Simple data frame with mixed types
#' df <- data.frame(
#'   id     = 1:2,
#'   value  = c(3.14, 2.71),
#'   active = c(TRUE, FALSE),
#'   date   = as.Date(c("2020-01-01", "2020-12-31"))
#' )
#'
#' # Short vs long-form URI:
#' xsd_convert(120L, shortform = TRUE)
#' xsd_convert(121L, shortform = FALSE)
#'
#' @section Class-specific examples:
#' ```r
#' xsd_convert(42L)                   # integer -> xsd:integer
#' xsd_convert(c(TRUE, FALSE, NA))    # logical -> xsd:boolean
#' xsd_convert(Sys.Date())            # Date -> xsd:date
#' xsd_convert(Sys.time())            # POSIXct -> xsd:dateTime
#' xsd_convert(factor("apple"))       # factor -> xsd:string
#' xsd_convert(c("apple", "banana"))  # character -> xsd:string
#' ```
#' @family RDF and linked data helpers
#' @export

xsd_convert <- function(x, idcol = NULL, shortform = TRUE, ...) {
  UseMethod("xsd_convert", x)
}


#' @keywords internal
get_type <- function(t, shortform = TRUE) {
  base_type <- if (any(class(t) %in% c("numeric", "double"))) {
    "decimal"
  } else if (any(class(t) == "integer")) {
    "integer"
  } else if (any(class(t) %in% c("character", "factor"))) {
    "string"
  } else if (any(class(t) == "logical")) {
    "boolean"
  } else if (any(class(t) == "Date")) {
    "date"
  } else if (any(class(t) == "POSIXct")) {
    "dateTime"
  } else if (any(class(t) == "difftime")) {
    "duration"
  } else {
    "string"
  }

  xsd_uri(base_type, shortform = shortform)
}

#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.haven_labelled_defined <- function(x, idcol = NULL,
                                               shortform = TRUE, ...) {
  type <- get_type(x, shortform)

  if (grepl("decimal$", type)) {
    return(xsd_convert(as_numeric(x), shortform = shortform, ...))
  }
  if (grepl("integer$", type)) {
    return(xsd_convert(as_numeric(x), shortform = shortform, ...))
  }
  if (grepl("string$", type)) {
    return(xsd_convert(as_character(x), shortform = shortform, ...))
  }
  if (grepl("boolean$", type)) {
    return(xsd_convert(as.logical(as_numeric(x)), shortform = shortform, ...))
  }

  stop("Unsupported haven_labelled_defined type: ", paste(class(x), collapse = ", "))
}

#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.data.frame <- function(x, idcol = NULL, shortform = TRUE, ...) {
  # Identify ID column (or default to row names)
  if (!is.null(idcol)) {
    id_idx <- idcol_find(x, idcol)
    ids <- as.character(x[[id_idx]])
    id_name <- names(x)[id_idx]
  } else {
    ids <- row.names(x)
    id_name <- ".rowid"
  }

  convert_cols <- setdiff(
    seq_along(x),
    if (!is.null(idcol)) id_idx else integer(0)
  )

  # Apply xsd_convert to all non-ID columns, forwarding shortform + ...
  xsd_list <- lapply(
    convert_cols,
    function(c) xsd_convert(x[[c]], shortform = shortform, ...)
  )
  names(xsd_list) <- names(x)[convert_cols]

  # Assemble result: ID column first, then converted columns
  result <- c(
    setNames(list(ids), id_name),
    xsd_list
  )

  as.data.frame(result, stringsAsFactors = FALSE)
}


#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.dataset_df <- function(x, idcol = "rowid",
                                   shortform = TRUE, ...) {
  NextMethod("xsd_convert", shortform = shortform, ...)
}

#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.tbl_df <- function(x, idcol = NULL, shortform = TRUE, ...) {
  if (!inherits(x, "data.frame")) {
    stop("xsd_convert.tbl_df: input must be a data frame or tibble.")
  }
  NextMethod("xsd_convert", shortform = shortform, ...)
}

#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.character <- function(x, idcol = NULL,
                                  shortform = TRUE, ...) {
  var_type <- xsd_uri("string", shortform)

  if (length(x) == 0) {
    return(paste0('""^^<', var_type, ">"))
  }

  ifelse(is.na(x),
    NA_character_,
    paste0('"', x, '"^^<', var_type, ">")
  )
}


#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.numeric <- function(x, idcol = NULL,
                                shortform = TRUE, ...) {
  var_type <- xsd_uri("decimal", shortform)
  if (length(x) == 0) {
    return(paste0('""^^<', var_type, ">"))
  }

  formatted <- ifelse(is.na(x), NA_character_, {
    str <- format(x, scientific = FALSE, trim = TRUE, justify = "none")
    sub("\\.0$", "", str)
  })

  ifelse(is.na(formatted),
    NA_character_,
    paste0('"', formatted, '"^^<', var_type, ">")
  )
}

#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.integer <- function(x, idcol = NULL, shortform = TRUE, ...) {
  var_type <- xsd_uri("integer", shortform)

  if (length(x) == 0) {
    return(paste0('""^^<', var_type, ">"))
  }

  ifelse(is.na(x),
    NA_character_,
    paste0('"', as.character(x), '"^^<', var_type, ">")
  )
}


#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.logical <- function(x, idcol = NULL, shortform = TRUE, ...) {
  var_type <- xsd_uri("boolean", shortform)

  if (length(x) == 0) {
    return(paste0('""^^<', var_type, ">"))
  }

  ifelse(
    is.na(x),
    NA_character_,
    paste0('"', tolower(as.character(x)), '"^^<', var_type, ">")
  )
}


#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.factor <- function(x, idcol = NULL, shortform = TRUE, ...) {
  args <- list(...)
  codelist <- args$codelist %||% NULL

  if (length(x) == 0) {
    return(paste0('""^^<', xsd_uri("string", shortform), ">"))
  }

  if (is.null(codelist)) {
    var_type <- xsd_uri("string", shortform)
    ifelse(
      is.na(x),
      NA_character_,
      paste0('"', as.character(x), '"^^<', var_type, ">")
    )
  } else {
    ifelse(
      is.na(x),
      NA_character_,
      paste0(codelist, ":", as.character(x))
    )
  }
}


#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.POSIXct <- function(x, idcol = NULL, shortform = TRUE, ...) {
  var_type <- xsd_uri("dateTime", shortform)

  if (length(x) == 0) {
    return(paste0('""^^<', var_type, ">"))
  }

  time_string <- strftime(x, format = "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")

  ifelse(
    is.na(x),
    NA_character_,
    paste0('"', time_string, '"^^<', var_type, ">")
  )
}

#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.Date <- function(x, idcol = NULL, shortform = TRUE, ...) {
  var_type <- xsd_uri("date", shortform)

  if (length(x) == 0) {
    return(paste0('""^^<', var_type, ">"))
  }

  date_str <- format(x, format = "%Y-%m-%d")

  ifelse(
    is.na(x),
    NA_character_,
    paste0('"', date_str, '"^^<', var_type, ">")
  )
}


#' @rdname xsd_convert
#' @exportS3Method
xsd_convert.difftime <- function(x, idcol = NULL, shortform = TRUE, ...) {
  var_type <- xsd_uri("duration", shortform)

  if (length(x) == 0) {
    return(paste0('""^^<', var_type, ">"))
  }

  seconds <- unclass(x)
  unit <- attr(x, "units")

  multiplier <- switch(unit,
    "secs" = 1,
    "mins" = 60,
    "hours" = 3600,
    "days" = 86400,
    stop("Unsupported difftime unit: ", unit)
  )
  seconds <- seconds * multiplier

  to_iso8601 <- function(s) {
    if (is.na(s)) {
      return(NA_character_)
    }
    h <- floor(s / 3600)
    m <- floor((s %% 3600) / 60)
    sec <- round(s %% 60)
    paste0(
      "PT",
      if (h > 0) paste0(h, "H") else "",
      if (m > 0) paste0(m, "M") else "",
      if (sec > 0 || (h == 0 && m == 0)) paste0(sec, "S") else ""
    )
  }

  iso_strs <- vapply(seconds, to_iso8601, character(1))

  ifelse(
    is.na(seconds),
    NA_character_,
    paste0('"', iso_strs, '"^^<', var_type, ">")
  )
}

#' @keywords internal
xsd_uri <- function(type, shortform = TRUE) {
  base <- "http://www.w3.org/2001/XMLSchema#"
  if (shortform) {
    paste0("xsd:", type)
  } else {
    paste0(base, type)
  }
}

## get_type internal function -------------------------------------
test_that("get_type returns correct XSD types (short and long)", {
  expect_equal(get_type(1.5), "xsd:decimal")
  expect_equal(get_type(1.5, shortform = FALSE), "http://www.w3.org/2001/XMLSchema#decimal")

  expect_equal(get_type(1L), "xsd:integer")
  expect_equal(get_type(1L, shortform = FALSE), "http://www.w3.org/2001/XMLSchema#integer")

  expect_equal(get_type("text"), "xsd:string")
  expect_equal(get_type("text", shortform = FALSE), "http://www.w3.org/2001/XMLSchema#string")

  expect_equal(get_type(TRUE), "xsd:boolean")
  expect_equal(get_type(TRUE, shortform = FALSE), "http://www.w3.org/2001/XMLSchema#boolean")

  expect_equal(get_type(Sys.Date()), "xsd:date")
  expect_equal(get_type(Sys.Date(), shortform = FALSE), "http://www.w3.org/2001/XMLSchema#date")

  expect_equal(get_type(Sys.time()), "xsd:dateTime")
  expect_equal(get_type(Sys.time(), shortform = FALSE), "http://www.w3.org/2001/XMLSchema#dateTime")

  expect_equal(get_type(as.difftime(1, units = "secs")), "xsd:duration")
  expect_equal(get_type(as.difftime(1, units = "secs"), shortform = FALSE), "http://www.w3.org/2001/XMLSchema#duration")
})


## Character conversions ------------------------------------------
test_that("xsd_convert.character handles standard strings", {
  expect_equal(
    xsd_convert(c("apple", " banana ", "cherry")),
    c('"apple"^^<xsd:string>', '" banana "^^<xsd:string>', '"cherry"^^<xsd:string>')
  )
})

test_that("xsd_convert.character handles NA values", {
  result <- xsd_convert(c("apple", NA, "cherry"))
  expect_equal(result[1], '"apple"^^<xsd:string>')
  expect_true(is.na(result[2]))
  expect_equal(result[3], '"cherry"^^<xsd:string>')
})

test_that("xsd_convert.character handles empty strings", {
  expect_equal(xsd_convert(""), '""^^<xsd:string>')
})

test_that("xsd_convert.character handles empty vector", {
  expect_equal(xsd_convert(character(0)), '""^^<xsd:string>')
})

## Numeric conversions ----------------------------------------------------

test_that("xsd_convert.numeric handles standard numeric values", {
  expect_equal(
    xsd_convert(c(1.0, 2.5, 3.0)),
    c('"1"^^<xsd:decimal>', '"2.5"^^<xsd:decimal>', '"3"^^<xsd:decimal>')
  )
})

test_that("xsd_convert.numeric handles NA values", {
  result <- xsd_convert(c(1.0, NA, 2.0))
  expect_equal(result[1], '"1"^^<xsd:decimal>')
  expect_true(is.na(result[2]))
  expect_equal(result[3], '"2"^^<xsd:decimal>')
})

test_that("xsd_convert.numeric handles empty vector", {
  expect_equal(xsd_convert(numeric(0)), '""^^<xsd:decimal>')
})

## Integer conversions ----------------------------------------------------
test_that("xsd_convert.integer handles normal integers", {
  expect_equal(xsd_convert(1:3), c('"1"^^<xsd:integer>', '"2"^^<xsd:integer>', '"3"^^<xsd:integer>'))
})

test_that("xsd_convert.integer handles NA", {
  x <- as.integer(c(42, NA))
  result <- xsd_convert(x)
  expect_equal(result[1], '"42"^^<xsd:integer>')
  expect_true(is.na(result[2]))
})

test_that("xsd_convert.integer handles empty input", {
  expect_equal(xsd_convert(integer(0)), '""^^<xsd:integer>')
})


## Factor conversions -----------------------------------------------------
test_that("xsd_convert.factor handles standard factors without codelist", {
  f <- factor(c("apple", "banana", "cherry"))
  result <- xsd_convert(f)
  expect_equal(result, c(
    '"apple"^^<xsd:string>',
    '"banana"^^<xsd:string>',
    '"cherry"^^<xsd:string>'
  ))
})

test_that("xsd_convert.factor handles factors with NA values", {
  f <- factor(c("apple", NA, "cherry"))
  result <- xsd_convert(f)
  expect_equal(result[1], '"apple"^^<xsd:string>')
  expect_true(is.na(result[2]))
  expect_equal(result[3], '"cherry"^^<xsd:string>')
})

test_that("xsd_convert.factor handles factors with codelist", {
  f <- factor(c("apple", "banana", "cherry"))
  result <- xsd_convert(f, codelist = "fruit")
  expect_equal(result, c(
    "fruit:apple",
    "fruit:banana",
    "fruit:cherry"
  ))
})

test_that("xsd_convert.factor handles factors with codelist and NA values", {
  f <- factor(c("apple", NA, "cherry"))
  result <- xsd_convert(f, codelist = "fruit")
  expect_equal(result[1], "fruit:apple")
  expect_true(is.na(result[2]))
  expect_equal(result[3], "fruit:cherry")
})

test_that("xsd_convert.factor handles empty factor", {
  f <- factor(character(0))
  result <- xsd_convert(f)
  expect_equal(result, '""^^<xsd:string>')
})

## Logical conversions -----------------------------------------------------

test_that("xsd_convert.logical handles TRUE and FALSE correctly", {
  expect_equal(xsd_convert(TRUE), '"true"^^<xsd:boolean>')
  expect_equal(xsd_convert(FALSE), '"false"^^<xsd:boolean>')
})

test_that("xsd_convert.logical returns NA for NA input", {
  expect_true(is.na(xsd_convert(NA)))
  expect_equal(
    xsd_convert(c(TRUE, NA, FALSE)),
    c('"true"^^<xsd:boolean>', NA, '"false"^^<xsd:boolean>')
  )
})

## Date conversions ------------------------------------------------------
test_that("xsd_convert.Date formats correctly", {
  d <- as.Date("2023-05-01")
  expect_equal(xsd_convert(d), '"2023-05-01"^^<xsd:date>')
})

test_that("xsd_convert.Date handles NA", {
  d <- as.Date(NA)
  expect_equal(xsd_convert(d), NA_character_)
})

test_that("xsd_convert.Date handles empty input", {
  d <- as.Date(character(0))
  expect_equal(xsd_convert(d), '""^^<xsd:date>')
})

test_that("xsd_convert.Date works when coerced from POSIXct with timezone", {
  dt <- as.POSIXct("2020-01-01 12:00:00", tz = "Europe/Berlin")
  d <- as.Date(dt)
  expect_equal(xsd_convert(d), '"2020-01-01"^^<xsd:date>')
})


test_that("xsd_convert.POSIXct converts datetime to xsd:dateTime in UTC", {
  ts <- as.POSIXct("2020-01-01 12:34:56", tz = "UTC")
  expect_equal(xsd_convert(ts), '"2020-01-01T12:34:56Z"^^<xsd:dateTime>')
})

test_that("xsd_convert.POSIXct handles POSIXct in local timezone correctly", {
  ts_local <- as.POSIXct("2020-01-01 12:34:56", tz = Sys.timezone())
  result <- xsd_convert(ts_local)

  # Convert to expected UTC
  ts_utc <- format(ts_local, format = "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")
  expected <- paste0('"', ts_utc, '"^^<xsd:dateTime>')

  expect_equal(result, expected)
})

## POSIXct conversions ---------------------------------------------------
test_that("xsd_convert.POSIXct handles a vector of times", {
  ts <- as.POSIXct(c("2020-01-01 12:00:00", "2021-01-01 15:30:00"), tz = "UTC")
  expect_equal(
    xsd_convert(ts),
    c('"2020-01-01T12:00:00Z"^^<xsd:dateTime>', '"2021-01-01T15:30:00Z"^^<xsd:dateTime>')
  )
})

test_that("xsd_convert.POSIXct handles NA datetime values", {
  ts <- as.POSIXct(NA, tz = "UTC")
  expect_equal(xsd_convert(ts), NA_character_)
})


test_that("xsd_convert.POSIXct handles empty vector", {
  ts <- as.POSIXct(character(0))
  expect_equal(xsd_convert(ts), '""^^<xsd:dateTime>')
})

## data.frame conversions ----------------------------------

test_that("xsd_convert.data.frame handles various column types", {
  df <- data.frame(
    id = 1:2,
    name = c("apple", NA),
    price = c(1.50, 2.30),
    available = c(TRUE, NA),
    date = as.Date(c("2020-01-01", NA)),
    stringsAsFactors = FALSE
  )

  result <- xsd_convert(df, idcol = "id")

  expect_equal(result$name[1], '\"apple\"^^<xsd:string>')
  expect_true(is.na(result$name[2]))
  expect_equal(result$price[2], '"2.3"^^<xsd:decimal>')
  expect_equal(result$id, as.character(c(1L, 2L)))
})
#' @rdname xsd_convert
#' @export
#' @exportS3Method
xsd_convert.data.frame <- function(x, idcol = NULL, ...) {
  # Identify ID column (or default to row names)
  if (!is.null(idcol)) {
    id_idx <- idcol_find(x, idcol)
    ids <- as.character(x[[id_idx]])
    id_name <- names(x)[id_idx]
  } else {
    ids <- row.names(x)
    id_name <- ".rowid"
  }

  convert_cols <- setdiff(seq_along(x), if (!is.null(idcol)) id_idx else integer(0))

  # Apply xsd_convert to all non-ID columns
  xsd_list <- lapply(convert_cols, function(c) xsd_convert(x[[c]], ...))
  names(xsd_list) <- names(x)[convert_cols]

  # Assemble result: ID column first, then converted columns
  result <- c(
    setNames(list(ids), id_name),
    xsd_list
  )

  as.data.frame(result, stringsAsFactors = FALSE)
}

## dataset_df  ---------------------------------------------
test_that("xsd_convert handles dataset_df", {
  orange_xsd <- xsd_convert(orange_df, idcol = "rowid")
  expect_equal(orange_xsd$rowid[1:3], c("orange:1", "orange:2", "orange:3"))
  expect_equal(orange_xsd$age[1], '"118"^^<xsd:decimal>')
})

## Empty strings ---------------------------------------------
test_that("xsd_convert handles empty vectors semantically", {
  expect_equal(xsd_convert(character(0)), '""^^<xsd:string>')
  expect_equal(xsd_convert(numeric(0)), '""^^<xsd:decimal>')
  expect_equal(xsd_convert(logical(0)), '""^^<xsd:boolean>')
})

## Conversion of difftime ----------------------------------------
test_that("xsd_convert.difftime handles hours and seconds", {
  x <- as.difftime(c(3600, 5400), units = "secs")
  result <- xsd_convert(x)
  expect_equal(result, c('"PT1H"^^<xsd:duration>', '"PT1H30M"^^<xsd:duration>'))
})

test_that("xsd_convert.difftime handles NA values", {
  x <- as.difftime(c(3600, NA, 7200), units = "secs")
  result <- xsd_convert(x)
  expect_equal(result[1], '"PT1H"^^<xsd:duration>')
  expect_true(is.na(result[2]))
  expect_equal(result[3], '"PT2H"^^<xsd:duration>')
})

test_that("xsd_convert.difftime handles empty input", {
  x <- as.difftime(numeric(0), units = "secs")
  expect_equal(xsd_convert(x), '""^^<xsd:duration>')
})

## Longform conversions -----------------------------------------------

test_that("xsd_convert produces longform URIs correctly", {
  # character
  expect_equal(
    xsd_convert("apple", shortform = FALSE),
    '"apple"^^<http://www.w3.org/2001/XMLSchema#string>'
  )

  # numeric
  expect_equal(
    xsd_convert(3.14, shortform = FALSE),
    '"3.14"^^<http://www.w3.org/2001/XMLSchema#decimal>'
  )

  # integer
  expect_equal(
    xsd_convert(42L, shortform = FALSE),
    '"42"^^<http://www.w3.org/2001/XMLSchema#integer>'
  )

  # logical
  expect_equal(
    xsd_convert(TRUE, shortform = FALSE),
    '"true"^^<http://www.w3.org/2001/XMLSchema#boolean>'
  )

  # factor
  f <- factor("banana")
  expect_equal(
    xsd_convert(f, shortform = FALSE),
    '"banana"^^<http://www.w3.org/2001/XMLSchema#string>'
  )

  # Date
  d <- as.Date("2023-05-01")
  expect_equal(
    xsd_convert(d, shortform = FALSE),
    '"2023-05-01"^^<http://www.w3.org/2001/XMLSchema#date>'
  )

  # POSIXct
  ts <- as.POSIXct("2020-01-01 12:34:56", tz = "UTC")
  expect_equal(
    xsd_convert(ts, shortform = FALSE),
    '"2020-01-01T12:34:56Z"^^<http://www.w3.org/2001/XMLSchema#dateTime>'
  )

  # difftime
  dt <- as.difftime(3600, units = "secs")
  expect_equal(
    xsd_convert(dt, shortform = FALSE),
    '"PT1H"^^<http://www.w3.org/2001/XMLSchema#duration>'
  )
})

test_that("xsd_convert.data.frame works with longform URIs", {
  df <- data.frame(
    id = 1:2,
    value = c(3.14, 2.71),
    active = c(TRUE, FALSE),
    date = as.Date(c("2020-01-01", "2020-12-31")),
    stringsAsFactors = FALSE
  )

  result <- xsd_convert(df, idcol = "id", shortform = FALSE)

  expect_equal(result$value[1], '"3.14"^^<http://www.w3.org/2001/XMLSchema#decimal>')
  expect_equal(result$active[2], '"false"^^<http://www.w3.org/2001/XMLSchema#boolean>')
  expect_equal(result$date[1], '"2020-01-01"^^<http://www.w3.org/2001/XMLSchema#date>')
})


test_that("xsd_convert handles dataset_df with longform URIs", {
  orange_xsd <- xsd_convert(orange_df, idcol = "rowid", shortform = FALSE)
  expect_equal(orange_xsd$age[1], '"118"^^<http://www.w3.org/2001/XMLSchema#decimal>')
})

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

if (!requireNamespace("rdflib",
  quietly = TRUE
)) {
  stop("Please install 'rdflib' to run this vignette.")
}

## ----definegdpdataset---------------------------------------------------------
library(dataset)

gdp <- defined(
  c(2355, 2592, 2884),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo <- defined(
  rep("AD", 3),
  label = "Geopolitical Entity",
  concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
  namespace = "https://www.geonames.org/countries/$1/"
)

gdp
geo

## ----smalldatasetexample------------------------------------------------------
small_dataset <- dataset_df(
  geo = geo,
  gdp = gdp,
  identifier = c(gdp = "http://example.com/dataset#gdp"),
  dataset_bibentry = dublincore(
    title = "Small GDP Dataset",
    creator = person("Jane", "Doe", role = "aut"),
    publisher = "Small Repository",
    subject = "Gross Domestic Product"
  )
)

small_dataset

## ----dublincoremetadata-------------------------------------------------------
as_dublincore(small_dataset)

## ----triplesexample-----------------------------------------------------------
triples <- dataset_to_triples(small_dataset,
  format = "nt"
)
triples

## ----ntexample----------------------------------------------------------------
mycon <- tempfile("my_dataset",
  fileext = "nt"
)
my_description <- describe(
  x = small_dataset,
  con = mycon
)

# Only three statements are shown:
readLines(mycon)[c(4, 8, 12)]

## ----provenancexample---------------------------------------------------------
## Show two lines of provenance:
provenance(small_dataset)[c(6, 7)]

## ----smalldf------------------------------------------------------------------
small_df <- as.data.frame(small_dataset,
  strip_attributes = FALSE
)

attr(small_dataset, "subject")

## ----smalltbl-----------------------------------------------------------------
small_tbl <- as_tibble(
  small_dataset,
  strip_attributes = TRUE
)

small_tbl

#' @title Bind strictly defined rows
#' @description Add rows of dataset \code{y} to dataset \code{x}, validating all
#'   semantic metadata. Metadata (labels, units, concept definitions,
#'   namespaces) must match exactly. Additional dataset-level metadata such as
#'   title and creator can be overridden using \code{...}.
#'
#' @details This function combines two semantically enriched datasets created
#'   with \code{dataset_df()}. All variable-level attributes — including labels,
#' units, concept definitions, and namespaces — must match. If \code{strict =
#' TRUE} (the default), the row identifier namespace (used in the \code{rowid}
#'   column) must also match exactly.
#'
#'   If \code{strict = FALSE}, row identifiers from \code{y} may differ and will
#'   be ignored; the output will inherit \code{x}'s row identifier scheme.
#'
#' @param x A `dataset_df` object.
#' @param y A `dataset_df` object to bind to `x`.
#' @param ... Optional dataset-level attributes such as \code{title} or
#'   \code{creator} to override.
#' @param strict Logical. If \code{TRUE} (default), require full semantic
#'   compatibility, including rowid.
#'
#' @return A new `dataset_df` object with rows from `x` and `y`, combined
#'   semantically.
#'
#' @export
#' @examples
#' A <- dataset_df(
#'   length = defined(c(10, 15),
#'     label = "Length",
#'     unit = "cm", namespace = "http://example.org"
#'   ),
#'   identifier = c(id = "http://example.org/dataset#"),
#'   dataset_bibentry = dublincore(
#'     title = "Dataset A",
#'     creator = person("Alice", "Smith")
#'   )
#' )
#'
#' B <- dataset_df(
#'   length = defined(c(20, 25),
#'     label = "Length",
#'     unit = "cm", namespace = "http://example.org"
#'   ),
#'   identifier = c(id = "http://example.org/dataset#")
#' )
#'
#' bind_defined_rows(A, B) # succeeds
#'
#' C <- dataset_df(
#'   length = defined(c(30, 35),
#'     label = "Length",
#'     unit = "cm", namespace = "http://example.org"
#'   ),
#'   identifier = c(id = "http://another.org/dataset#")
#' )
#'
#' \dontrun{
#' bind_defined_rows(A, C, strict = TRUE) # fails: mismatched rowid
#' }
#'
#' bind_defined_rows(A, C, strict = FALSE) # succeeds: rowid inherited
bind_defined_rows <- function(x, y, ..., strict = FALSE) {
  dots <- list(...)

  if (!inherits(x, "dataset_df")) {
    stop(
      "`x` must be a dataset_df object. Got: ",
      paste(class(x), collapse = ", ")
    )
  }

  if (!inherits(y, "dataset_df")) {
    stop(
      "`y` must be a dataset_df object. Got: ",
      paste(class(y), collapse = ", ")
    )
  }


  # Strict mode: also validate rowid identifier
  if (strict) {
    ns_x <- namespace_attribute(x[["rowid"]])
    ns_y <- namespace_attribute(y[["rowid"]])

    if (!identical(ns_x, ns_y)) {
      stop("Row identifier namespaces must match in strict mode.")
    }
  }

  # Exclude rowid from semantic checks
  vars <- setdiff(names(x), "rowid")

  if (!identical(names(x)[vars], names(y)[vars])) {
    stop("Error: Column names must match between datasets.")
  }

  if (!identical(var_label(x[vars]), var_label(y[vars]))) {
    stop("Error: Variable labels must match in the two datasets.")
  }

  if (!identical(lapply(x[vars], var_unit), lapply(y[vars], var_unit))) {
    stop("Error: Variable units must match in the two datasets.")
  }

  if (!identical(
    lapply(x[vars], function(i) attr(i, "concept")),
    lapply(y[vars], function(j) attr(j, "concept"))
  )) {
    stop("Variable concept definitions must match in the two datasets.")
  }

  if (!identical(
    lapply(x[vars], namespace_attribute),
    lapply(y[vars], namespace_attribute)
  )
  ) {
    stop("Variable namespaces must match in the two datasets.")
  }

  # Bind only variable columns (exclude rowid)
  new_data <- mapply(function(a, b) c(a, b), x[vars], y[vars], SIMPLIFY = FALSE)

  # Use the rowid namespace from x; allow override later if needed
  new_dataset <- do.call(dataset_df, c(new_data, list(identifier = namespace_attribute(x[["rowid"]]))))

  # Handle title/creator if supplied
  if (!is.null(dots$title)) dataset_title(new_dataset, overwrite = TRUE) <- dots$title
  if (!is.null(dots$creator)) {
    creator(new_dataset) <- dots$creator
  } else {
    creator(new_dataset) <- compare_creators(x, y)
  }

  attr(new_dataset, "dataset_bibentry") <- attr(x, "dataset_bibentry")
  new_dataset
}

test_that("bind_defined_rows() works", {
  A <- dataset_df(
    a = defined(c(11, 14, 16), label = "length", unit = "cm"),
    dataset_bibentry = dublincore(
      title = "Test", creator = person("Jane Doe"),
      dataset_date = Sys.Date()
    )
  )
  B <- dataset_df(
    a = defined(c(12, 17, 19), label = "length", unit = "cm"),
    dataset_bibentry = dublincore(
      title = "Test", creator = person("Jane Doe")
    )
  )
  D <- bind_defined_rows(x = A, y = B)
  expect_equal(as_numeric(D$a), c(11, 14, 16, 12, 17, 19))
  expect_equal(D$a, defined(c(11, 14, 16, 12, 17, 19),
    label = "length", unit = "cm"
  ))
  expect_equal(dataset_title(A), dataset_title(D))
  expect_equal(get_bibentry(A)$year, get_bibentry(D)$year)
})

test_that("bind_defined_rows() detects if not dataset_df is
          the input parameter", {
  A <- dataset_df(
    a = defined(c(11, 14, 16),
      label = "length",
      unit = "cm"
    ),
    dataset_bibentry = dublincore(
      title = "Test", creator = person("Jane Doe"),
      dataset_date = Sys.Date()
    )
  )
  expect_error(
    bind_defined_rows(
      y = A,
      x = data.frame()
    ),
    regexp = "x\\` must be a dataset_df object"
  )
  expect_error(
    bind_defined_rows(A,
      y = data.frame()
    ),
    regexp = "y\\` must be a dataset_df"
  )
})

test_that("bind_defined_rows() detects different labels", {
  A <- dataset_df(
    width = defined(c(1, 2), label = "width", unit = "cm"),
    height = defined(c(3, 4), label = "height", unit = "cm")
  )
  B <- dataset_df(
    width = defined(c(3, 4), unit = "cm"),
    height = defined(c(2, 2), label = "height", unit = "cm")
  )
  expect_error(bind_defined_rows(x = A, y = B),
    regexp = "Variable labels"
  )
})

test_that("bind_defined_rows() detects different units", {
  A <- dataset_df(
    width = defined(c(1, 2), label = "width", unit = "cm"),
    height = defined(c(3, 4), label = "height", unit = "mm")
  )
  B <- dataset_df(
    width = defined(c(3, 4), label = "width", unit = "cm"),
    height = defined(c(2, 2), label = "height", unit = "cm")
  )
  expect_error(bind_defined_rows(x = A, y = B),
    regexp = "Variable units"
  )
})

test_that("bind_defined_rows() detects different definitions", {
  A <- dataset_df(
    width = defined(c(1, 2), label = "width", concept = "cm"),
    height = defined(c(3, 4), label = "height", concept = "mm")
  )
  B <- dataset_df(
    width = defined(c(3, 4), label = "width", concept = "cm"),
    height = defined(c(2, 2), label = "height", concept = "cm")
  )
  expect_error(bind_defined_rows(x = A, y = B),
    regexp = "Variable concept definitions"
  )
})


test_that("bind_defined_rows() detects different namespace", {
  A <- dataset_df(
    width = defined(c(1, 2),
      label = "width",
      unit = "cm", namespace = "http://example.com"
    ),
    height = defined(c(3, 4),
      label = "height",
      unit = "cm", namespace = "http://example.com"
    )
  )
  B <- dataset_df(
    width = defined(c(3, 4),
      label = "width",
      unit = "cm", namespace = "http://example.com"
    ),
    height = defined(c(2, 2), label = "height", unit = "cm")
  )
  expect_error(bind_defined_rows(x = A, y = B),
    regexp = "Variable namespaces"
  )
})

test_that("bind_defined_rows() has the same rowid namespace", {
  A <- dataset_df(
    width = defined(c(1, 2),
      label = "width",
      unit = "cm", namespace = "http://example.com"
    ),
    height = defined(c(3, 4),
      label = "height",
      unit = "cm", namespace = "http://example.com"
    ),
    identifier = c(wbi = "https:://example.com/")
  )
  B <- dataset_df(
    width = defined(c(3, 4),
      label = "width",
      unit = "cm", namespace = "http://example.com"
    ),
    height = defined(c(2, 2),
      label = "height",
      unit = "cm", namespace = "http://example.com"
    ),
    identifier = c(wbi = "https:://example.com/")
  )
  D <- dataset_df(
    width = defined(c(3, 4),
      label = "width",
      unit = "cm", namespace = "http://example.com"
    ),
    height = defined(c(2, 2),
      label = "height",
      unit = "cm", namespace = "http://example.com"
    ),
    identifier = c(wbi = "https:://example.net/")
  )
  expect_equal(
    namespace_attribute(bind_defined_rows(x = A, y = B)$rowid),
    namespace_attribute(B$rowid)
  )
  expect_error(bind_defined_rows(x = A, y = D, strict = TRUE),
    regexp = "Row identifier namespaces must match"
  )
})

#' Create N-Triples
#'
#' Create RDF triple statements to annotate your dataset with standard,
#' interoperable metadata.
#'
#' N-Triples is a line-based serialization format for RDF. It is easy to parse
#' and widely supported. For details, see the
#' [W3C RDF 1.2 N-Triples specification](https://www.w3.org/TR/rdf12-n-triples/).
#'
#' @param triples A character vector of concatenated N-Triples, created with
#'   [`n_triple()`].
#'
#' @return A character vector of unique N-Triple strings.
#'
#' @examples
#' triple_1 <- n_triple(
#'   "http://example.org/show/218",
#'   "http://www.w3.org/2000/01/rdf-schema#label",
#'   "That Seventies Show"
#' )
#'
#' triple_2 <- n_triple(
#'   "http://example.org/show/218",
#'   "http://example.org/show/localName",
#'   '"Cette Série des Années Septante"@fr-be'
#' )
#'
#' n_triples(c(triple_1, triple_2, triple_1))
#'
#' @export


n_triples <- function(triples) {
  unique(triples)
}

#' @title Create an N-Triple
#' @description Create a single N-Triple triple.
#' @details N-Triples is an easy to parse line-based subset of Turtle to serialize
#' RDF. An N-Triple triple is a sequence of RDF terms representing the subject,
#'  predicate and object of an RDF Triple. Use [n_triples()] to serialize
#'  multiple statements.
#' @source [RDF 1.1 N-Triples](https://www.w3.org/TR/n-triples/)
#' @param s The subject of a triplet.
#' @param p The predicate of a triplet.
#' @param o The object of a triplet.
#' @return A character vector containing one N-Triple string.
#' @examples
#' s <- "http://example.org/show/218"
#' p <- "http://www.w3.org/2000/01/rdf-schema#label"
#' o <- "That Seventies Show"
#' n_triple(s, p, o)
#' @export
n_triple <- function(s, p, o) {
  if (length(o) != 1) {
    stop("n_triple(): object 'o' must be a scalar (length = 1), got: ", length(o))
  }

  s <- create_iri(s)
  p <- create_iri(p)

  # Don't convert `o` if it's already quoted with ^^<...>
  if (grepl('^".+"\\^\\^<.+>$', o) || grepl("^<.+>$", o)) {
    # o is already a well-formed literal or URI
  } else {
    o <- create_iri(o)
  }

  sprintf("%s %s %s .", s, p, o)
}


#' @keywords internal
create_iri <- function(x) {
  if (length(x) != 1) {
    stop("create_iri(): input must be a scalar value (length = 1).")
  }

  if (any(c("list", "data.frame", "tbl", "data.table") %in% class(x))) {
    stop("Error: create_iri(x) must be a scalar URI, string, integer, double, Date, dateTime, or person.")
  }

  double_string <- "^^<http://www.w3.org/2001/XMLSchema#double>"
  integer_string <- "^^<http://www.w3.org/2001/XMLSchema#integer>"
  character_string <- "^^<http://www.w3.org/2001/XMLSchema#string>"
  date_string <- "^^<http://www.w3.org/2001/XMLSchema#date>"
  datetime_string <- "^^<http://www.w3.org/2001/XMLSchema#dateTime>"

  if (inherits(x, "person")) {
    if ("isni" %in% tolower(names(x$comment))) {
      x <- paste0("https://isni.org/isni/", x$comment[which(tolower(names(x$comment)) == "isni")])
    } else if ("orcid" %in% tolower(names(x$comment))) {
      x <- paste0("https://orcid.org/", x$comment[which(tolower(names(x$comment)) == "orcid")])
    } else if ("viaf" %in% tolower(names(x$comment))) {
      x <- paste0("https://viaf.org/viaf/", x$comment[which(tolower(names(x$comment)) == "viaf")])
    } else if ("wikidata" %in% tolower(names(x$comment))) {
      qid <- x$comment[which(tolower(names(x$comment)) == "wikidata")]
      qid <- gsub("https://www.wikidata.org/wiki/", "", qid)
      x <- paste0("https://www.wikidata.org/wiki/", qid)
    } else {
      name_parts <- c(x$given, x$family)
      role_part <- if (!is.null(x$role)) paste0("[", x$role, "]") else ""
      x <- paste(trimws(paste(name_parts, collapse = " ")), role_part)
    }
  }

  if (is.integer(x)) {
    sprintf('"%s"%s', as.character(x), integer_string)
  } else if (inherits(x, "POSIXct")) {
    xsd_convert(x)
  } else if (is.character(x) && substr(x, 1, 5) %in% c("http:", "https")) {
    sprintf("<%s>", as.character(x))
  } else if (grepl("^_\\:", x)) {
    x # return blank node unquoted
  } else if (grepl("@", x)) {
    sprintf('"%s"', x)
  } else if (inherits(x, "Date")) {
    sprintf('"%s"%s', as.character(x), date_string)
  } else if (is.numeric(x)) {
    sprintf('"%s"%s', as.character(x), double_string)
  } else if (x == "a") {
    "<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>"
  } else if (is.character(x)) {
    x <- gsub("DCMITYPE\\:", "http://purl.org/dc/dcmitype/", x)
    sprintf('"%s"%s', as.character(x), character_string)
  }
}


#' @keywords internal
prov_author <- function(author_person) {
  # Handle multiple authors
  if (length(author_person) > 1) {
    return(unlist(lapply(author_person, prov_author), use.names = FALSE))
  }

  person_iri <- NULL # <-- initialize early
  print_name <- ""

  if (inherits(author_person, "person")) {
    print_name <- "_:"
    if (!is.null(author_person$family)) print_name <- paste0(print_name, tolower(author_person$family))
    if (!is.null(author_person$given)) print_name <- paste0(print_name, tolower(author_person$given))
    person_iri <- get_person_iri(author_person)
  } else if (is.character(attr(author_person, "person"))) {
    print_name <- paste0(attr(author_person, "person"), ": ")
  }

  if (!is.null(person_iri)) {
    n_triple(person_iri, "a", "http://www.w3.org/ns/prov#Agent")
  } else {
    n_triple(print_name, "a", "http://www.w3.org/ns/prov#Agent")
  }
}

#' @keywords internal
get_person_iri <- function(p) {
  assertthat::assert_that(inherits(p, "person"),
    msg = "Error: get_person_iri(p): p is not a utils::person object."
  )

  if (!is.null(p$comment)) {
    comment_names <- tolower(names(p$comment))
    comment_values <- p$comment

    if ("orcid" %in% comment_names) {
      id <- comment_values[which(comment_names == "orcid")[1]]
      if (!grepl("^https://orcid.org/", id)) id <- paste0("https://orcid.org/", id)
      return(id)
    } else if ("isni" %in% comment_names) {
      id <- comment_values[which(comment_names == "isni")[1]]
      if (!grepl("^https://isni.org/isni/", id)) id <- paste0("https://isni.org/isni/", id)
      return(id)
    } else if ("viaf" %in% comment_names) {
      id <- comment_values[which(comment_names == "viaf")[1]]
      if (!grepl("^https://viaf.org/viaf/", id)) id <- paste0("https://viaf.org/viaf/", id)
      return(id)
    } else if ("wikidata" %in% comment_names) {
      qid <- comment_values[which(comment_names == "wikidata")[1]]
      qid <- sub("^https://www.wikidata.org/wiki/", "", qid) # strip if already full
      return(paste0("https://www.wikidata.org/wiki/", qid))
    }
  }

  NULL
}


#' @keywords internal
expand_triples <- function(dataset_id, predicate_uri, values) {
  if (is.null(values)) {
    return(character(0))
  }

  # Normalize: wrap scalar in list
  if (!is.list(values)) values <- as.list(values)

  # Filter invalid/placeholder values
  values <- Filter(function(x) {
    !is.null(x) &&
      length(x) == 1 &&
      !is.na(x) &&
      !(as.character(x) %in% c("", ":unas", ":tba"))
  }, values)

  if (length(values) == 0) {
    return(character(0))
  }

  vapply(values, function(val) {
    if (inherits(val, "person")) {
      val <- format(val)
    }
    n_triple(dataset_id, predicate_uri, val)
  }, character(1))
}

test_that("n_triple and related helpers produce correct N-Triple strings", {
  # String literal object
  triple_1 <- n_triple(
    "http://example.org/show/218",
    "http://www.w3.org/2000/01/rdf-schema#label",
    "That Seventies Show"
  )

  # Language-tagged literal object
  triple_2 <- n_triple(
    "http://example.org/show/218",
    "http://example.org/show/localName",
    '"Cette Série des Années Septante"@fr-be'
  )

  # XSD datetime literal from POSIXct
  triple_time <- n_triple(
    "http://example.com/creation",
    "http://www.w3.org/ns/prov#generatedAtTime",
    as.POSIXct(10000, origin = "2024-01-01", tz = "UTC")
  )

  # ORCID agent as subject
  triple_agent <- n_triple(
    "https://orcid.org/0000-0001-7513-6760",
    "a",
    "http://www.w3.org/ns/prov#Agent"
  )

  # Person object with ORCID
  author_person <- person(
    given = "Daniel",
    family = "Antal",
    comment = c(ORCID = "https://orcid.org/0000-0001-7513-6760")
  )

  # Expectations
  expect_equal(
    triple_1,
    '<http://example.org/show/218> <http://www.w3.org/2000/01/rdf-schema#label> "That Seventies Show"^^<http://www.w3.org/2001/XMLSchema#string> .'
  )

  expect_equal(
    triple_time,
    '<http://example.com/creation> <http://www.w3.org/ns/prov#generatedAtTime> "2024-01-01T02:46:40Z"^^<xsd:dateTime> .'
  )

  expect_equal(length(n_triples(c(triple_1, triple_2, triple_1))), 2)
  expect_equal(length(n_triples(c(triple_1, triple_2))), 2)

  expect_equal(
    triple_agent,
    "<https://orcid.org/0000-0001-7513-6760> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Agent> ."
  )

  # create_iri() datatype coercion checks
  expect_equal(create_iri(23), '"23"^^<http://www.w3.org/2001/XMLSchema#double>')
  expect_equal(create_iri("23"), '"23"^^<http://www.w3.org/2001/XMLSchema#string>')
  expect_equal(create_iri(as.integer(23)), '"23"^^<http://www.w3.org/2001/XMLSchema#integer>')
  expect_equal(create_iri(as.Date("2024-10-30")), '"2024-10-30"^^<http://www.w3.org/2001/XMLSchema#date>')

  # prov_author
  expect_equal(
    prov_author(author_person),
    "<https://orcid.org/0000-0001-7513-6760> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Agent> ."
  )
})

test_that("n_triple() gives error", {
  expect_error(n_triple(s = c("1", "2"), p = "author", o = "Jane Doe"),
    regexp = "a scalar value"
  )
})


test_that("expand_triples handles multiple values and filtering", {
  dataset_id <- "http://example.org/demo"
  predicate <- "http://purl.org/dc/terms/creator"

  # Test with single string
  single <- expand_triples(dataset_id, predicate, "Jane Doe")
  expect_type(single, "character")
  expect_length(single, 1)
  expect_true(grepl("Jane Doe", single))

  # Test with multiple strings
  multi <- expand_triples(dataset_id, predicate, c("Jane Doe", "John Smith"))
  expect_length(multi, 2)
  expect_true(any(grepl("Jane Doe", multi)))
  expect_true(any(grepl("John Smith", multi)))

  # Test with person objects
  creators <- c(
    person(given = "Ada", family = "Lovelace"),
    person(given = "Grace", family = "Hopper")
  )
  people <- expand_triples(dataset_id, predicate, creators)
  expect_length(people, 2)
  expect_true(any(grepl("Lovelace", people)))
  expect_true(any(grepl("Hopper", people)))

  # Test filtering of placeholder values
  placeholders <- expand_triples(dataset_id, predicate, c(":tba", ":unas", "", NA, NULL))
  expect_length(placeholders, 0)

  # Mixed valid and invalid
  mixed <- expand_triples(dataset_id, predicate, c("Valid Name", ":tba", NA, "", ":unas"))
  expect_length(mixed, 1)
  expect_true(grepl("Valid Name", mixed))
})


test_that("create_iri() helper function", {
  author_person <- person(
    given = "Daniel", family = "Antal",
    email = "daniel.antal@dataobservatory.eu",
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0001-7513-6760")
  )
  expect_error(create_iri(list(a = 1:2)))
  # expect_output(print(create_iri(as.POSIXct(10000, origin = "2024-01-01", tz="UTC"))), "2024-01-01T03:46:40Z")
  # expect_output(print(create_iri(as.POSIXct(10000, origin = "2024-01-01", tz="UTC"))), "\\^\\^<xsd:dateTime>")
  expect_equal(create_iri(author_person), "<https://orcid.org/0000-0001-7513-6760>")
  jane_doe <- person(given = "Jane", family = "Doe", role = "aut", email = "example@example.com")
  joe_doe <- person(
    given = "Joe", family = "Doe", role = "aut", email = "example@example.com",
    comment = c(Wikidata = "https://www.wikidata.org/wiki/Q000")
  )
  expect_equal(create_iri(x = joe_doe), "<https://www.wikidata.org/wiki/Q000>")
  joe_doe <- person(
    given = "Joe", family = "Doe", role = "aut", email = "example@example.com",
    comment = c(ISNI = "1234")
  )
  expect_equal(create_iri(x = joe_doe), "<https://isni.org/isni/1234>")
  viaf_doe <- person(
    given = "Joe", family = "Doe", role = "aut", email = "example@example.com",
    comment = c(VIAF = "1234")
  )
  expect_equal(create_iri(x = viaf_doe), "<https://viaf.org/viaf/1234>")
  expect_equal(create_iri(x = 2), "\"2\"^^<http://www.w3.org/2001/XMLSchema#double>")
  expect_true(grepl("http://www.w3.org/2001/XMLSchema#date>", create_iri(Sys.Date())))
})

test_that("get_person_iri() provides the reformatted iri for the persons", {
  author_person <- person(
    given = "Daniel", family = "Antal",
    email = "daniel.antal@dataobservatory.eu",
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0001-7513-6760")
  )
  expect_equal(get_person_iri(author_person), "https://orcid.org/0000-0001-7513-6760")
  expect_equal(get_person_iri(person("Jane Doe")), NULL)
  expect_equal(
    get_person_iri(p = person(
      given = "Daniel", family = "Antal",
      role = "cre", comment = c(ORCID = "0000-0001-7513-6760")
    )),
    "https://orcid.org/0000-0001-7513-6760"
  )
  expect_equal(
    get_person_iri(
      p =
        person(
          given = "Edgar", family = "Anderson",
          role = "cre", comment = c(VIAF = "https://viaf.org/viaf/6440526")
        )
    ),
    c(VIAF = "https://viaf.org/viaf/6440526")
  )
  expect_equal(
    get_person_iri(
      p =
        person(given = "Taylor", family = "Swift", role = "cre", comment = c(ISNI = "https://isni.org/isni/0000000078519858"))
    ),
    c(ISNI = "https://isni.org/isni/0000000078519858")
  )
})

test_that("create_iri() handles scalar types correctly", {
  # Numeric (double)
  expect_equal(create_iri(3.14), '"3.14"^^<http://www.w3.org/2001/XMLSchema#double>')

  # Integer
  expect_equal(create_iri(as.integer(42)), '"42"^^<http://www.w3.org/2001/XMLSchema#integer>')

  # Date
  expect_equal(create_iri(as.Date("2024-01-01")), '"2024-01-01"^^<http://www.w3.org/2001/XMLSchema#date>')

  # POSIXct
  dt <- as.POSIXct("2024-01-01 12:34:56", tz = "UTC")
  expect_equal(create_iri(dt), '"2024-01-01T12:34:56Z"^^<xsd:dateTime>')

  # URI string
  expect_equal(create_iri("https://example.org/id"), "<https://example.org/id>")

  # Language-tagged literal
  expect_equal(create_iri('"Some value"@en'), '"\"Some value\"@en"')

  # Blank node (now unquoted)
  expect_equal(create_iri("_:anon1"), "_:anon1")

  # RDF type shortcut
  expect_equal(create_iri("a"), "<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>")

  # DCMI type abbreviation
  expect_equal(
    create_iri("DCMITYPE:Dataset"),
    '"http://purl.org/dc/dcmitype/Dataset"^^<http://www.w3.org/2001/XMLSchema#string>'
  )

  # Plain string
  expect_equal(create_iri("plain string"), '"plain string"^^<http://www.w3.org/2001/XMLSchema#string>')
})

test_that("create_iri() handles person() objects correctly", {
  # Person with VIAF
  p1 <- person("Jane", "Doe", comment = c(viaf = "12345"))
  expect_equal(create_iri(p1), "<https://viaf.org/viaf/12345>")

  # Person with ORCID
  p2 <- person("J", "Doe", comment = c(orcid = "0000-0001-2345-6789"))
  expect_equal(create_iri(p2), "<https://orcid.org/0000-0001-2345-6789>")
})

test_that("create_iri() fallback person gets a string literal", {
  p3 <- person("Harper", "Lee", role = "aut", email = "test@example.com")
  iri_p3 <- create_iri(p3)

  expect_match(iri_p3, '^"[^"]+"\\^\\^<http://www.w3.org/2001/XMLSchema#string>$')
  expect_true(grepl("Harper|Lee", iri_p3))
})

test_that("create_iri() fails for vector inputs", {
  expect_error(create_iri(c("a", "b")), "input must be a scalar value")
})

test_that("create_iri() fails for list-like or tabular inputs", {
  expect_error(create_iri(list(a = 1)), "must be a scalar")
  expect_error(create_iri(data.frame(x = 1)), "must be a scalar")
})


test_that("prov_author() handles different person identifiers correctly", {
  # ORCID
  p_orcid <- person("Jane", "Doe", comment = c(ORCID = "0000-0002-1825-0097"))
  triple_orcid <- prov_author(p_orcid)
  expect_match(triple_orcid, "<https://orcid.org/0000-0002-1825-0097> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Agent> \\.")

  # ISNI
  p_isni <- person("Alan", "Smith", comment = c(ISNI = "0000000121032683"))
  triple_isni <- prov_author(p_isni)
  expect_match(triple_isni, "<https://isni.org/isni/0000000121032683> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Agent> \\.")

  # VIAF
  p_viaf <- person("Nick", "Carraway", comment = c(VIAF = "12345678"))
  triple_viaf <- prov_author(p_viaf)
  expect_match(triple_viaf, "<https://viaf.org/viaf/12345678> <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.w3.org/ns/prov#Agent> \\.")

  # Wikidata
  p_qid <- person("Fitzgerald", "Scott", comment = c(wikidata = "Q39232"))
  triple_qid <- prov_author(p_qid)
  expect_match(
    triple_qid,
    "^<https://www\\.wikidata\\.org/wiki/Q39232> <http://www\\.w3\\.org/1999/02/22-rdf-syntax-ns#type> <http://www\\.w3\\.org/ns/prov#Agent> \\.$"
  )

  # Fallback to blank node with label
  p_fallback <- person("Harper", "Lee", role = "aut")
  triple_fallback <- prov_author(p_fallback)
  expect_match(
    triple_fallback,
    "^_:[a-z]+ <http://www\\.w3\\.org/1999/02/22-rdf-syntax-ns#type> <http://www\\.w3\\.org/ns/prov#Agent> \\.$"
  )

  # Multiple persons
  multi <- personList(p_orcid, p_isni)
  triples_multi <- prov_author(multi)
  expect_length(triples_multi, 2)
  expect_true(all(grepl("^<https://", triples_multi)))
})

test_that("dataset_df sets default provenance on creation", {
  data("gdp", package = "dataset")

  test_dataset <- dataset_df(
    rowid = defined(paste0("eg:", 1:2),
      namespace = "http://example.com/dataset#"
    ),
    geo = defined(
      gdp$geo[1:2],
      label = "Geopolitical entity",
      concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
      namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/geo/$1"
    ),
    dataset_bibentry = dublincore(
      title = "Test Dataset",
      creator = person("Daniel", "Antal",
        comment = c(ORCID = "0000-0001-7513-6760")
      ),
      publisher = "Test Publisher"
    )
  )

  prov <- provenance(test_dataset)

  # Check that some default triples are included
  expect_type(prov, "character")
  expect_true(any(grepl("prov#Entity", prov)))
  expect_true(any(grepl("prov#Agent", prov)))
  expect_true(any(grepl("generatedAtTime", prov)))
})


test_that("provenance(x) works", {
  expect_error(provenance(mtcars))
  tested <- dataset_df(x = mtcars)
  expect_true(class(provenance(tested)[1]) == "character")
  expect_output(print(provenance(tested)), "<http://example.com/dataset#>")
  provenance(x = tested) <- n_triple("https://doi.org/10.5281/zenodo.10396807", "http://www.w3.org/ns/prov#wasInformedBy", "http://example.com/source#1")
  expect_output(print(provenance(tested)), "<http://example.com/source#1>")
})

#' Create a Modern Metadata Object Compatible with bibentry
#'
#' Constructs a [utils::bibentry()] object extended with Dublin Core and
#' DataCite-compatible fields. This unified structure supports use with
#' functions such as [dublincore()] and [datacite()], and is the internal
#' format for storing rich metadata with datasets.
#'
#' @param title A character string specifying the dataset title.
#' @param author A [utils::person()] or list/vector of person objects. Mapped to
#'   `creator` in DataCite and DCMI.
#' @param contributor Optional list or vector of [utils::person()] objects.
#'   Contributor roles are merged if duplicated.
#' @param publisher A character string or [utils::person()] representing the
#'   publishing entity.
#' @param year Publication year. Automatically derived from `date` if not
#'   provided explicitly.
#' @param date A [Date] object or character string in ISO format.
#' @param identifier A persistent identifier (e.g., DOI or URL).
#' @param subject Optional keyword, tag, or controlled vocabulary term.
#' @param ... Additional fields such as `language`, `format`, `rights`, or
#'   `description`.
#'
#' @return
#' An object of class `"bibrecord"` and `"bibentry"`, suitable for citation and
#' embedding in metadata-aware structures such as [dataset_df()].
#'
#' @examples
#' bibrecord(
#'   title = "Gross domestic product, volumes",
#'   author = person("Eurosat"),
#'   publisher = person("Eurostat"),
#'   identifier = "https://doi.org/10.2908/TEINA011",
#'   date = as.Date("2025-05-20")
#' )
#'
#' @seealso
#' Learn more in the vignette:
#' [`bibrecord`](https://dataset.dataobservatory.eu/articles/bibrecord.html)
#' @importFrom utils bibentry
#' @family bibrecord functions
#' @export
bibrecord <- function(
  title,
  author,
  contributor = NULL,
  publisher = NULL,
  year = NULL,
  date = Sys.Date(),
  identifier = NULL,
  subject = NULL,
  ...
) {
  stopifnot(inherits(author, "person") ||
    all(vapply(author, inherits, logical(1), "person")))

  # Normalize author roles and ensure vector of class "person"
  author <- normalize_roles(as.list(author))
  author <- do.call("c", author)

  # Normalize contributor if present
  if (!is.null(contributor)) {
    if (inherits(contributor, "person")) {
      contributor <- list(contributor)
    }
    contributor <- normalize_roles(as.list(contributor))
    contributor <- do.call("c", contributor)
  }

  # Infer year from date if missing
  year <- year %||% substr(as.character(date), 1, 4)

  meta <- bibentry(
    bibtype = "Misc",
    title = title,
    author = author,
    year = year,
    date = as.character(date),
    identifier = identifier,
    publisher = publisher,
    subject = subject,
    ...
  )

  if (!is.null(contributor)) {
    attr(meta, "contributor") <- contributor
  }

  class(meta) <- c("bibrecord", class(meta))
  meta
}

#' @keywords internal
`%||%` <- function(a, b) {
  if (!is.null(a)) a else b
}

#' @export
print.bibrecord <- function(x, ...) {
  NextMethod("print")

  contributors <- attr(x, "contributor")
  if (!is.null(contributors)) {
    cat("\nContributors:\n")
    cat(fix_contributor(contributors), "\n")
  }

  invisible(x)
}

test_that("print.bibrecord dispatches and adds contributor section", {
  rec <- bibrecord(
    title = "X",
    author = person("A", "B"),
    contributor = person("C", "D")
  )

  out <- capture.output(print(rec))
  expect_true(any(grepl("Contributors:", out)))
})


test_that("bibrecord() returns a bibentry-compatible object", {
  a <- person("Jane", "Doe", role = "cre")
  b <- person("Alice", "Smith", role = "dtm")

  rec <- bibrecord(
    title = "Sample Dataset",
    author = list(a),
    contributor = list(b),
    publisher = "Open Science Org",
    identifier = "doi:10.1234/test",
    date = "2023-12-31",
    subject = "Example"
  )

  expect_s3_class(rec, "bibrecord")
  expect_s3_class(rec, "bibentry")
  expect_equal(rec$title, "Sample Dataset")
  expect_equal(rec$publisher, "Open Science Org")
  expect_equal(rec$identifier, "doi:10.1234/test")
  expect_true(inherits(rec$author, "person"))
  expect_true(inherits(attr(rec, "contributor")[[1]], "person"))
})

test_that("bibrecord() infers year from date if missing", {
  rec <- bibrecord(
    title = "Auto Year Test",
    author = person("Jane", "Doe"),
    date = "2022-01-01"
  )
  expect_equal(rec$year, "2022")
})

test_that("print.bibrecord() prints without error and includes contributor", {
  a <- person("Jane", "Doe", role = "cre")
  b <- person("Alice", "Smith", role = "dtm")

  rec <- bibrecord(
    title = "Printable Record",
    author = list(a),
    contributor = list(b),
    date = "2021-01-01"
  )

  expect_output(print(rec), "Contributors:")
  expect_output(print(rec), "Alice Smith")
})

test_that("bibrecord() works without contributor field", {
  a <- person("Jane", "Doe", role = "cre")

  rec <- bibrecord(
    title = "No Contributor",
    author = list(a),
    date = "2020-12-01"
  )

  expect_null(attr(rec, "contributor"))
  expect_output(print(rec), "Doe J")
})

#' @title Get or Set the Geolocation of a Dataset Object
#'
#' @description
#' Access or assign the optional `geolocation` attribute to a semantically rich
#' dataset object.
#'
#' @details
#' The `geolocation` field describes the spatial region or named place where
#' the data was collected or that the dataset is about. This field is
#' recommended for data discovery in DataCite Metadata Schema 4.4.
#'
#' See: [DataCite: Geolocation Guidance](https://support.datacite.org/docs/datacite-metadata-schema-v44-recommended-and-optional-properties#18-geolocation)
#'
#' @param x A dataset object created by [dataset_df()] or
#'   `dataset::as_dataset_df()`.
#' @param value A character string specifying the `geolocation`.
#' @param overwrite Logical. If `TRUE` (default), the existing `geolocation`
#'   attribute is replaced with `value`. If `FALSE`, the function returns a
#'   message and does not overwrite the existing value.
#'
#' @return A character string of length 1, representing the `geolocation`
#'   attribute attached to `x`.
#'
#' @examples
#' orange_dataset <- orange_df
#' geolocation(orange_df) <- "US"
#' geolocation(orange_df)
#'
#' geolocation(orange_df, overwrite = FALSE) <- "GB"
#'
#' @family bibliographic helper functions
#' @export

geolocation <- function(x) {
  attr(x, "Geolocation")
}

#' @rdname geolocation
#' @export
`geolocation<-` <- function(x, overwrite = TRUE, value) {
  if (is.null(attr(x, "Geolocation"))) {
    if (is.null(value)) {
      attr(x, "Geolocation") <- NA_character_
    } else {
      attr(x, "Geolocation") <- value
    }
  } else if (overwrite) {
    attr(x, "Geolocation") <- value
  } else {
    message("The dataset has already an Geolocation: ", geolocation(x))
  }
  x
}

test_that("geolocation() returns NULL if not set", {
  df <- data.frame(a = 1:3)
  expect_null(geolocation(df))
})

test_that("geolocation() returns the value after being set", {
  df <- data.frame(b = 1:3)
  geolocation(df) <- "US"
  expect_equal(geolocation(df), "US")
})

test_that("geolocation()<- sets NA when value is NULL and no existing attribute", {
  df <- data.frame(c = 1:3)
  geolocation(df) <- NULL
  expect_true(is.na(geolocation(df)))
})

test_that("geolocation()<- overwrites existing attribute when overwrite = TRUE", {
  df <- data.frame(d = 1:3)
  geolocation(df) <- "US"
  geolocation(df, overwrite = TRUE) <- "GB"
  expect_equal(geolocation(df), "GB")
})

test_that("geolocation()<- does not overwrite when overwrite = FALSE", {
  df <- data.frame(e = 1:3)
  geolocation(df) <- "US"
  expect_message(
    geolocation(df, overwrite = FALSE) <- "FR",
    regexp = "The dataset has already an Geolocation"
  )
  expect_equal(geolocation(df), "US")
})

test_that("geolocation()<- sets value when attribute exists but is NULL", {
  df <- data.frame(f = 1:3)
  attr(df, "Geolocation") <- NULL
  geolocation(df) <- "IT"
  expect_equal(geolocation(df), "IT")
})

test_that("geolocation() works on dataset_df object", {
  skip_if_not(exists("dataset_df"), message = "dataset_df() not defined")
  df <- dataset_df(data.frame(g = 1:3))
  geolocation(df) <- "CA"
  expect_equal(geolocation(df), "CA")
})

#' Create a semantically enriched vector with variable-level metadata
#'
#' `defined()` constructs a vector that behaves like a base R vector but carries
#' semantic metadata used for documentation, validation, and interoperability.
#' The resulting object inherits from [`haven::labelled()`] (for numeric,
#' character, and factor data) or from base date/time classes, and adds:
#'
#' * a human-readable variable label,
#' * an optional unit of measurement,
#' * a **concept URI** identifying the meaning of the variable
#'   (formerly called *definition*),
#' * an optional namespace used for value-level URI expansion,
#' * optional labelled values (where supported).
#'
#' The `concept` attribute is a general semantic anchor and may refer to:
#' a measure or dimension concept (SDMX-style), a property IRI (e.g. Wikibase),
#' or any URI that defines or describes the variable.
#'
#' `defined()` vectors preserve their metadata during subsetting, printing,
#' summarizing, comparisons, and many tidyverse operations. They integrate
#' smoothly with [`dataset_df()`] objects and can be safely flattened via
#' [`as.data.frame()`], [`as_tibble()`], or coercion helpers such as
#' [`as_numeric()`] and [`as_character()`].
#'
#' @section Supported input types:
#' * numeric (integer or double)
#' * character
#' * factor (converted via [`labelled::to_labelled()`])
#' * [`Date`]
#' * [`POSIXct`]
#' * [`haven::labelled()`]
#' * logical (with restrictions: logical vectors **cannot** have value labels)
#'
#' @param x A vector to annotate.
#' @param labels Optional named vector of value labels. Only supported for
#'   numeric or character vectors (not for logical).
#' @param label A short human-readable variable label (character of length 1).
#' @param unit Unit of measurement (character length 1) or `NULL`.
#' @param concept A URI or identifier describing the meaning or definition
#'   of the variable. This replaces the deprecated `definition` argument.
#' @param namespace Optional string or named character vector used to generate
#'   value-level URIs via substitution (`$1` macro).
#' @param ... For backward compatibility; the deprecated `definition`
#'   argument is still accepted and mapped to `concept`.
#'
#' @return A vector of class `"haven_labelled_defined"` or `"datetime_defined"`,
#'   depending on the input type.
#'
#' @importFrom labelled is.labelled
#'
#' @seealso
#'   [`is.defined()`],
#'   [`as_numeric()`],
#'   [`as_character()`],
#'   [`as_logical()`],
#'   [`strip_defined()`],
#'   [`dataset_df()`]
#'   [`print.haven_labelled_defined()`]
#'
#' @examples
#' gdp_vector <- defined(
#'   c(3897, 7365, 6753),
#'   label = "Gross Domestic Product",
#'   unit = "million dollars",
#'   concept = "http://data.europa.eu/83i/aa/GDP"
#' )
#'
#' is.defined(gdp_vector)
#' print(gdp_vector)
#' summary(gdp_vector)
#' gdp_vector[1:2]
#'
#' @export
# Main generic ---------------------------------------------------------------
defined <- function(x,
                    labels = NULL,
                    label = NULL,
                    unit = NULL,
                    concept = NULL,
                    namespace = NULL,
                    ...) {
  dots <- list(...)

  # ------------------------------------------------------------------
  # DEPRECATED ARGUMENT SUPPORT
  # ------------------------------------------------------------------
  if (!is.null(dots$definition)) {
    warning("`definition` is deprecated; use `concept` instead.", call. = FALSE)
    if (is.null(concept)) {
      concept <- dots$definition
    }
  }

  # ------------------------------------------------------------------
  # ARGUMENT VALIDATION
  # ------------------------------------------------------------------
  if (!is.null(label) && (!is.character(label) || length(label) != 1)) {
    stop("`label` must be a character(1) or NULL.", call. = FALSE)
  }

  if (!is.null(unit) && (!is.character(unit) || length(unit) != 1)) {
    stop("`unit` must be a character(1) or NULL.", call. = FALSE)
  }

  if (!is.null(concept) && (!is.character(concept) || length(concept) != 1)) {
    stop("`concept` must be a character(1) or NULL.", call. = FALSE)
  }

  if (!is.null(namespace) &&
    (!is.character(namespace) || length(namespace) != 1)) {
    stop("`namespace` must be a character(1) or NULL.", call. = FALSE)
  }


  # ------------------------------------------------------------------
  # DISPATCH BY TYPE
  # ------------------------------------------------------------------

  # LOGICAL -----------------------------------------------------------
  if (is.logical(x)) {
    if (!is.null(labels)) {
      stop(
        "defined(logical): value labels are not supported for logical vectors.\n",
        "Use factor or character instead.",
        call. = FALSE
      )
    }

    tmp <- x
    attr(tmp, "label") <- label
    attr(tmp, "unit") <- unit
    attr(tmp, "concept") <- concept
    attr(tmp, "namespace") <- namespace
    class(tmp) <- c("haven_labelled_defined", "logical")

    return(tmp)
  }

  # NUMERIC / INTEGER ------------------------------------------------
  if (is.numeric(x)) {
    x_raw <- vctrs::vec_data(x)
    labels <- vec_cast_named(labels, x_raw, x_arg = "labels", to_arg = "x")

    return(new_labelled_defined(
      x_raw,
      labels = labels,
      label = label,
      unit = unit,
      concept = concept,
      namespace = namespace
    ))
  }

  # CHARACTER ---------------------------------------------------------
  if (is.character(x)) {
    return(new_labelled_defined(
      x,
      labels = labels,
      label = label,
      unit = unit,
      concept = concept,
      namespace = namespace
    ))
  }

  # FACTOR ------------------------------------------------------------
  if (is.factor(x)) {
    labelled_x <- labelled::to_labelled(x)
    var_unit(labelled_x) <- unit
    var_concept(labelled_x) <- concept
    var_namespace(labelled_x) <- namespace

    class(labelled_x) <- c("haven_labelled_defined", class(labelled_x))
    return(labelled_x)
  }

  # HAVEN LABELLED ----------------------------------------------------
  if (is.labelled(x)) {
    x2 <- x
    var_unit(x2) <- unit
    var_concept(x2) <- concept
    var_namespace(x2) <- namespace

    class(x2) <- c("haven_labelled_defined", class(x2))
    return(x2)
  }

  # DATE --------------------------------------------------------------
  if (inherits(x, "Date")) {
    tmp <- x
    attr(tmp, "label") <- label
    attr(tmp, "unit") <- unit
    attr(tmp, "concept") <- concept
    attr(tmp, "namespace") <- namespace
    class(tmp) <- c("haven_labelled_defined", "Date")
    return(tmp)
  }

  # POSIXct -----------------------------------------------------------
  if (inherits(x, "POSIXct")) {
    tmp <- x
    attr(tmp, "label") <- label
    attr(tmp, "unit") <- unit
    attr(tmp, "concept") <- concept
    attr(tmp, "namespace") <- namespace
    class(tmp) <- c("haven_labelled_defined", "POSIXct", "POSIXt")
    return(tmp)
  }

  # UNSUPPORTED -------------------------------------------------------
  stop(
    "defined(x): `x` must be logical, numeric, character, factor, Date, ",
    "POSIXct, or a labelled vector.",
    call. = FALSE
  )
}


#' @rdname defined
#' @export
is.defined <- function(x) {
  any(inherits(x, "haven_labelled_defined"), inherits(x, "datetime_defined"))
}

#' @keywords internal
vec_ptype_abbr.haven_labelled_defined <- function(x, ...) {
  "defined"
}

#' @keywords internal
vec_ptype2.double.haven_labelled_defined <- function(x, y, ...) double()

#' @keywords internal
vec_cast.double.haven_labelled_defined <- function(x, to, ...) vctrs::vec_data(x)

#' @keywords internal
vec_cast.character.haven_labelled_defined <- function(x, to, ...) vctrs::vec_data(x)

#' From haven
#' @keywords internal
#' @importFrom vctrs vec_cast
vec_cast_named <- function(x, to, ...) {
  stats::setNames(vctrs::vec_cast(x, to, ...), names(x))
}

#' @importFrom tibble new_tibble
#' @importFrom haven labelled
#' @keywords internal
new_labelled_defined <- function(x = double(),
                                 labels = NULL,
                                 label = NULL,
                                 unit = NULL,
                                 concept = NULL,
                                 namespace = NULL) {
  if (!is.null(unit) && (!is.character(unit) || length(unit) != 1)) {
    stop("defined(..., unit): 'unit' must be a character vector of length one.")
  }

  if (!is.null(concept) && (!is.character(concept) || length(concept) != 1)) {
    stop("defined(..., defintion): 'concept' must be a character vector of length one or NULL.")
  }

  if (!is.null(label) && (!is.character(label) || length(label) != 1)) {
    stop("defined(..., label): 'label' must be a character vector of length one or NULL.")
  }

  if (!is.null(namespace) && (!is.character(namespace) || length(namespace) != 1)) {
    stop("defined(..., namespace): 'namespace' must be a character vector of length one or NULL.")
  }

  tmp <- haven::labelled(x, labels = labels, label = label)

  attr(tmp, "unit") <- unit
  attr(tmp, "concept") <- concept
  attr(tmp, "namespace") <- namespace
  attr(tmp, "class") <- c("haven_labelled_defined", class(tmp))

  tmp
}

#' @importFrom tibble new_tibble
#' @keywords internal
new_datetime_defined <- function(x,
                                 label = NULL,
                                 unit = NULL,
                                 concept = NULL,
                                 namespace = NULL) {
  if (!is.null(unit) && (!is.character(unit) || length(unit) != 1)) {
    stop("defined(..., unit): 'unit' must be a character vector of length one.")
  }

  if (!is.null(concept) && (!is.character(concept) || length(concept) != 1)) {
    stop("defined(..., defintion): 'concept' must be a character vector of length one.")
  }

  if (!is.null(label) && (!is.character(label) || length(label) != 1)) {
    stop("defined(..., label): 'label' must be a character vector of length one.")
  }

  if (!is.null(namespace) && (!is.character(namespace) || length(namespace) != 1)) {
    stop("defined(..., namespace): 'namespace' must be a character vector of length one or NULL.")
  }

  tmp <- x

  attr(tmp, "unit") <- unit
  attr(tmp, "concept") <- concept
  attr(tmp, "namespace") <- namespace
  attr(tmp, "class") <- c("datetime_defined", class(tmp))

  tmp
}


## Subsetting ------------------------------------------------------

#' @export
`[.haven_labelled_defined` <- function(x, i, ...) {
  result <- NextMethod("[")
  most_attrs <- c("label", "unit", "concept", "namespace", "labels")
  for (attr_name in most_attrs) {
    attr(result, attr_name) <- attr(x, attr_name)
  }
  class(result) <- class(x)
  result
}

#' @export
#' @importFrom vctrs vec_data
`[[.haven_labelled_defined` <- function(x, i, ...) {
  defined(vctrs::vec_data(x)[[i]],
    label = var_label(x),
    unit = var_unit(x),
    concept = var_concept(x),
    namespace = var_namespace(x),
    labels = attr(x, "labels")
  )
}


#' @export
#' @importFrom vctrs vec_data
Ops.haven_labelled_defined <- function(e1, e2) {
  # Comparisons work as expected
  lhs <- if (inherits(e1, "haven_labelled_defined")) vctrs::vec_data(e1) else e1
  rhs <- if (inherits(e2, "haven_labelled_defined")) vctrs::vec_data(e2) else e2
  .Generic <- .Generic
  do.call(.Generic, list(lhs, rhs))
}

#' @export
#' @importFrom vctrs vec_data
length.haven_labelled_defined <- function(x) {
  length(vctrs::vec_data(x))
}

#' @export
#' @importFrom vctrs vec_data
#' @importFrom utils head
head.haven_labelled_defined <- function(x, n = 6L, ...) {
  x[seq_len(min(n, length(x)))]
}

#' @export
#' @importFrom vctrs vec_data
#' @importFrom utils tail
tail.haven_labelled_defined <- function(x, n = 6L, ...) {
  x[seq.int(to = length(x), length.out = min(n, length(x)))]
}


## Print & Summary --------------------------------------------------

#' Print a defined (haven_labelled_defined) vector
#'
#' @description
#' Custom print method for [`haven_labelled_defined`] vectors created with
#' [defined()]. It prints the variable name, label, and a short semantic
#' summary before the underlying values.
#'
#' @param x A `haven_labelled_defined` vector.
#' @param ... Passed on to [base::print()].
#'
#' @return `x`, invisibly.
#'
#' @seealso [defined()], [summary.haven_labelled_defined()]
#'
#' @examples
#' sex <- defined(
#'   c(0, 1, 1, 0),
#'   label  = "Sex",
#'   labels = c("Female" = 0, "Male" = 1)
#' )
#'
#' print(sex)
#'
#' @export
print.haven_labelled_defined <- function(x, ...) {
  has_def <- !is.null(var_concept(x)) && !is.na(var_concept(x)) && nzchar(var_concept(x))
  has_unit <- !is.null(var_unit(x)) && !is.na(var_unit(x)) && nzchar(var_unit(x))
  has_label <- !is.null(var_label(x))

  cat(deparse(substitute(x)))

  if (has_label) cat(paste0(": ", var_label(x)))
  cat("\n")

  if (has_def && has_unit) {
    msg <- paste0("Defined as ", var_concept(x), ", measured in ", var_unit(x))
  } else if (has_def) {
    msg <- paste0("Defined as ", var_concept(x))
  } else if (has_unit) {
    msg <- paste0("Measured in ", var_unit(x))
  } else {
    msg <- "Defined vector"
  }

  cat(msg, "\n")
  print(vctrs::vec_data(x), ...)
  invisible(x)
}


#' @export
format.haven_labelled_defined <- function(x, ...) {
  base <- format(vec_data(x), ...)
  unit <- var_unit(x)
  def <- var_concept(x)

  if (!is.null(unit) && nzchar(unit)) {
    suffix <- paste0(" (", unit, ")")
  } else if (!is.null(def) && nzchar(def) && nchar(def) < 30) {
    suffix <- paste0(" [", def, "]")
  } else {
    suffix <- ""
  }

  paste0(base, suffix)
}

#' @rdname defined
#' @param object An R object to be summarised.
#' @export
summary.haven_labelled_defined <- function(object, ...) {
  label <- var_label(object)
  unit <- var_unit(object)

  if (!is.null(label) && nzchar(label)) {
    if (!is.null(unit) && nzchar(unit)) {
      cat(paste0(label, " (", unit, ")\n"))
    } else {
      cat(paste0(label, "\n"))
    }
  }
  NextMethod()
}

#' Combine defined vectors with metadata checks
#'
#' The `c()` method for `defined` vectors ensures that all semantic metadata
#' (label, unit, concept, namespace, and value labels) match exactly. This
#' prevents accidental loss or mixing of incompatible definitions during
#' concatenation.
#'
#' All input vectors must:
#' - Have identical `label` attributes
#' - Have identical `unit`, `concept`, and `namespace`
#' - Have identical value labels (or none)
#'
#' @param ... One or more vectors created with [defined()].
#'
#' @return A single `defined` vector with concatenated values and retained
#'   metadata.
#'
#' @examples
#' a <- defined(1:3, label = "Length", unit = "meter")
#' b <- defined(4:6, label = "Length", unit = "meter")
#' c(a, b)
#'
#' @seealso [defined()]
#' @export
c.haven_labelled_defined <- function(...) {
  dots <- list(...)

  var_labels <- unlist(lapply(dots, var_label))
  val_labels <- lapply(dots, function(x) attr(x, "labels"))
  units <- unlist(lapply(dots, var_unit))
  concepts <- unlist(lapply(dots, var_concept))
  namespaces <- unlist(lapply(dots, namespace_attribute))

  all_identical <- function(l) {
    all(mapply(identical, head(l, 1), tail(l, -1)))
  }

  if (length(unique(as.character(var_labels))) > 1) {
    stop("c(): var_label must be identical or NULL across inputs")
  }

  if (length(unique(as.character(units))) > 1) {
    stop("c(): unit must be identical or NULL across inputs")
  }

  if (length(unique(as.character(concepts))) > 1) {
    stop("c(): concept must be identical or NULL across inputs")
  }

  if (length(unique(as.character(namespaces))) > 1) {
    stop("c(): namespace must be identical or NULL across inputs")
  }

  if (!all_identical(val_labels)) {
    stop("c(): value labels must be identical or NULL across inputs")
  }

  defined(
    unname(do.call(c, lapply(dots, vctrs::vec_data))),
    label = var_labels[[1]],
    labels = val_labels[[1]],
    concept = concepts[[1]],
    namespace = namespaces[[1]],
    unit = units[[1]]
  )
}

## Coercion ----------------------------------------------------------
## as_numeric, as_character, as_factor in separate files

#' @export
as.list.haven_labelled_defined <- function(x, ...) {
  lapply(seq_along(x), function(i) x[[i]])
}

#' @export
as.vector.haven_labelled_defined <- function(x, mode = "any") {
  as.vector(vctrs::vec_data(x), mode = mode)
}

#' @title Strip the class from a defined vector
#' @description Converts a `defined` vector to a base R numeric or character,
#'   retaining metadata as passive attributes.
#' @param x A `defined` vector.
#' @return A base R vector with attributes (`label`, `unit`, etc.) intact.
#' @seealso [as_numeric()], [as_character()]
#' @examples
#' gdp <- defined(c(3897L, 7365L), label = "GDP", unit = "million dollars")
#' strip_defined(gdp)
#'
#' fruits <- defined(c("apple", "avocado", "kiwi"),
#'   label = "Fruit", unit = "kg"
#' )
#'
#' strip_defined(fruits)
#' @export
strip_defined <- function(x) {
  if (!inherits(x, "haven_labelled_defined")) {
    return(x)
  }

  base_class <- typeof(vctrs::vec_data(x)) # typically "double" or "integer"
  class(x) <- base_class
  x
}

#' @importFrom pillar type_sum tbl_sum
#' @export
type_sum.haven_labelled_defined <- function(x) {
  "defined"
}

test_that("defined(POSIXct) preserves values, can be
          coerced back", {
  p <- as.POSIXct("2024-01-01 12:00:00",
    tz = "UTC"
  ) + (0:2) * 3600
  p_def <- defined(p, label = "Time variable", unit = "hour")

  expect_true(is.defined(p_def))
  expect_true(inherits(p_def, "haven_labelled_defined"))
  expect_true(inherits(p_def, "POSIXct"))

  expect_equal(var_unit(p_def), "hour")
  expect_equal(var_label(p_def), "Time variable")
  expect_equal(attr(as.POSIXct(p_def), "tzone"), "UTC")

  expect_equal(
    as.numeric(unclass(p_def)),
    as.numeric(unclass(p))
  )
})

test_that("as.Date coercion can strip attributes if needed", {
  p <- as.POSIXct("2024-01-01 12:00:00",
    tz = "UTC"
  ) + (0:2) * 3600
  p_def <- defined(p, label = "Time variable", unit = "hour")

  stripped_attributes <- names(attributes(as.POSIXct(p_def, TRUE)))
  expect_equal(stripped_attributes, c("class", "tzone"))
})


test_that("as.POSIXct.haven_labelled_defined respects
          user-initiated tz change", {
  # base POSIXct in UTC
  base_time <- as.POSIXct("2024-01-01 12:00:00", tz = "UTC")

  # wrap in defined()/haven_labelled_defined with metadata
  x <- defined(
    base_time,
    label = "Timestamp",
    unit  = "seconds"
  )

  target_tz <- "Europe/Budapest"

  # user explicitly requests a new time zone
  out <- as.POSIXct(x, tz = target_tz, strip_attributes = FALSE)

  # what base R would do without the wrapper
  expected <- base::as.POSIXct(base_time, tz = target_tz)

  # 1) Class and wrapper removal
  expect_s3_class(out, "POSIXct")
  expect_true(inherits(out, "POSIXt"))
  expect_false(inherits(out, "haven_labelled_defined"))

  # 2) Instant in time: same as base R's conversion
  expect_identical(as.numeric(out), as.numeric(expected))

  # 3) Time zone: whatever base R uses, we mirror
  expect_identical(attr(out, "tzone"), attr(expected, "tzone"))

  # 4) Metadata: by default strip_attributes = TRUE, so we keep it
  expect_identical(attr(out, "label"), "Timestamp")
  expect_identical(attr(out, "unit"), "seconds")
})

test_that("as.POSIXct.haven_labelled_defined
          strips attributes when requested", {
  base_time <- as.POSIXct("2024-01-01 12:00:00",
    tz = "UTC"
  )

  x <- defined(
    base_time,
    label = "Timestamp",
    unit  = "seconds"
  )

  target_tz <- "Europe/Budapest"

  out <- as.POSIXct(x, tz = target_tz, strip_attributes = TRUE)
  expected <- base::as.POSIXct(base_time, tz = target_tz)

  # Still matches base R conversion
  expect_identical(as.numeric(out), as.numeric(expected))
  expect_identical(attr(out, "tzone"), attr(expected, "tzone"))

  # Metadata stripped
  expect_null(attr(out, "label"))
  expect_null(attr(out, "unit"))
})

# These methods are related to the defined() class but
# tested in a separate test suite.

test_that("as_numeric() drops metadata by default (strip_attributes = TRUE)", {
  gdp <- defined(
    c(3897L, 7365L),
    label = "GDP",
    unit = "million dollars",
    concept = "http://example.org/GDP",
    namespace = "http://example.org/ns/"
  )

  num <- as_numeric(gdp)

  expect_type(num, "integer")
  expect_null(attr(num, "label"))
  expect_null(attr(num, "unit"))
  expect_null(attr(num, "concept"))
  expect_null(attr(num, "namespace"))
  expect_false(inherits(num, "haven_labelled_defined"))
})


test_that("as_numeric() preserves metadata when
          strip_attributes = FALSE", {
  gdp <- defined(
    c(1000L, 2000L),
    label = "GDP",
    unit = "million dollars",
    concept = "http://example.org/GDP",
    namespace = "http://ns/"
  )

  num <- as_numeric(gdp, strip_attributes = FALSE)

  expect_equal(attr(num, "label"), "GDP")
  expect_equal(attr(num, "unit"), "million dollars")
  expect_equal(attr(num, "concept"), "http://example.org/GDP")
  expect_equal(attr(num, "namespace"), "http://ns/")
  expect_type(num, "integer")

  # Should no longer be a defined vector
  expect_false(inherits(num, "haven_labelled_defined"))
})


test_that("as_numeric() preserves metadata when strip_attributes = FALSE", {
  gdp <- defined(
    c(10, 20, 30),
    label = "GDP",
    unit = "EUR",
    concept = "c:gdp"
  )

  num <- as_numeric(gdp, strip_attributes = FALSE)

  expect_equal(attr(num, "label"), "GDP")
  expect_equal(attr(num, "unit"), "EUR")
  expect_equal(attr(num, "concept"), "c:gdp")
})


test_that("as.numeric() base method always drops metadata and class", {
  gdp <- defined(c(1, 2, 3),
    label = "GDP",
    unit = "million USD",
    concept = "x"
  )

  rawnum <- as.numeric(gdp)

  expect_equal(rawnum, c(1, 2, 3))
  expect_type(rawnum, "double")

  expect_null(attributes(rawnum))
})


test_that("as_numeric() errors on non-numeric underlying data", {
  x <- defined(
    c("a", "b", "c"),
    label = "Letters"
  )
  expect_error(as_numeric(x), "underlying data is not numeric")
})


test_that("as_numeric() works with double underlying types", {
  x <- defined(
    c(1.1, 2.2, 3.3),
    label = "Float"
  )

  y <- as_numeric(x)

  expect_type(y, "double")
  expect_equal(y, c(1.1, 2.2, 3.3))
})


test_that("vec_cast(double) drops metadata", {
  x <- defined(
    c(10, 20),
    label = "Test",
    unit = "kg"
  )

  y <- vctrs::vec_cast(x, double())

  expect_equal(y, c(10, 20))
  expect_null(attr(y, "label"))
  expect_null(attr(y, "unit"))
  expect_type(y, "double")
})


test_that("strip_attributes = FALSE keeps *all* metadata consistently", {
  x <- defined(
    c(5, 6, 7),
    label = "Var",
    unit = "U",
    concept = "C",
    namespace = "NS"
  )

  y <- as_numeric(x, strip_attributes = FALSE)

  expect_equal(attr(y, "label"), "Var")
  expect_equal(attr(y, "unit"), "U")
  expect_equal(attr(y, "concept"), "C")
  expect_equal(attr(y, "namespace"), "NS")
})


test_that("as_numeric() returns base numeric with no defined class", {
  x <- defined(1:3, label = "X")

  y <- as_numeric(x)

  expect_false(inherits(y, "haven_labelled_defined"))
  expect_false(inherits(y, "defined"))
  expect_type(y, "integer")
})

test_that("as_numeric() drops metadata by default", {
  gdp <- defined(c(3897L, 7365L),
    label = "GDP",
    unit = "million dollars"
  )
  num <- as_numeric(gdp)
  expect_type(num, "integer")
  expect_null(attr(num, "label"))
  expect_null(attr(num, "unit"))
  expect_false(inherits(num, "defined"))
})

test_that("as_numeric(strip_attributes = FALSE) keeps
          only semantic attributes", {
  x <- defined(
    1:3,
    label = "Count",
    unit = "n",
    concept = "http://example.org/count",
    namespace = "http://example.org/ns"
  )
  # add an extra, non-semantic attribute to x
  attr(x, "extra_attr") <- "should_not_survive"

  out <- as_numeric(x, strip_attributes = FALSE)

  expect_type(out, "integer")
  expect_false(inherits(out, "haven_labelled_defined"))

  # Semantic attributes preserved
  expect_equal(attr(out, "label"), "Count")
  expect_equal(attr(out, "unit"), "n")
  expect_equal(attr(out, "concept"), "http://example.org/count")
  expect_equal(attr(out, "namespace"), "http://example.org/ns")

  # Extra, non-semantic attribute should NOT be kept
  expect_null(attr(out, "extra_attr"))
})

test_that("as_numeric() strips ALL semantic attributes by default", {
  x <- defined(
    1:3,
    label = "Count",
    unit = "n",
    concept = "http://example.org/count",
    namespace = "http://example.org/ns"
  )

  out <- as_numeric(x) # default strip_attributes = TRUE

  expect_type(out, "integer")
  expect_false(inherits(out, "haven_labelled_defined"))

  # All semantic attributes should be gone
  expect_null(attr(out, "label"))
  expect_null(attr(out, "unit"))
  expect_null(attr(out, "concept"))
  expect_null(attr(out, "namespace"))
})

# These methods are related to the defined() class but
# tested in a separate test suite.

test_that("as_factor() converts defined vector with labels to factor", {
  x <- defined(
    c(0, 1, 1, 0),
    label = "Sex",
    labels = c(
      "Female" = 0,
      "Male" = 1
    )
  )
  f <- as_factor(x)

  expect_s3_class(f, "factor")
  expect_equal(levels(f), c("Female", "Male"))
  expect_equal(as.character(f), c(
    "Female", "Male",
    "Male", "Female"
  ))
})

test_that("as_factor() preserves order of labels", {
  x <- defined(
    c(2, 3, 1),
    label = "Priority",
    labels = c("Low" = 1, "Medium" = 2, "High" = 3)
  )
  f <- as_factor(x)
  expect_equal(levels(f), c("Low", "Medium", "High"))
  expect_equal(as.character(f), c("Medium", "High", "Low"))
})

test_that("as_factor() without labels returns numeric-like factor", {
  x <- defined(c(1, 2, 1, 3), label = "Unlabelled")
  f <- as_factor(x)
  expect_s3_class(f, "factor")
  expect_equal(as.character(f), c("1", "2", "1", "3"))
})

test_that("as_factor() preserves metadata when
          strip_attributes = FALSE", {
  x <- defined(
    c(0, 1, 0),
    label = "Yes/No",
    labels = c("No" = 0, "Yes" = 1),
    unit = "boolean",
    concept = "http://example.org/yesno",
    namespace = "http://example.org/ns/"
  )

  f <- as_factor(x, strip_attributes = FALSE)

  expect_s3_class(f, "factor")
  expect_equal(attr(f, "unit"), "boolean")
  expect_equal(attr(f, "concept"), "http://example.org/yesno")
  expect_equal(attr(f, "namespace"), "http://example.org/ns/")
  expect_false("haven_labelled_defined" %in% class(f))
})

test_that("as_factor() strips metadata when
          strip_attributes = TRUE", {
  x <- defined(
    c(1, 2, 1),
    label = "Test",
    labels = c("One" = 1, "Two" = 2),
    unit = "u",
    concept = "c",
    namespace = "ns"
  )

  f <- as_factor(x, strip_attributes = TRUE)

  expect_s3_class(f, "factor")
  expect_null(attr(f, "unit"))
  expect_null(attr(f, "concept"))
  expect_null(attr(f, "namespace"))
})


test_that("as_factor() works for defined character vectors", {
  x <- defined(
    c("low", "medium", "high", "medium"),
    label = "Severity"
  )

  f <- as_factor(x)

  expect_s3_class(f, "factor")
  expect_equal(as.character(f), c("low", "medium", "high", "medium"))
  expect_equal(levels(f), sort(unique(c("low", "medium", "high"))))
})

test_that("as_factor() handles logical defined vectors", {
  x <- defined(
    c(TRUE, FALSE, TRUE),
    label = "Flag"
  )

  f <- as_factor(x)

  expect_s3_class(f, "factor")
  expect_equal(as.character(f), c("TRUE", "FALSE", "TRUE"))
  expect_equal(levels(f), c("FALSE", "TRUE"))
})

test_that("as_factor() handles missing values correctly", {
  x <- defined(
    c(1, NA, 2, NA),
    labels = c("One" = 1, "Two" = 2)
  )

  f <- as_factor(x)

  expect_s3_class(f, "factor")
  expect_equal(as.character(f), c("One", NA, "Two", NA))
})

test_that("as_factor() works with empty vectors", {
  x <- defined(
    numeric(0),
    label = "Empty"
  )

  f <- as_factor(x)

  expect_s3_class(f, "factor")
  expect_equal(length(f), 0)
  expect_equal(levels(f), character(0))
})

test_that("as_factor() errors gracefully when input is not defined", {
  expect_error(
    as_factor(1:3),
    "no applicable method"
  )
})

test_that("as_factor() retains level order stability for
          numeric but unlabeled", {
  x <- defined(
    c(10, 20, 10, 30),
    label = "Numbers!"
  )

  f <- as_factor(x)

  expect_equal(levels(f), c("10", "20", "30"))
})

test_that("as_logical coerces back booleans with or without
          semantic metadata", {
  x <- defined(
    x = c(TRUE, FALSE, TRUE),
    label = "Flag",
    unit = "boolean",
    concept = "https://example.org/flag",
    namespace = "test"
  )

  x_coerced <- as_logical(x, strip_attributes = FALSE)
  expect_equal(
    attr(x_coerced, "label"),
    "Flag"
  )

  expect_equal(as_logical(x), c(TRUE, FALSE, TRUE))
})

test_that("var_labels() retrieves all variable labels", {
  df <- dataset_df(
    id = defined(1:3, label = "Observation ID"),
    temp = defined(c(22.5, 23.0, 21.8), label = "Temperature (°C)"),
    site = defined(c("A", "B", "A"))
  )

  labs <- var_labels(df)
  expect_type(labs, "list")
  expect_equal(labs$id, "Observation ID")
  expect_equal(labs$temp, "Temperature (°C)")
  expect_null(labs$site) # unlabeled by default
})

test_that("var_labels(null_action) handles fill, skip, na, empty", {
  df <- dataset_df(
    id = defined(1:3, label = "Observation ID"),
    site = defined(c("A", "B", "A"))
  )

  expect_equal(
    var_labels(df, null_action = "fill")$site,
    "site"
  )
  expect_false("site" %in% names(var_labels(df, null_action = "skip")))
  expect_true(is.na(var_labels(df, null_action = "na")$site))
  expect_equal(var_labels(df, null_action = "empty")$site, "")
})


test_that("var_labels<- sets multiple labels from list", {
  df <- dataset_df(
    id = defined(1:3),
    site = defined(c("A", "B", "A"))
  )
  var_labels(df) <- list(id = "Identifier", site = "Site code")

  expect_equal(var_label(df$id), "Identifier")
  expect_equal(var_label(df$site), "Site code")
  # Check dataset-level var_labels attribute
  expect_equal(attr(df, "var_labels")$id, "Identifier")
})

test_that("var_labels<- sets from named character vector", {
  df <- dataset_df(
    id = defined(1:3),
    site = defined(c("A", "B", "A"))
  )
  var_labels(df) <- c(id = "Identifier", site = "Site code")

  expect_equal(var_label(df$id), "Identifier")
  expect_equal(var_label(df$site), "Site code")
})

test_that("var_labels<- ignores non-matching names", {
  df <- dataset_df(
    id = defined(1:3),
    site = defined(c("A", "B", "A"))
  )
  var_labels(df) <- list(nonexistent = "Should be ignored", id = "Identifier")
  expect_equal(var_label(df$id), "Identifier")
  expect_null(var_label(df$site))
})

test_that("var_labels<- errors if character vector is unnamed", {
  df <- dataset_df(
    id = defined(1:3),
    site = defined(c("A", "B", "A"))
  )
  expect_error(var_labels(df) <- c("a", "b"), "must be named")
})


test_that("var_labels(unlist = TRUE) returns named vector", {
  df <- dataset_df(
    rowid = defined(1:3, label = "Observation ID"),
    site = defined(c("A", "B", "A"))
  )
  labs <- var_labels(df, unlist = TRUE, null_action = "empty")
  expect_type(labs, "character")
  expect_named(labs, c("rowid", "site"))
})

test_that("defined() constructor creates expected vectors", {
  z <- defined(c(1, 1, 1, 0, 0, 0),
    label = "",
    labels = c("F" = 0, "M" = 1, "_N" = 99),
    concept = "https://example.org/sex"
  )
  x <- defined(c(0, 1, 0, 1, 1, 0),
    label = "sex",
    labels = c("F" = 0, "M" = 1, "_N" = 99),
    concept = "https://example.org/sex"
  )
  v <- defined(c(1, 0),
    label = "sex",
    labels = c("F" = 0, "M" = 1, "_N" = 99),
    concept = "https://example.org/sex"
  )
  y <- defined(c(1, 1, 1, 0, 0, 0),
    label = "sex",
    labels = c("F" = 0, "M" = 1, "_N" = 99),
    concept = "https://example.org/sex"
  )
  expect_equal(c(1:3, y), c(1, 2, 3, 1, 1, 1, 0, 0, 0))
  expect_equal(c("a", "b", y), c("a", "b", as.character(y)))
  expect_equal(
    c(x, y),
    defined(c(0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0),
      label = "sex",
      labels = c("F" = 0, "M" = 1, "_N" = 99),
      concept = "https://example.org/sex"
    )
  )
  expect_equal(
    c(x, y, v),
    defined(c(0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 0),
      label = "sex",
      labels = c("F" = 0, "M" = 1, "_N" = 99),
      concept = "https://example.org/sex"
    )
  )
  a <- defined(c("a", "b"), label = "letters")
  d <- defined(c("d", "e"), label = "letters")
  expect_equal(
    c(a, d),
    defined(c("a", "b", "d", "e"), label = "letters")
  )
})

test_that("defined() attributes can be retrieved", {
  numeric_vec <- defined(1:5,
    label = "Length",
    unit = "meters",
    concept = "http://example.org/length",
    namespace = "http://example.org/ns"
  )

  factor_vec <- defined(
    x = factor(c("low", "medium", "high")),
    label = "Severity",
    concept = "http://example.org/severity",
    namespace = "severity"
  )

  expect_true(is.defined(numeric_vec))
  expect_equal(var_label(numeric_vec), "Length")
  expect_equal(var_unit(numeric_vec), "meters")
  expect_equal(var_concept(numeric_vec), "http://example.org/length")
  expect_equal(var_namespace(factor_vec), "severity")
  expect_equal(as.numeric(numeric_vec), 1:5)
})

#-------------------------- logical ------------------------------

test_that("defined.logical() works with metadata and labels", {
  x <- defined(
    x = c(TRUE, FALSE, TRUE),
    label = "Flag",
    unit = "boolean",
    concept = "https://example.org/flag",
    namespace = "test"
  )

  expect_true(is.defined(x))
  expect_s3_class(x, "haven_labelled_defined")
  expect_s3_class(x, "logical")
  expect_equal(
    attr(x, "concept"),
    "https://example.org/flag"
  )


  expect_equal(var_label(x), "Flag")
  expect_equal(var_unit(x), "boolean")
  expect_equal(var_concept(x), "https://example.org/flag")
  expect_equal(var_namespace(x), "test")
})

test_that("defined.logical() forbids value labels", {
  expect_error(
    defined(
      x      = c(TRUE, FALSE),
      labels = c("no" = FALSE, "yes" = TRUE)
    ),
    "value labels are not supported for logical vectors"
  )
})

#-------------------- subsetting ------------------------
test_that("Subsetting defined vectors works correctly", {
  vec <- defined(100:110, label = "Measurement", unit = "kg")
  sub <- vec[1:3]
  one <- vec[[2]]

  expect_true(is.defined(sub))
  expect_equal(var_label(sub), "Measurement")
  expect_equal(var_unit(one), "kg")
  expect_equal(as_numeric(one), 101)
})

test_that("summary and print methods work as expected", {
  vec <- defined(1:3, label = "Depth", unit = "m")

  expect_output(summary(vec), "Depth \\(m\\)")
  expect_output(print(vec), "Measured in m")
})

test_that("coercion to base types works", {
  vec <- defined(1:3, label = "Count", unit = "n")

  expect_equal(as.numeric(vec), c(1, 2, 3))
  expect_equal(as_character(defined(c("a", "b"), label = "Letter")), c("a", "b"))

  # Check that coercion to numeric fails if not numeric
  nonnum <- defined(c("a", "b"), label = "Text")
  expect_error(as_numeric(nonnum), "underlying data is not numeric")
})

test_that("comparison operations work correctly", {
  a <- defined(1:3)
  b <- defined(3:1)

  expect_equal(a < b, c(TRUE, FALSE, FALSE))
  expect_equal(a == c(1, 2, 3), c(TRUE, TRUE, TRUE))
})

test_that("combining works only with identical metadata", {
  a <- defined(1:3,
    label = "height",
    unit = "m", concept = "def", namespace = "http://ns"
  )
  b <- defined(4:6,
    label = "height",
    unit = "m", concept = "def", namespace = "http://ns"
  )

  expect_equal(
    c(a, b),
    defined(1:6,
      label = "height", unit = "m",
      concept = "def", namespace = "http://ns"
    )
  )

  c_diff <- defined(4:6,
    label = "length",
    unit = "m", concept = "def", namespace = "http://ns"
  )
  expect_error(c(a, c_diff), "var_label must be identical")
})

test_that("type_sum returns defined", {
  x <- defined(1:3)
  expect_equal(type_sum(x), "defined")
})


test_that("c() combines compatible defined vectors", {
  a <- defined(1:3,
    label = "Length",
    unit = "meter",
    concept = "http://example.org/def",
    namespace = "http://example.org/"
  )

  b <- defined(4:6,
    label = "Length",
    unit = "meter",
    concept = "http://example.org/def",
    namespace = "http://example.org/"
  )

  ab <- c(a, b)

  expect_s3_class(ab, "haven_labelled_defined")
  expect_equal(length(ab), 6)
  expect_equal(var_label(ab), "Length")
  expect_equal(var_unit(ab), "meter")
  expect_equal(var_concept(ab), "http://example.org/def")
  expect_equal(var_namespace(ab), "http://example.org/")
})

test_that("c() throws error on mismatched labels", {
  a <- defined(1:3, label = "Height")
  b <- defined(4:6, label = "Length")

  expect_error(
    c(a, b),
    "var_label must be identical "
  )
})

test_that("c() throws error on mismatched units", {
  a <- defined(1:3, label = "Length", unit = "meter")
  b <- defined(4:6, label = "Length", unit = "centimeter")

  expect_error(
    c(a, b),
    "unit must be identical "
  )
})

test_that("c() throws error on mismatched concepts", {
  a <- defined(1:3, label = "Length", concept = "http://example.org/def1")
  b <- defined(4:6, label = "Length", concept = "http://example.org/def2")

  expect_error(
    c(a, b),
    "concept must be identical or NULL across inputs"
  )
})

test_that("c() throws error on mismatched namespaces", {
  a <- defined(1:3, label = "Length", namespace = "http://example.org/")
  b <- defined(4:6, label = "Length", namespace = "http://example.com/")

  expect_error(
    c(a, b),
    "namespace must be identical or NULL across inpu"
  )
})

test_that("c() throws error on mismatched value labels", {
  a <- defined(1:3,
    label = "Sex",
    labels = c("M" = 1, "F" = 2)
  )
  b <- defined(4:6,
    label = "Sex",
    labels = c("Male" = 1, "Female" = 2)
  )

  expect_error(
    c(a, b),
    "value labels must be identical"
  )
})

test_that("describe() writes provenance and metadata to file", {
  data("gdp", package = "dataset")

  test_ds <- dataset_df(
    rowid = defined(c("eg:1", "eg:2"), namespace = "http://example.com/dataset#"),
    geo = defined(
      gdp$geo[1:2],
      label = "Country",
      concept = "http://example.com/prop/geo",
      namespace = "https://eionet.europa.eu/geo/$1"
    ),
    dataset_bibentry = dublincore(
      title = "Example Dataset",
      creator = person("John", "Doe")
    )
  )

  tmpfile <- tempfile(fileext = ".nt")
  describe(x = test_ds, con = tmpfile)

  lines <- readLines(tmpfile)
  expect_true(any(grepl("prov#", lines)))
  expect_true(any(grepl("ttp://purl.org/dc/terms/title", lines)))
  expect_true(length(lines) > 3)
})

x <- orange_df
con <- tempfile()

test_that("defined(Date) constructs and preserves values,
          can be coerced back", {
  d <- as.Date("2024-01-01") + 0:2
  d_def <- defined(d, label = "Date variable", unit = "day")

  # class inheritance
  expect_true(is.defined(d_def))
  expect_true(inherits(d_def, "haven_labelled_defined"))
  expect_true(inherits(d_def, "Date"))

  # metadata preserved
  expect_equal(var_unit(d_def), "day")
  expect_equal(var_label(d_def), "Date variable")

  # values preserved
  expect_equal(as.numeric(unclass(d_def)), as.numeric(unclass(d)))
})


test_that("as.Date coercion can strip attributes if needed", {
  d <- as.Date("2024-01-01") + 0:2
  d_def <- defined(d, label = "Date variable", unit = "day")

  stripped_attributes <- names(attributes(as.Date(d_def, TRUE)))
  expect_equal(stripped_attributes, "class")
})

# These methods are related to the defined() class but
# tested in a separate test suite.

test_that("as.character() drops class and metadata from
          defined vector", {
  x <- defined(c("apple", "banana", "cherry"),
    label = "Fruit",
    unit = "kg",
    concept = "http://example.org/fruit"
  )
  result <- as.character(x)
  expect_type(result, "character")
  expect_false(inherits(result, "haven_labelled_defined"))
  expect_false("unit" %in% names(attributes(result)))
})

test_that("as_character() without preserve_attributes drops
          metadata", {
  x <- defined(c("red", "green", "blue"),
    label = "Color",
    unit = "rgb",
    concept = "http://example.org/color"
  )
  result <- as_character(x)
  expect_type(result, "character")
  expect_false("unit" %in% names(attributes(result)))
  expect_false(inherits(result, "haven_labelled_defined"))
})

test_that("as_character() with strip_attributes=FALSE
          keeps metadata", {
  x <- defined(c("yes", "no"),
    label = "Binary",
    unit = "boolean",
    concept = "http://example.org/binary",
    namespace = "http://example.org/ns"
  )
  result <- as_character(x, strip_attributes = FALSE)
  expect_type(result, "character")
  expect_equal(attr(result, "unit"), "boolean")
  expect_equal(
    attr(result, "concept"),
    "http://example.org/binary"
  )
  expect_equal(
    attr(result, "namespace"),
    "http://example.org/ns"
  )
})

# -------------------------------------------------------------------
# Additional tests for attribute stripping / preservation
# -------------------------------------------------------------------

test_that("as_character() strips ALL semantic attributes by default", {
  x <- defined(
    c("apple", "banana"),
    label = "Fruit",
    unit = "kg",
    concept = "http://example.org/fruit",
    namespace = "http://example.org/ns"
  )
  attr(x, "extra_attr") <- "xyz"

  out <- as_character(x) # default strip_attributes = TRUE

  expect_type(out, "character")
  expect_false(inherits(out, "haven_labelled_defined"))

  # All semantic attributes removed
  expect_null(attr(out, "label"))
  expect_null(attr(out, "unit"))
  expect_null(attr(out, "concept"))
  expect_null(attr(out, "namespace"))

  # Non-semantic attributes also removed
  expect_null(attr(out, "extra_attr"))
})

test_that("as_character(strip_attributes = FALSE) preserves
          only semantic attributes", {
  x <- defined(
    c("red", "green"),
    label = "Color",
    unit = "rgb",
    concept = "http://example.org/color",
    namespace = "http://example.org/ns"
  )
  attr(x, "extra_attr") <- "to_be_dropped"

  out <- as_character(x, strip_attributes = FALSE)

  expect_type(out, "character")
  expect_false(inherits(out, "haven_labelled_defined"))

  # Semantic attributes preserved
  expect_equal(attr(out, "label"), "Color")
  expect_equal(attr(out, "unit"), "rgb")
  expect_equal(attr(out, "concept"), "http://example.org/color")
  expect_equal(attr(out, "namespace"), "http://example.org/ns")

  # Extra attribute should not be carried over
  expect_null(attr(out, "extra_attr"))
})

test_that("as.character() (base) always returns plain character
          with no attributes", {
  x <- defined(
    c("yes", "no"),
    label = "Binary",
    unit = "boolean",
    concept = "http://example.org/binary",
    namespace = "http://example.org/ns"
  )
  attr(x, "extra_attr") <- "something"

  out <- as.character(x)

  expect_type(out, "character")
  expect_false(inherits(out, "haven_labelled_defined"))

  # No attributes at all
  expect_equal(names(attributes(out)) %||% character(), character(0))
})

test_that("var_label() works with defined() objects", {
  x <- defined(1:5, label = "Defined label")
  expect_equal(var_label(x), "Defined label")

  var_label(x) <- "Updated label"
  expect_equal(var_label(x), "Updated label")
})

test_that("var_label<- works with haven_labelled_defined", {
  x <- haven::labelled(1:5, labels = c(a = 1, b = 2))
  x <- defined(x)
  var_label(x) <- "Labelled defined"
  expect_equal(var_label(x), "Labelled defined")
})

test_that("var_label.dataset_df returns expected label list", {
  df <- dataset_df(
    x = defined(1:3, label = "Label X"),
    y = defined(4:6),
    z = defined(letters[1:3], label = "Label Z")
  )

  expect_equal(var_label(df)$x, "Label X")
  expect_null(var_label(df)$y)
  expect_equal(var_label(df)$z, "Label Z")
})

test_that("var_label.dataset_df supports null_action options", {
  df <- dataset_df(a = defined(1:2, label = "Has Label"), b = 3:4)

  expect_equal(var_label(df, null_action = "fill")$b, "b")
  expect_true(is.na(var_label(df, null_action = "na")$b))
  expect_equal(var_label(df, null_action = "empty")$b, "")
  expect_null(var_label(df, null_action = "keep")$b)

  skipped <- var_label(df, null_action = "skip")
  expect_false("b" %in% names(skipped))
})

test_that("var_label.dataset_df returns named vector with unlist = TRUE", {
  df <- dataset_df(
    u = defined(1:3, label = "U label"),
    v = 4:6
  )
  out <- var_label(df, unlist = TRUE, null_action = "empty")
  expect_type(out, "character")
  expect_equal(out[["u"]], "U label")
  expect_equal(out[["v"]], "")
})

test_that("var_label.dataset_df handles recurse = TRUE on packed columns", {
  packed <- dataset_df(
    a = defined(1:3, label = "A"),
    b = tibble::tibble(x = defined(1:3, label = "SubX"), y = 4:6)
  )

  out <- var_label(packed, recurse = TRUE, null_action = "empty")
  expect_type(out$b, "list")
  expect_equal(out$b$x, "SubX")
})


test_that("var_label() null_action options on dataset_df", {
  df <- dataset_df(x = defined(1:3, label = "has label"), y = 4:6)

  expect_equal(var_label(df, null_action = "keep")$y, NULL)
  expect_equal(var_label(df, null_action = "fill")$y, "y")
  expect_equal(var_label(df, null_action = "skip")$y, NULL)
  expect_true(is.na(var_label(df, null_action = "na")$y))
  expect_equal(var_label(df, null_action = "empty")$y, "")
})

test_that("var_label() with unlist = TRUE returns named vector", {
  df <- dataset_df(x = defined(1:3, label = "x label"), y = defined(4:6))
  out <- var_label(df, unlist = TRUE, null_action = "empty")
  expect_type(out, "character")
  expect_named(out, c("rowid", "x", "y"))
  expect_equal(out[["x"]], "x label")
  expect_equal(out[["y"]], "")
})

test_that("label_attribute() returns exact label attribute or NULL", {
  x <- 1:5
  expect_null(label_attribute(x))

  attr(x, "label") <- "simple label"
  expect_equal(label_attribute(x), "simple label")
})

test_that("var_label<- removes label when value is NULL", {
  x <- defined(1:3)
  var_label(x) <- "temporary label"
  expect_equal(var_label(x), "temporary label")
  var_label(x) <- NULL
  expect_null(var_label(x))
})

#' @title Get or update provenance information
#'
#' @description
#' Retrieve or append provenance statements (in N‑Triples form) stored on a
#' [`dataset_df()`] object.
#'
#' @details
#' Provenance is stored in the `"prov"` attribute as N‑Triples text. Use
#' [n_triple()] or [n_triples()] to construct valid statements that follow
#' PROV‑O (e.g., `prov:wasGeneratedBy`, `prov:wasInformedBy`).
#'
#' @param x A dataset created with [dataset_df()].
#' @param value Character vector of N‑Triples created by [n_triple()] or
#'   [n_triples()] to append to existing provenance.
#'
#' @return
#' * `provenance(x)` returns the contents of the `"prov"` attribute (character
#'   vector of N‑Triples), or `NULL` if none is set.
#' * `provenance(x) <- value` appends `value` to the `"prov"` attribute and
#'   returns the modified dataset invisibly.
#'
#' @examples
#' provenance(orange_df)
#'
#' # Add a provenance statement:
#' provenance(orange_df) <- n_triple(
#'   "https://doi.org/10.5281/zenodo.10396807",
#'   "http://www.w3.org/ns/prov#wasInformedBy",
#'   "http://example.com/source#1"
#' )
#'
#' @importFrom utils citation
#' @export
provenance <- function(x) {
  if (!is.dataset_df(x)) {
    stop("provenance(x): x must be a dataset_df object with standardised provenance metadata.")
  }
  attr(x, "prov")
}

#' @rdname provenance
#' @export
`provenance<-` <- function(x, value) {
  if (!is.dataset_df(x)) {
    stop("provenance(x)<- : x must be a dataset object created with dataset_df() or as_dataset_df().")
  }

  old_provenance <- provenance(x)
  new_provenance <- old_provenance

  attr(x, "prov") <- c(new_provenance, value)
  invisible(x)
}

#' @title Build default provenance bundle
#' @description
#' Construct a small PROV bundle (as N‑Triples) describing the dataset, the
#' software agent, and an optional creation time.
#'
#' @details
#' This helper is used internally to seed provenance metadata. It emits a set of
#' PROV statements including an `Entity` for the dataset, an `Activity` for
#' creation, and `SoftwareAgent` entries for the package citation.
#'
#' @param dataset_id Base IRI for the dataset (used as the `Entity` subject).
#' @param author Optional creator/author agent.
#' @param dtm Optional data team/maintainer agent.
#' @param generated_at_time Optional POSIXct time; defaults to [Sys.time()].
#'
#' @return A character vector of N‑Triples suitable for the `"prov"` attribute.
#'
#' @keywords internal
#' @importFrom utils citation
default_provenance <- function(dataset_id = "http://example.com/dataset#",
                               author = NULL,
                               dtm = NULL,
                               generated_at_time = NULL) {
  cite_dataset <- utils::citation("dataset")

  # See prov_author in n_triple.R
  agent_triples <- c()
  if (!is.null(author)) agent_triples <- c(agent_triples, prov_author(author))
  if (!is.null(dtm)) agent_triples <- c(agent_triples, prov_author(dtm))

  if (is.null(generated_at_time)) {
    generated_at_time <- Sys.time()
  }

  bundle_id <- gsub("#", "_prov.nt", dataset_id)

  prov <- n_triples(
    c(
      n_triple(bundle_id, "a", "http://www.w3.org/ns/prov#Bundle"),
      n_triple(dataset_id, "a", "http://www.w3.org/ns/prov#Entity"),
      n_triple(dataset_id, "a", "http://purl.org/linked-data/cube#DataSet"),
      agent_triples,
      n_triple("https://doi.org/10.32614/CRAN.package.dataset", "a", "http://www.w3.org/ns/prov#SoftwareAgent"),
      n_triple("http://example.com/creation", "a", "http://www.w3.org/ns/prov#Activity"),
      n_triple("http://example.com/creation", "http://www.w3.org/ns/prov#generatedAtTime", xsd_convert(generated_at_time)),
      n_triple(paste0("https://doi.org/", cite_dataset[[2]]$doi), "a", "http://www.w3.org/ns/prov#SoftwareAgent")
    )
  )

  prov
}

#' @title Set the Primary Language of a Dataset
#'
#' @description
#' Assign the primary language of a semantically rich dataset object using an
#' ISO 639 language code or full language name. This sets the `language`
#' attribute in the dataset's metadata.
#'
#' @details
#' This function supports recognition of:
#' - 2-letter codes (ISO 639-1, e.g., `"en"`, `"fr"`)
#' - 3-letter codes from both:
#'   - `Alpha_3_B` (bibliographic, e.g., `"fre"`)
#'   - `Alpha_3_T` (terminologic, e.g., `"fra"`)
#' - Full language names (e.g., `"English"`, `"French"`)
#'
#' For compatibility with open science repositories and modern metadata
#' standards, this function **returns the terminologic code** (`Alpha_3_T`)
#' when available. If `Alpha_3_T` is missing for a language, the legacy
#' bibliographic code (`Alpha_3_B`) is used as a fallback.
#'
#' Full language names (e.g., `"English"`, `"Spanish"`) are matched
#' case-insensitively against the ISO 639-2 Name field. Exact matches are
#' attempted first; if none are found, a prefix match is used. For example:
#' - `"English"` returns `"eng"`
#' - `"English, Old"` returns `"ang"`
#'
#' This means that:
#' - Both `"fra"` (terminologic) and `"fre"` (bibliographic) will be accepted
#'   as valid input for French
#' - The resulting value stored and returned will be `"fra"`
#'
#' This behaviour aligns with:
#' - [DataCite Metadata Schema 4.4](https://support.datacite.org/docs/datacite-metadata-schema-v44-recommended-and-optional-properties#9-language)
#' - [schema.org](https://schema.org/inLanguage)
#' - Common repository practices (Zenodo, OSF, Figshare)
#'
#' If `value` is `NULL`, the language is marked as `":unas"` (unspecified).
#'
#' In some cases<U+2014>especially for historical or moribund languages<U+2014>multiple
#' similar names may exist. In such cases, it is safer to use a specific
#' language code (e.g., `"ang"` instead of `"English, Old"` and `"enm"`
#'  for `"English, Middle (1100-1500)"`). You can also
#' refer directly to the definitions in [`ISOcodes::ISO_639_2`]
#' for clarity.
#'
#' @param x A dataset object created by [dataset_df()] or [as_dataset_df()].
#' @param value A 2-letter or 3-letter language code (ISO 639-1 or ISO 639-2),
#'   or a full language name (case-insensitive).
#' @param iso_639_code A character string indicating the desired return format:
#'   either `"639-3"` (default; terminologic) or `"639-1"` (2-letter code).
#'
#' @return The dataset with an updated `language` attribute, typically an ISO
#' 639-2/T code (`Alpha_3_T`) such as `"fra"`, `"eng"`, `"spa"`, etc.
#'
#' @examples
#' df <- dataset_df(data.frame(x = 1:3))
#'
#' language(df) <- "English" # Returns "eng"
#' language(df) <- "fre" # Legacy code; returns "fra"
#' language(df) <- "fra" # Returns "fra"
#' language(df, iso_639_code = "639-1") <- "fra" # Returns "fr"
#'
#' language(df) <- NULL # Sets ":unas"
#'
#' @family bibliographic helper functions
#' @export

language <- function(x) {
  assert_that(is.dataset_df(x),
    msg = "language(x): x must be a dataset object created with dataset() or as_dataset_df()."
  )

  ds_bibentry <- get_bibentry(x)
  as.character(ds_bibentry$language)
}


#' @rdname language
#' @export
`language<-` <- function(x, iso_639_code = "639-3", value) {
  assert_that(
    is.dataset_df(x),
    msg = "language(x)<- value: x must be a dataset object created with dataset() or as_dataset_df()."
  )

  ds_bibentry <- get_bibentry(x)

  # Handle NULL input by assigning placeholder
  if (is.null(value)) {
    ds_bibentry$language <- ":unas"
    attr(x, "dataset_bibentry") <- ds_bibentry
    return(x)
  }

  # Load and normalize ISO language data
  ISO_639 <- ISOcodes::ISO_639_2
  ISO_639 <- as.data.frame(
    lapply(ISO_639, as.character),
    stringsAsFactors = FALSE
  )

  value <- tolower(value)
  lang_entry <- data.frame()

  # Try matching by Alpha_2 (2-letter code)
  if (nchar(value) == 2) {
    lang_entry <- ISO_639[which(tolower(ISO_639$Alpha_2) == value), ]
  }

  # Try matching by Alpha_3_B or Alpha_3_T (3-letter code)
  if (nrow(lang_entry) == 0 && nchar(value) == 3) {
    lang_entry <- ISO_639[
      tolower(ISO_639$Alpha_3_B) == value |
        tolower(ISO_639$Alpha_3_T) == value,
    ]
  }

  # Try matching by full language name
  if (nrow(lang_entry) == 0) {
    lang_entry <- ISO_639[which(tolower(ISO_639$Name) == value), ]
  }

  # If Still not found, fail gracefully
  if (nrow(lang_entry) == 0) {
    stop(paste0(
      "Language='", value,
      "' is not a valid ISO 639 language code."
    ))
  }

  # Assign value according to iso_639_code preference
  if (iso_639_code == "639-1") {
    ds_bibentry$language <- as.character(lang_entry$Alpha_2[1])
  } else {
    # Prefer terminologic code if present and non-empty
    lang3t <- as.character(lang_entry$Alpha_3_T[1])
    lang3b <- as.character(lang_entry$Alpha_3_B[1])

    ds_bibentry$language <- if (!is.na(lang3t) && lang3t != "") {
      lang3t
    } else {
      lang3b
    }
  }

  attr(x, "dataset_bibentry") <- ds_bibentry
  invisible(x)
}

#' @rdname language
#' @export
`language<-` <- function(x, iso_639_code = "639-3", value) {
  assert_that(
    is.dataset_df(x),
    msg = "language(x)<- value: x must be a dataset object created with dataset() or as_dataset_df()."
  )

  ds_bibentry <- get_bibentry(x)

  # Handle NULL input by assigning placeholder
  if (is.null(value)) {
    ds_bibentry$language <- ":unas"
    attr(x, "dataset_bibentry") <- ds_bibentry
    return(x)
  }

  # Load and normalize ISO language data
  ISO_639 <- ISOcodes::ISO_639_2
  ISO_639 <- as.data.frame(
    lapply(ISO_639, as.character),
    stringsAsFactors = FALSE
  )

  value <- tolower(value)
  lang_entry <- data.frame()

  # Try matching by Alpha_2 (2-letter code)
  if (nchar(value) == 2) {
    matches <- which(tolower(ISO_639$Alpha_2) == value)
    if (length(matches) > 0) {
      lang_entry <- ISO_639[matches, ]
    }
  }

  # Try matching by Alpha_3_B or Alpha_3_T (3-letter code)
  if (nrow(lang_entry) == 0 && nchar(value) == 3) {
    matches <- which(
      tolower(ISO_639$Alpha_3_B) == value |
        tolower(ISO_639$Alpha_3_T) == value
    )
    if (length(matches) > 0) {
      lang_entry <- ISO_639[matches, ]
    }
  }

  # Try matching by full language name
  if (nrow(lang_entry) == 0) {
    matches <- which(value == tolower(ISO_639$Name))
    if (length(matches) == 0) {
      matches <- which(grepl(paste0("^", value), tolower(ISO_639$Name)))
    }
    lang_entry <- ISO_639[matches, ]
  }

  # If still not found, fail gracefully
  if (nrow(lang_entry) == 0) {
    stop(
      paste0(
        "Language='", value,
        "' is not a valid ISO 639 language code or name."
      )
    )
  }

  # Assign value according to iso_639_code preference
  if (iso_639_code == "639-1") {
    ds_bibentry$language <- as.character(lang_entry$Alpha_2[1])
  } else {
    # Prefer terminologic code if present and non-empty
    lang3t <- as.character(lang_entry$Alpha_3_T[1])
    lang3b <- as.character(lang_entry$Alpha_3_B[1])

    ds_bibentry$language <- if (!is.na(lang3t) && lang3t != "") {
      lang3t
    } else {
      lang3b
    }
  }

  attr(x, "dataset_bibentry") <- ds_bibentry
  invisible(x)
}

test_that("language() sets and retrieves ISO 639-3 language code", {
  df <- dataset_df(data.frame(x = 1:3))
  language(df) <- "en"
  expect_equal(language(df), "eng")
})

test_that("language() sets and retrieves ISO 639-1 code", {
  df <- dataset_df(data.frame(x = 1:3))
  language(df, iso_639_code = "639-1") <- "eng"
  expect_equal(language(df), "en")
})

test_that("language() sets and retrieves ISO 639-1
          code with multiple hits", {
  df <- dataset_df(data.frame(x = 1:3))
  language(df, iso_639_code = "639-3") <- "English, Old"
  expect_equal(language(df), "ang")
})

test_that("language() accepts full language names", {
  df <- dataset_df(data.frame(x = 1:3))
  language(df) <- "French"
  expect_equal(language(df), "fra") # 639-3 default
})

test_that("language() is case-insensitive for names and codes", {
  df <- dataset_df(data.frame(x = 1:3))
  language(df) <- "SpaniSH"
  expect_equal(language(df), "spa")
})

test_that("language() throws error on unknown language code", {
  df <- dataset_df(data.frame(x = 1:3))
  expect_error(
    language(df) <- "xx",
    regexp = "is not a valid ISO 639 language code"
  )
})

test_that("language() throws error on unknown full name", {
  df <- dataset_df(data.frame(x = 1:3))
  expect_error(
    language(df) <- "Elvish",
    regexp = "is not a valid ISO 639 language code"
  )
})

test_that("language() rejects non-dataset_df input", {
  df <- data.frame(x = 1:3)
  expect_error(
    language(df),
    regexp = "language\\(x\\): x must be a dataset object"
  )
  expect_error(
    language(df) <- "en",
    regexp = "language\\(x\\)<- value: x must be a dataset object"
  )
})

#' @keywords internal

normalize_roles <- function(people, default_role = "ctb") {
  lapply(people, function(p) {
    if (!inherits(p, "person")) {
      return(p)
    }
    if (is.null(p$role) || !any(nzchar(p$role))) {
      p$role <- default_role
    }
    p
  })
}

#' Coerce a defined vector to numeric
#'
#' @description
#' `as_numeric()` converts a [`defined()`][defined] vector to a numeric vector.
#' It validates that the underlying data are numeric, and optionally preserves
#' or strips semantic metadata.
#'
#' @details
#' Use \code{strip_attributes = TRUE} when flattening or preparing data for
#' external pipelines, but keep the default when working with defined
#' vectors directly.
#'
#' `as.numeric()` drops all metadata and returns only the numeric values in
#' a vector.
#'
#' @param x A vector created with [defined()].
#' @param strip_attributes Logical; whether to remove semantic metadata
#'   (`label`, `unit`, `concept`, `namespace`). Defaults to `TRUE`.
#' @param ... Reserved for future use.
#'
#' @return A numeric vector with or without preserved attributes.
#' @examples
#' x <- defined(
#'   1:3,
#'   label = "Count",
#'   unit = "n",
#'   concept = "http://example.org/count",
#'   namespace = "http://example.org/ns"
#' )
#'
#' as_numeric(x)
#'
#' @export
as_numeric <- function(x, ...) {
  UseMethod("as_numeric", x)
}

#' @rdname as_numeric
#' @export
as_numeric.haven_labelled_defined <- function(x,
                                              strip_attributes = TRUE,
                                              ...) {
  underlying <- vctrs::vec_data(x)

  if (!is.numeric(underlying)) {
    stop("as_numeric(): underlying data is not numeric.", call. = FALSE)
  }

  out <- underlying

  if (!strip_attributes) {
    attr(out, "label") <- attr(x, "label", exact = TRUE)
    attr(out, "unit") <- attr(x, "unit", exact = TRUE)
    attr(out, "concept") <- attr(x, "concept", exact = TRUE)
    attr(out, "namespace") <- attr(x, "namespace", exact = TRUE)
  }

  out
}

# Base R method: always drop metadata and class
#' @export
as.numeric.haven_labelled_defined <- function(x, ...) {
  vctrs::vec_data(x)
}

# vctrs casting: drop metadata
#' @export
#' @importFrom vctrs vec_data
vec_cast.double.haven_labelled_defined <- function(x, to, ...) {
  vctrs::vec_data(x)
}

#' @keywords internal
idcol_find <- function(x, idcol) {
  if (is.null(idcol)) {
    stop("idcol_find: 'idcol' must be specified.")
  }

  # Case: character column name
  if (is.character(idcol)) {
    if (!idcol %in% names(x)) {
      stop(sprintf("idcol_find: column '%s' not found in data frame.", idcol))
    }
    selected <- which(names(x) == idcol)

    # Case: numeric column index
  } else if (is.numeric(idcol)) {
    if (!idcol %in% seq_along(x)) {
      stop(sprintf("idcol_find: column index %s is out of bounds.", idcol))
    }
    selected <- idcol

    # Unsupported type
  } else {
    stop("idcol_find: 'idcol' must be a column name (character) or index (numeric).")
  }

  if (length(selected) != 1) {
    stop("idcol_find: 'idcol' must refer to exactly one column.")
  }

  selected
}

test_that("idcol_find() works", {
  expect_equal(idcol_find(orange_df, "rowid"), 1)
})

test_that("idcol_find works with valid character input", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_equal(idcol_find(df, "a"), 1)
  expect_equal(idcol_find(df, "b"), 2)
})

test_that("idcol_find works with valid numeric input", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_equal(idcol_find(df, 1), 1)
  expect_equal(idcol_find(df, 2), 2)
})

test_that("idcol_find returns error for invalid character input", {
  df <- data.frame(a = 1:3)
  expect_error(idcol_find(df, "c"), regexp = "not found")
})

test_that("idcol_find returns error for out-of-bounds numeric input", {
  df <- data.frame(a = 1:3)
  expect_error(idcol_find(df, 5), regexp = "out of bounds")
})

test_that("idcol_find returns error for wrong input type", {
  df <- data.frame(a = 1:3)
  expect_error(idcol_find(df, TRUE), "must be a column name")
  expect_error(idcol_find(df, list("a")), "must be a column name")
})

test_that("idcol_find returns error if character
          name matches multiple columns", { # creates duplicate names
  df <- data.frame(x = 1:3, x = 4:6, check.names = FALSE)
  expect_error(idcol_find(df, "x"), "exactly one column")
})

test_that("idcol_find returns correct index in real datasets", {
  mtcars2 <- mtcars
  mtcars2$rowid <- row.names(mtcars)
  expect_equal(idcol_find(mtcars2, "rowid"), ncol(mtcars2))
  expect_error(idcol_find(mtcars2, "bowid"), "not found")
})

#' @keywords internal
fix_publisher <- function(publishers) {
  if (is.null(publishers)) {
    return(":unas")
  }

  if (inherits(publishers, "person")) {
    full_names <- vapply(as.list(publishers), function(p) {
      if (!is.null(p$family) && nzchar(p$family)) {
        paste(p$given, p$family)
      } else {
        p$given
      }
    }, character(1))

    if (length(full_names) > 1) {
      return(paste0("{", paste(full_names, collapse = "} and {"), "}"))
    } else {
      return(full_names)
    }
  }

  if (is.character(publishers)) {
    return(publishers)
  }

  stop("fix_publisher(): Unsupported publisher type")
}

test_that("fix_publisher() works", {
  expect_equal(fix_publisher(publishers = "publisher"), "publisher")
  expect_equal(fix_publisher(publishers = NULL), ":unas")
  expect_equal(
    fix_publisher(publishers = person("Jane", "Doe", role = "pbl")),
    "Jane Doe"
  )
  expect_equal(fix_publisher(publishers = person("Open Data Institute", role = "pbl")), "Open Data Institute")
  expect_equal(fix_publisher(publishers = c(
    person("Jane", "Doe", role = "pbl"),
    person("Open Data Institute")
  )), "{Jane Doe} and {Open Data Institute}")
})

#' Coerce a defined vector to a factor
#'
#' @description
#' `as_factor()` converts a [`defined()`][defined] vector into a standard R
#' factor. If value labels are present, they are turned into factor levels
#' via [haven::as_factor()]. Otherwise, the underlying values are converted
#' with [base::factor()].
#' @details
#' Use \code{strip_attributes = TRUE} when flattening or preparing data for
#' external pipelines, but keep the default when working with defined
#' vectors directly.
#' @param x A vector created with [defined()].
#' @param ... Reserved for future extensions.
#'
#' @return A factor vector.
#'
#' @examples
#' sex <- defined(
#'   c(0, 1, 1, 0),
#'   label  = "Sex",
#'   labels = c("Female" = 0, "Male" = 1)
#' )
#'
#' as_factor(sex)
#' as_factor(sex, strip_attributes = FALSE)
#'
#' @export
as_factor <- function(x, ...) {
  UseMethod("as_factor")
}

#' @rdname as_factor
#' @export
#' @param strip_attributes Logical; should semantic metadata attributes
#'   (such as \code{label}, \code{unit}, \code{definition}, and
#'   \code{namespace}) be removed from the returned vector?
#'   Defaults to \code{TRUE}.
#' @importFrom haven as_factor labelled
#' @importFrom vctrs vec_data
as_factor.haven_labelled_defined <- function(
  x,
  strip_attributes = TRUE,
  ...
) {
  vals <- vctrs::vec_data(x)
  lbls <- attr(x, "labels", exact = TRUE)

  # CASE 1: value labels present → labelled → haven::as_factor()
  if (!is.null(lbls)) {
    fac <- haven::as_factor(
      haven::labelled(vals, labels = lbls),
      ...
    )
  } else {
    # CASE 2: no value labels → plain factor on values
    fac <- factor(vals)
  }

  # (We don't touch class(fac): it's already a plain factor)
  # Optionally copy semantic metadata
  if (!strip_attributes) {
    attr(fac, "unit") <- attr(x, "unit", exact = TRUE)
    attr(fac, "concept") <- attr(x, "concept", exact = TRUE)
    attr(fac, "namespace") <- attr(x, "namespace", exact = TRUE)
  }

  fac
}

#' @title Get or Set the Namespace of a Variable
#'
#' @description
#' Retrieve or assign the namespace part of a permanent, global variable
#' identifier, independent of the current R session or instance.
#'
#' @details
#' The `namespace` attribute is useful when working with remote, linked, or
#' open data sources. Variable identifiers in such datasets are often qualified
#' with a common namespace prefix. When combined, the prefix and namespace form
#' a persistent URI or IRI for the variable.
#'
#' Retaining the namespace ensures the identifiers remain valid and resolvable
#' during validation, merging, or future updates of the vector (such as when it
#' is used as a column in a dataset).
#'
#' `get_variable_namespaces()` is an alias for `var_namespace()`.
#' `namespace_attribute()` and `set_namespace_attribute()` are internal helpers.
#'
#' For full usage, see:
#' `vignette("defined", package = "dataset")` <U+2014> demonstrating integration of
#' variable labels, namespaces, units of measure, and machine-independent
#' identifiers.
#'
#' @param x A vector.
#' @param value A character string specifying the namespace, or `NULL` to
#'   remove it.
#' @param ... Additional arguments for method compatibility with other classes.
#'
#' @return A character string representing the namespace attribute of a vector
#'   constructed with [defined()]. Returns the updated object (in setter forms).
#'
#' @examples
#' # Define a vector with a namespace
#' x <- defined("Q42", namespace = c(wd = "https://www.wikidata.org/wiki/"))
#'
#' # Get the namespace
#' var_namespace(x)
#' get_variable_namespaces(x)
#'
#' # Set the namespace
#' var_namespace(x) <- "https://example.org/ns/"
#'
#' # Remove the namespace
#' var_namespace(x) <- NULL
#'
#' # Use lower-level helpers (not typically used directly)
#' namespace_attribute(x)
#' namespace_attribute(x) <- "https://example.org/custom/"
#'
#' @family defined metadata methods and functions
#' @rdname var_namespace
#' @export

var_namespace <- function(x, ...) {
  # rlang::check_dots_used()
  UseMethod("var_namespace")
}

#' @export
var_namespace.default <- function(x, ...) {
  attr(x, "namespace", exact = TRUE)
}

#' @rdname var_namespace
#' @export
`var_namespace<-` <- function(x, value) {
  UseMethod("var_namespace<-")
}

#' @export
`var_namespace<-.default` <- function(x, value) {
  namespace_attribute(x) <- value
  x
}

#' @rdname var_namespace
#' @export
get_variable_namespaces <- var_namespace


#' @rdname var_namespace
#' @export
namespace_attribute <- function(x) {
  attr(x, "namespace", exact = TRUE)
}


#' @rdname var_namespace
#' @export
get_namespace_attribute <- function(x) {
  namespace_attribute(x)
}


#' @rdname var_namespace
#' @export
set_namespace_attribute <- function(x, value) {
  if ((!is.character(value) && !is.null(value)) || length(value) > 1) {
    stop(
      "`namespace` should be a single character string or NULL",
      call. = FALSE,
      domain = "R-dataset"
    )
  }
  attr(x, "namespace") <- value
  x
}


#' @rdname var_namespace
#' @export
`namespace_attribute<-` <- set_namespace_attribute

test_that("var_namespace() retrieves the namespace correctly", {
  x <- defined("Q42",
    namespace = "https://www.wikidata.org/wiki/"
  )
  expect_equal(var_namespace(x), "https://www.wikidata.org/wiki/")
})

test_that("var_namespace() returns NULL if no namespace is set", {
  x <- defined("Q42")
  expect_null(var_namespace(x))
})

test_that("var_namespace<-() sets namespace correctly", {
  x <- defined("Q42")
  var_namespace(x) <- "https://example.org/ns/"
  expect_equal(var_namespace(x), "https://example.org/ns/")
})

test_that("var_namespace<-() removes the namespace with NULL", {
  x <- defined("Q42", namespace = "https://example.org/ns/")
  var_namespace(x) <- NULL
  expect_null(var_namespace(x))
})

test_that("get_variable_namespaces() is an alias of var_namespace()", {
  x <- defined("Q42", namespace = "https://wikidata.org/")
  expect_equal(get_variable_namespaces(x), var_namespace(x))
})

test_that("namespace_attribute() and its setter work as expected", {
  x <- defined("Q123")
  namespace_attribute(x) <- "https://custom.org/"
  expect_equal(namespace_attribute(x), "https://custom.org/")
})

test_that("set_namespace_attribute() validates input correctly", {
  x <- defined("Q123")

  # No namespace is set
  expect_null(namespace_attribute(x), "https://valid.org/")

  # Valid input
  expect_no_error(set_namespace_attribute(x, "https://valid.org/"))
  x <- (set_namespace_attribute(x, "https://valid.org/"))
  expect_equal(var_namespace(x), "https://valid.org/")

  # NULL input removes namespace
  x <- set_namespace_attribute(x, NULL)
  expect_null(namespace_attribute(x))

  # Invalid: non-character
  expect_error(
    set_namespace_attribute(x, 123),
    regexp = "`namespace` should be a single character string or NULL"
  )

  # Invalid: multiple strings
  expect_error(
    set_namespace_attribute(x, c("ns1", "ns2")),
    regexp = "`namespace` should be a single character string or NULL"
  )
})

test_that("namespace_attribute<-() uses set_namespace_attribute", {
  x <- defined("Q42")
  x <- `namespace_attribute<-`(x, "https://ns.org/")
  expect_equal(namespace_attribute(x), "https://ns.org/")
})

#' Create a new `dataset_df` object
#'
#' The `dataset_df()` constructor creates semantically rich modern data frames.
#' These inherit from [`tibble::tibble`] and carry structured metadata using
#' attributes.
#'
#' Use `is.dataset_df()` to check class membership.
#'
#' S3 methods for `dataset_df` include:
#' - `print()` to display the dataset with metadata
#' - `summary()` to summarize both data and metadata
#' - `names()` to retrieve variable names using standard data frame semantics
#'
#' For full details, see `vignette("dataset_df", package = "dataset")`.
#'
#' @param ... Vectors (columns) that should be included in the dataset.
#' @param identifier A named vector of one or more URI prefixes for row IDs.
#'   Defaults to `c(eg = "http://example.com/dataset#")`. For example, if your
#'   dataset will be published under DOI `https://doi.org/1234`, you may use
#'   `c(obs = "https://doi.org/1234#")`, which will generate row URIs such as
#'   `https://doi.org/1234#1`, ..., `#n`.
#' @param dataset_bibentry A bibliographic metadata record for the dataset,
#'   created using [`datacite()`] or [`dublincore()`].
#' @param var_labels A named list of human-readable labels for each variable.
#' @param units A named list of measurement units for measured variables.
#' @param concepts A named list of linked concepts (URIs) for variables or
#'   dimensions.
#' @param dataset_subject A subject descriptor created with [`subject()`] or
#'   [`subject_create()`].
#' @param x A `dataset_df` object (used in method dispatch).
#' @param df A `data.frame` to convert to a `dataset_df`.
#'
#' @return A `dataset_df` object: a tibble with attached metadata stored in
#'   attributes.
#'
#' @note A simple, serverless scaffolding for publishing `dataset_df` objects
#'   on the web (with HTML + RDF exports) is available at
#'   <https://github.com/dataobservatory-eu/dataset-template>.
#'
#' @examples
#' my_dataset <- dataset_df(
#'   country_name = defined(
#'     c("AD", "LI"),
#'     concept = "http://data.europa.eu/bna/c_6c2bb82d",
#'     namespace = "https://www.geonames.org/countries/$1/"
#'   ),
#'   gdp = defined(
#'     c(3897, 7365),
#'     label = "Gross Domestic Product",
#'     unit = "million dollars",
#'     concept = "http://data.europa.eu/83i/aa/GDP"
#'   ),
#'   identifier = c(
#'     obs = "https://dataobservatory-eu.github.io/dataset-template#"
#'   ),
#'   dataset_bibentry = dublincore(
#'     title = "GDP of Andorra and Liechtenstein",
#'     description = "A small but semantically rich dataset example.",
#'     creator = person("Jane", "Doe", role = "cre"),
#'     publisher = "Open Data Institute",
#'     language = "en"
#'   )
#' )
#'
#' # Basic usage
#' print(my_dataset)
#' head(my_dataset)
#' summary(my_dataset)
#'
#' # Metadata access
#' as_dublincore(my_dataset)
#' as_datacite(my_dataset)
#'
#' # Export description as RDF triples
#' my_description <- describe(my_dataset, con = tempfile())
#' my_description
#'
#' @seealso [defined()], [dublincore()], [datacite()], [subject()]
#'
#' @export

# User constructor
dataset_df <- function(
  ...,
  identifier = c(obs = "http://example.com/dataset#obs"),
  var_labels = NULL,
  units = NULL,
  concepts = NULL,
  dataset_bibentry = NULL,
  dataset_subject = NULL
) {
  dots <- list(...)

  if (!"rowid" %in% names(dots)) {
    add_rowid <- TRUE
  } else {
    add_rowid <- FALSE
  }

  sys_time <- Sys.time()
  year <- substr(as.character(sys_time), 1, 4)

  if (is.null(dataset_subject)) {
    dataset_subject <- default_subject # See: subject.R
  }

  if (is.null(dataset_bibentry)) {
    Title <- "Untitled Dataset"
    Creator <- person("Author", "Unknown")
    dataset_bibentry <- datacite(
      Title = Title,
      Creator = Creator,
      Subject = dataset_subject,
      Date = Sys.Date()
    )
  }

  tmp <- new_dataset(
    x = tibble::tibble(...),
    identifier = identifier,
    dataset_bibentry = dataset_bibentry,
    var_labels = var_labels,
    units = units,
    concepts = concepts
  )

  dataset_bibentry <- get_bibentry(tmp)
  if (dataset_bibentry$year == ":tba") dataset_bibentry$year <- year
  if (dataset_bibentry$date == ":tba") {
    dataset_bibentry$date <- as.character(Sys.Date())
  }

  attr(tmp, "dataset_bibentry") <- dataset_bibentry
  attr(tmp, "subject") <- dataset_subject
  tmp
}

#' @rdname dataset_df
#' @export
as_dataset_df <- function(
  df,
  identifier = c(obs = "http://example.com/dataset#obs"),
  var_labels = NULL,
  units = NULL,
  concepts = NULL,
  dataset_bibentry = NULL,
  dataset_subject = NULL, ...
) {
  dots <- list(...)

  if (is.null(dots$dataset_bibentry)) {
    dataset_bibentry <- set_default_bibentry()
  }

  new_dataset(df,
    identifier = identifier,
    dataset_bibentry = dataset_bibentry,
    var_labels = var_labels,
    units = units,
    concepts = concepts
  )
}

# Developer constructor
#' @importFrom tibble new_tibble
#' @keywords internal
new_dataset <- function(x,
                        add_rowid = TRUE,
                        identifier,
                        dataset_bibentry = NULL,
                        var_labels = NULL,
                        units = NULL,
                        concepts = NULL) {
  assertthat::assert_that(is.data.frame(x),
    msg = "Error: new_dataset(x): x is not a data frame"
  )

  generated_at_time <- Sys.time()

  tmp <- tibble::new_tibble(
    x,
    class = "dataset_df",
    nrow = nrow(x)
  )

  add_rowid <- !"rowid" %in% names(tmp)

  if (add_rowid) {
    tmp <- tibble::rowid_to_column(tmp)
    prefix <- names(identifier)[1]
    tmp$rowid <- defined(paste0(prefix, tmp$rowid),
      namespace = identifier
    )
  }

  if (is.null(dataset_bibentry)) {
    dataset_bibentry <- set_default_bibentry()
  }

  attr(tmp, "dataset_bibentry") <- dataset_bibentry

  # tmp <- set_var_labels(tmp, var_labels = var_labels)

  prov <- default_provenance(
    generated_at_time = generated_at_time,
    author = dataset_bibentry$author
  )

  attr(tmp, "prov") <- prov

  tmp
}

#' @rdname dataset_df
#' @return `is.dataset_df` returns a logical value
#' (if the object is of class `dataset_df`.)
#' @export
is.dataset_df <- function(x) {
  inherits(x, "dataset_df")
}

#' @rdname dataset_df
#' @export
print.dataset_df <- function(x, ...) {
  dataset_bibentry <- get_bibentry(x)
  if (is.null(dataset_bibentry)) {
    dataset_bibentry <- set_default_bibentry()
  }

  # Extract fields
  authors <- dataset_bibentry$author
  year <- dataset_bibentry$year
  title <- dataset_bibentry$title
  doi <- dataset_bibentry$identifier
  dataset_date <- dataset_bibentry$Date

  # Format author(s)
  author_fmt <- function(authors) {
    if (length(authors) == 1) {
      return(authors[[1]]$family %||% format(authors[[1]]))
    }

    is_institutional <- vapply(
      authors,
      function(a) is.null(a$given) && !is.null(a$family),
      logical(1)
    )
    if (all(is_institutional)) {
      return(paste(
        vapply(
          authors,
          function(a) a$family, character(1)
        ),
        collapse = "-"
      ))
    }

    if (length(authors) == 2) {
      return(paste(vapply(
        authors,
        function(a) a$family,
        character(1)
      ), collapse = "-"))
    }

    return(paste0(authors[[1]]$family, " et al."))
  }

  apa_header <- sprintf(
    "%s (%s): %s [dataset]",
    author_fmt(authors),
    year,
    title
  )

  if (!is.null(doi) && grepl("doi.org", doi)) {
    apa_header <- paste0(apa_header, ", ", doi)
  }

  cat(trimws(apa_header), "\n", sep = "")

  # Generate the tibble-like format
  df_fmt <- format(x)
  table_header <- df_fmt[1]
  table_body <- df_fmt[-1]

  # Extract column header line
  col_line <- table_body[1]

  # Print column header, label row, and table body
  cat(col_line, "\n")
  cat(paste0(table_body[-1], collapse = "\n"), "\n")

  invisible(x)
}


#' @export
tbl_sum.dataset_df <- function(x, ...) {
  NextMethod()
}

#' @export
summary.dataset_df <- function(object, ...) {
  dataset_bibentry <- get_bibentry(object)
  if (is.null(dataset_bibentry)) {
    dataset_bibentry <- set_default_bibentry()
  }
  # Extract fields
  authors <- dataset_bibentry$author
  year <- dataset_bibentry$year
  title <- dataset_bibentry$title
  doi <- dataset_bibentry$identifier

  # Format author(s)
  author_fmt <- function(authors) {
    if (length(authors) == 1) {
      return(authors[[1]]$family %||% format(authors[[1]]))
    }

    is_institutional <- vapply(
      authors,
      function(a) is.null(a$given) && !is.null(a$family),
      logical(1)
    )
    if (all(is_institutional)) {
      return(paste(
        vapply(
          authors,
          function(a) a$family, character(1)
        ),
        collapse = "-"
      ))
    }

    if (length(authors) == 2) {
      return(paste(vapply(
        authors,
        function(a) a$family,
        character(1)
      ), collapse = "-"))
    }

    return(paste0(authors[[1]]$family, " et al."))
  }

  apa_header <- sprintf(
    "%s (%s): Summary of %s [dataset]",
    author_fmt(authors),
    year,
    title
  )


  if (!is.null(doi) && grepl("doi.org", doi)) {
    apa_header <- paste0(apa_header, ", ", doi)
  }

  cat(trimws(apa_header), "\n\n", sep = "")

  NextMethod()
}

#' @export
plot.dataset_df <- function(x, y = NULL, ..., main = NULL, sub = NULL) {
  title <- dataset_title(x)
  bib <- get_bibentry(x)
  author <- tryCatch(as.character(bib$author), error = function(e) "")
  year <- bib$year %||% substr(bib$date, 1, 4)
  publisher <- bib$publisher %||% ""

  main <- main %||% title
  sub <- sub %||% paste(author, "(", year, ")", "-", publisher)

  df <- as.data.frame(x)

  # Identify numeric or defined columns
  numeric_like_cols <- which(vapply(df, function(col) {
    inherits(col, "numeric") || inherits(col, "double") || inherits(col, "integer")
  }, logical(1)))

  if (length(numeric_like_cols) < 2) {
    stop("Not enough numeric or defined columns to create a plot.")
  }

  for (j in numeric_like_cols) {
    df[[j]] <- as_numeric(df[[j]])
  }

  xcol <- df[[numeric_like_cols[1]]]
  ycol <- df[[numeric_like_cols[2]]]

  x_label <- ifelse(is.null(var_label(x)[numeric_like_cols[1]][[1]]),
    names(x)[numeric_like_cols[1]],
    var_label(x)[numeric_like_cols[1]]
  )

  y_label <- ifelse(is.null(var_label(x)[numeric_like_cols[2]][[1]]),
    names(x)[numeric_like_cols[2]],
    var_label(x)[numeric_like_cols[2]]
  )

  plot(xcol, ycol,
    xlab = x_label,
    ylab = y_label,
    main = main,
    ...
  )
}


#' @rdname dataset_df
#' @export
is_dataset_df <- function(x) {
  inherits(x, "dataset_df")
}

#' @rdname dataset_df
#' @export
names.dataset_df <- function(x) {
  NextMethod("names")
}

#' @export
`[.dataset_df` <- function(x, i, j, drop = FALSE) {
  out <- NextMethod("[")
  attributes_to_preserve <- c("dataset_bibentry", "subject", "prov")

  for (attr_name in attributes_to_preserve) {
    attr(out, attr_name) <- attr(x, attr_name)
  }

  class(out) <- class(x)
  out
}

test_that("get_bibentry() throws error for non-dataset_df", {
  expect_error(get_bibentry(mtcars),
    regexp = "dataset has no dataset_bibentry"
  )
})

test_that("get_bibentry() returns the dataset_bibentry object", {
  orange_bibentry <- get_bibentry(orange_df)
  expect_equal(orange_bibentry$title, "Growth of Orange Trees")
  expect_equal(orange_bibentry$date, "1998")
})

test_that("set_bibentry() sets a new bibentry", {
  orange_df_2 <- orange_df
  new_bibentry <- dublincore(
    title = "Test",
    creator = person("Jane", "Doe"),
    dataset_date = 2013
  )
  set_bibentry(dataset = orange_df_2) <- new_bibentry
  expect_equal(get_bibentry(orange_df_2)$title, "Test")
})

test_that("set_bibentry(NULL) populates sensible defaults", {
  df <- dataset_df(data.frame(x = 1))
  # start with no bibentry
  attr(df, "dataset_bibentry") <- NULL
  # setting NULL triggers default dublincore(...)
  set_bibentry(df) <- NULL
  be <- get_bibentry(df)
  expect_equal(be$title, "Untitled Dataset")
  # year defaults to current year
  expect_equal(be$year, substr(as.character(Sys.time()), 1, 4))
})

test_that("set_bibentry() does not overwrite provided year", {
  df <- dataset_df(data.frame(x = 1))
  provided_year <- "2010"
  be <- dublincore(
    title = "Keep My Year",
    creator = person("Jane", "Doe"),
    dataset_date = provided_year
  )
  # ensure be$year is set by constructor to provided_year
  expect_equal(be$year, provided_year)
  set_bibentry(df) <- be
  expect_equal(get_bibentry(df)$year, provided_year)
})

test_that("set_bibentry() returns invisibly and updates only attributes", {
  df <- dataset_df(data.frame(x = 1))
  be <- dublincore(
    title = "Attr Only",
    creator = person("Jane", "Doe"),
    dataset_date = 2022
  )
  expect_invisible(set_bibentry(df) <- be)
  expect_equal(ncol(df), 2) # data part unchanged, but rowid added
  expect_equal(get_bibentry(df)$title, "Attr Only")
})

test_that("get/set bibentry roundtrip works with DataCite bibentry too", {
  df <- dataset_df(data.frame(x = 1))
  be <- datacite(
    Title = "DataCite Title",
    Creator = person("Jane", "Doe"),
    PublicationYear = "2021"
  )
  set_bibentry(df) <- be
  got <- get_bibentry(df)
  expect_equal(got$title, "DataCite Title")
  expect_equal(got$year, "2021")
})

test_that("as_dublincore/as_datacite list views reflect bibentry", {
  df <- dataset_df(data.frame(x = 1))
  be <- dublincore(
    title = "List View",
    creator = person("Jane", "Doe"),
    dataset_date = "2023",
    publisher = "DataPub"
  )
  set_bibentry(df) <- be

  dc_list <- as_dublincore(df, type = "list")
  expect_equal(dc_list$title, "List View")
  expect_equal(as.character(dc_list$creator), as.character(be$author))

  dcite_list <- as_datacite(df, type = "list")
  expect_equal(dcite_list$Title, "List View")
  expect_equal(as.character(dcite_list$Creator), as.character(be$author))
  expect_true(dcite_list$Publisher %in% c("DataPub", "")) # depending on defaults
})

test_that("get_bibentry() error message is informative", {
  # plain data.frame → no attribute
  df <- data.frame(x = 1)
  expect_error(
    get_bibentry(df),
    regexp = "dataset has no dataset_bibentry attribute"
  )
})

test_that("publisher() retrieves publisher from dataset_df", {
  df <- dataset_df(x = 1:3)
  publisher(df) <- "Open Science Repository"
  expect_equal(publisher(df), "Open Science Repository")
})

test_that("publisher() errors if object is not a dataset_df", {
  regular_df <- data.frame(x = 1:3)
  expect_error(
    publisher(regular_df),
    "publisher\\(x\\): x must be a dataset object"
  )
})

test_that("publisher<- assigns publisher if none exists", {
  df <- dataset_df(x = 1:3)
  expect_silent(publisher(df) <- "Data Publisher Inc.")
  expect_equal(publisher(df), "Data Publisher Inc.")
})

test_that("publisher<- assigns publisher if current is :tba", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry")$publisher <- ":tba"
  publisher(df) <- "Example Press"
  expect_equal(publisher(df), "Example Press")
})

test_that("publisher<- does not overwrite by default if set", {
  df <- dataset_df(x = 1:3)
  publisher(df) <- "Original Publisher"
  expect_message(publisher(df, overwrite = FALSE) <- "New Publisher",
    regexp = "dataset has already an Publisher"
  )
  expect_equal(publisher(df), "Original Publisher")
})

test_that("publisher<- does overwrite if overwrite = TRUE", {
  df <- dataset_df(x = 1:3)
  publisher(df) <- "Old Publisher"
  publisher(df, overwrite = TRUE) <- "New Publisher"
  expect_equal(publisher(df), "New Publisher")
})

test_that("publisher<- errors with invalid value length", {
  df <- dataset_df(x = 1:3)
  expect_error(
    publisher(df) <- c("a", "b"),
    "value must be of length 1"
  )
})

test_that("publisher<- sets to :tba when value is NULL", {
  df <- dataset_df(x = 1:3)
  publisher(df) <- NULL
  expect_equal(publisher(df), ":tba")
})

test_that("dataset_df() creates and validates basic object", {
  df <- dataset_df(mtcars)
  expect_s3_class(df, "dataset_df")
  expect_true(is.dataset_df(df))
  expect_false(is.dataset_df(mtcars))

  expect_equal(get_bibentry(df)$author, person("Author", "Unknown"))
})

test_that("dataset_df() respects identifier for rowid creation", {
  df <- dataset_df(mtcars,
    identifier = c(mt = "http://mtcars.com/dataset#")
  )
  expect_s3_class(df$rowid, "haven_labelled_defined")
  expect_true(all(grepl("^mt", df$rowid)))
})


test_that("dataset_df() stores variable metadata", {
  df <- dataset_df(
    foo = defined(1:3, label = "Example Var", unit = "count")
  )
  expect_equal(var_label(df$foo), "Example Var")
  expect_equal(var_unit(df$foo), "count")
})

test_that("dataset_df() creates with full metadata", {
  bib <- dublincore(
    title = "Growth of Orange Trees",
    creator = person("N.R.", "Draper", role = "cre"),
    publisher = "Wiley",
    identifier = "https://doi.org/10.5281/zenodo.10396807",
    dataset_date = 1998,
    description = "35 rows of orange growth"
  )

  df <- dataset_df(
    tree = defined(Orange$Tree, label = "Tree ID"),
    age = defined(Orange$age, unit = "days"),
    circumference = defined(Orange$circumference, label = "Size"),
    dataset_bibentry = bib
  )

  expect_equal(dataset_title(df), "Growth of Orange Trees")
  expect_equal(creator(df)[[1]]$family, "Draper")
})

test_that("as_dataset_df() converts from data.frame", {
  df <- as_dataset_df(Orange)
  expect_s3_class(df, "dataset_df")
})

test_that("new_dataset() validates input", {
  expect_error(new_dataset(42, identifier = c(x = "http://example.com")))
  expect_s3_class(
    new_dataset(Orange, identifier = c(x = "http://example.com")),
    "dataset_df"
  )
})

test_that("subsetting preserves class and metadata", {
  df <- dataset_df(Orange)
  sub <- df[1:3, ]
  expect_s3_class(sub, "dataset_df")
  expect_true(!is.null(attr(sub, "dataset_bibentry")))
})

test_that("rowid not duplicated if already present", {
  df <- dataset_df(
    rowid = defined(paste0("id:", 1:3)),
    x = 1:3
  )
  expect_equal(names(df)[1], "rowid")
})

test_that("summary() outputs metadata", {
  df <- dataset_df(
    x = 1:3,
    dataset_bibentry = datacite(
      Title = "Summary Test",
      Creator = person("Jane", "Doe"),
      PublicationYear = 2023
    )
  )
  expect_output(summary(df), "Summary Test")
  expect_output(summary(df), "Doe")
})

test_that("print() outputs full APA-style citation and data", {
  df <- dataset_df(
    x = 1:2,
    dataset_bibentry = dublincore(
      title = "Tiny Data",
      creator = list(person("A.", "Smith")),
      identifier = "https://doi.org/10.1234/example",
      dataset_date = 2021
    )
  )
  expect_output(print(df), "Smith \\(2021\\): Tiny Data \\[dataset\\], https://doi.org/10.1234/example")
  expect_output(print(df), "1 obs1\\s+1")
  expect_output(print(df), "2 obs2\\s+2")
})

test_that("dataset_df supports multiple authors in APA formatting", {
  df <- dataset_df(
    x = 1:2,
    dataset_bibentry = dublincore(
      title = "Multi Author Example",
      creator = list(
        person("A.", "Alpha"),
        person("B.", "Beta"),
        person("C.", "Gamma")
      ),
      dataset_date = 2020
    )
  )
  expect_output(print(df), "Alpha et al. \\(2020\\)")
})

test_that("dataset_df supports institutional authors", {
  df <- dataset_df(
    x = 1:2,
    dataset_bibentry = dublincore(
      title = "Inst Author Example",
      creator = person(family = "OpenDataOrg"),
      dataset_date = 2022
    )
  )
  expect_output(print(df), "OpenDataOrg \\(2022\\)")
})

test_that("rbind works with bind_defined_rows()", {
  df1 <- dataset_df(a = defined(1:2, label = "A"))
  df2 <- dataset_df(a = defined(3:4, label = "A"))
  out <- bind_defined_rows(df1, df2)
  expect_equal(nrow(out), 4)
  expect_s3_class(out, "dataset_df")
})

test_that("names.dataset_df returns correct column names", {
  df <- dataset_df(a = 1:3, b = 4:6)
  expect_equal(names(df)[1], "rowid")
  expect_length(names(df), 3)
})

test_that("dataset_df handles empty input gracefully", {
  empty <- dataset_df()
  expect_s3_class(empty, "dataset_df")
  expect_equal(nrow(empty), 0)
})

test_that("dataset_df rejects invalid variable types", {
  expect_error(dataset_df(a = new.env()), "must be vectors")
})

test_that("dataset_title() returns the current title", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_title(df, overwrite = TRUE) <- "Test Dataset"
  expect_equal(dataset_title(df), "Test Dataset")
})

test_that("dataset_title<-() sets a title when none is present", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_title(df, overwrite = TRUE) <- "Initial Title"
  expect_equal(dataset_title(df), "Initial Title")
})

test_that("dataset_title<-() overwrites title when allowed", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_title(df, overwrite = TRUE) <- "Original Title"
  dataset_title(df, overwrite = TRUE) <- "New Title"
  expect_equal(dataset_title(df), "New Title")
})

test_that("dataset_title<-() does not overwrite by default", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_title(df, overwrite = TRUE) <- "Do Not Change"
  expect_warning(
    dataset_title(df) <- "New Attempt",
    regexp = "The dataset already has a title"
  )
  expect_equal(dataset_title(df), "Do Not Change")
})

test_that("dataset_title<-() sets ':tba' if value is NULL", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_title(df) <- NULL
  expect_equal(dataset_title(df), ":tba")
})

test_that("dataset_title<-() errors on character vector of length > 1", {
  df <- dataset_df(data.frame(x = 1:3))
  expect_error(
    dataset_title(df) <- c("Title A", "Title B"),
    regexp = "multiple titles"
  )
})

test_that("dataset_title<-() errors on non-dataset_df input", {
  x <- data.frame(x = 1:3)
  expect_error(dataset_title(x),
    regexp = "must be a dataset object"
  )
  expect_error(dataset_title(x) <- "Invalid",
    regexp = "must be a dataset object"
  )
})

test_that("contributor() returns only contributor persons", {
  df <- dataset_df(data.frame(x = 1))
  creator(df) <- person("Alice", "Smith", role = "aut")
  contributor(df, overwrite = FALSE) <- person("Infra",
    role = "ctb",
    comment = c(contributorType = "hostingInstitution")
  )
  cons <- contributor(df)
  chr <- vapply(cons, as.character, character(1))
  expect_true(any(grepl("Infra.*\\[ctb\\]", chr)))
  # ensure creators not included
  all_people <- vapply(creator(df), as.character, character(1))
  expect_true(any(grepl("Alice Smith", all_people)))
  expect_false(any(grepl("Alice Smith", chr)))
})

test_that("contributor() returns only
          contributors (role = 'ctb' or contributorType)", {
  df <- dataset_df(data.frame(x = 1))
  creator(df) <- utils::person("Alice", "Author", role = "aut")
  creator(df, overwrite = FALSE) <- utils::person("Bob", "Builder", role = "ctb")
  creator(df, overwrite = FALSE) <- utils::person("Carol", "Curator", comment = c(contributorType = "DataCurator"))

  ctb <- contributor(df)

  expect_true(all(vapply(ctb, function(p) {
    identical(p$role, "ctb") ||
      (!is.null(p$comment) && !is.null(p$comment[["contributorType"]]))
  }, logical(1))))
})

test_that("contributor() returns empty when no contributors", {
  df <- dataset_df(data.frame(x = 1))
  creator(df) <- utils::person("Alice", "Author", role = "aut")
  expect_length(contributor(df), 0)
})

test_that("contributor<-() adds contributor and sets role to 'ctb' if missing", {
  df <- dataset_df(data.frame(x = 1))
  creator(df) <- utils::person("Alice", "Author", role = "aut")

  contributor(df) <- utils::person("Jane", "Doe")
  ctb <- contributor(df)

  expect_equal(length(ctb), 1)
  expect_equal(ctb[[1]]$role, "ctb")
})

test_that("contributor<-() appends when overwrite = FALSE", {
  df <- dataset_df(data.frame(x = 1))
  contributor(df) <- utils::person("Jane", "Doe", role = "ctb")
  contributor(df, overwrite = FALSE) <- utils::person("John", "Smith", role = "ctb")

  ctb <- contributor(df)
  expect_equal(length(ctb), 2)
})

test_that("contributor<-() overwrites contributors when overwrite = TRUE", {
  df <- dataset_df(data.frame(x = 1))
  contributor(df) <- utils::person("Jane", "Doe", role = "ctb")
  contributor(df, overwrite = TRUE) <- utils::person("John", "Smith", role = "ctb")

  ctb <- contributor(df)
  expect_equal(length(ctb), 1)
  expect_equal(ctb[[1]]$given, "John")
})

test_that("contributor<-() errors on wrong type", {
  df <- dataset_df(data.frame(x = 1))
  expect_error(contributor(df) <- "not a person", "utils::person object")
})

test_that("contributor<-() errors if role is not 'ctb'", {
  df <- dataset_df(data.frame(x = 1))
  expect_error(contributor(df) <- utils::person("Jane", "Doe", role = "aut"), "role must be 'ctb'")
})

test_that("returns datacite bibentry by default", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- datacite(
    Title = "Test Dataset",
    Creator = person("Jane", "Doe")
  )
  result <- as_datacite(df)
  expect_s3_class(result, "datacite")
  expect_s3_class(result, "bibentry")
})

test_that("returns list if type = 'list'", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- datacite(
    Title = "List Output",
    Creator = person("Jane", "Doe")
  )
  result <- as_datacite(df, type = "list")
  expect_type(result, "list")
  expect_named(result, c(
    "Title", "Creator", "Identifier", "Publisher", "PublicationYear",
    "Subject", "Type", "Contributor", "Date", "Language",
    "AlternateIdentifier", "RelatedIdentifier", "Format", "Version",
    "Rights", "Description", "Geolocation", "FundingReference"
  ))
})

test_that("returns dataset_df if type = 'dataset_df'", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- datacite(
    Title = "Structured Output",
    Creator = person("Jane", "Doe"),
    Identifier = "doi:10.1234/example",
    Publisher = "DataPub",
    Subject = subject_create(
      term = "Oranges",
      schemeURI = "http://id.loc.gov/authorities/subjects",
      valueURI = "http://id.loc.gov/authorities/subjects/sh85095257",
      subjectScheme = "LCCH",
      prefix = "lcch:"
    ),
    Contributor = person("Contributor", "Name"),
    Date = "2022",
    Language = "en",
    AlternateIdentifier = "alt-id",
    RelatedIdentifier = "rel-id",
    Format = "text/csv",
    Version = "v1.0",
    Rights = "CC-BY",
    Description = "A dataset about something",
    Geolocation = "Global",
    FundingReference = "Funder X"
  )
  result <- as_datacite(df, type = "dataset_df")
  expect_s3_class(result, "dataset_df")
  expect_true(all(c("Title", "Creator", "Identifier", "Publisher") %in% names(result)))
})

test_that("returns ntriples string if type = 'ntriples'", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- datacite(
    Title = "Triple Output",
    Creator = person("Jane", "Doe"),
    Contributor = person("Contributor", "Name")
  )
  result <- as_datacite(df, type = "ntriples")
  expect_type(result, "character")
  expect_true(grepl("http", result[1]))
})

test_that("invalid type falls back to bibentry with warning", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- datacite(
    Title = "Fallback",
    Creator = person("Jane", "Doe")
  )
  expect_warning(
    result <- as_datacite(df, type = "nonsense"),
    "type cannot be"
  )
  expect_s3_class(result, "datacite")
})

test_that("errors if author argument is not a person", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- datacite(
    Title = "Bad Author",
    Creator = person("Jane", "Doe")
  )
  expect_error(
    as_datacite(df, author = "not a person"),
    "author must be created with utils\\:\\:person"
  )
})

test_that("rights() retrieves rights from dataset_df", {
  df <- dataset_df(x = 1:3)
  rights(df, overwrite = TRUE) <- "CC-BY-4.0"
  expect_equal(rights(df), "CC-BY-4.0")
})

test_that("rights() errors for non-dataset_df object", {
  df <- data.frame(x = 1:3)
  expect_error(
    rights(df),
    "rights\\(x\\): x must be a dataset object"
  )
})

test_that("rights<- assigns rights when none exists", {
  df <- dataset_df(x = 1:3)
  expect_silent(rights(df, overwrite = TRUE) <- "GPL-3")
  expect_equal(rights(df), "GPL-3")
})

test_that("rights<- assigns rights if current is ':unas'", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry")$rights <- ":unas"
  rights(df) <- "MIT"
  expect_equal(rights(df), "MIT")
})

test_that("rights<- emits message if overwrite = FALSE and field exists", {
  df <- dataset_df(x = 1:3)
  rights(df, overwrite = TRUE) <- "Original"
  expect_message(
    rights(df, overwrite = FALSE) <- "Attempted overwrite",
    "already a rights field"
  )
  expect_equal(rights(df), "Original")
})

test_that("rights<- overwrites if overwrite = TRUE", {
  df <- dataset_df(x = 1:3)
  rights(df) <- "Old License"
  rights(df, overwrite = TRUE) <- "New License"
  expect_equal(rights(df), "New License")
})

test_that("rights<- errors if value is not length 1", {
  df <- dataset_df(x = 1:3)
  expect_error(
    rights(df) <- c("A", "B"),
    regexp = "must have length=1"
  )
})


test_that("rights<- sets :unas when value is NULL", {
  df <- dataset_df(x = 1:3)
  rights(df) <- NULL
  expect_equal(rights(df), ":unas")
})

test_that("dataset_format() returns default when unset", {
  df <- dataset_df(data.frame(x = 1:3))
  # Remove format to simulate unset
  be <- get_bibentry(df)
  be$format <- NULL
  attr(df, "dataset_bibentry") <- be

  expect_equal(dataset_format(df), "application/r-rds")
})

test_that("dataset_format() returns current value when set", {
  df <- dataset_df(data.frame(x = 1:3))
  be <- get_bibentry(df)
  be$format <- "text/csv"
  attr(df, "dataset_bibentry") <- be

  expect_equal(dataset_format(df), "text/csv")
})

test_that("dataset_format<- sets value when default or unset", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_format(df) <- "text/csv"
  expect_equal(dataset_format(df), "text/csv")
})

test_that("dataset_format<- resets to default on NULL", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_format(df) <- "text/csv"
  dataset_format(df) <- NULL
  expect_equal(dataset_format(df), "application/r-rds")
})

test_that("dataset_format<- respects overwrite = FALSE", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_format(df) <- "text/csv"

  expect_message(
    dataset_format(df) <- "application/parquet",
    "already has a format"
  )
  expect_equal(dataset_format(df), "text/csv")
})

test_that("dataset_format<- overwrites when overwrite = TRUE", {
  df <- dataset_df(data.frame(x = 1:3))
  dataset_format(df) <- "text/csv"
  dataset_format(df, overwrite = TRUE) <- "application/parquet"
  expect_equal(dataset_format(df), "application/parquet")
})

test_that("dataset_format() errors for non-dataset_df", {
  expect_error(
    dataset_format(mtcars),
    "must be a dataset object created with dataset_df"
  )
})

test_that("dataset_format<- errors for invalid value length", {
  df <- dataset_df(data.frame(x = 1:3))
  expect_error(
    dataset_format(df) <- c("a", "b"),
    "must be length 1 or NULL"
  )
})

test_that("as_dublincore() handles multiple creators and metadata correctly", {
  orange_bibentry <- dublincore(
    title = "Growth of Orange Trees",
    creator = c(
      person(
        given = "N.R.",
        family = "Draper",
        role = "cre",
        comment = c(VIAF = "http://viaf.org/viaf/84585260")
      ),
      person(
        given = "H",
        family = "Smith",
        role = "cre"
      )
    ),
    contributor = person(
      given = "Antal",
      family = "Daniel",
      role = "dtm"
    ),
    publisher = "Wiley",
    datasource = "https://isbnsearch.org/isbn/9780471170822",
    dataset_date = 1998,
    identifier = "https://doi.org/10.5281/zenodo.14917851",
    language = "en",
    description = "The Orange data frame has 35 rows and 3 columns of records of the growth of orange trees."
  )

  test <- dataset_df(a = 1:3, dataset_bibentry = orange_bibentry)

  dc <- as_dublincore(test, type = "ntriples")

  expect_type(dc, "character")
  expect_true(all(grepl("^<https://doi.org/10.5281/zenodo.14917851> <http://purl.org/dc/terms/", dc)))

  # Check presence of key predicates
  expect_true(any(grepl("dc/terms/title", dc)))
  expect_true(any(grepl("dc/terms/creator", dc)))
  expect_true(any(grepl("dc/terms/contributor", dc)))
  expect_true(any(grepl("dc/terms/publisher", dc)))
  expect_true(any(grepl("dc/terms/identifier", dc)))
  expect_true(any(grepl("dc/terms/source", dc)))
  expect_true(any(grepl("dc/terms/description", dc)))
  expect_true(any(grepl("dc/terms/date", dc)))
  expect_true(any(grepl("dc/terms/language", dc)))
  expect_true(any(grepl("dc/terms/type", dc)))

  # Specific value checks
  expect_true(any(grepl("Draper", dc)))
  expect_true(any(grepl("Smith", dc)))
  expect_true(any(grepl("Daniel", dc)))
  expect_true(any(grepl("Wiley", dc)))
  expect_true(any(grepl("zenodo.14917851", dc)))
  expect_true(any(grepl("Growth of Orange Trees", dc)))
  expect_true(any(grepl("Orange data frame", dc)))
  expect_true(any(grepl("9780471170822", dc)))
})

test_that("identifier() retrieves the identifier from dataset_df", {
  df <- dataset_df(data.frame(x = 1:3))
  identifier(df) <- "http://example.org/dataset/123"
  expect_equal(identifier(df), "http://example.org/dataset/123")
})

test_that("identifier() retrieves the identifier from bibentry", {
  be <- dublincore(title = "Example", creator = person("Jane", "Doe"))
  identifier(be) <- "http://example.org/resource"
  expect_equal(identifier(be), "http://example.org/resource")
})

test_that("identifier<- sets DOI and extracts correctly", {
  df <- dataset_df(data.frame(x = 1:3))
  identifier(df) <- "https://doi.org/10.1234/example.doi"
  expect_equal(identifier(df), "https://doi.org/10.1234/example.doi")
  expect_equal(attr(df, "doi"), "10.1234/example.doi")
})

test_that("identifier<- accepts unnamed character string", {
  df <- dataset_df(data.frame(x = 1:3))
  identifier(df) <- "https://example.org/id/abc123"
  expect_equal(identifier(df), "https://example.org/id/abc123")
})

test_that("identifier<- accepts numeric and coerces to character", {
  df <- dataset_df(data.frame(x = 1:3))
  identifier(df) <- 12345
  expect_equal(identifier(df), "12345")
})

test_that("identifier<- sets default ':unas' on NULL", {
  df <- dataset_df(data.frame(x = 1:3))
  identifier(df) <- NULL
  expect_equal(identifier(df), ":unas")
})

test_that("identifier<- doesn't overwrite unless allowed", {
  df <- dataset_df(data.frame(x = 1:3))
  identifier(df) <- "https://example.org/id/1"
  expect_warning(
    identifier(df, overwrite = FALSE) <- "https://example.org/id/2",
    regexp = "The dataset has already an identifier"
  )
  expect_equal(identifier(df), "https://example.org/id/1")
})

test_that("identifier<- overwrites when allowed", {
  df <- dataset_df(data.frame(x = 1:3))
  identifier(df) <- "https://example.org/id/1"
  identifier(df, overwrite = TRUE) <- "https://example.org/id/2"
  expect_equal(identifier(df), "https://example.org/id/2")
})

test_that("identifier<- errors on invalid object types", {
  expect_error(
    identifier(data.frame(x = 1)),
    regexp = "x must be a dataset_df or a bibentry object"
  )
  df <- data.frame(x = 1)
  expect_error(
    do.call("identifier<-", list(df, value = "id")),
    regexp = "x must be a dataset_df or a bibentry object"
  )
})

test_that("identifier<- errors on invalid value types", {
  df <- dataset_df(data.frame(x = 1:3))
  expect_error(
    identifier(df) <- list("invalid"),
    regexp = "value must be a named or not named character string"
  )
})

test_that("publication_year() gets the publication year correctly", {
  df <- dataset_df(data.frame(a = 1:3))
  publication_year(df) <- "2020"
  expect_equal(publication_year(df), "2020")
})

test_that("publication_year() returns character", {
  df <- dataset_df(data.frame(b = 1:3))
  publication_year(df) <- "1999"
  expect_type(publication_year(df), "character")
})

test_that("publication_year()<- sets the year when no existing value", {
  df <- dataset_df(data.frame(c = 1:3))
  publication_year(df) <- "2015"
  expect_equal(publication_year(df), "2015")
})

test_that("publication_year()<- overwrites when overwrite = TRUE", {
  df <- dataset_df(data.frame(d = 1:3))
  publication_year(df) <- "2010"
  publication_year(df, overwrite = TRUE) <- "2022"
  expect_equal(publication_year(df), "2022")
})

test_that("publication_year()<- does not overwrite when overwrite = FALSE", {
  df <- dataset_df(data.frame(e = 1:3))
  publication_year(df) <- "2001"
  expect_warning(
    publication_year(df, overwrite = FALSE) <- "2005",
    regexp = "The dataset has already an publication_year"
  )
  expect_equal(publication_year(df), "2001") # Should not change
})

test_that("publication_year()<- handles NULL value by setting ':unas'", {
  df <- dataset_df(data.frame(f = 1:3))
  publication_year(df) <- NULL
  expect_equal(publication_year(df), ":unas")
})

test_that("publication_year() fails on non-dataset_df input", {
  df <- data.frame(x = 1:3)
  expect_error(publication_year(df),
    regexp = "publication_year\\(x\\): x must be a dataset object"
  )
  expect_error(publication_year(df) <- "2020",
    regexp = "publication_year\\(x\\) <- value: x must be a dataset object"
  )
})

test_that("agent()<- assignment works", {
  p1 <- person(given = "Jane", family = "Doe", role = c("cre", "aut"))
  p2 <- person(given = "Joe", family = "Doe", role = c("aut", "dtm"))
  p3 <- person(given = "Publisher Inc", role = c("pbl"))
  test_df <- dataset_df(data.frame(a = 1, b = 2))
  expect_error(agent(test_df) <- 1:2,
    regexp = "value must be a vector of utils\\:\\:persons\\(\\) class"
  )
  expect_equal(get_bibentry(test_df)$author, person("Author", "Unknown"))
  expect_error(agent(x = 1) <- person("Jane Doe"))
})

test_that("agent() works", {
  expect_error(agent(x = 1:3),
    regexp = "x must be a dataset_df, a vector of persons, a dublincore or datacite object"
  )
  p1 <- person("Jane", "Doe", role = c("cre", "aut"))
  p2 <- person("Joe", "Doe", role = c("aut", "dtm"))
  p3 <- person("Publisher Inc", role = c("pbl"))
  expect_equal(length(agent(x = c(p1, p2, p3))), 3)
  expect_equal(
    agent(x = c(p1, p2, p3))$creators[1],
    person("Jane", "Doe", role = c("cre", "aut"))
  )
  expect_null(agent(x = c(p1, p2, p3))$contributors)
  expect_equal(
    agent(x = c(p1, p2, p3))$publisher,
    person("Publisher Inc", role = c("pbl"))
  )
})

test_that("returns dublincore bibentry by default", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- dublincore(
    title = "Test Dataset",
    creator = person("Jane", "Doe")
  )
  result <- as_dublincore(df)
  expect_s3_class(result, "dublincore")
  expect_s3_class(result, "bibentry")
})

test_that("returns list if type = 'list'", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- dublincore(
    title = "List Output",
    creator = person("Jane", "Doe")
  )
  result <- as_dublincore(df, type = "list")
  expect_type(result, "list")
  expect_named(result, c(
    "title", "creator", "identifier", "publisher", "subject", "type",
    "contributor", "date", "language", "relation", "dataset_format", "rights",
    "datasource", "description", "coverage"
  ))
})

test_that("returns dataset_df if type = 'dataset_df'", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- dublincore(
    title = "Structured Output",
    creator = person("Jane", "Doe"),
    identifier = "doi:10.1234/example",
    publisher = "DataPub",
    subject = "Science",
    contributor = person("Contributor", "Name"),
    dataset_date = "2022",
    language = "en",
    relation = "https://related.example.org",
    dataset_format = "text/csv",
    rights = "CC-BY",
    datasource = "https://example.org/data",
    description = "A dataset about something",
    coverage = "Global"
  )
  result <- as_dublincore(df, type = "dataset_df")
  expect_s3_class(result, "dataset_df")
  expect_true(all(c("title", "creator", "identifier", "publisher") %in% names(result)))
})

test_that("returns ntriples string if type = 'ntriples'", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- dublincore(
    title = "Triple Output",
    creator = person("Jane", "Doe"),
    contributor = person("Contributor", "Name")
  )
  result <- as_dublincore(df, type = "ntriples")
  expect_type(result, "character")
  expect_true(grepl("http", result[1]))
})

test_that("invalid type falls back to bibentry with warning", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- dublincore(
    title = "Fallback",
    creator = person("Jane", "Doe")
  )
  expect_warning(
    result <- as_dublincore(df, type = "nonsense"),
    "type cannot be"
  )
  expect_s3_class(result, "dublincore")
})

test_that("errors if author argument is not a person", {
  df <- dataset_df(x = 1:3)
  attr(df, "dataset_bibentry") <- dublincore(
    title = "Bad Author",
    creator = person("Jane", "Doe")
  )
  expect_error(
    as_dublincore(df, author = "not a person"),
    "author must be created with utils\\:\\:person"
  )
})

test_that("creator() retrieves the current creator from dataset_df", {
  df <- dataset_df(data.frame(x = 1:3))
  creator(df) <- person("Alice", "Smith", role = "aut")
  result <- creator(df)
  expect_s3_class(result, "person")
  expect_equal(as.character(result), "Alice Smith [aut]")
})

test_that("creator<-() sets a new creator", {
  df <- dataset_df(data.frame(x = 1:3))
  creator(df) <- person("Jane", "Doe", role = "aut")
  expect_equal(as.character(creator(df)), "Jane Doe [aut]")
})

test_that("creator<-() appends new creator when overwrite = FALSE", {
  df <- dataset_df(data.frame(x = 1:3))
  creator(df) <- person("Alice", "Smith", role = "aut")
  creator(df, overwrite = FALSE) <- person("Bob", "Jones", role = "ctb")
  names <- vapply(creator(df), as.character, character(1))
  expect_true("Alice Smith [aut]" %in% names)
  expect_true("Bob Jones [ctb]" %in% names)
  creator(df, overwrite = FALSE) <- person("Bob", "Jones", role = "aut")
  names2 <- vapply(creator(df), as.character, character(1))
  expect_true("Bob Jones [aut]" %in% names2)
  creator(df, overwrite = TRUE) <- person("Joe", "Doe", role = "aut")
  names3 <- vapply(creator(df), as.character, character(1))
  expect_false("Bob Jones [aut]" %in% names3)
  expect_true("Joe Doe [aut]" %in% names3)
})

test_that("creator<-() does not change anything when value is NULL", {
  df <- dataset_df(data.frame(x = 1:3))
  creator(df) <- person("Test", "User", role = "aut")

  creator(df) <- NULL # This should return `df` unchanged
  expect_equal(as.character(creator(df)), "Test User [aut]")
})

test_that("creator<-() errors when value is not a person object", {
  df <- dataset_df(data.frame(x = 1:3))
  expect_error(
    creator(df) <- "Jane Doe",
    regexp = "value` must be a utils::person object"
  )
})

test_that("creator() errors on non-dataset_df input", {
  x <- data.frame(x = 1:3)
  expect_error(
    creator(x),
    regexp = "creator\\(x\\): x must be a dataset object"
  )
})

test_that("creator<-() errors on non-dataset_df input", {
  x <- data.frame(x = 1:3)
  expect_error(
    creator(x) <- person("Fake", "User", role = "aut"),
    regexp = "creator\\(x\\) <- value: x must be a dataset object"
  )
})

#' Get or set the technical format of a dataset
#'
#' Adds or retrieves the optional `"format"` field of a dataset's bibentry.
#' This field is the dataset's technical/media type (e.g., a MIME type).
#'
#' @details
#' The format field corresponds to
#' [dct:format](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/format/)
#' in Dublin Core and to `format` in
#' [DataCite](https://schema.datacite.org/).
#' It is useful for indicating serialization such as `"text/csv"`,
#' `"application/parquet"`, or `"application/r-rds"`.
#'
#' If no format is set, this helper uses the package default
#' `"application/r-rds"`.
#'
#' @param x A semantically rich data frame created with [dataset_df()] or
#'   [as_dataset_df()].
#' @param value A length‑one character string specifying the format
#'   (e.g., `"text/csv"`). Use `NULL` to reset to the default.
#' @param overwrite Logical. Replace an existing non‑default value? If `FALSE`
#'   and a non‑default value already exists, a message is emitted and the value
#'   is kept. Defaults to `FALSE`.
#'
#' @return
#' The `"format"` (technical format) as a character string (length 1).
#' When assigning, the updated object `x` is returned invisibly.
#'
#' @examples
#' dataset_format(orange_df) <- "text/csv"
#' dataset_format(orange_df)
#'
#' # Reset to the package default
#' dataset_format(orange_df) <- NULL
#'
#' @family bibliographic helper functions
#' @importFrom assertthat assert_that
#' @export
dataset_format <- function(x) {
  assertthat::assert_that(
    is.dataset_df(x),
    msg = "dataset_format(x): x must be a dataset object created with dataset_df() or as_dataset_df()."
  )
  be <- get_bibentry(x)
  val <- be$format
  if (is.null(val)) "application/r-rds" else as.character(val)
}

#' @rdname dataset_format
#' @export
`dataset_format<-` <- function(x, overwrite = FALSE, value) {
  assertthat::assert_that(
    is.dataset_df(x),
    msg = "dataset_format(x) <- value: x must be a dataset object created with dataset_df() or as_dataset_df()."
  )
  assertthat::assert_that(
    is.null(value) || length(value) == 1,
    msg = "dataset_format(x) <- value: `value` must be length 1 or NULL."
  )

  be <- get_bibentry(x)

  if (is.null(value)) {
    be$format <- "application/r-rds"
    attr(x, "dataset_bibentry") <- be
    return(invisible(x))
  }

  # Treat these as non-user values we can safely replace without overwrite=TRUE
  is_default <- is.null(be$format) ||
    identical(be$format, "application/r-rds") ||
    identical(be$format, ":tba")

  if (is_default || overwrite) {
    be$format <- as.character(value)
  } else {
    message(
      "The dataset already has a format: ", be$format,
      ". Use overwrite = TRUE to replace it."
    )
  }

  attr(x, "dataset_bibentry") <- be
  invisible(x)
}

#' @title Get or Set the Title of a Dataset
#'
#' @description Retrieve or assign the main title of a dataset, typically used
#' as the primary label in metadata exports (e.g., DataCite or Dublin Core).
#'
#' @details According to the [Dublin Core specification for
#' `title`](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/title/),
#' the title represents the name by which the resource is formally known.
#'
#' The DataCite metadata schema supports multiple titles (e.g., translated,
#' alternative), but this function currently supports only a single main title.
#'
#' @param x A dataset object created by [dataset_df()] or [as_dataset_df()].
#' @param value A character string representing the new title. If `NULL`, a
#'   placeholder value `":tba"` is assigned. If `value` is a character vector of
#'   length > 1, an error is raised.
#' @param overwrite Logical. If `TRUE`, the existing title is replaced. If
#'   `FALSE` (default) and a title is already present, a warning is issued and
#'   the title is not changed.
#'
#' @return `dataset_title()` returns the current dataset title as a character
#' string. `dataset_title<-()` returns the updated dataset object (invisible).
#'
#' @examples
#' dataset_title(orange_df)
#'
#' # Set a new title with overwrite = TRUE
#' dataset_title(orange_df, overwrite = TRUE) <- "The Growth of Orange Trees"
#' dataset_title(orange_df)
#'
#' @family bibliographic helper functions
#' @rdname dataset_title
#' @export

dataset_title <- function(x) {
  if (!is.dataset_df(x)) {
    stop("dataset_title(x) must be a dataset object created with dataset() or as_dataset_df().")
  }

  ds_bibentry <- get_bibentry(x)
  ds_bibentry$title
}

#' @rdname dataset_title
#' @importFrom stats setNames
#' @export
`dataset_title<-` <- function(x, overwrite = FALSE, value) {
  if (!is.dataset_df(x)) {
    stop("title(x) <- x must be a dataset object created with dataset() or as_dataset_df().")
  }

  ds_bibentry <- invisible(get_bibentry(x))

  if (is.null(value)) {
    ds_bibentry$title <- ":tba"
    attr(x, "dataset_bibentry") <- ds_bibentry
    return(x)
  }

  if (inherits(value, "factor") || inherits(value, "character")) {
    if (length(value) > 1) {
      stop("title(x) <- value: if you have multiple titles, use dataset_title_create()")
    }
  }

  if (!ds_bibentry$title %in% c(":unas", ":tba", "") && !overwrite) {
    warning("The dataset already has a title: ", ds_bibentry$title)
  } else {
    ds_bibentry$title <- value
  }

  attr(x, "dataset_bibentry") <- ds_bibentry

  invisible(x)
}

#' @keywords internal
`agent<-` <- function(x, value) {
  return_type <- NULL

  if ("dataset_bibentry" %in% names(attributes(x))) {
    dataset_bibentry <- get_bibentry(x)
    return_type <- "dataset"
  }

  if (inherits(x, "dublincore")) {
    dataset_bibentry <- x
    return_type <- "dublincore"
  }

  if (inherits(x, "datacite")) {
    dataset_bibentry <- x
    return_type <- "datacite"
  }

  assertthat::assert_that(
    !is.null(x),
    msg = "Error: agent(x)<- x must be a dataset_df, a dublincore or a datacite object."
  )


  assertthat::assert_that(all(inherits(value, "person")),
    msg = "Error: agent(x) <- value: value must be a vector of utils::persons() class."
  )

  creators <- ifelse(is.null(dataset_bibentry$author), ":tba", dataset_bibentry$author)
  publishers <- ifelse(is.null(dataset_bibentry$publisher), ":unas", dataset_bibentry$publisher)
  contributors <- ifelse(is.null(dataset_bibentry$contributor), ":unas", dataset_bibentry$contributor)

  get_creator <- function(x) {
    if (!is.null(x$role)) ifelse("cre" %in% x$role, TRUE, FALSE) else FALSE
  }

  get_author <- function(x) {
    if (!is.null(x$role)) ifelse("aut" %in% x$role, TRUE, FALSE) else FALSE
  }

  get_publisher <- function(x) {
    if (!is.null(x$role)) ifelse("pbl" %in% x$role, TRUE, FALSE) else FALSE
  }

  is_creator <- vapply(value, get_creator, logical(1))
  is_author <- vapply(value, get_author, logical(1))
  is_publisher <- vapply(value, get_publisher, logical(1))


  new_creators <- c(value[is_creator], value[is_author[!is_creator]])
  new_publishers <- c(value[is_publisher])
  new_contributors <- c(value[!value %in% c(creators, publishers)])

  creators <- ifelse(length(new_creators) > 0, new_creators, creators)
  publishers <- ifelse(length(new_publishers) > 0, new_publishers, publishers)
  contributors <- ifelse(length(new_creators) > 0,
    new_contributors,
    contributors
  )

  dataset_bibentry$author <- ifelse(length(new_creators) > 0,
    new_creators,
    dataset_bibentry[[1]]$author
  )
  dataset_bibentry$contributor <- contributors
  dataset_bibentry$publisher <- publishers

  if (return_type %in% c("datacite", "dublincore")) {
    dataset_bibentry
  } else if (return_type == "dataset") {
    attr(x, "dataset_bibentry") <- dataset_bibentry
    invisible(x)
  }
}

#' @keywords internal
agent <- function(x) {
  return_type <- NULL

  if (inherits(x, "dataset_df")) {
    dataset_bibentry <- get_bibentry(x)
    return_type <- "dataset"
  }

  if (inherits(x, "datacite")) {
    dataset_bibentry <- x
    creators <- ifelse(is.null(dataset_bibentry$author), ":tba", dataset_bibentry$author)
    publishers <- ifelse(is.null(dataset_bibentry$publisher), ":unas", dataset_bibentry$publisher)
    contributors <- ifelse(is.null(dataset_bibentry$contributor), ":unas", dataset_bibentry$contributor)
    return_type <- "datacite"
  }

  if (inherits(x, "dublincore")) {
    dataset_bibentry <- x
    creators <- ifelse(is.null(dataset_bibentry$author), ":tba", dataset_bibentry$author)
    publishers <- ifelse(is.null(dataset_bibentry$publisher), ":unas", dataset_bibentry$publisher)
    contributors <- ifelse(is.null(dataset_bibentry$contributor), ":unas", dataset_bibentry$contributor)

    return_type <- "dublincore"
  }

  if (all(inherits(x, "person"))) {
    contributors <- x
    publishers <- x
    creators <- x
    return_type <- "persons_vector"
  }

  assertthat::assert_that(
    any(return_type %in% c("dataset", "datacite", "dublincore", "persons_vector")),
    msg = "Error: agent(x)<- x must be a dataset_df, a vector of persons, a dublincore or datacite object."
  )

  get_creator <- function(x) {
    if (!is.null(x$role)) ifelse("cre" %in% x$role, TRUE, FALSE) else FALSE
  }

  get_author <- function(x) {
    if (!is.null(x$role)) ifelse("aut" %in% x$role, TRUE, FALSE) else FALSE
  }

  get_contributor <- function(x) {
    if (!is.null(x$role)) ifelse("ctb" %in% x$role, TRUE, FALSE) else FALSE
  }

  get_publisher <- function(x) {
    if (!is.null(x$role)) ifelse("pbl" %in% x$role, TRUE, FALSE) else FALSE
  }

  is_creator <- vapply(creators, get_creator, logical(1))
  is_author <- vapply(creators, get_author, logical(1))
  is_publisher <- vapply(creators, get_publisher, logical(1))
  is_contributor <- vapply(creators, get_contributor, logical(1))

  new_creators <- c(creators[is_creator], creators[is_author[!is_creator]])
  new_contributors <- c(contributors[!contributors %in% c(creators, publishers)])
  new_publishers <- publishers[is_publisher]
  new_contributors <- contributors[is_contributor]

  creators <- if (length(new_creators) > 0) creators <- new_creators
  contributors <- if (length(new_contributors) > 0) contributors <- new_contributors
  publishers <- if (length(new_publishers) > 0) publishers <- new_publishers

  list(
    creators = creators,
    contributors = contributors,
    publishers = publishers
  )
}

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(dataset)

## ----dataframe----------------------------------------------------------------
data.frame(
  geo = c("LI", "SM"),
  CPI = c("0.8", "0.9"),
  GNI = c("8976", "9672")
)

## ----attributes---------------------------------------------------------------
x <- 2457
attr(x, "unit") <- "CP_MEUR"
attr(x, "concept") <- "http://data.europa.eu/83i/aa/GDP"

## -----------------------------------------------------------------------------
library(dataset)

gdp <- defined(
  c(2355, 2592, 2884),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo <- defined(
  rep("AD", 3),
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

## -----------------------------------------------------------------------------
var_concept(gdp)
var_unit(gdp)
var_namespace(geo)

## ----smalldataset-------------------------------------------------------------
small_dataset <- dataset_df(
  geo = geo,
  gdp = gdp,
  identifier = c(gdp = "http://example.com/dataset#gdp"),
  dataset_bibentry = dublincore(
    title = "Small GDP Dataset",
    creator = person("Jane", "Doe", role = "aut"),
    publisher = "Small Repository",
    subject = "Gross Domestic Product"
  )
)

## ----bibrecord----------------------------------------------------------------
as_dublincore(small_dataset)
as_datacite(small_dataset)

## ----provenance---------------------------------------------------------------
provenance(small_dataset)

## ----description--------------------------------------------------------------
description_nt <- tempfile(pattern = "small_dataset", fileext = ".nt")
describe(small_dataset, description_nt)

# Only a few lines shown:
readLines(description_nt)[5:8]

## ----eurogpd------------------------------------------------------------------
euro_gdp <- defined(
  c(2355, 2592),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo_europe <- defined(
  c("AD", "LI"),
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

euros_dataset <- dataset_df(
  geo = geo_europe,
  gdp = euro_gdp,
  dataset_bibentry = dublincore(
    title = "European Microstates GDP",
    creator = person("Statistical Unit", role = "aut"),
    publisher = "Eurostat",
    subject = "Gross Domestic Product"
  )
)

## ----usdgdp-------------------------------------------------------------------
usd_gdp <- defined(
  56,
  label = "Gross Domestic Product",
  unit = "USD_MILLIONS",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo_tuvalu <- defined(
  "TV",
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

tuvalu_dataset <- dataset_df(
  geo = geo_tuvalu,
  gdp = usd_gdp,
  dataset_bibentry = dublincore(
    title = "Tuvalu GDP (USD)",
    creator = person("Island", "Bureau", role = "aut"),
    publisher = "PacificStats",
    subject = "Gross Domestic Product"
  )
)

## -----------------------------------------------------------------------------
binded <- try(bind_defined_rows(euros_dataset, tuvalu_dataset), silent = TRUE)

## ----mutate-------------------------------------------------------------------
exchange_rate <- 1.02
eur_tuv_gdp <- defined(
  56 * exchange_rate,
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

tuvalu_dataset <- dataset_df(
  geo = geo_tuvalu,
  gdp = eur_tuv_gdp,
  dataset_bibentry = dublincore(
    title = "Tuvalu GDP (USD)",
    creator = person("Island", "Bureau", role = "aut"),
    publisher = "PacificStats",
    subject = "Gross Domestic Product"
  )
)

## -----------------------------------------------------------------------------
var_unit(eur_tuv_gdp) <- "M_EUR"

## ----metadatachanges----------------------------------------------------------
global_dataset <- bind_defined_rows(euros_dataset, tuvalu_dataset)
dataset_title(global_dataset, overwrite = TRUE) <- "Global Microstates GDP"
publisher(global_dataset) <- "My Research Institute"
creator(global_dataset) <- person("Jane Doe", role = "aut")
language(global_dataset) <- "en"
description(global_dataset) <- "A dataset created from various sources about the GDP of very small states."
global_dataset

## -----------------------------------------------------------------------------
as_dublincore(global_dataset)

## -----------------------------------------------------------------------------
dataset_to_triples(global_dataset)

## -----------------------------------------------------------------------------
dataset_to_triples(global_dataset, format = "nt")

#' @title Create, add, or retrieve a subject
#'
#' @description
#' Manage the subject metadata of a dataset. The subject can be stored as a
#' simple character term or as a structured object with subproperties created by
#' [subject_create()].
#'
#' @details
#' The subject property records what the dataset is about.
#' The [DataCite subject property](https://schema.datacite.org/meta/kernel-4/)
#' allows multiple subproperties, but these cannot be stored directly in a
#' standard [`utils::bibentry`] object.
#' Therefore:
#'
#' * If you set a character string as the subject, it is stored in both the
#'   bibentry and the `"subject"` attribute.
#' * If you set a structured subject (via [subject_create()]), the `$term` value
#'   is stored in the bibentry, and the full object is stored in the `"subject"`
#'   attribute of the [`dataset_df`] object.
#'
#' @param x A dataset object created with [dataset_df()] or [as_dataset_df()].
#' @param term A subject term, for example `"Data sets"`.
#' @param schemeURI URI of the subject identifier scheme, for example
#'   `"http://id.loc.gov/authorities/subjects"`.
#' @param valueURI URI of the subject term, for example
#'   `"https://id.loc.gov/authorities/subjects/sh2018002256"`.
#' @param prefix Abbreviated prefix for a scheme URI, for example `"lcch:"`.
#'   Widely used namespaces (schemes) have conventional abbreviations.
#' @param subjectScheme Name of the subject scheme, classification code, or
#'   authority if one is used. This acts as a namespace.
#' @param classificationCode Classification code for schemes that do not have
#'   `valueURI` entries for each subject term (e.g., ANZSRC).
#' @param value A subject object created by [subject_create()] or a character
#'   string. Used by `subject<-` to replace the subject.
#'
#' @return
#' * `subject(x)` returns:
#'   - a single `"subject"` object if only one is present,
#'   - a list of `"subject"` objects if multiple are present,
#'   - otherwise falls back to the plain string from the bibentry.
#' * `subject(x) <- value` accepts a character vector, a `"subject"` object, or
#'   a list of `"subject"` objects, and updates both the bibentry slot and the
#'   `"subject"` attribute. Returns the dataset invisibly.
#' * `subject_create()` returns a structured `"subject"` object — or a list of
#'   them if multiple terms are provided.
#' * `is.subject(x)` returns `TRUE` if `x` inherits from class `"subject"`.
#'
#' @examples
#' # Set a structured subject
#' subject(orange_df) <- subject_create(
#'   term = "Oranges",
#'   schemeURI = "http://id.loc.gov/authorities/subjects",
#'   valueURI = "http://id.loc.gov/authorities/subjects/sh85095257",
#'   subjectScheme = "LCCH",
#'   prefix = "lcch:"
#' )
#'
#' # Retrieve subject with subproperties
#' subject(orange_df)
#'
#' @family bibliographic helper functions
#' @importFrom assertthat assert_that
#' @export
#' @export
subject <- function(x) {
  assertthat::assert_that(
    is.dataset_df(x),
    msg = "subject(x): x must be a dataset_df object created with dataset_df() or as_dataset_df()."
  )

  subj_attr <- attr(x, "subject", exact = TRUE)
  if (!is.null(subj_attr)) {
    if (is.subject(subj_attr)) {
      return(subj_attr)
    }
    if (is.list(subj_attr) && all(vapply(subj_attr, is.subject, logical(1)))) {
      return(if (length(subj_attr) == 1) subj_attr[[1]] else subj_attr)
    }
    return(subj_attr)
  }

  subj_bib <- get_bibentry(x)$subject
  if (!is.null(subj_bib)) {
    return(subj_bib)
  }

  message("No subject is recorded.")
  NULL
}


#' @rdname subject
#' @export
subject_create <- function(term,
                           schemeURI = NULL,
                           valueURI = NULL,
                           prefix = NULL,
                           subjectScheme = NULL,
                           classificationCode = NULL) {
  if (is.null(term)) term <- ":tba"

  if (length(term) > 1) {
    dataset_subject <- lapply(seq_along(term), function(i) {
      new_Subject(
        term[i],
        subjectScheme = subjectScheme[i],
        schemeURI = schemeURI[i],
        valueURI = if (!is.null(valueURI)) valueURI[i] else NULL,
        classificationCode = if (!is.null(classificationCode)) classificationCode[i] else NULL,
        prefix = prefix[i]
      )
    })
    # don’t force class on the whole list
    return(dataset_subject)
  } else {
    return(new_Subject(
      term = term,
      subjectScheme = subjectScheme,
      schemeURI = schemeURI,
      valueURI = valueURI,
      classificationCode = classificationCode,
      prefix = prefix
    ))
  }
}

#' @inheritParams Subject
#' @keywords internal
new_Subject <- function(term,
                        schemeURI = NULL,
                        valueURI = NULL,
                        prefix = NULL,
                        subjectScheme = NULL,
                        classificationCode = NULL) {
  if (is.null(subjectScheme)) subjectScheme <- ""
  if (is.null(schemeURI)) schemeURI <- ""
  if (is.null(valueURI)) valueURI <- ""
  if (is.null(prefix)) prefix <- ""

  if (!is.null(classificationCode)) {
    dataset_subject <- list(
      term = term,
      subjectScheme = subjectScheme,
      schemeURI = schemeURI,
      classificationCode = classificationCode,
      prefix = prefix
    )
  } else {
    dataset_subject <- list(
      term = term,
      subjectScheme = subjectScheme,
      schemeURI = schemeURI,
      valueURI = valueURI,
      classificationCode = classificationCode,
      prefix = prefix
    )
  }

  class(dataset_subject) <- c("subject", class(dataset_subject))

  dataset_subject
}

#' @rdname subject
#' @export
`subject<-` <- function(x, value) {
  assert_that(
    is.dataset_df(x),
    msg = "subject<-(x, value): x must be a dataset object created with dataset_df() or as_dataset_df()."
  )

  ds_bibentry <- get_bibentry(x)

  # normalize input
  if (is.null(value)) {
    value <- list(new_Subject(term = ":tba"))
  } else if (is.character(value)) {
    value <- lapply(value, new_Subject)
  } else if (is.subject(value)) {
    value <- list(value) # wrap single subject
  } else if (is.list(value) && all(vapply(value, is.subject, logical(1)))) {
    # already a list of subjects, ok
  } else {
    stop("subject(x, value)<- : value must be created with `subject_create()` or be a character string (or list thereof).")
  }

  # flatten terms into bibentry
  ds_bibentry$subject <- vapply(value, function(s) s$term, character(1))
  attr(x, "dataset_bibentry") <- ds_bibentry

  # keep full structured objects
  attr(x, "subject") <- value

  invisible(x)
}

#' @rdname subject
is.subject <- function(x) {
  ifelse(inherits(x, "subject"), TRUE, FALSE)
}

#' @keywords internal
default_subject <- subject_create(
  term = "Data sets",
  subjectScheme = "LCSH",
  schemeURI = "http://id.loc.gov/authorities/subjects",
  valueURI = "http://id.loc.gov/authorities/subjects/sh2018002256",
  prefix = "lcsh:"
)

test_that("subject() errors for non-dataset_df", {
  expect_error(
    subject(mtcars),
    "subject\\(x\\): x must be a dataset"
  )
})

test_that("subject() returns the structured subject
          from attribute when present", {
  df <- dataset_df(data.frame(x = 1:3))
  subject(df) <- subject_create("Biodiversity")
  s <- subject(df)
  expect_s3_class(s, "subject")
  expect_equal(s$term, "Biodiversity")
})

test_that("subject() returns the structured subject
          from attribute when present", {
  df <- dataset_df(data.frame(x = 1:3))
  subject(df) <- subject_create("Biodiversity")
  s <- subject(df)
  expect_s3_class(s, "subject")
  expect_equal(s$term, "Biodiversity")
})

test_that("subject() falls back to bibentry$subject when attr is missing", {
  df <- dataset_df(data.frame(x = 1:3))
  # set via character → bibentry$subject set to term + attr(subject) set
  subject(df) <- "Forestry"
  # remove attr to force fallback
  attr(df, "subject") <- NULL
  got <- subject(df)
  expect_equal(got, "Forestry")
})

test_that("subject() returns default subject for a fresh dataset_df", {
  df <- dataset_df(data.frame(x = 1:3))
  s <- subject(df)
  expect_s3_class(s, "subject")
  expect_equal(s$term, default_subject$term) # e.g., "Data sets"
})


test_that("subject() messages and returns NULL when subject truly missing", {
  df <- dataset_df(data.frame(x = 1:3))
  # Nuke both attr and bibentry entry to simulate a broken object
  attr(df, "subject") <- NULL
  be <- get_bibentry(df)
  be$subject <- NULL
  attr(df, "dataset_bibentry") <- be

  expect_message(out <- subject(df), "No subject is recorded\\.")
  expect_null(out)
})


# --- subject_create / new_Subject ---

test_that("subject_create() creates a valid subject object", {
  s <- subject_create(
    term = "Climate Data",
    subjectScheme = "LCSH",
    schemeURI = "http://id.loc.gov/authorities/subjects",
    valueURI = "https://id.loc.gov/authorities/subjects/sh2001000174",
    prefix = "lcsh:"
  )
  expect_s3_class(s, "subject")
  expect_equal(s$term, "Climate Data")
  expect_equal(s$subjectScheme, "LCSH")
  expect_equal(s$schemeURI, "http://id.loc.gov/authorities/subjects")
  expect_equal(s$valueURI, "https://id.loc.gov/authorities/subjects/sh2001000174")
  expect_equal(s$prefix, "lcsh:")
})

test_that("subject_create() handles NULL term gracefully", {
  s <- subject_create(term = NULL)
  expect_s3_class(s, "subject")
  expect_equal(s$term, ":tba")
})

test_that("subject_create() handles multiple terms", {
  terms <- c("Climate", "Agriculture")
  schemes <- c("LCSH", "LCSH")
  s <- subject_create(term = terms, subjectScheme = schemes)
  expect_type(s, "list")
  expect_s3_class(s[[1]], "subject")
  expect_equal(s[[1]]$term, "Climate")
  expect_equal(s[[2]]$term, "Agriculture")
})

test_that("new_Subject() constructs with/without classificationCode", {
  s1 <- new_Subject("Environment")
  expect_s3_class(s1, "subject")
  expect_equal(s1$term, "Environment")

  s2 <- new_Subject("Biology", classificationCode = "06")
  expect_s3_class(s2, "subject")
  expect_equal(s2$classificationCode, "06")
})

# --- is.subject ---

test_that("is.subject() returns TRUE for subject objects", {
  s <- subject_create("Health")
  expect_true(is.subject(s))
})

test_that("is.subject() returns FALSE for non-subject", {
  expect_false(is.subject(list(term = "NotClassed")))
  expect_false(is.subject("Just a string"))
})

# --- setter semantics ---

test_that("subject<- handles NULL and character input", {
  df <- dataset_df(data.frame(x = 1:3))

  subject(df) <- NULL
  expect_equal(subject(df)$term, ":tba")

  subject(df) <- "Oceanography"
  expect_equal(subject(df)$term, "Oceanography")
})

test_that("subject<- errors for invalid input types", {
  df <- dataset_df(data.frame(x = 1:3))
  expect_error(
    {
      subject(df) <- list(term = "Invalid")
    },
    regexp = "value must be created with"
  )
})


test_that("subject<- replaces both attr and bibentry field", {
  df <- dataset_df(data.frame(x = 1:3))
  s <- subject_create("Volcanology")
  subject(df) <- s

  be <- get_bibentry(df)

  expect_equal(subject(df)$term, "Volcanology")
  expect_s3_class(subject(df), "list")
})

# --- integration with as_datacite(dataset_df) flattening ---

test_that("as_datacite(dataset_df) flattens subject to its term", {
  df <- dataset_df(data.frame(x = 1))
  creator(df) <- person("Jane", "Doe", role = "cre")
  subject(df) <- subject_create(
    term = "Oranges",
    schemeURI = "http://id.loc.gov/authorities/subjects",
    valueURI = "http://id.loc.gov/authorities/subjects/sh85095257",
    subjectScheme = "LCCH",
    prefix = "lcch:"
  )
  out <- as_datacite(df, type = "dataset_df")
  expect_s3_class(out, "dataset_df")
  expect_equal(as.character(out$Subject), "Oranges")
})

test_that("subject<- handles multiple subjects as list", {
  df <- dataset_df(data.frame(x = 1:3))

  subject(df) <- subject_create(c("Forests", "Agriculture"))

  s <- attr(df, "subject")
  expect_type(s, "list")
  expect_s3_class(s[[1]], "subject")
  expect_equal(s[[1]]$term, "Forests")
  expect_equal(s[[2]]$term, "Agriculture")

  be <- get_bibentry(df)
  expect_true(all(c("Forests", "Agriculture") %in% be$subject))

  got <- subject(df)
  expect_type(got, "list")
  expect_equal(
    vapply(got, `[[`, "term", FUN.VALUE = character(1)),
    c("Forests", "Agriculture")
  )
})

test_that("id_to_column works on a basic data.frame", {
  df <- data.frame(a = 1:3, b = letters[1:3])
  out <- id_to_column(df)
  expect_true("rowid" %in% names(out))
  expect_equal(out$rowid, paste0("eg:", 1:3))
})

test_that("id_to_column replaces existing rowid column", {
  df <- data.frame(rowid = c("x", "y", "z"), a = 1:3)
  out <- id_to_column(df, prefix = "custom:")
  expect_equal(out$rowid, paste0("custom:", 1:3))
})

test_that("id_to_column accepts custom ids", {
  df <- data.frame(a = 1:2)
  out <- id_to_column(df, prefix = "ex:", ids = c("foo", "bar"))
  expect_equal(out$rowid, c("ex:foo", "ex:bar"))
})

test_that("id_to_column errors on mismatched ids length", {
  df <- data.frame(a = 1:3)
  expect_error(
    id_to_column(df, ids = c("a", "b")),
    "ids must be of same lengths as nrow"
  )
})

test_that("id_to_column handles special characters in row names", {
  df <- data.frame(a = 1:2)
  rownames(df) <- c("first row", "second#row!")
  out <- id_to_column(df)
  expect_equal(out$rowid, c("eg:first-row", "eg:second-row-"))
})

test_that("id_to_column works with prefix = NULL", {
  df <- data.frame(a = 1:2)
  out <- id_to_column(df, prefix = NULL)
  expect_equal(out$rowid, c("1", "2"))
})

test_that("id_to_column preserves dataset_df structure", {
  df <- dataset_df(a = 1:2)
  out <- id_to_column(df)
  expect_s3_class(out, "dataset_df")
  expect_true("rowid" %in% names(out))
})

test_that("id_to_column preserves dataset_df metadata", {
  df <- dataset_df(
    a = 1:2,
    dataset_bibentry = dublincore(
      title = "Test",
      creator = person("A", "B")
    )
  )
  subject(df) <- subject_create(
    term = "Test",
    valueURI = "http://example.org/test"
  )
  attr(df, "prov") <- list(generated = "now")

  out <- id_to_column(df, ids = c("a", "b"))
  expect_equal(attr(out, "dataset_bibentry")$title, "Test")
  expect_equal(subject(out)$term, "Test")
  expect_equal(subject(out)$valueURI, "http://example.org/test")
  expect_equal(attr(out, "prov")$generated, "now")
})

test_that("id_to_column works on dataset_df with custom ids", {
  df <- dataset_df(a = 1:2)
  out <- id_to_column(df, prefix = "ex:", ids = c("x", "y"))
  expect_equal(out$rowid, c("ex:x", "ex:y"))
})

test_that("id_to_column handles prefix = NULL with existing rowid", {
  df <- data.frame(rowid = c("old1", "old2"), a = 1:2)
  out <- id_to_column(df, prefix = NULL)
  # Should replace rowid, no prefix applied
  expect_equal(out$rowid, c("1", "2"))
})

test_that("id_to_column works with dataset_df with no provenance", {
  df <- dataset_df(
    a = 1:2,
    dataset_bibentry = dublincore(title = "NoProv", creator = person("X", "Y"))
  )
  # Explicitly remove provenance to test NULL handling
  attr(df, "prov") <- NULL
  out <- id_to_column(df)
  expect_s3_class(out, "dataset_df")
  # Rowids should still be present
  expect_equal(out$rowid, c("eg:1", "eg:2"))
})

test_that("id_to_column final return path is exercised", {
  df <- data.frame(a = 1:2)
  # No rowid column means it should go through lhs/rhs binding and end with tmp
  out <- id_to_column(df, prefix = "z:")
  expect_equal(out$rowid, c("z:1", "z:2"))
  expect_equal(names(out)[1], "rowid")
})

#' @title Add or retrieve related items (DataCite/Dublin Core)
#'
#' @description Manage related resources for a dataset using a unified accessor.
#' - For **DataCite 4.x**, this maps to `relatedIdentifier` (+ type & relation).
#' - For **Dublin Core**, this maps to `dct:relation` (string).
#'
#' @details To remain compatible with [utils::bibentry()], the bibentry stores
#' only the
#' **string identifier** (e.g., DOI/URL). The full structured object created by
#' [related_create()] is preserved in the `"relation"` attribute.
#'
#' A `"related"` object is a small S3 list with the following elements:
#' - `relatedIdentifier`: the related resource identifier (DOI, URL, etc.)
#' - `relationType`: the DataCite relation type (e.g., `"IsPartOf"`, `"References"`)
#' - `relatedIdentifierType`: the type of identifier (`"DOI"`, `"URL"`, etc.)
#' - `resourceTypeGeneral`: optional, the general type of the related resource (e.g., `"Text"`, `"Dataset"`)
#'
#' @param x A dataset object created with [dataset_df()] or [as_dataset_df()].
#' @param value A `related` object from [related_create()] or a character.
#'   Vectors of characters are also supported and will be converted to a list of
#'   `"related"` objects.
#' @param relatedIdentifier A string with the identifier of the related
#'   resource.
#' @param relationType A string naming the relation type (per DataCite
#'   vocabulary).
#' @param relatedIdentifierType A string naming the identifier type (`"DOI"`,
#'   `"URL"`, etc.).
#' @param resourceTypeGeneral Optional: a string naming the general type of the
#'   related resource.
#'
#' @return
#' * `relation(x)` returns:
#'   - a single structured `"related"` object (from [related_create()]) if only
#' one relation is present,
#'   - a list of `"related"` objects if multiple relations are present,
#'   - otherwise it falls back to the bibentry field (`relatedidentifier` for
#' DataCite or `relation` for Dublin Core).
#' * `relation(x) <- value` sets the `"relation"` attribute (structured object
#' or list of objects) and the bibentry string fields (`relatedidentifier` and
#' `relation`), and returns the dataset invisibly.
#' * `related_create()` constructs a structured `"related"` object.
#' * `is.related(x)` returns `TRUE` if `x` inherits from class `"related"`.
#'
#' @examples
#' df <- dataset_df(data.frame(x = 1))
#' relation(df) <- related_create(
#'   relatedIdentifier = "10.1234/example",
#'   relationType = "IsPartOf",
#'   relatedIdentifierType = "DOI"
#' )
#' relation(df) # structured object
#' get_bibentry(df)$relation # "10.1234/example"
#' get_bibentry(df)$relatedidentifier # "10.1234/example"
#'
#' # Character input is normalized to a DOI/URL with default types
#' relation(df) <- "https://doi.org/10.5678/xyz"
#' relation(df) # structured object (relationType/Type filled with defaults)
#'
#' # Create related object directly
#' rel <- related_create("https://doi.org/10.5678/xyz", "References", "DOI")
#' is.related(rel) # TRUE
#'
#' @family bibliographic helper functions
#' @export
relation <- function(x) {
  assertthat::assert_that(
    is.dataset_df(x),
    msg = "relation(x): x must be a dataset_df object."
  )

  rel_attr <- attr(x, "relation", exact = TRUE)
  if (!is.null(rel_attr)) {
    if (is.related(rel_attr)) {
      return(rel_attr)
    }
    if (is.list(rel_attr) && all(vapply(rel_attr, is.related, logical(1)))) {
      return(if (length(rel_attr) == 1) rel_attr[[1]] else rel_attr)
    }
    return(rel_attr)
  }

  be <- get_bibentry(x)
  if (!is.null(be$relatedidentifier)) {
    return(be$relatedidentifier)
  }
  if (!is.null(be$relation)) {
    return(be$relation)
  }

  message("No related item is recorded.")
  NULL
}

#' @rdname relation
#' @export
`relation<-` <- function(x, value) {
  assertthat::assert_that(
    is.dataset_df(x),
    msg = "relation<-(x, value): x must be a dataset_df object created with dataset_df() or as_dataset_df()."
  )

  be <- get_bibentry(x)

  # --- normalize input ---
  if (is.null(value)) {
    value <- list(related_create(":unas", "IsPartOf", "URL"))
  } else if (is.character(value)) {
    # allow vector of strings too
    value <- lapply(value, function(v) {
      related_create(
        relatedIdentifier = v,
        relationType = "IsPartOf",
        relatedIdentifierType = if (grepl("^https?://", v)) "URL" else "DOI"
      )
    })
  } else if (is.related(value)) {
    value <- list(value) # wrap single related in a list
  } else if (is.list(value) && all(vapply(value, is.related, logical(1)))) {
    # already a list of related objects → nothing to do
  } else {
    stop("relation(x, value)<- : value must be created with `related_create()` or be a character string (or list thereof).")
  }

  # --- store in dataset_bibentry ---
  # keep only the identifiers (vector of strings)
  be$relatedidentifier <- vapply(value, function(v) v$relatedIdentifier, character(1))
  be$relation <- be$relatedidentifier
  attr(be, "relation") <- value # structured list of related items

  attr(x, "dataset_bibentry") <- be
  attr(x, "relation") <- value

  invisible(x)
}

#' @rdname relation
#' @export
related_create <- function(relatedIdentifier,
                           relationType,
                           relatedIdentifierType,
                           resourceTypeGeneral = NULL) {
  rel <- list(
    relatedIdentifier = as.character(relatedIdentifier),
    relationType = as.character(relationType),
    relatedIdentifierType = as.character(relatedIdentifierType),
    resourceTypeGeneral = if (is.null(resourceTypeGeneral)) NULL else as.character(resourceTypeGeneral)
  )
  class(rel) <- c("related", "list")
  rel
}

#' @rdname relation
#' @export
is.related <- function(x) inherits(x, "related")

#' @rdname relation
#' @export
related_item <- relation

#' @rdname relation
#' @export
`related_item<-` <- `relation<-`

test_that("datacite_to_triples errors if title missing", {
  dc <- datacite(
    Title = "",
    Creator = person("Jane", "Doe", role = "cre"),
    Publisher = "ACME"
  )
  expect_error(datacite_to_triples(dc), "title is required")
})

test_that("datacite_to_triples includes core fields", {
  dc <- datacite(
    Title = "Climate Data",
    Creator = person("Jane", "Doe", role = "cre"),
    Publisher = "ACME",
    Identifier = "10.1234/xyz",
    Language = "en",
    Rights = "CC-BY-4.0",
    Description = "Test dataset",
    Format = "text/csv",
    Version = "1.0.0"
  )
  triples <- datacite_to_triples(dc)

  expect_true(any(grepl("title", triples)))
  expect_true(any(grepl("creator", triples)))
  expect_true(any(grepl("publisher", triples)))
  expect_true(any(grepl("identifier", triples)))
  expect_true(any(grepl("language", triples)))
  expect_true(any(grepl("rights", triples)))
  expect_true(any(grepl("descriptions", triples)))
  expect_true(any(grepl("formats", triples)))
  expect_true(any(grepl("version", triples)))
})

test_that("datacite_to_triples flattens subject to terms", {
  dc <- datacite(
    Title = "Biodiversity Data",
    Creator = person("Jane", "Doe", role = "cre"),
    Publisher = "ACME",
    Subject = subject_create("Forests", subjectScheme = "LCSH")
  )
  triples <- datacite_to_triples(dc)
  expect_true(any(grepl("subjects", triples)))
  expect_true(any(grepl("Forests", triples)))
})

test_that("datacite_to_triples uses structured relation attribute", {
  dc <- datacite(
    Title = "Climate Data",
    Creator = person("Jane", "Doe", role = "cre"),
    Publisher = "ACME"
  )

  df <- dataset_df(data.frame(x = 1), dataset_bibentry = dc)

  # Set relation on the dataset_df (works via setter)
  relation(df) <- related_create(
    relatedIdentifier = "10.9999/abc",
    relationType = "IsPartOf",
    relatedIdentifierType = "DOI"
  )

  # Extract bibentry back out
  dc <- get_bibentry(df)
  triples <- datacite_to_triples(dc)

  expect_true(any(grepl("relatedIdentifier", triples)))
  expect_true(any(grepl("relationType", triples)))
  expect_true(any(grepl("relatedIdentifierType", triples)))
})


test_that("datacite_to_triples handles multiple related items", {
  df <- dataset_df(data.frame(x = 1),
    dataset_bibentry = datacite(
      Title = "Climate Data",
      Creator = person("Jane", "Doe", role = "cre"),
      Publisher = "ACME"
    )
  )

  relation(df) <- list(
    related_create("10.1111/one", "IsPartOf", "DOI"),
    related_create("https://example.com/rel", "References", "URL", "Text")
  )

  triples <- datacite_to_triples(get_bibentry(df))

  expect_true(sum(grepl("relatedIdentifier", triples)) >= 2)
  expect_true(any(grepl("IsPartOf", triples)))
  expect_true(any(grepl("References", triples)))
  expect_true(any(grepl("relatedIdentifierType", triples)))
  expect_true(any(grepl("resourceTypeGeneral", triples)))
})

test_that("datacite_to_triples falls back to flat relatedidentifier", {
  # here you don’t use the setter, so calling directly on the bibentry is fine
  dc <- datacite(
    Title = "Climate Data",
    Creator = person("Jane", "Doe", role = "cre"),
    Publisher = "ACME",
    RelatedIdentifier = "10.5555/fallback"
  )
  triples <- datacite_to_triples(dc)
  expect_true(any(grepl("10.5555/fallback", triples)))
})

test_that("datacite_to_triples handles mixed subject lists", {
  dc <- datacite(
    Title = "Biodiversity Data",
    Creator = person("Jane", "Doe", role = "cre"),
    Publisher = "ACME",
    Subject = list(
      "Forests",
      subject_create("Agriculture", subjectScheme = "LCSH")
    )
  )

  triples <- datacite_to_triples(dc)

  expect_true(any(grepl("subjects", triples)))
  expect_true(any(grepl("Forests", triples)))
  expect_true(any(grepl("Agriculture", triples)))
})

test_that("relation() gets/sets structured and flat variants", {
  df <- dataset_df(data.frame(x = 1))

  # character → normalized
  relation(df) <- "https://doi.org/10.1234/abc"
  r <- relation(df)
  expect_s3_class(r, "related")
  expect_equal(r$relatedIdentifier, "https://doi.org/10.1234/abc")
  expect_equal(get_bibentry(df)$relation, "https://doi.org/10.1234/abc")
  expect_equal(
    get_bibentry(df)$relatedidentifier,
    "https://doi.org/10.1234/abc"
  )

  # structured
  rel <- related_create("10.5678/def", "References", "DOI", "Text")
  relation(df) <- rel
  r2 <- relation(df)
  expect_equal(r2$relationType, "References")
  expect_equal(get_bibentry(df)$relation, "10.5678/def")
  expect_equal(get_bibentry(df)$relatedidentifier, "10.5678/def")
})


test_that("relation<- errors for invalid input types", {
  df <- dataset_df(data.frame(x = 1))
  expect_error(
    {
      relation(df) <- list(relatedIdentifier = "not-classed")
    },
    regexp = "value must be created with `related_create\\(\\)`"
  )
})

test_that("relation() falls back to bibentry when attribute missing", {
  df <- dataset_df(data.frame(x = 1))
  relation(df) <- related_create("10.1/xyz", "IsPartOf", "DOI")
  attr(df, "relation") <- NULL # remove structured attr
  got <- relation(df)
  expect_true(is.character(got))
  expect_equal(got, "10.1/xyz")
})

test_that("relation() handles multiple related objects", {
  df <- dataset_df(data.frame(x = 1))

  rels <- list(
    related_create("10.1111/one", "IsPartOf", "DOI"),
    related_create("https://example.com/rel", "References", "URL", "Text")
  )

  relation(df) <- rels

  r <- relation(df)

  # should be a list
  expect_type(r, "list")
  expect_length(r, 2)

  # each element should be a `related` object
  expect_true(all(vapply(r, is.related, logical(1))))

  # identifiers should match
  expect_equal(r[[1]]$relatedIdentifier, "10.1111/one")
  expect_equal(r[[2]]$relatedIdentifier, "https://example.com/rel")

  # bibentry slots should contain the flat identifiers
  be <- get_bibentry(df)
  expect_equal(
    be$relatedidentifier,
    c("10.1111/one", "https://example.com/rel")
  )
  expect_equal(be$relation, c("10.1111/one", "https://example.com/rel"))
})

#' Semantic labelled vector class
#'
#' @name haven_labelled_defined
#'
#' @description
#' The class \code{haven_labelled_defined} represents a semantically
#' enriched labelled vector created by \code{\link{defined}}.
#'
#' Objects of this class inherit from:
#'
#' \itemize{
#'   \item \code{\link[haven:labelled]{haven::labelled}} for numeric,
#'     character, and factor data;
#'   \item \code{Date} for temporal dates;
#'   \item \code{POSIXct} and \code{POSIXt} for timestamps;
#'   \item \code{logical} for boolean data.
#' }
#'
#' The class tag enables S3 method dispatch for:
#'
#' \itemize{
#'   \item \code{\link{as_character}}
#'   \item \code{\link{as_numeric}}
#'   \item \code{\link{as_logical}}
#'   \item \code{as.Date}
#'   \item \code{as.POSIXct}
#'   \item \code{\link{print.haven_labelled_defined}}
#'   \item \code{\link{summary.haven_labelled_defined}}
#' }
#'
#' Users normally do not construct this class directly; it is returned
#' as the result of calling \code{\link{defined}}.
#'
#' @return No value is returned. This documentation page exists so that
#'   the class can be referenced in help topics.
#'
#' @keywords internal
NULL

#' @rdname datacite
#' @param x A dataset object created with [dataset_df()].
#' @param type A DataCite 4.4 metadata can be returned as:
#'   `"list"`, `"dataset_df"`, `"bibentry"` (default), or `"ntriples"`.
#' @param ... Optional parameters to add to a `datacite` object.
#'   For example, `author = person("Jane", "Doe")` adds an author if
#'   `type = "dataset_df"`.
#' @return `as_datacite(x, type)` returns the DataCite bibliographical metadata
#'   of `x` either as a list, a bibentry object, an N-Triples text serialisation
#'   or a dataset_df object.
#' @export
as_datacite <- function(x, type = "bibentry", ...) {
  citation_author <- person(NULL, NULL)

  is_person <- function(p) inherits(p, "person")

  arguments <- list(...)
  if (!is.null(arguments$author)) {
    if (is_person(arguments$author)) {
      citation_author <- arguments$author
    } else {
      stop("as_datacite(x, ..., author = ): author must be created with utils::person().")
    }
  }

  if (!type %in% c("bibentry", "list", "dataset_df", "ntriples")) {
    warning(paste0(
      "as_datacite(ds, type=...) type cannot be ",
      type, ". Reverting to 'bibentry'."
    ))
    type <- "bibentry"
  }

  ds_bibentry <- get_bibentry(x)
  Title <- ds_bibentry$title
  Creator <- ds_bibentry$author

  if (!is_person(Creator)) {
    stop('attr(x, "dataset_bibentry")$author is not a person object.')
  }

  # Date / Year fallback
  if (!is.null(ds_bibentry$year)) {
    if (is.null(ds_bibentry$dataset_date)) {
      Date <- as.character(ds_bibentry$year)
    } else {
      Date <- as.character(ds_bibentry$date)
    }
    PublicationYear <- as.character(ds_bibentry$year)
  } else if (!is.null(ds_bibentry$date)) {
    Date <- as.character(ds_bibentry$date)
    PublicationYear <- substr(Date, 1, 4)
  } else {
    Date <- ":tba"
    PublicationYear <- ":unas"
  }

  Publisher <- ifelse(is.null(ds_bibentry$publisher), "", as.character(ds_bibentry$publisher))
  Identifier <- ifelse(is.null(ds_bibentry$identifier), ":tba", as.character(ds_bibentry$identifier))
  Version <- ifelse(is.null(ds_bibentry$version), ":unas", as.character(ds_bibentry$version))
  Description <- ifelse(is.null(ds_bibentry$description), ":unas", as.character(ds_bibentry$description))
  Language <- ifelse(is.null(ds_bibentry$language), ":unas", as.character(ds_bibentry$language))
  Format <- ifelse(is.null(ds_bibentry$format), "application/r-rds", as.character(ds_bibentry$format))
  AlternateIdentifier <- ifelse(is.null(ds_bibentry$alternateidentifier), ":unas", ds_bibentry$alternateidentifier)
  RelatedIdentifier <- ifelse(is.null(ds_bibentry$relatedidentifier), ":unas", ds_bibentry$relatedidentifier)
  Rights <- ifelse(is.null(ds_bibentry$rights), ":tba", as.character(ds_bibentry$rights))
  Geolocation <- ifelse(is.null(ds_bibentry$geolocation), ":unas", as.character(ds_bibentry$geolocation))
  FundingReference <- ifelse(is.null(ds_bibentry$fundingreference), ":unas", as.character(ds_bibentry$fundingreference))

  # Contributors — check attribute if available
  Contributor <- if (!is.null(attr(ds_bibentry, "contributor"))) {
    attr(ds_bibentry, "contributor")
  } else if (!is.null(ds_bibentry$contributor)) {
    ds_bibentry$contributor
  } else {
    ""
  }

  # Subject handling (always structured or NULL)
  if (is.null(ds_bibentry$subject)) {
    Subject <- NULL
  } else if (is.subject(ds_bibentry$subject)) {
    Subject <- ds_bibentry$subject
  } else if (is.character(ds_bibentry$subject)) {
    Subject <- new_Subject(term = ds_bibentry$subject)
  } else {
    stop("Unsupported subject type in dataset_bibentry$subject")
  }

  if (type == "bibentry") {
    new_datacite(
      Title = Title,
      Creator = Creator,
      Identifier = Identifier,
      Publisher = Publisher,
      PublicationYear = PublicationYear,
      Subject = Subject,
      Type = "Dataset",
      Contributor = Contributor,
      Date = Date,
      Language = Language,
      AlternateIdentifier = AlternateIdentifier,
      RelatedIdentifier = RelatedIdentifier,
      Format = Format,
      Version = Version,
      Rights = Rights,
      Description = Description,
      Geolocation = Geolocation,
      FundingReference = FundingReference
    )
  } else if (type == "list") {
    if (identical(Contributor, "")) Contributor <- NULL
    if (identical(Subject, "")) Subject <- NULL

    list(
      Title = Title,
      Creator = Creator,
      Identifier = Identifier,
      Publisher = Publisher,
      PublicationYear = PublicationYear,
      Subject = Subject,
      Type = "Dataset",
      Contributor = Contributor,
      Date = Date,
      Language = Language,
      AlternateIdentifier = AlternateIdentifier,
      RelatedIdentifier = RelatedIdentifier,
      Format = Format,
      Version = Version,
      Rights = Rights,
      Description = Description,
      Geolocation = Geolocation,
      FundingReference = FundingReference
    )
  } else if (type == "dataset_df") {
    # Flatten multi-length values to single strings
    collapse_if_needed <- function(x) {
      if (length(x) > 1) paste(as.character(x), collapse = "; ") else x
    }

    flatten_subject <- function(s) {
      if (is.null(s)) {
        return("")
      }
      if (is.subject(s)) {
        # Use only the term for dataset_df export
        return(s$term)
      }
      as.character(s)
    }

    dataset_df(
      data.frame(
        Title = collapse_if_needed(Title),
        Creator = paste(
          vapply(Creator, function(p) p$family, character(1)),
          collapse = "; "
        ),
        Identifier = collapse_if_needed(Identifier),
        Publisher = collapse_if_needed(Publisher),
        PublicationYear = collapse_if_needed(PublicationYear),
        Subject = collapse_if_needed(flatten_subject(Subject)),
        Type = "Dataset",
        Contributor = if (is.null(Contributor)) {
          ":unas"
        } else {
          collapse_if_needed(as.character(Contributor))
        },
        Date = collapse_if_needed(Date),
        Language = collapse_if_needed(Language),
        AlternateIdentifier = collapse_if_needed(AlternateIdentifier),
        RelatedIdentifier = collapse_if_needed(RelatedIdentifier),
        Format = collapse_if_needed(Format),
        Version = collapse_if_needed(Version),
        Rights = collapse_if_needed(Rights),
        Description = collapse_if_needed(Description),
        Geolocation = collapse_if_needed(Geolocation),
        FundingReference = collapse_if_needed(FundingReference)
      ),
      dataset_bibentry = datacite(
        Title = paste0("The DataCite metadata of `", collapse_if_needed(Title), "`"),
        Creator = Creator,
        Identifier = collapse_if_needed(Identifier),
        Date = collapse_if_needed(Date)
      )
    )
  } else if (type == "ntriples") {
    dataset_id <- if (is.null(Identifier) || Identifier == ":tba") {
      "http://example.com/dataset_tba/"
    } else {
      Identifier
    }

    dc_list <- list(
      title = Title,
      creator = paste(vapply(Creator, function(p) p$family, character(1)), collapse = "; "),
      identifier = Identifier,
      publisher = Publisher,
      publicationyear = PublicationYear,
      subject = Subject,
      type = "http://purl.org/dc/dcmitype/Dataset",
      contributor = Contributor,
      date = Date,
      language = Language,
      alternateidentifier = AlternateIdentifier,
      relatedidentifier = RelatedIdentifier,
      format = Format,
      version = Version,
      rights = Rights,
      description = Description,
      geolocation = Geolocation,
      fundingreference = FundingReference
    )

    datacite_to_triples(dc_list, dataset_id = dataset_id)
  }
}

#' Coerce a defined Date vector to a base R Date
#'
#' @description
#' Coerces a [`haven_labelled_defined`] vector whose underlying type is
#' [`Date`] into a base R `Date` vector.
#'
#' This method preserves the underlying date values and, by default,
#' also retains any semantic metadata attached to the variable.
#'
#' @details
#' Use \code{strip_attributes = TRUE} when flattening or preparing data for
#' external pipelines, but keep the default when working with defined
#' vectors directly.
#'
#' Base R's [`as.Date()`] also works, as it dispatches to this method via
#' S3. However, using `as.Date()` on defined vectors is considered safe
#' because this method ensures metadata is handled predictably.
#'
#' @param x A vector created with [`defined()`] with underlying type
#'   \code{Date}.
#' @param strip_attributes Logical; should the semantic metadata attributes
#'   (label, unit, definition, namespace) be removed from the returned
#'   vector? Defaults to \code{FALSE}.
#'
#' @param ... Additional arguments passed to [base::as.Date()].
#'
#' @return A `Date` vector, optionally carrying semantic metadata.
#'
#' @examples
#' d <- defined(Sys.Date() + 0:2, label = "Observation date")
#'
#' # Recommended usage
#' as.Date(d)
#'
#' # Stripping metadata
#' as.Date(d, strip_attributes = TRUE)
#'
#' @seealso
#'   [`as.POSIXct()`],
#'   [`as_numeric()`], [`as_character()`], [`as_logical()`],
#'   [`defined()`]
#'
#' @export
as.Date.haven_labelled_defined <- function(x,
                                           strip_attributes = FALSE,
                                           ...) {
  if (!inherits(x, "Date")) {
    stop(
      "as.Date.haven_labelled_defined() requires underlying Date vector.",
      call. = FALSE
    )
  }

  if (strip_attributes) {
    attr(x, "label") <- NULL
    attr(x, "unit") <- NULL
    attr(x, "definition") <- NULL
    attr(x, "namespace") <- NULL
  }

  # Remove only the defined wrapper, keep the
  # real POSIXct classes + attributes
  class(x) <- setdiff(class(x), "haven_labelled_defined")
  x
}

#' Get or Set a Unit of Measure
#'
#' Adds or retrieves a unit of measure (UoM) attribute to a vector. Units
#' provide semantic meaning for numeric or character data — such as currency,
#' weight, or time — helping prevent incorrect operations like merging values
#' measured in incompatible units.
#'
#' @param x A vector.
#' @param value A character string specifying the unit of measure, or `NULL` to
#'   remove the unit.
#' @param ... Further arguments for method extensions.
#'
#' @details The `"unit"` attribute stores a machine-readable representation of a
#' unit of measure (e.g., `"kg"`, `"USD"`, `"days"`). This is useful when
#' working with linked open data or when combining data from multiple sources
#' where silent mismatches in units could cause errors.
#'
#' For full integration with semantic metadata (e.g., labels, concepts,
#' namespaces), use [defined()] vectors or [dataset_df()] objects.
#'
#' `get_variable_units()` is an alias for `var_unit()`.
#'
#' See `vignette("defined", package = "dataset")` for end-to-end examples
#' involving semantic enrichment.
#'
#' @return
#' - `var_unit(x)` returns the `"unit"` attribute as a character string.
#' - `var_unit(x) <- value` sets, updates, or removes the unit and returns
#' the modified vector invisibly.
#'
#' @examples
#' # Retrieve the unit of measure (if defined)
#' var_unit(orange_df$circumference)
#'
#' # Regular data.frame columns have no unit by default
#' var_unit(mtcars$wt)
#'
#' # Add a unit to a column
#' var_unit(mtcars$wt) <- "1000 lbs"
#'
#' # Remove the unit
#' var_unit(mtcars$wt) <- NULL
#'
#' @family defined metadata methods and functions
#' @export
var_unit <- function(x, ...) {
  # rlang::check_dots_used()
  UseMethod("var_unit")
}

#' @export
var_unit.default <- function(x, ...) {
  attr(x, "unit", exact = TRUE)
}

#' @rdname var_unit
#'
#' @description
#' The `var_unit<-` assignment method sets, updates, or removes the `"unit"`
#' attribute of a vector. This can be used with `defined()` vectors or base
#' vectors to ensure consistent semantic annotation.
#'
#' @param value A character string defining the unit of measure, or `NULL` to
#'   remove the unit.
#'
#' @return
#' The modified object `x`, returned invisibly with the updated `"unit"`
#' attribute.
#'
#' @export
`var_unit<-` <- function(x, value) {
  UseMethod("var_unit<-")
}


#' @rdname var_unit
#' @export
`var_unit<-.default` <- function(x, value) {
  unit_attribute(x) <- value
  x
}


#' @rdname var_unit
#' @export
get_variable_units <- var_unit


#' @rdname var_unit
#'
#' @description
#' `unit_attribute()` is a low-level helper to directly access the `"unit"`
#' attribute of a vector, without applying fallback logic. It is mainly used
#' internally.
#'
#' @return
#' The `"unit"` attribute of the object `x`, or `NULL` if not set.
#'
#' @export
unit_attribute <- function(x) {
  attr(x, "unit", exact = TRUE)
}

#' @rdname var_unit
#'
#' @description
#' `get_unit_attribute()` is an alias for `unit_attribute()`, included for naming
#' consistency in codebases that distinguish getter/setter patterns.
#'
#' @export
get_unit_attribute <- function(x) {
  unit_attribute(x)
}


#' @rdname var_unit
#'
#' @description
#' `set_unit_attribute()` is the low-level assignment function that sets or
#' removes the `"unit"` attribute of an object. Used internally by
#' `unit_attribute<-`.
#'
#' @param value A single character string or `NULL`. If not of length one, an
#'   error is thrown.
#'
#' @return The object `x` with updated `"unit"` attribute.
#'
#' @export
set_unit_attribute <- function(x, value) {
  if ((!is.character(value) && !is.null(value)) || length(value) > 1) {
    stop(
      "`unit` should be a single character string or NULL",
      call. = FALSE,
      domain = "R-dataset"
    )
  }
  attr(x, "unit") <- value
  x
}

#' @rdname var_unit
#' @export
`unit_attribute<-` <- set_unit_attribute

test_that("var_unit() gets the unit attribute from a vector", {
  x <- 1:3
  attr(x, "unit") <- "meters"
  expect_equal(var_unit(x), "meters")
})

test_that("var_unit() returns NULL if no unit is set", {
  x <- 1:3
  expect_null(var_unit(x))
})

test_that("var_unit<- sets the unit on a vector", {
  x <- 1:3
  var_unit(x) <- "kg"
  expect_equal(attr(x, "unit"), "kg")
})

test_that("var_unit<- removes the unit if set to NULL", {
  x <- 1:3
  attr(x, "unit") <- "dollars"
  var_unit(x) <- NULL
  expect_null(attr(x, "unit"))
})

test_that("get_variable_units is an alias for var_unit", {
  x <- 1:3
  attr(x, "unit") <- "liters"
  expect_equal(get_variable_units(x), "liters")
})

test_that("unit_attribute gets the exact 'unit' attribute", {
  x <- 1:3
  attr(x, "unit") <- "seconds"
  expect_equal(unit_attribute(x), "seconds")
})

test_that("get_unit_attribute is an alias for unit_attribute", {
  x <- 1:3
  attr(x, "unit") <- "minutes"
  expect_equal(get_unit_attribute(x), "minutes")
})

test_that("unit_attribute<- sets a unit using the low-level setter", {
  x <- 1:3
  unit_attribute(x) <- "°C"
  expect_equal(attr(x, "unit"), "°C")
})

test_that("set_unit_attribute sets and removes units correctly", {
  x <- 1:3

  # Set
  y <- set_unit_attribute(x, "EUR")
  expect_equal(attr(y, "unit"), "EUR")

  # Remove
  z <- set_unit_attribute(x, NULL)
  expect_null(attr(z, "unit"))
})

test_that("set_unit_attribute throws for invalid input", {
  expect_error(
    set_unit_attribute(1:3, c("USD", "EUR")),
    "`unit` should be a single character string or NULL"
  )

  expect_error(
    set_unit_attribute(1:3, 42),
    "`unit` should be a single character string or NULL"
  )

  expect_error(
    set_unit_attribute(1:3, TRUE),
    "`unit` should be a single character string or NULL"
  )
})

#' Get or Set the Rights of a Dataset Object
#'
#' Adds or retrieves the optional `"rights"` attribute of a dataset object.
#' This field contains information about intellectual property or usage rights.
#'
#' @details
#' The `"rights"` field corresponds to
#' [dct:rights](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/rights/)
#' from Dublin Core, and to `rights` in [DataCite](https://schema.datacite.org/).
#'
#' Rights information typically includes statements about legal ownership,
#' licensing, or usage conditions. It helps ensure that users understand how
#' a dataset may be reused, cited, or shared.
#'
#' @param x A semantically rich data frame created with [dataset_df()] or
#'   [as_dataset_df()].
#' @param value A character string specifying the rights (e.g., `"CC-BY-4.0"`).
#' @param overwrite Logical. Should the existing value be replaced? If `FALSE`
#'   and a value already exists, the function emits a message instead of
#'   overwriting. Defaults to `FALSE`.
#'
#' @return
#' The `"rights"` attribute of the dataset as a character string (length 1).
#' When assigning, the updated object `x` is returned invisibly.
#'
#' @examples
#' rights(orange_df) <- "CC-BY-SA"
#' rights(orange_df)
#'
#' @family bibliographic helper functions
#' @export

rights <- function(x) {
  assertthat::assert_that(is.dataset_df(x),
    msg = "rights(x): x must be a dataset object created with dataset_df() or as_dataset_df()."
  )

  DataBibentry <- get_bibentry(x)
  as.character(DataBibentry$rights)
}

#' @rdname rights
#' @export
`rights<-` <- function(x, overwrite = FALSE, value) {
  assertthat::assert_that(is.dataset_df(x),
    msg = "rights(x): x must be a dataset object created with dataset_f() or as_dataset_df()."
  )

  assertthat::assert_that(is.null(value) | length(value) == 1,
    msg = "rights(x): x must have length=1 (or set to NULL)."
  )

  DataBibentry <- invisible(get_bibentry(x))

  if (is.null(value)) {
    DataBibentry$rights <- ":unas"
    attr(x, "dataset_bibentry") <- DataBibentry
    return(invisible(x))
  }

  if (length(value) > 1) {
    stop("rights(x) <- value: value must be of length 1.")
  }

  is_unas <- DataBibentry$rights == ":unas"

  if (is.null(DataBibentry$rights)) {
    DataBibentry$rights <- value
  } else if (is_unas) {
    DataBibentry$rights <- value
  } else if (overwrite) {
    DataBibentry$rights <- value
  } else {
    message("The dataset has already a rights field: ", DataBibentry$rights)
  }

  attr(x, "dataset_bibentry") <- DataBibentry
  invisible(x)
}

#' Growth of Orange Trees
#'
#' A dataset recording the growth of orange trees, replicated from the classic
#' [`datasets::Orange`](https://stat.ethz.ch/R-manual/R-devel/library/datasets/html/Orange.html)
#' dataset and implemented as a `dataset_df` S3 class with enhanced semantic metadata.
#'
#' @format A data frame with 35 rows and 4 variables:
#' - `rowid`: A unique identifier for each row (character)
#' - `tree`: Tree identifier (ordered factor)
#' - `age`: Age of the tree in days (numeric)
#' - `circumference`: Trunk circumference in mm (numeric)
#'
#' @details
#' This is a semantically enriched version of the classic Orange dataset,
#' constructed using the `dataset_df()` and `dublincore()` constructors.
#' Each column includes semantic metadata such as units, labels, concepts,
#' or namespace identifiers. The dataset also embeds a machine-readable citation
#' for reproducibility and provenance tracking.
#'
#' ### Constructor Example
#' ```r
#' orange_bibentry <- dublincore(
#'   title = "Growth of Orange Trees",
#'   creator = c(
#'     person(
#'       given = "N.R.",
#'       family = "Draper",
#'       role = "cre",
#'       comment = c(VIAF = "http://viaf.org/viaf/84585260")
#'     ),
#'     person(
#'       given = "H",
#'       family = "Smith",
#'       role = "cre"
#'     )
#'   ),
#'   contributor = person(
#'     given = "Antal",
#'     family = "Daniel",
#'     role = "dtm"
#'   ),
#'   publisher = "Wiley",
#'   datasource = "https://isbnsearch.org/isbn/9780471170822",
#'   dataset_date = 1998,
#'   identifier = "https://doi.org/10.5281/zenodo.14917851",
#'   language = "en",
#'   description = "The Orange data frame has 35 rows and 3 columns of records of the growth of orange trees."
#' )
#'
#' orange_df <- dataset_df(
#'   rowid = defined(paste0("orange:", row.names(Orange)),
#'     label = "ID in the Orange dataset",
#'     namespace = c("orange" = "datasets::Orange")
#'   ),
#'   tree = defined(Orange$Tree,
#'     label = "The number of the tree"
#'   ),
#'   age = defined(Orange$age,
#'     label = "The age of the tree",
#'     unit = "days since 1968/12/31"
#'   ),
#'   circumference = defined(Orange$circumference,
#'     label = "circumference at breast height",
#'     unit = "milimeter",
#'     concept = "https://www.wikidata.org/wiki/Property:P2043"
#'   ),
#'   dataset_bibentry = orange_bibentry
#' )
#'
#' orange_df$rowid <- defined(orange_df$rowid,
#'   namespace = "https://doi.org/10.5281/zenodo.14917851"
#' )
#' ```
#' @references
#' - Draper, N. R. & Smith, H. (1998). *Applied Regression Analysis* (3rd ed.). Wiley.
#' - Pinheiro, J. C. & Bates, D. M. (2000). *Mixed-effects Models in S and S-PLUS*. Springer.
#' - Becker, R. A., Chambers, J. M. & Wilks, A. R. (1988). *The New S Language*. Wadsworth & Brooks/Cole.
#'
#' @examples
#' # Print with semantic citation and data preview
#' print(orange_df)
#'
#' # Access semantic metadata associated with variables
#' print(orange_df$age)
#'
#' # Retrieve the embedded bibliographic record
#' as_dublincore(orange_df)
#'
#' @keywords datasets
"orange_df"

#' @title Get or set the bibentry
#'
#' @description
#' Retrieve or replace the bibliographic entry stored in a dataset's attributes.
#' The entry is a [`utils::bibentry`] used to hold citation metadata for
#' [`dataset_df()`] objects.
#'
#' @details
#' New datasets are initialized with reasonable defaults. To build a new
#' bibentry with sensible defaults and field names, use [datacite()] (DataCite)
#' or [dublincore()] (Dublin Core), then assign it with
#' `set_bibentry(dataset) <- value`.
#'
#' See the vignette for more background:
#' `vignette("bibentry", package = "dataset")`.
#'
#' @param dataset A dataset created with [dataset_df()].
#' @param value A [`utils::bibentry`] to store on the dataset. If `NULL`, a
#'   minimal default entry is created.
#'
#' @return
#' * `get_bibentry(dataset)` returns the [`utils::bibentry`] stored in
#'   `dataset`'s attributes.
#' * `set_bibentry(dataset) <- value` sets the attribute and returns the
#'   modified dataset invisibly.
#'
#' @examples
#' # Get the bibentry of a dataset_df object:
#' be <- get_bibentry(orange_df)
#'
#' # Create a well-formed bibentry (DataCite-style):
#' be2 <- datacite(
#'   Creator = person("Jane", "Doe"),
#'   Title = "The Orange Trees Dataset",
#'   Publisher = "MyOrg"
#' )
#'
#' # Assign the new bibentry:
#' set_bibentry(orange_df) <- be2
#'
#' # Inspect in different notations:
#' as_datacite(orange_df, type = "list")
#' as_dublincore(orange_df, type = "list")
#'
#' @family bibliographic helper functions
#' @importFrom utils bibentry person
#' @export
get_bibentry <- function(dataset) {
  assertthat::assert_that("dataset_bibentry" %in% names(attributes(dataset)),
    msg = "Error: get_bibentry(dataset): dataset has no dataset_bibentry attribute"
  )
  attr(dataset, "dataset_bibentry")
}

#' @rdname get_bibentry
#' @export
`set_bibentry<-` <- function(dataset, value) {
  sys_time <- Sys.time()
  year <- substr(as.character(sys_time), 1, 4)

  if (is.null(value)) {
    value <- dublincore(
      title = "Untitled Dataset",
      creator = person("Unknown Author"),
      dataset_date = year
    )
  }

  if (is.null(value$year)) value$year <- year

  attr(dataset, "dataset_bibentry") <- value
  invisible(dataset)
}

#' @keywords internal
set_default_bibentry <- function() {
  sys_time <- Sys.time()
  year <- substr(as.character(sys_time), 1, 4)
  Title <- "Untitled Dataset"
  Creator <- person("Unknown", "Author")
  datacite(Title = Title, Creator = Creator, PublicationYear = year)
}

#' @rdname dublincore
#'
#' @param type Output format. One of `"bibentry"`, `"dataset_df"`, `"list"`, or
#'   `"ntriples"`. See Details.
#' @param ... Optional fields to attach to the `dublincore` object.
#'   For example, `author = person("Jane", "Doe")` adds an author to the
#'   citation if `type = "dataset_df"`.
#'
#' @export

as_dublincore <- function(x, type = "bibentry", ...) {
  citation_author <- person(NULL, NULL)

  is_person <- function(p) ifelse(inherits(p, "person"), TRUE, FALSE)

  arguments <- list(...)

  if (!is.null(arguments$author)) {
    if (is_person(arguments$author)) {
      citation_author <- arguments$author
    } else {
      stop("as_dublincore(x, ..., author = ): author must be created with utils::person().")
    }
  }

  if (!type %in% c("bibentry", "list", "dataset_df", "ntriples")) {
    warning(paste0(
      "as_dublincore(ds, type=...) type cannot be ",
      type, ". Reverting to 'bibentry'."
    ))
    type <- "bibentry"
  }

  dataset_bibentry <- get_bibentry(x)
  dataset_title <- dataset_bibentry$title
  dataset_creator <- dataset_bibentry$author

  if (!is_person(dataset_creator)) {
    stop('attr(x, "dataset_bibentry")$author is not a person object.')
  }

  if (!is.null(dataset_bibentry$year)) {
    if (is.null(dataset_bibentry$dataset_date)) {
      dataset_date <- as.character(dataset_bibentry$year)
    } else {
      dataset_date <- as.character(dataset_bibentry$date)
    }
  } else if (!is.null(dataset_bibentry$date)) {
    dataset_date <- dataset_bibentry$date
  } else {
    dataset_date <- ":tba"
  }

  dataset_relation <- ifelse(
    is.null(dataset_bibentry$relation), ":unas", as.character(dataset_bibentry$relation)
  )
  dataset_identifier <- ifelse(
    is.null(dataset_bibentry$identifier), ":tba", as.character(dataset_bibentry$identifier)
  )
  dataset_version <- ifelse(
    is.null(dataset_bibentry$version), ":unas", as.character(dataset_bibentry$version)
  )
  dataset_description <- ifelse(
    is.null(dataset_bibentry$description), ":unas", as.character(dataset_bibentry$description)
  )
  dataset_language <- ifelse(
    is.null(dataset_bibentry$language), ":unas", as.character(dataset_bibentry$language)
  )
  dataset_format <- ifelse(
    is.null(dataset_bibentry$format), "application/r-rds", as.character(dataset_bibentry$format)
  )
  dataset_rights <- ifelse(
    is.null(dataset_bibentry$rights), ":tba", as.character(dataset_bibentry$rights)
  )
  dataset_coverage <- ifelse(
    is.null(dataset_bibentry$coverage), ":unas", as.character(dataset_bibentry$coverage)
  )
  datasource <- ifelse(
    is.null(dataset_bibentry$datasource), ":unas", as.character(dataset_bibentry$datasource)
  )
  dataset_contributor <- ifelse(
    is.null(dataset_bibentry$contributor), "", as.character(dataset_bibentry$contributor)
  )
  dataset_subject <- ifelse(
    is.null(dataset_bibentry$subject), "", as.character(dataset_bibentry$subject)
  )
  dataset_publisher <- ifelse(
    is.null(dataset_bibentry$publisher), "", as.character(dataset_bibentry$publisher)
  )

  properties <- c(
    length(dataset_title),
    length(as.character(dataset_creator)),
    length(dataset_identifier),
    length(dataset_publisher),
    length(dataset_subject),
    length("DCMITYPE:Dataset"),
    length(dataset_contributor),
    length(dataset_date),
    length(dataset_language),
    length(dataset_relation),
    length(dataset_format),
    length(dataset_rights),
    length(datasource),
    length(dataset_description),
    length(dataset_coverage)
  )

  if (type == "bibentry") {
    new_dublincore(
      title = dataset_title,
      creator = dataset_creator,
      identifier = dataset_identifier,
      publisher = dataset_publisher,
      subject = dataset_subject,
      type = "DCMITYPE:Dataset",
      contributor = dataset_contributor,
      dataset_date = dataset_date,
      language = dataset_language,
      relation = dataset_relation,
      dataset_format = dataset_format,
      rights = dataset_rights,
      datasource = datasource,
      description = dataset_description,
      coverage = dataset_coverage
    )
  } else if (type == "list") {
    if (dataset_contributor == "") dataset_contributor <- NULL
    if (dataset_subject == "") dataset_subject <- NULL

    if (is.null(attr(dataset_bibentry, "contributor"))) {
      dataset_contributor <- ""
    } else {
      dataset_contributor <- attr(dataset_bibentry, "contributor")
    }

    list(
      title = dataset_title,
      creator = dataset_creator,
      identifier = dataset_identifier,
      publisher = dataset_publisher,
      subject = dataset_subject,
      type = "DCMITYPE:Dataset",
      contributor = dataset_contributor,
      date = dataset_date,
      language = dataset_language,
      relation = dataset_relation,
      dataset_format = dataset_format,
      rights = dataset_rights,
      datasource = datasource,
      description = dataset_description,
      coverage = dataset_coverage
    )
  } else if (type == "dataset_df") {
    assertthat::assert_that(
      all(properties == 1),
      msg = "In as_dublincore(), not all properties have length 1 and cannot be exported to dataset_df."
    )
    dataset_df(
      data.frame(
        title = dataset_title,
        creator = as.character(dataset_creator),
        identifier = dataset_identifier,
        publisher = dataset_publisher,
        subject = dataset_subject,
        type = "DCMITYPE:Dataset",
        contributor = dataset_contributor,
        date = dataset_date,
        language = dataset_language,
        relation = dataset_relation,
        dataset_format = dataset_format,
        rights = dataset_rights,
        datasource = datasource,
        description = dataset_description,
        coverage = dataset_coverage
      ),
      reference = list(
        title = paste0(
          "The Dublin Core Metadata of `",
          dataset_bibentry$title, "'"
        ),
        author = citation_author,
        year = substr(as.character(Sys.Date()), 1, 4)
      )
    )
  } else if (type == "ntriples") {
    dataset_id <- if (is.null(dataset_identifier) ||
      dataset_identifier == ":tba") {
      "http://example.com/dataset_tba/"
    } else {
      dataset_identifier
    }


    dataset_contributor <- attr(dataset_bibentry, "contributor")

    # Create full DC metadata list and delegate filtering/serialization
    dclist <- list(
      title       = dataset_title,
      creator     = dataset_creator,
      identifier  = dataset_identifier,
      publisher   = dataset_publisher,
      subject     = dataset_subject,
      type        = "http://purl.org/dc/dcmitype/Dataset",
      contributor = dataset_contributor,
      date        = dataset_date,
      language    = dataset_language,
      relation    = dataset_relation,
      format      = dataset_format,
      rights      = dataset_rights,
      datasource  = datasource,
      description = dataset_description,
      coverage    = dataset_coverage
    )

    return(dublincore_to_triples(
      dclist = dclist,
      dataset_id = dataset_id
    ))
  }
}

#' Get or Set the Publisher of a Dataset Object
#'
#' Adds or retrieves the optional `"publisher"` attribute for a dataset object.
#' This property aligns with `dct:publisher` (Dublin Core) and `publisher`
#' (DataCite).
#'
#' @description
#' The publisher is the entity responsible for holding, archiving, releasing,
#' or distributing the resource. It is typically included in dataset citation
#' metadata.
#'
#' For software, this might refer to a code repository (e.g., GitHub). If both
#' a hosting platform and a producing institution are involved, use the
#' publisher for the institution and [creator()] with
#' `contributorType = "hostingInstitution"` for the platform.
#'
#' @param x A dataset object created with [dataset_df()] or
#'   [as_dataset_df()].
#' @param overwrite Logical. Should existing publisher metadata be overwritten?
#'   Defaults to `FALSE`. If `FALSE` and the field exists, a warning is issued.
#' @param value A character string specifying the publisher.
#'
#' @return
#' A character string of length one containing the `"publisher"` attribute.
#' When assigning, the updated object `x` is returned invisibly.
#'
#' @examples
#' publisher(orange_df) <- "Wiley"
#' publisher(orange_df)
#'
#' @family bibliographic helper functions
#' @importFrom assertthat assert_that
#' @export

publisher <- function(x) {
  assert_that(is.dataset_df(x),
    msg = "publisher(x): x must be a dataset object created with dataset() or as_dataset()."
  )

  DataBibentry <- get_bibentry(x)
  as.character(DataBibentry$publisher)
}

#' @rdname publisher
#' @export
`publisher<-` <- function(x, overwrite = TRUE, value) {
  if (!is.dataset_df(x)) {
    stop("publisher(x): x must be a dataset object created with dataset() or as_dataset().")
  }

  DataBibentry <- invisible(get_bibentry(x))

  if (is.null(value)) {
    DataBibentry$publisher <- ":tba"
    attr(x, "dataset_bibentry") <- DataBibentry
    return(x)
  }

  if (length(value) > 1) {
    stop("publisher(x) <- value: value must be of length 1.")
  }

  is_tba <- DataBibentry$publisher == ":tba"

  if (is.null(DataBibentry$publisher)) {
    DataBibentry$publisher <- value
  } else if (is_tba) {
    DataBibentry$publisher <- value
  } else if (overwrite) {
    DataBibentry$publisher <- value
  } else {
    message(
      "The dataset has already an Publisher: ",
      DataBibentry$publisher
    )
  }

  attr(x, "dataset_bibentry") <- DataBibentry
  invisible(x)
}

#' @title Get or Set the Identifier of a Dataset or Metadata Record
#'
#' @description Retrieve or assign the `identifier` attribute of a dataset or
#' bibliographic metadata object.
#'
#' @details An *identifier* provides an unambiguous reference to a resource.
#' Recommended practice is to supply a persistent identifier string, such as a
#' DOI, ISBN, or URN, that conforms to a recognized identification system.
#'
#' Both [Dublin
#' Core](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/#identifier)
#' and [DataCite
#' 4.4](https://support.datacite.org/docs/datacite-metadata-schema-v44-mandatory-properties#1-identifier)
#' define `identifier` as a core property. If the identifier is a DOI, it will
#' also be stored in the `doi` field of the metadata record.
#'
#' Although `identifier` is not part of the minimal Dublin Core term set, it is
#' always included in `dataset` metadata for compatibility with publishing and
#' indexing systems. You may omit it if working under a strict DC profile.
#'
#' For best practice in choosing identifier schemes, see the [IANA-registered
#' URI schemes](https://www.iana.org/assignments/uri-schemes/uri-schemes.xhtml).
#'
#' @param x A [dataset_df()] object or a [`utils::bibentry`] object (including
#'   [dublincore()] or [datacite()] records).
#' @param value A character string giving the identifier. Can be named (e.g.,
#'   `c(doi = "...")`) or unnamed. Numeric values are coerced to character.
#' @param overwrite Logical. If `TRUE` (default), any existing identifier is
#'   replaced. If `FALSE`, an existing identifier is preserved unless it is
#'   `":unas"` or `":tba"`.
#'
#' @return For `identifier()`, the current identifier as a character string. For
#' `identifier<-()`, the updated object (invisible).
#'
#' @examples
#' orange_copy <- orange_df
#'
#' # Get the current identifier
#' identifier(orange_copy)
#'
#' # Set a new identifier (e.g., a DOI)
#' identifier(orange_copy) <- "https://doi.org/10.9999/example.doi"
#'
#' # Prevent accidental overwrite
#' identifier(orange_copy, overwrite = FALSE) <- "https://example.org/id"
#'
#' # Use numeric and NULL values
#' identifier(orange_copy) <- 12345
#' identifier(orange_copy) <- NULL # Sets ":unas"
#'
#' @family Reference metadata functions
#' @rdname identifier
#' @export

identifier <- function(x) {
  if (!(inherits(x, "bibentry") || inherits(x, "dataset_df"))) {
    stop("identifier(x): x must be a dataset_df or a bibentry object.")
  }

  if (is.dataset_df(x)) {
    ds_bibentry <- get_bibentry(x)
  } else {
    ds_bibentry <- x
  }

  ds_bibentry$identifier
}

#' @rdname identifier
#' @export
`identifier<-` <- function(x, overwrite = TRUE, value) {
  if (is.numeric(value)) value <- as.character(value)

  if (!(inherits(x, "bibentry") || inherits(x, "dataset_df"))) {
    stop("identifier(x): x must be a dataset_df or a bibentry object.")
  }

  if (!(is.null(value) || inherits(value, "character"))) {
    stop("identifier(x) <- value: value must be a named or not named character string of length 1.")
  }

  if (is.dataset_df(x)) {
    ds_bibentry <- get_bibentry(x)
  } else {
    ds_bibentry <- x
  }

  old_identifier <- ds_bibentry$identifier

  if (is.null(value)) {
    value <- ":unas"
  }

  is_doi <- function(i) {
    if (!is.null(names(i))) {
      if (tolower(names(i)) == "doi") {
        return(TRUE)
      }
    } else {
      ifelse(grepl("https://doi.org", i), TRUE, FALSE)
    }
  }

  if (overwrite || old_identifier %in% c(":unas", ":tba")) {
    ds_bibentry$identifier <- value
    if (is_doi(value)) {
      doi <- gsub("https://doi.org/", "", value)
      doi <- gsub("/$", "", doi)
      ds_bibentry$doi <- doi
    }
  } else {
    warning(
      "The dataset has already an identifier: ",
      old_identifier, ".\nYou can overwrite this message with identifier(x, overwrite = TRUE) <- value"
    )
  }

  if (inherits(x, "bibentry")) {
    ds_bibentry
  } else {
    if (is_doi(value)) attr(x, "doi") <- gsub("https://doi.org/", "", value)
    attr(x, "dataset_bibentry") <- ds_bibentry
    invisible(x)
  }
}

#' Get or Set a Variable Label
#'
#' Adds or retrieves a human-readable label as a metadata attribute for a
#' variable or vector. This label is useful for making variables easier to
#' understand than their programmatic names (e.g., column names).
#'
#' @details
#' This interface builds on [labelled::var_label()] and is compatible with
#' the `defined()` infrastructure for semantic metadata (labels, namespaces,
#' units, and variable identifiers).
#'
#' See [labelled::var_label()] for low-level usage. For a comprehensive
#' guide to working with variable labels and semantic metadata, see:
#' `vignette("defined", package = "dataset")`.
#'
#' @param x A vector or data frame.
#' @param value A character string or `NULL` to remove the label.
#'   When used with `var_labels()` on a data frame, this can also be a named list
#'   or a character vector of the same length as the number of columns in `x`.
#' @param ... Further arguments passed to or used by methods.
#'
#' @return
#' - `var_label(x)` returns the `"label"` attribute of `x` as a character string.
#' - `var_label(x) <- value` sets, removes, or replaces the label attribute of `x`,
#' returning the updated object invisibly.
#'
#' @examples
#' # Retrieve the label attribute
#' var_label(orange_df$circumference)
#'
#' # Set or update the label attribute
#' var_label(orange_df$circumference) <- "circumference (breast height)"
#'
#' @seealso [labelled::var_label()], [var_labels()], [defined()]
#' @family defined metadata methods and functions
#'
#' @name var_label
#' @export
#' @importFrom labelled var_label
#' @importFrom labelled `var_label<-`
NULL

#' @rdname var_label
#' @export
#' @importFrom labelled `var_label<-`
var_label.defined <- function(x, ...) {
  NextMethod()
}
#' @rdname var_label
#'
#' @description
#' `label_attribute()` is a low-level helper that retrieves the `"label"` attribute
#' of an object without any fallback or printing logic. It is primarily used internally.
#'
#' @return
#' A character string if the `"label"` attribute exists, or `NULL` if not present.
#'
#' @export
#' @importFrom labelled label_attribute
label_attribute <- function(x) {
  attr(x, "label", exact = TRUE)
}

#' @rdname var_label
#'
#' @description
#' The `var_label<-` assignment method sets or removes the `"label"` attribute
#' of a vector or data frame column. This allows attaching human-readable
#' descriptions to variables for interpretability and downstream metadata use.
#'
#' @param value A character string to assign as the label, or `NULL` to remove it.
#'
#' @return
#' The modified object `x`, returned invisibly with the updated `"label"` attribute.
#'
#' @export
`var_label<-` <- function(x, value) {
  UseMethod("var_label<-")
}

#' @rdname var_label
#' @export
#' @importFrom labelled `var_label<-`
`var_label<-.haven_labelled_defined` <- function(x, value) {
  attr(x, "label") <- value
  x
}

#' @rdname var_label
#'
#' @param unlist For data frames, return a named vector instead of a list.
#' @param null_action For data frames, controls how to handle columns without a variable label.
#'   Options are:
#'   - `"keep"` (default): keep `NULL` for unlabeled columns
#'   - `"fill"`: use the column name as a fallback
#'   - `"skip"`: exclude columns with no label from the result
#'   - `"na"`: use `NA_character_`
#'   - `"empty"`: use an empty string `""`
#' @param recurse If `TRUE`, applies `var_label()` recursively on packed columns
#'   (as created by [tidyr::pack()]) to retrieve sub-column labels. If `FALSE`,
#'   only the outer (grouped) column label is returned.
#' @examples
#' # Example: Retrieve variable labels from a dataset_df
#' df <- dataset_df(
#'   id = defined(1:3, label = "Observation ID"),
#'   temp = defined(c(22.5, 23.0, 21.8), label = "Temperature (°C)"),
#'   site = defined(c("A", "B", "A"))
#' )
#'
#' # List form (default)
#' var_label(df)
#'
#' # Character vector form
#' var_label(df, unlist = TRUE, null_action = "empty")
#'
#' # Exclude variables without labels
#' var_label(df, null_action = "skip")
#'
#' # Replace missing labels with column names
#' var_label(df, null_action = "fill")
#' @export
var_label.dataset_df <- function(x,
                                 unlist = FALSE,
                                 null_action = c("keep", "fill", "skip", "na", "empty"),
                                 recurse = FALSE,
                                 ...) {
  null_action <- match.arg(null_action)

  result <- lapply(names(x), function(name) {
    col <- x[[name]]

    if (recurse && inherits(col, "data.frame")) {
      return(var_label(col, unlist = unlist, null_action = null_action, recurse = recurse, ...))
    }

    label <- var_label(col)

    if (is.null(label)) {
      label <- switch(null_action,
        keep = NULL,
        fill = name,
        skip = NULL,
        na = NA_character_,
        empty = ""
      )
    }

    label
  })

  names(result) <- names(x)

  if (null_action == "skip") {
    result <- result[!vapply(result, is.null, logical(1))]
  }

  if (unlist) {
    return(unlist(result, use.names = TRUE))
  }

  result
}

#' @title Describe a dataset in N-Triples format
#' @description Writes provenance and Dublin Core metadata of a dataset to a
#'   file or connection in N-Triples format.
#' @param x A `dataset_df` object.
#' @param con A connection or a character string path (e.g. from `tempfile()`).
#' @return Writes N-Triples to `con` and invisibly returns `x`.
#' @examples
#' test_ds <- dataset_df(
#'   rowid = defined(c("eg:1", "eg:2"),
#'     namespace = "http://example.com/dataset#"
#'   ),
#'   geo = defined(
#'     gdp$geo[1:2],
#'     label = "Country",
#'     concept = "http://example.com/prop/geo",
#'     namespace = "https://eionet.europa.eu/geo/$1"
#'   ),
#'   dataset_bibentry = dublincore(
#'     title = "Example Dataset",
#'     creator = person("John", "Doe")
#'   )
#' )
#'
#' # returns invisibly the contents of the text file serialisation:
#' testdescription <- describe(test_ds, con = tempfile())
#' testdescription
#' @export
describe <- function(x, con) {
  assertthat::assert_that(
    is.dataset_df(x),
    msg = "describe(x, con): x must be a dataset_df object."
  )

  # Collect provenance and DC metadata
  prov_text <- provenance(x)
  dc_text <- as_dublincore(x, type = "ntriples")

  prov_text <- if (is.null(prov_text)) character(0) else as.character(prov_text)
  dc_text <- if (is.null(dc_text)) character(0) else as.character(dc_text)

  ntriples_text <- c(prov_text, dc_text)

  if (!is.character(ntriples_text)) {
    stop("describe(): expected character vector of N-Triples.")
  }

  # Handle connection or file path
  if (is.character(con)) {
    con_file <- file(con, open = "w", encoding = "UTF-8")
    on.exit(close(con_file), add = TRUE)
    writeLines(ntriples_text, con = con_file)
  } else {
    writeLines(ntriples_text, con = con)
  }

  invisible(x)
}

#' Convert a \code{dataset_df} to a base \code{data.frame}
#'
#' @description
#' Converts a [`dataset_df`] into a plain `data.frame`. By default this
#' strips semantic metadata (label, unit, concept/definition, namespace) from
#' each column, but this can be controlled via the
#' \code{strip_attributes} argument.
#'
#' Dataset-level metadata remains attached as inert attributes.
#'
#' @param x A [`dataset_df`].
#' @param strip_attributes Logical: should column-level semantic metadata
#'   be stripped? Default: \code{TRUE}.
#' @param ... Passed to \code{base::as.data.frame()}.
#' @inheritParams base::as.data.frame
#'
#' @return A base R `data.frame` without the \code{dataset_df} class.
#'
#' @examples
#' data(orange_df)
#' as.data.frame(orange_df)
#'
#' @rdname as.data.frame.dataset_df
#' @export
as.data.frame.dataset_df <- function(x, ...,
                                     strip_attributes = TRUE,
                                     optional = FALSE,
                                     stringsAsFactors = FALSE) {
  # ---- base data.frame conversion ----
  df <- base::as.data.frame(
    unclass(x),
    optional = optional,
    stringsAsFactors = stringsAsFactors
  )

  # ---- process each column ----
  df <- lapply(df, function(col) {
    if (inherits(col, "haven_labelled_defined")) {
      ## ---- Date / POSIXct cases ----
      if (inherits(col, "POSIXct")) {
        # keep underlying time values, drop wrapper class
        base <- structure(
          unclass(col),
          class = c("POSIXct", "POSIXt"),
          tzone = attr(col, "tzone", exact = TRUE)
        )

        if (!strip_attributes) {
          # preserve semantics if requested
          attr(base, "label") <- attr(col, "label", exact = TRUE)
          attr(base, "unit") <- attr(col, "unit", exact = TRUE)
          # support both old "definition" and new "concept" naming
          def_or_concept <- attr(col, "concept", exact = TRUE)
          if (is.null(def_or_concept)) {
            def_or_concept <- attr(col, "definition", exact = TRUE)
          }
          attr(base, "concept") <- def_or_concept
          attr(base, "namespace") <- attr(col, "namespace", exact = TRUE)
        }
        col <- base
      } else if (inherits(col, "Date")) {
        base <- structure(
          unclass(col),
          class = "Date"
        )

        if (!strip_attributes) {
          attr(base, "label") <- attr(col, "label", exact = TRUE)
          attr(base, "unit") <- attr(col, "unit", exact = TRUE)
          def_or_concept <- attr(col, "concept", exact = TRUE)
          if (is.null(def_or_concept)) {
            def_or_concept <- attr(col, "definition", exact = TRUE)
          }
          attr(base, "concept") <- def_or_concept
          attr(base, "namespace") <- attr(col, "namespace", exact = TRUE)
        }
        col <- base
      } else {
        ## ---- non-date/time defined vectors ----
        lbls <- attr(col, "labels", exact = TRUE)
        underlying <- vctrs::vec_data(col)

        if (!is.null(lbls)) {
          # labelled / categorical: go to character
          fac <- as_factor(col) # returns factor with correct levels
          col <- as.character(fac) # convert factor to characters
        } else if (is.numeric(underlying)) {
          # numeric: use semantic-aware numeric coercion
          col <- as_numeric(
            col,
            strip_attributes = strip_attributes
          )
        } else {
          # fallback: character with semantic awareness
          col <- as_character(
            col,
            strip_attributes = strip_attributes
          )
        }
      }

      # Ensure the "haven_labelled_defined" wrapper class is gone
      if (inherits(col, "haven_labelled_defined")) {
        class(col) <- setdiff(class(col), "haven_labelled_defined")
      }
    }

    # ---- final attribute stripping if requested ----
    if (strip_attributes) {
      # Handle both legacy "definition" and current "concept"
      attributes(col)[c("label", "unit", "concept", "definition", "namespace")] <- NULL
    }

    col
  })

  df <- as.data.frame(df,
    stringsAsFactors = stringsAsFactors,
    optional = optional
  )

  # ---- preserve dataset-level metadata (inert) ----
  for (att in c("dataset_bibentry", "subject", "prov")) {
    if (!is.null(attr(x, att))) {
      attr(df, att) <- attr(x, att)
    }
  }

  df
}

#' Coerce a `dataset_df` to a tibble
#'
#' @description
#' [as_tibble()] method for `dataset_df` objects.
#'
#' Converts a `dataset_df` into a tibble. By default, additional metadata
#' attached to the object is removed unless `strip_attributes = FALSE` is
#' specified.
#'
#' @rdname as_tibble.dataset_df
#' @name as_tibble.dataset_df
#'
#' @return
#' A [`tibble`][tibble::tibble()] containing the data (and optionally some
#' attributes) of the `dataset_df`.
#'
#' @param x A `dataset_df` object.
#' @inheritParams as.data.frame.dataset_df
#' @param ... Additional arguments passed to [as.data.frame.dataset_df()].
#' @param .name_repair Treatment of problematic column names:
#'   * `"minimal"`: No name checks.
#'   * `"unique"`: Enforce uniqueness.
#'   * `"check_unique"`: (default) require unique names.
#'   * `"universal"`: Make unique and syntactic.
#'   * A function or purrr-style anonymous function, see [rlang::as_function()].
#'
#' @seealso
#' * [tibble::as_tibble()]
#' * [as.data.frame.dataset_df()]
#'
#' @family tibble.methods
#'
#' @section Metadata handling:
#' The `strip_attributes` argument controls which attributes of the `dataset_df`
#' object are preserved. When `strip_attributes = TRUE` (the default), only
#' column-level information is kept.
#'
#' @section About column names:
#' The `dataset_df` class may internally use reserved names for indexing,
#' identifiers, or metadata. These are never exposed in the resulting tibble.
#' Column names in the output tibble may be repaired according to
#' `.name_repair`.
#'
#' @examples
#' # Convert a dataset_df to a tibble
#' x <- dataset_df(orange_df)
#' as_tibble(x)
#'
#' # Keep attributes
#' as_tibble(x, strip_attributes = FALSE)
#'
#' @export
as_tibble <- function(x, ...) {
  UseMethod("as_tibble")
}

#' @rdname as_tibble.dataset_df
#' @export
as_tibble.dataset_df <- function(
  x,
  ...,
  strip_attributes = TRUE,
  .name_repair = "check_unique"
) {
  df <- as.data.frame.dataset_df(
    x,
    strip_attributes = strip_attributes,
    ...
  )

  tibble::as_tibble(df, .name_repair = .name_repair)
}

#' @rdname as_tibble.dataset_df
#' @export
as.tibble.dataset_df <- as_tibble.dataset_df

test_that("as.data.frame.dataset_df() returns
          a plain data.frame", {
  ds <- dataset_df(
    a = defined(1:3,
      label = "A label",
      unit = "cm",
      concept = "http://example.com/a"
    ),
    b = defined(c("x", "y", "z"),
      namespace = "http://example.com/ns/$1"
    )
  )

  df <- as.data.frame(ds)

  # Class should be plain data.frame
  expect_true(is.data.frame(df))
  expect_false(inherits(df, "dataset_df"))
  expect_false(any(vapply(
    df, inherits, logical(1),
    "haven_labelled_defined"
  )))

  # Structure and values preserved
  expect_equal(nrow(df), 3)
  expect_equal(ncol(df), 3)
  expect_equal(df$a, 1:3)
  expect_equal(df$b, c("x", "y", "z"))

  # Column attributes must be removed
  expect_null(attr(df$a, "label"))
  expect_null(attr(df$a, "unit"))
  expect_null(attr(df$a, "concept"))
  expect_null(attr(df$a, "namespace"))
})

test_that("as.data.frame.dataset_df() preserves
          high-level metadata (inert)", {
  ds <- dataset_df(a = 1:2)
  df <- as.data.frame(ds)

  # Metadata preserved as non-actionable attributes
  expect_true(!is.null(attr(df, "dataset_bibentry")))
  expect_true(!is.null(attr(df, "prov")))
  expect_true(!is.null(attr(df, "subject")))

  # But the object must not behave as dataset_df
  expect_false(is.dataset_df(df))
  expect_error(creator(df), "dataset")
})

test_that("as.data.frame.dataset_df()
          handles empty and edge cases", {
  ds_empty <- dataset_df()
  df_empty <- as.data.frame(ds_empty)

  expect_true(is.data.frame(df_empty))
  expect_equal(ncol(df_empty), 1) # rowid column only
  expect_equal(nrow(df_empty), 0)

  # With only rowid defined explicitly
  ds_rowid <- dataset_df(rowid = defined(c("eg:1", "eg:2")))
  df_rowid <- as.data.frame(ds_rowid)

  expect_equal(df_rowid$rowid, c("eg:1", "eg:2"))
})

test_that("as.data.frame.dataset_df()
          strips defined semantics correctly", {
  ds <- dataset_df(
    numeric_def = defined(1:3, unit = "kg"),
    char_def = defined(c("A", "B", "C"), label = "X"),
    factor_def = defined(as.factor(c("Low", "High", "Low")),
      namespace = "ns:"
    )
  )

  df <- as.data.frame(ds)

  expect_equal(df$numeric_def, 1:3)
  expect_equal(df$char_def, c("A", "B", "C"))
  expect_equal(df$factor_def, c("Low", "High", "Low"))

  # Important: no semantic attributes must remain
  for (col in df) {
    expect_null(attr(col, "unit"))
    expect_null(attr(col, "label"))
    expect_null(attr(col, "concept"))
    expect_null(attr(col, "namespace"))
  }
})

test_that("as_tibble.dataset_df() behaves
          like as.data.frame but returns tibble", {
  ds <- dataset_df(
    x = defined(1:3, label = "x"),
    y = defined(c("A", "B", "C"))
  )

  tbl <- as_tibble(ds)

  expect_true(tibble::is_tibble(tbl))
  expect_false(inherits(tbl, "dataset_df"))
  expect_equal(tbl$x, 1:3)
  expect_equal(tbl$y, c("A", "B", "C"))

  # No defined-class left
  expect_false(inherits(tbl$x, "haven_labelled_defined"))
})

test_that("as.data.frame.dataset_df() does not alter
          values of defined(Date/POSIXct)", {
  d <- Sys.Date() + 0:2
  p <- as.POSIXct("2024-01-01 12:00:00",
    tz = "UTC"
  ) + (0:2) * 3600

  ds <- dataset_df(
    d = defined(d),
    p = defined(p)
  )

  df <- as.data.frame(ds)

  expect_equal(df$d, d)
  expect_equal(df$p, p)
})

test_that("as.data.frame.dataset_df() drops only
          dataset_df-specific classes", {
  ds <- dataset_df(
    x = I(Orange[1:3, ]) # I() keeps inner data.frame
  )

  df <- as.data.frame(ds)

  expect_true(is.data.frame(df))
  expect_false(inherits(df, "dataset_df"))
  expect_true(is.data.frame(df$x))
})


test_that("as.data.frame.dataset_df(strip_attributes = TRUE)
          strips all semantic attributes", {
  d <- Sys.Date() + 0:2
  p <- as.POSIXct("2024-01-01 12:00:00", tz = "UTC") + (0:2) * 3600

  ds <- dataset_df(
    num  = defined(1:3, label = "Num", unit = "kg", concept = "x"),
    chr  = defined(c("A", "B", "C"), label = "Letters", namespace = "http://ns"),
    date = defined(d, label = "DateLabel", unit = "day"),
    time = defined(p, label = "TimeLabel", unit = "hour")
  )

  df <- as.data.frame(ds, strip_attributes = TRUE)

  # Values preserved
  expect_equal(df$num, 1:3)
  expect_equal(df$chr, c("A", "B", "C"))
  expect_equal(df$date, d)
  expect_equal(df$time, p)

  # All defined-class removed
  expect_false(any(vapply(df, inherits, logical(1), "haven_labelled_defined")))

  # All metadata attributes removed
  for (col in df) {
    expect_null(attr(col, "label"))
    expect_null(attr(col, "unit"))
    expect_null(attr(col, "concept"))
    expect_null(attr(col, "namespace"))
  }
})

test_that("as.data.frame.dataset_df(strip_attributes = FALSE) keeps semantic attributes", {
  d <- Sys.Date() + 0:2
  p <- as.POSIXct("2024-01-01 12:00:00", tz = "UTC") + (0:2) * 3600

  ds <- dataset_df(
    num = defined(1:3, label = "Num", unit = "kg", concept = "x"),
    chr = defined(c("A", "B", "C"),
      label = "Letters",
      namespace = "http://ns"
    ),
    date = defined(d, label = "DateLabel", unit = "day"),
    time = defined(p, label = "TimeLabel", unit = "hour")
  )

  df <- as.data.frame(ds, strip_attributes = FALSE)

  ## ---- Value equality ignoring attributes ----
  # Use base coercers to drop attributes and compare pure values
  expect_equal(as.numeric(df$num), 1:3)
  expect_equal(as.character(df$chr), c("A", "B", "C"))
  expect_equal(attr(df$date, "unit"), "day")
  expect_equal(attr(df$time, "unit"), "hour")

  df_s <- as.data.frame(ds, strip_attributes = TRUE)
  expect_equal(as.Date(df_s$date), d)
  expect_equal(as.POSIXct(df_s$time), p)


  ## ---- Metadata preserved ----
  expect_equal(attr(df$num, "label"), "Num")
  expect_equal(attr(df$num, "unit"), "kg")
  expect_equal(attr(df$num, "concept"), "x")

  expect_equal(attr(df$chr, "label"), "Letters")
  expect_equal(attr(df$chr, "namespace"), "http://ns")

  expect_equal(attr(df$date, "label"), "DateLabel")
  expect_equal(attr(df$date, "unit"), "day")

  expect_equal(attr(df$time, "label"), "TimeLabel")
  expect_equal(attr(df$time, "unit"), "hour")

  ## ---- Wrapper class removed ----
  expect_false(any(vapply(
    df,
    inherits, logical(1),
    "haven_labelled_defined"
  )))
})

test_that("as.data.frame.dataset_df(strip_attributes = TRUE)
          does not keep semantic attributes", {
  d <- Sys.Date() + 0:2
  p <- as.POSIXct("2024-01-01 12:00:00", tz = "UTC") + (0:2) * 3600

  ds <- dataset_df(
    num  = defined(1:3, label = "Num", unit = "kg", concept = "x"),
    chr  = defined(c("A", "B", "C"), label = "Letters", namespace = "http://ns"),
    date = defined(d, label = "DateLabel", unit = "day"),
    time = defined(p, label = "TimeLabel", unit = "hour")
  )

  df_s <- as.data.frame(ds, strip_attributes = TRUE)
  expect_equal(as.Date(df_s$date), d)
  expect_equal(as.POSIXct(df_s$time), p)
  expect_null(attr(df_s, "unit"))
})

#' Coerce a defined vector to logical
#'
#' @description
#' Coerces a [`haven_labelled_defined`] vector created with [`defined()`]
#' into a base R logical vector.
#'
#' This function is the recommended, semantic-aware interface for converting
#' defined logical vectors. It preserves the underlying logical values and,
#' unless otherwise requested, retains the semantic metadata attached to the
#' variable.
#'
#' @details
#' Use \code{strip_attributes = TRUE} when flattening or preparing data for
#' external pipelines, but keep the default when working with defined
#' vectors directly.
#'
#' Users may also call base R's [`as.logical()`], which will dispatch to this
#' method automatically via S3. However, `as_logical()` is preferred because
#' it makes the semantics explicit and supports the `strip_attributes`
#' argument.
#'
#' @param x A vector created with [`defined()`].
#' @param strip_attributes Logical; should semantic metadata attributes
#'   (such as \code{label}, \code{unit}, \code{definition}, and
#'   \code{namespace}) be removed from the returned vector?
#'   Defaults to \code{FALSE}.
#'
#'
#' @param ... Additional arguments (currently unused).
#'
#' @return A logical vector. If \code{strip_attributes = FALSE}, any semantic
#'   metadata attached to \code{x} will be carried over to the returned vector.
#'
#' @examples
#' # Basic usage
#' flg <- defined(c(TRUE, FALSE, TRUE), label = "Flag")
#' as_logical(flg)
#'
#' # Metadata preserved by default
#' attr(as_logical(flg), "label")
#'
#' # Stripping metadata
#' as_logical(flg, strip_attributes = TRUE)
#'
#' # Base R coercion also works (via S3 method dispatch)
#' as.logical(flg)
#'
#' @seealso
#'   [`as_numeric()`], [`as_character()`],
#'   [`as.Date()`], [`as.POSIXct()`],
#'   [`defined()`]
#'
#' @export
as_logical <- function(x, ...) {
  UseMethod("as_logical", x)
}

#' @rdname as_logical
#' @export
as_logical.haven_labelled_defined <- function(x,
                                              strip_attributes = TRUE,
                                              ...) {
  out <- as.logical(unclass(x))

  if (!strip_attributes) {
    attr(out, "label") <- attr(x, "label", exact = TRUE)
    attr(out, "unit") <- attr(x, "unit", exact = TRUE)
    attr(out, "definition") <- attr(x, "definition", exact = TRUE)
    attr(out, "namespace") <- attr(x, "namespace", exact = TRUE)
  }

  out
}

#' Coerce a defined POSIXct vector to a base R POSIXct
#'
#' @description
#' Coerces a [`haven_labelled_defined`] vector whose underlying type is
#' [`POSIXct`] into a base R `POSIXct` time vector.
#'
#' This method preserves both the timestamp values and the original time zone.
#' By default, semantic metadata is also retained.
#'
#' @details
#' Use \code{strip_attributes = TRUE} when flattening or preparing data for
#' external pipelines, but keep the default when working with defined
#' vectors directly.\cr
#' Base R's [`as.POSIXct()`] also works, as it dispatches to this method via
#' S3. Using this method directly is preferred when metadata preservation
#' matters.
#'
#' @param x A vector created with [`defined()`] with underlying type
#'   \code{POSIXct}.
#' @param tz a character string. The time zone specification to be used for
#' the conversion, if one is required. System-specific timezones
#' (see [`base::timezones()`],
#' but "" is the current time zone, and "GMT" is UTC (Universal Time,
#' Coordinated). Invalid values are most commonly treated as UTC, on
#' some platforms with a warning.
#' @param strip_attributes Logical; should semantic metadata attributes
#'   (label, unit, definition, namespace) be removed? Defaults to
#'   \code{FALSE}.
#'
#' @param ... Additional arguments passed to [base::as.POSIXct()].
#'
#' @return A `POSIXct` vector with timestamp values preserved.
#'
#' @examples
#' p <- defined(
#'   as.POSIXct("2024-01-01 12:00:00", tz = "UTC"),
#'   label = "Timestamp"
#' )
#'
#' # Recommended usage
#' as.POSIXct(p)
#'
#' # Explicit attribute stripping
#' as.POSIXct(p, strip_attributes = TRUE)
#'
#' @seealso
#'   [`as.Date()`],
#'   [`as_numeric()`], [`as_character()`], [`as_logical()`],
#'   [`defined()`]
#'
#' @export
as.POSIXct.haven_labelled_defined <- function(
  x,
  tz = "",
  strip_attributes = TRUE,
  ...
) {
  if (!inherits(x, "POSIXct")) {
    stop(
      "as.POSIXct.haven_labelled_defined() requires underlying POSIXct.",
      call. = FALSE
    )
  }

  # Save semantic metadata before we manipulate the object
  meta <- list(
    label      = attr(x, "label", exact = TRUE),
    unit       = attr(x, "unit", exact = TRUE),
    definition = attr(x, "definition", exact = TRUE),
    namespace  = attr(x, "namespace", exact = TRUE)
  )

  # Drop only the wrapper class so base::as.POSIXct() can handle tz
  class(x) <- setdiff(class(x), "haven_labelled_defined")

  # If the user explicitly requests a time zone, delegate to base R
  if (!identical(tz, "")) {
    x <- base::as.POSIXct(x, tz = tz, ...)
  }

  if (strip_attributes) {
    attr(x, "label") <- NULL
    attr(x, "unit") <- NULL
    attr(x, "definition") <- NULL
    attr(x, "namespace") <- NULL
  } else {
    # Restore metadata in case base::as.POSIXct() dropped anything
    for (nm in names(meta)) {
      if (!is.null(meta[[nm]])) {
        attr(x, nm) <- meta[[nm]]
      }
    }
  }

  x
}

#' @keywords internal
dublincore_to_triples <- function(
  dclist,
  dataset_id = "http://example.com/dataset"
) {
  if (is.null(dclist) || is.null(dclist$title) || nchar(dclist$title) == 0) {
    stop("Error: dublincore_to_triples(dclist, dataset_id): no title found in dclist")
  }

  # Normalize type field URI if needed
  if (!is.null(dclist$type)) {
    dclist$type <- gsub("DCMITYPE:", "http://purl.org/dc/terms/DCMIType", dclist$type)
  }

  # Accumulate all triples (many Dublin Core fields may have multiple values):
  dctriples <- c(
    expand_triples(dataset_id, "http://purl.org/dc/terms/title", dclist$title),
    expand_triples(dataset_id, "http://purl.org/dc/terms/description", dclist$description),
    expand_triples(dataset_id, "http://purl.org/dc/terms/creator", dclist$creator),
    expand_triples(dataset_id, "http://purl.org/dc/terms/contributor", dclist$contributor),
    expand_triples(dataset_id, "http://purl.org/dc/terms/publisher", dclist$publisher),
    expand_triples(dataset_id, "http://purl.org/dc/terms/identifier", dclist$identifier),
    expand_triples(dataset_id, "http://purl.org/dc/terms/subject", dclist$subject),
    expand_triples(dataset_id, "http://purl.org/dc/terms/type", dclist$type),
    expand_triples(dataset_id, "http://purl.org/dc/terms/date", dclist$date),
    expand_triples(dataset_id, "http://purl.org/dc/terms/language", dclist$language),
    expand_triples(dataset_id, "http://purl.org/dc/terms/relation", dclist$relation),
    expand_triples(dataset_id, "http://purl.org/dc/terms/format", dclist$format),
    expand_triples(dataset_id, "http://purl.org/dc/terms/rights", dclist$rights),
    expand_triples(dataset_id, "http://purl.org/dc/terms/source", dclist$datasource),
    expand_triples(dataset_id, "http://purl.org/dc/terms/coverage", dclist$coverage)
  )

  n_triples(dctriples)
}

#' Create a Bibentry Object with DataCite Metadata Fields
#'
#' Constructs a bibliographic metadata record conforming to the
#' [DataCite Metadata Schema](https://schema.datacite.org/). The resulting
#' object is stored as a modified [utils::bibentry()] enriched with structured
#' Dublin Core and DataCite-compliant metadata.
#'
#' @details
#' DataCite is a leading non-profit organization that provides persistent
#' identifiers (DOIs) for research data and other research outputs. Members of
#' the research community use DataCite to register datasets with globally
#' resolvable metadata for citation and discovery.
#'
#' This function sets `"Dataset"` as the default resource type. The `Size`
#' attribute (e.g., bytes, pages, etc.) is automatically added if available.
#'
#' @param Title The name(s) by which the resource is known. Similar to
#'   [dct:title](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/title/).
#' @param Creator One or more [utils::person()] objects describing the main
#'   authors or contributors responsible for creating the resource.
#' @param Identifier A persistent identifier (e.g., DOI or URI). May refer to a
#'   specific version or all versions of the resource.
#' @param Publisher The name of the organization that holds, publishes, or
#'   distributes the resource. Required by DataCite. See [publisher()].
#' @param PublicationYear The year of public availability (in `YYYY` format).
#'   See [publication_year()].
#' @param Subject A topic, keyword, or classification term. See [subject()] and
#'   [subject_create()] for structured vocabularies.
#' @param Contributor An individual or institution that contributed to the
#'   development, distribution, or curation of the resource.
#' @param Type The resource type. Defaults to `"Dataset"` for general use. See
#'   [dcm:type](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/type/).
#' @param Language Language code as per IETF BCP 47 / ISO 639-1. See [language()].
#' @param AlternateIdentifier Optional local or secondary identifier. Defaults
#'   to `":unas"`.
#' @param RelatedIdentifier Related resources (e.g., prior versions, papers).
#'   Defaults to `":unas"`.
#' @param Format A technical format (e.g., `"application/pdf"`, `"text/csv"`).
#' @param Version A free-text version string (e.g., `"1.0.0"`). Defaults to
#'   `"0.1.0"`. See [version()].
#' @param Rights Licensing or usage restrictions for the resource. Defaults to
#'   `":tba"`. See [rights()].
#' @param Date A date in `"YYYY"`, `"YYYY-MM-DD"` or ISO datetime format.
#'   Can also be a [Date] or [POSIXct] object.
#' @param DateList A list of multiple dates. Currently not supported.
#' @param Description Free-text summary or additional information. Defaults to
#'   `":tba"`.
#' @param Geolocation Geographic location covered or referenced by the resource.
#'   See [geolocation()].
#' @param FundingReference Information about funding or financial support.
#'   Defaults to `":unas"`. Structured funding metadata not yet implemented.
#'
#' @return
#' A [utils::bibentry()] object with DataCite-compliant fields. Use
#' [as_datacite()] to extract the metadata as a list or bibentry object.
#'
#' @source
#' - [DataCite 4.3 Mandatory Properties](https://support.datacite.org/docs/schema-mandatory-properties-v43)
#' - [DataCite 4.3 Optional Properties](https://support.datacite.org/docs/schema-optional-properties-v43)
#'
#' @family bibrecord functions
#' @seealso
#' Learn more in the vignette:
#' [`bibrecord`](https://dataset.dataobservatory.eu/articles/bibrecord.html)
#' @importFrom utils person bibentry
#'
#' @examples
#' datacite(
#'   Title = "Growth of Orange Trees",
#'   Creator = c(
#'     person(
#'       given = "N.R.",
#'       family = "Draper",
#'       role = "cre",
#'       comment = c(VIAF = "http://viaf.org/viaf/84585260")
#'     ),
#'     person(
#'       given = "H",
#'       family = "Smith",
#'       role = "cre"
#'     )
#'   ),
#'   Publisher = "Wiley",
#'   Date = 1998,
#'   Language = "en"
#' )
#'
#' # Extract bibliographic metadata
#' as_datacite(orange_df)
#'
#' # As a list
#' as_datacite(orange_df, "list")
#'
#' @export

datacite <- function(Title,
                     Creator,
                     Identifier = NULL,
                     Publisher = NULL,
                     PublicationYear = NULL,
                     Subject = subject_create(
                       term = "data sets",
                       subjectScheme = "Library of Congress Subject Headings (LCSH)",
                       schemeURI = "https://id.loc.gov/authorities/subjects.html",
                       valueURI = "http://id.loc.gov/authorities/subjects/sh2018002256"
                     ),
                     Type = "Dataset",
                     Contributor = NULL,
                     Date = ":tba",
                     DateList = NULL,
                     Language = NULL,
                     AlternateIdentifier = ":unas",
                     RelatedIdentifier = ":unas",
                     Format = ":tba",
                     Version = "0.1.0",
                     Rights = ":tba",
                     Description = ":tba",
                     Geolocation = ":unas",
                     FundingReference = ":unas") {
  if (is.null(PublicationYear)) {
    if (!is.null(Date) && grepl("^\\d{4}", as.character(Date))) {
      PublicationYear <- substr(as.character(Date), 1, 4)
    } else {
      PublicationYear <- ":tba"
    }
  }

  Date <- ifelse(is.null(DateList), ":tba", as.character(Date))
  DateList <- ifelse(is.null(DateList), ":tba", as.character(DateList))
  Format <- ifelse(is.null(Format), ":tba", as.character(Format))
  AlternateIdentifier <- ifelse(is.null(AlternateIdentifier),
    ":unas", AlternateIdentifier
  )
  if (is.null(RelatedIdentifier)) {
    RelatedIdentifie <- ":unas"
  }
  Rights <- ifelse(is.null(Rights), ":tba", as.character(Rights))
  Geolocation <- ifelse(is.null(Geolocation), ":unas",
    as.character(Geolocation)
  )
  FundingReference <- ifelse(is.null(FundingReference),
    ":unas", as.character(FundingReference)
  )

  new_datacite(
    Title = Title,
    Creator = Creator,
    Identifier = Identifier,
    Publisher = Publisher,
    PublicationYear = PublicationYear,
    Subject = Subject,
    Type = "Dataset",
    Contributor = Contributor,
    Date = Date,
    DateList = DateList,
    Language = Language,
    AlternateIdentifier = AlternateIdentifier,
    RelatedIdentifier = RelatedIdentifier,
    Format = Format,
    Version = Version,
    Rights = Rights,
    Description = Description,
    Geolocation = Geolocation,
    FundingReference = FundingReference
  )
}

#' @keywords internal
new_datacite <- function(Title,
                         Creator,
                         Identifier,
                         Publisher,
                         PublicationYear,
                         Subject,
                         Type = "Dataset",
                         Contributor,
                         Date,
                         DateList,
                         Language,
                         AlternateIdentifier,
                         RelatedIdentifier,
                         Format,
                         Version,
                         Rights,
                         Description,
                         Geolocation,
                         FundingReference) {
  # bibentry: subject slot must be character
  datacite_object <- bibentry(
    bibtype = "Misc",
    title = Title,
    author = Creator,
    year = as.character(PublicationYear),
    identifier = Identifier,
    publisher = Publisher,
    date = Date,
    language = Language,
    subject = if (is.subject(Subject)) Subject$term else as.character(Subject),
    alternateidentifier = AlternateIdentifier,
    relatedidentifier = if (is.related(RelatedIdentifier)) {
      RelatedIdentifier$relatedIdentifier
    } else if (is.null(RelatedIdentifier)) {
      ":unas"
    } else {
      as.character(RelatedIdentifier)
    },
    format = Format,
    version = Version,
    rights = Rights,
    description = Description,
    geolocation = Geolocation,
    fundingreference = FundingReference
  )

  # keep the structured subject object as an attribute
  if (!is.null(Subject) && is.subject(Subject)) {
    attr(datacite_object, "subject") <- Subject
  }

  # keep the structured related item object as an attribute
  if (!is.null(RelatedIdentifier) && is.related(RelatedIdentifier)) {
    attr(datacite_object, "relatedIdentifier") <- RelatedIdentifier
  }

  # add structured contributor list
  if (!is.null(Contributor)) {
    attr(datacite_object, "contributor") <- Contributor
  }

  class(datacite_object) <- c("datacite", class(datacite_object))
  datacite_object
}

#' @rdname datacite
#' @return \code{is.datacite(x)} returns a logical values (if the object
#' \code{x} is of class \code{datacite}).
is.datacite <- function(x) {
  UseMethod("is.datacite", x)
}

#' @rdname datacite
#' @param x An object that is tested if it has a class "datacite".
#' @exportS3Method
is.datacite.datacite <- function(x) inherits(x, "datacite")

#' @rdname datacite
#' @exportS3Method
print.datacite <- function(x, ...) {
  cat("DataCite Metadata Record\n")
  cat("--------------------------\n")

  pr <- function(label, value) {
    if (!is.null(value) && length(value) > 0 && any(nzchar(value))) {
      # %-12s = left-justify label into 12 chars
      cat(sprintf("%-12s %s\n", paste0(label, ":"), paste(value, collapse = "; ")))
    }
  }

  pr("Title", x$title)
  pr("Creator(s)", paste(format(x$author), collapse = "; "))

  contributor <- attr(x, "contributor")
  if (!is.null(contributor)) {
    pr("Contributor(s)", fix_contributor(contributor))
  }

  subj <- attr(x, "subject")
  if (!is.null(subj) && is.subject(subj)) {
    subj_val <- subj$term
    if (!is.null(subj$schemeURI) && nzchar(subj$schemeURI)) {
      subj_val <- paste0(subj_val, " [", subj$schemeURI, "]")
    }
    pr("Subject(s)", subj_val)
  } else if (!is.null(x$subject)) {
    pr("Subject(s)", x$subject)
  }

  pr("Identifier", x$identifier)
  pr("Publisher", x$publisher)
  pr("Year", x$year)
  pr("Language", x$language)
  pr("Description", x$description)

  invisible(x)
}

test_that("datacite() creates a datacite object with minimal input", {
  dc <- datacite(
    Title = "A Simple Dataset",
    Creator = person("John", "Doe", role = "cre"),
    Publisher = "Test Publisher",
    Date = "2024-01-01",
    Language = "en"
  )

  expect_s3_class(dc, "datacite")
  expect_s3_class(dc, "bibentry")
  expect_equal(dc$title, "A Simple Dataset")
  expect_equal(dc$publisher, "Test Publisher")
  expect_equal(dc$year, "2024")
  expect_equal(dc$language, "en")
  expect_equal(dc$format, ":tba")
})

test_that("datacite() calculates PublicationYear from Date if not supplied", {
  dc <- datacite(
    Title = "Auto-Year Dataset",
    Creator = person("Jane", "Smith", role = "cre"),
    Publisher = "Test Publisher",
    Date = "2017-06-01"
  )
  expect_equal(dc$year, "2017")
})

test_that("datacite() accepts subject from subject_create()", {
  subject_obj <- subject_create("Climate Change", subjectScheme = "LCSH")
  dc <- datacite(
    Title = "Climate Data",
    Creator = person("Eve", "Rivera", role = "cre"),
    Publisher = "Climate Org",
    Subject = subject_obj
  )
  expect_equal(dc$subject, "Climate Change")
  expect_equal(attr(dc, "subject"), subject_obj)
})

test_that("datacite() handles optional identifiers and contributors", {
  contributor <- person("Sam", "Lee", role = "ctb")
  dc <- datacite(
    Title = "Dataset with DOI",
    Creator = person("John", "Doe", role = "cre"),
    Identifier = "https://doi.org/10.1234/testdoi",
    Publisher = "OrgX",
    Contributor = contributor
  )
  expect_equal(dc$identifier, "https://doi.org/10.1234/testdoi")
  expect_equal(attr(dc, "contributor")[[1]]$family, "Lee")
})

test_that("datacite() fills defaults when values are NULL", {
  dc <- datacite(
    Title = "Null Defaults",
    Creator = person("Ana", "Ng", role = "cre"),
    Publisher = "Default Org"
  )
  expect_equal(dc$description, ":tba")
  expect_equal(dc$rights, ":tba")
  expect_equal(dc$geolocation, ":unas")
  expect_equal(dc$format, ":tba")
  expect_equal(dc$alternateidentifier, ":unas")
})

test_that("datacite stores structured related as attribute,
          flat string in slot", {
  rel <- related_create("10.1234/abc",
    relationType =
      "IsPartOf", relatedIdentifierType = "DOI"
  )
  dc <- datacite(
    Title = "X",
    Creator = person("A", "B", role = "cre"),
    Publisher = "Org",
    Subject = subject_create("Test"),
    RelatedIdentifier = rel
  )
  expect_equal(dc$relatedidentifier, "10.1234/abc") # flat string
  expect_s3_class(attr(dc, "relatedIdentifier"), "related") # structured
  expect_equal(attr(dc, "relatedIdentifier")$relationType, "IsPartOf")
})


test_that("is.datacite() behave as expected", {
  dc <- datacite(
    Title = "Null Defaults",
    Creator = person("Ana", "Ng", role = "cre"),
    Publisher = "Default Org"
  )
  expect_true(is.datacite(dc))
})

test_that("print.datacite() behaves as expected", {
  subject_obj <- subject_create("Climate Change", subjectScheme = "LCSH")
  dc <- datacite(
    Title = "Climate Data",
    Creator = person("Eve", "Rivera", role = "cre"),
    Publisher = "Climate Org",
    Subject = subject_obj
  )
  expect_output(print(dc), "DataCite Metadata Record")

  out <- capture_output(print(dc))

  # Header
  expect_match(out, "DataCite Metadata Record")
  expect_match(out, "--------------------------")

  # Fields
  expect_match(out, "Title:\\s+Climate Data")
  expect_match(out, "Creator\\(s\\):\\s+Eve Rivera \\[cre\\]")
  expect_match(out, "Subject\\(s\\):\\s+Climate Change")
  expect_match(out, "Publisher:\\s+Climate Org")
  expect_match(out, "Year:\\s+:tba")
  expect_match(out, "Description:\\s+:tba")

  # Alignment: all labels should have the colon at same position
  lines <- strsplit(out, "\n")[[1]]
  meta_lines <- grep(":", lines, value = TRUE)

  # find where values start (first non-space after colon)
  value_start <- vapply(meta_lines, function(l) {
    m <- regexpr(":", l)
    rest <- substr(l, m + 1, nchar(l))
    m + regexpr("[^ ]", rest)
  }, integer(1))

  expect_true(length(unique(value_start)) == 1)
})

test_that("datacite_to_triples() fails when title is missing", {
  dc <- list() # missing required field
  expect_error(
    datacite_to_triples(dc),
    regexp = "title is required"
  )
})

#' A Small GDP Dataset
#'
#' A compact sample of GDP and main aggregates from Eurostat's annual
#' international cooperation dataset. This data subset contains illustrative
#' records for select countries and time periods.
#'
#' @format A data frame with 10 rows and 5 variables:
#' - `geo`: Country name (character)
#' - `year`: Reference year (integer)
#' - `gdp`: Gross Domestic Product value (numeric)
#' - `unit`: Unit of measurement, e.g., "Million EUR" (character)
#' - `freq`: Observation frequency, e.g., "Annual" (character)
#'
#' @details
#' This dataset is intended for examples, tests, and demonstration purposes. It
#' reflects simplified GDP data as published by Eurostat. The actual Eurostat
#' dataset includes more countries, breakdowns, and metadata.
#'
#' @source Eurostat (2021). GDP and main aggregates - international data
#' cooperation (annual data). \doi{10.2908/NAIDA_10_GDP}
#'
#' @keywords datasets
#' @examples
#' head(gdp)
"gdp"

#' @title Get or set all variable labels on a dataset
#'
#' @description
#' Retrieve or assign labels for all variables (columns) in a dataset.
#'
#' @details
#' This is the dataset-level equivalent of [var_label()].
#' It works with any `data.frame`-like object, including [dataset_df()], and
#' returns/sets the `"label"` attribute of each column.
#'
#' Labels are useful for storing human-readable descriptions of variables that
#' may have short or cryptic column names.
#'
#' For internal purposes, this function uses the `"var_labels"` dataset
#' attribute and delegates to [var_label()] and
#' [var_label<-()] on individual columns.
#'
#' @param x A `data.frame` or [`dataset_df`] object.
#' @param value
#'   - For setting: a named list or named character vector of labels.
#'     Names must match column names in `x`.
#'     Unnamed elements are ignored.
#'   - For getting: ignored.
#' @param unlist Logical; if `TRUE`, return a named character vector instead
#'   of a list. Defaults to `FALSE`.
#' @param null_action How to handle columns without labels. One of:
#'   - `"keep"` (default): keep `NULL` values for unlabeled columns.
#'   - `"fill"`: use the column name as a fallback label.
#'   - `"skip"`: exclude unlabeled columns from the result.
#'   - `"na"`: use `NA_character_` for unlabeled columns.
#'   - `"empty"`: use an empty string `""` for unlabeled columns.
#'
#' @return
#' * Getter: a named list (or vector if `unlist = TRUE`) of variable labels.
#' * Setter: the modified `x` with updated labels, returned invisibly.
#'
#' @examples
#' df <- dataset_df(
#'   id = defined(1:3, label = "Observation ID"),
#'   temp = defined(c(22.5, 23.0, 21.8), label = "Temperature (°C)"),
#'   site = defined(c("A", "B", "A"))
#' )
#'
#' # Get all variable labels
#' var_labels(df)
#'
#' # Set multiple labels at once
#' var_labels(df) <- list(site = "Site code")
#'
#' # Return as a named vector with empty string for unlabeled vars
#' var_labels(df, unlist = TRUE, null_action = "empty")
#'
#' @seealso [var_label()]
#' @family defined metadata methods and functions
#' @export
var_labels <- function(x,
                       unlist = FALSE,
                       null_action = c("keep", "fill", "skip", "na", "empty")) {
  null_action <- match.arg(null_action)
  stopifnot(is.data.frame(x))

  result <- lapply(names(x), function(name) {
    lbl <- var_label(x[[name]])
    if (is.null(lbl)) {
      lbl <- switch(null_action,
        keep = NULL,
        fill = name,
        skip = NULL,
        na = NA_character_,
        empty = ""
      )
    }
    lbl
  })
  names(result) <- names(x)

  if (null_action == "skip") {
    result <- result[!vapply(result, is.null, logical(1))]
  }
  if (unlist) {
    return(unlist(result, use.names = TRUE))
  }
  result
}

#' @rdname var_labels
#' @export
`var_labels<-` <- function(x, value) {
  stopifnot(is.data.frame(x))
  stopifnot(is.list(value) || is.character(value))

  # Convert character vector to named list if needed
  if (is.character(value) && !is.list(value)) {
    if (is.null(names(value))) {
      stop("Character vector for var_labels() must be named.")
    }
    value <- as.list(value)
  }

  # Only set labels for matching names
  for (nm in intersect(names(x), names(value))) {
    var_label(x[[nm]]) <- value[[nm]]
  }

  # Update dataset-level "var_labels" attribute too
  x <- set_var_labels(x, value)

  invisible(x)
}

#' @keywords internal
set_var_labels <- function(dataset, var_labels) {
  var_label_list <- list()
  var_label_list <- lapply(colnames(dataset), function(i) i)
  names(var_label_list) <- colnames(dataset)

  for (rn in which(names(var_label_list) %in% names(var_labels))) {
    var_label_list[[rn]] <- var_labels[[which(names(var_label_list)[rn] == names(var_labels))]]
  }

  attr(dataset, "var_labels") <- var_label_list

  dataset
}

#' @title Get / set a concept definition for a vector or a dataset
#' @param x a vector
#' @param value a character string or `NULL` to remove the concept definition of
#'   a measure.
#' @param ... Further parameters for inheritance, not in use.
#' @details `get_variable_concepts()` is identical to `var_concept()`.
#' @examples
#' small_country_dataset <- dataset_df(
#'   country_name = defined(c("Andorra", "Lichtenstein"), label = "Country"),
#'   gdp = defined(c(3897, 7365),
#'     label = "Gross Domestic Product",
#'     unit = "million dollars"
#'   )
#' )
#' var_concept(small_country_dataset$country_name) <- "http://data.europa.eu/bna/c_6c2bb82d"
#' var_concept(small_country_dataset$country_name)
#' # To remove a concept definition of variable
#' var_concept(small_country_dataset$country_name) <- NULL
#' @return The (linked) concept of the meaning of the data contained by a
#'   vector constructed with[defined()].
#' @export

var_concept <- function(x, ...) {
  # rlang::check_dots_used()
  UseMethod("var_concept")
}

#' @export
var_concept.default <- function(x, ...) {
  concept <- attr(x, "concept", exact = TRUE)

  # Fallback to deprecated 'definition' attribute
  if (is.null(concept)) {
    legacy <- attr(x, "definition", exact = TRUE)
    if (!is.null(legacy)) {
      warning("Attribute `definition` is deprecated; please rename to `concept`.", call. = FALSE)
      return(legacy)
    }
  }

  concept
}

#' @rdname var_concept
#' @export
`var_concept<-` <- function(x, value) {
  UseMethod("var_concept<-")
}

#' @rdname var_concept
#' @param x A vector to which the concept URI will be assigned.
#' @param value A character string with a concept URI or NULL to remove the concept.
#' @description Assigns a concept URI to a vector created with `defined()`. This
#' method updates the `concept` attribute and validates that the input is a single
#' character string or NULL.
#' @return The modified vector with updated `concept` metadata.
#' @export
#' @examples
#' x <- defined(c(1, 2, 3), label = "Example Variable")
#' var_concept(x) <- "http://example.org/concept/XYZ"
#' var_concept(x)
`var_concept<-.default` <- function(x, value) {
  if ((!is.character(value) && !is.null(value)) || length(value) > 1) {
    stop("`concept` must be a single character string or NULL", call. = FALSE)
  }
  attr(x, "concept") <- value
  x
}

#' @title Get concepts for all variables in a dataset_df
#' @description Returns a named list of concept URIs (or NULLs) for all
#'   variables.
#' @param x A `dataset_df` object.
#' @return A named list of concept URIs for each variable.
#' @examples get_variable_concepts(orange_df)
#' @export
get_variable_concepts <- function(x) {
  if (!is.dataset_df(x)) {
    stop("get_variable_concepts(x): x must be a dataset_df object.", call. = FALSE)
  }

  lapply(x, var_concept)
}

test_that("var_concept() works for defined vectors", {
  x <- defined(c(1, 2, 3), label = "A")
  expect_null(var_concept(x))

  var_concept(x) <- "http://example.org/concept/ABC"
  expect_equal(var_concept(x), "http://example.org/concept/ABC")

  var_concept(x) <- NULL
  expect_null(var_concept(x))
})

test_that("var_concept() errors for invalid inputs", {
  x <- defined(c(1, 2, 3), label = "A")
  expect_error(
    var_concept(x) <- c("uri1", "uri2"),
    "must be a single character string or NULL"
  )
  expect_error(
    var_concept(x) <- 123,
    "must be a single character string or NULL"
  )
  expect_error(
    var_concept(x) <- list("not a string"),
    "must be a single character string or NULL"
  )
})

test_that("get_variable_concepts() returns named list of concepts", {
  df <- dataset_df(
    var1 = defined(c(1, 2), label = "V1", concept = "http://concept.org/v1"),
    var2 = defined(c(3, 4), label = "V2")
  )
  concepts <- get_variable_concepts(df)

  expect_type(concepts, "list")
  expect_named(concepts, c("rowid", "var1", "var2"))
  expect_equal(concepts$var1, "http://concept.org/v1")
  expect_null(concepts$var2)
})

test_that("get_variable_concepts() errors on non-dataset_df input", {
  expect_error(
    get_variable_concepts(data.frame(x = 1:3)),
    "must be a dataset_df object"
  )
})

#' Format contributors into a citation string
#'
#' Format a list of `utils::person` objects into a compact string, merging roles
#' per person and normalizing names. Contributors without explicit roles are
#' assigned `"ctb"`. If `NULL` or `":unas"` is supplied, returns `":unas"`.
#'
#' @param contributors A vector or list of `person` objects, or `NULL`, or the
#'   character string `":unas"`.
#'
#' @return A single character string, e.g. `"{Jane Doe [dtm, ctb]} and
#'  {John Smith [ctb]}"`.
#' @keywords internal
fix_contributor <- function(contributors = NULL) {
  if (is.null(contributors) ||
    (is.character(contributors) &&
      length(contributors) == 1 &&
      contributors == ":unas")) {
    return(":unas")
  }

  # Normalize into list of persons
  if (inherits(contributors, "person")) {
    contributors_list <- as.list(contributors)
  } else if (is.list(contributors) && all(vapply(contributors, inherits, "person", FUN.VALUE = logical(1)))) {
    contributors_list <- contributors
  } else {
    return(":unas")
  }

  # Group by identity (given|family)
  identity_keys <- vapply(contributors_list, function(p) {
    paste0(p$given %||% "", "|", p$family %||% "")
  }, character(1))

  merged <- vapply(unique(identity_keys), function(key) {
    matches <- contributors_list[identity_keys == key]

    # Collect unique roles (default to ctb)
    roles <- unique(unlist(lapply(matches, function(p) {
      if (is.null(p$role) || length(p$role) == 0 || all(!nzchar(p$role))) {
        "ctb"
      } else {
        p$role
      }
    })))

    base <- matches[[1]]
    full_name <- clean_person_name(base)
    paste0("{", full_name, " [", paste(roles, collapse = ", "), "]}")
  }, character(1))

  paste(merged, collapse = " and ")
}

#' Remove role suffixes from formatted person names
#'
#' @param p A `person` object.
#'
#' @return Character string without role annotations, e.g. `"Jane Doe"`.
#' @keywords internal
clean_person_name <- function(p) {
  name <- format(p)
  sub("\\s*\\[[^]]+\\]$", "", name)
}

#' Map R person roles to schema.org-style roles
#'
#' @param role A character vector of roles (e.g. `"cre"`, `"ctb"`).
#'
#' @return A character vector with schema.org-style roles.
#' @keywords internal
map_role_to_schema <- function(role) {
  role_map <- c(
    cre = "creator",
    aut = "author",
    ctb = "contributor",
    dtm = "editor",
    fnd = "funder",
    cph = "copyrightHolder",
    own = "copyrightHolder",
    pbl = "publisher"
  )
  ifelse(role %in% names(role_map), role_map[role], "contributor")
}

test_that("returns :unas for NULL input", {
  expect_equal(fix_contributor(NULL), ":unas")
})

test_that("returns :unas for ':unas' input", {
  expect_equal(fix_contributor(":unas"), ":unas")
})

test_that("handles a single person with explicit role", {
  p <- person(given = "Jane", family = "Doe", role = "dtm")
  expect_equal(fix_contributor(p), "{Jane Doe [dtm]}")
})

test_that("handles a single person with no role (defaults to ctb)", {
  p <- person(given = "Jane", family = "Doe")
  expect_equal(fix_contributor(p), "{Jane Doe [ctb]}")
})

test_that("merges multiple roles for the same person", {
  p1 <- person(given = "Jane", family = "Doe", role = "dtm")
  p2 <- person(given = "Jane", family = "Doe", role = "ctb")
  expect_equal(fix_contributor(list(p1, p2)), "{Jane Doe [dtm, ctb]}")
})

test_that("handles a list of persons with mixed roles", {
  p1 <- person(given = "Jane", family = "Doe", role = "dtm")
  p2 <- person(given = "John", family = "Smith") # defaults to ctb
  expect_equal(
    fix_contributor(list(p1, p2)),
    "{Jane Doe [dtm]} and {John Smith [ctb]}"
  )
})

test_that("clean_person_name strips role suffixes from person", {
  p <- person(given = "Jane", family = "Doe", role = "dtm")
  expect_equal(clean_person_name(p), "Jane Doe")
})

test_that("clean_person_name handles simple strings", {
  expect_equal(clean_person_name("Jane Doe [ctb]"), "Jane Doe")
  expect_equal(clean_person_name("John Smith"), "John Smith")
})

test_that("map_role_to_schema maps known roles", {
  expect_equal(map_role_to_schema("cre"), "creator")
  expect_equal(map_role_to_schema("ctb"), "contributor")
  expect_equal(map_role_to_schema("dtm"), "editor") # or whatever mapping you want
})

test_that("map_role_to_schema defaults to 'contributor' for unknown roles", {
  expect_equal(map_role_to_schema("xyz"), "contributor")
})

#' Add Identifier to First Column of a Dataset
#'
#' Adds a prefixed identifier (e.g., `eg:`) to the first column of a dataset,
#' useful for generating semantic row IDs (e.g., for RDF serialization).
#'
#' @param x A dataset created with [dataset_df()], or a regular data frame.
#' @param prefix A character string used as the prefix for row identifiers.
#'   Defaults to `"eg:"` (referring to [example.com](https://example.com)).
#' @param ids Optional. A character vector of custom IDs to use instead of row names.
#'
#' @return
#' A dataset of the same class as `x`, with the first column updated to include
#' unique prefixed identifiers.
#'
#' @examples
#' # Example with a dataset_df object:
#' id_to_column(orange_df)
#'
#' # Example with a regular data.frame:
#' id_to_column(Orange, prefix = "orange:")
#'
#' @export
id_to_column <- function(x, prefix = "eg:", ids = NULL) {
  is_dataset <- is.dataset_df(x)

  lastcol <- ncol(x)

  if (is.null(ids)) {
    ids <- gsub("[^[:alnum:]]", "-", row.names(x))
  } else if (nrow(x) != length(ids)) {
    stop("id_to_column(x, ..., ids) : ids must be of same lengths as nrow(x).")
  }

  if (is.null(prefix)) {
    prefix <- ""
  }

  if ("rowid" %in% names(x)) {
    x$rowid <- paste0(prefix, ids)
    return(x)
  } else {
    rhs <- x
    x$rowid <- paste0(prefix, ids)
    lhs <- x[, "rowid", drop = FALSE]

    if (is_dataset) {
      DataBibentry <- get_bibentry(rhs)
      dataset_subject <- subject(rhs)
      dataset_prov <- provenance(x)
      tmp <- as_dataset_df(cbind(lhs, rhs),
        reference = list(
          author = DataBibentry$author,
          title = DataBibentry$title
        )
      )
      attr(tmp, "dataset_bibentry") <- DataBibentry
      attr(tmp, "prov") <- dataset_prov
      subject(tmp) <- dataset_subject
    } else {
      tmp <- cbind(lhs, rhs)
    }
  }

  tmp
}

#' @title Get or set the dataset Description
#'
#' @description Get or set the optional `Description` property as an attribute
#'   on a dataset object.
#'
#' @details The `Description` is recommended for discovery in DataCite. It
#'   captures additional information that does not fit other metadata categories
#'   — such as technical notes or dataset usage. It is a free-text field. See
#'   [dct:description](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/description/).
#'
#' @param x A dataset object created with [`dataset_df()`] or
#'   [`as_dataset_df()`].
#' @param value The new description, as a character string.
#' @param overwrite Logical. If `TRUE`, will overwrite any existing description.
#'   If `FALSE` (default), will warn and keep the existing description.
#'
#' @return The `Description` attribute as a character vector of length 1.
#'
#' @examples
#' description(orange_df)
#' description(orange_df, overwrite = TRUE) <- "This dataset records orange tree growth."
#' description(orange_df)
#'
#' @family bibliographic helper functions
#' @export

description <- function(x) {
  assert_that(is.dataset_df(x),
    msg = "description(x): x must be a dataset object created with dataset_df() or as_dataset_df()."
  )

  ds_bibentry <- get_bibentry(x)
  as.character(ds_bibentry$description)
}

#' @rdname description
#' @export
`description<-` <- function(x, overwrite = FALSE, value) {
  assert_that(is.dataset_df(x),
    msg = "description(x) <- value: x must be a dataset object created with dataset_df() or as_dataset_df()."
  )

  ds_bibentry <- get_bibentry(x)
  existing_description <- as.character(ds_bibentry$description)

  if (is.null(value)) {
    value <- ":unas"
  }

  if (overwrite ||
    length(existing_description) == 0 ||
    existing_description %in% c("", ":unas", ":tba")) {
    ds_bibentry$description <- as.character(value)
    attr(x, "dataset_bibentry") <- ds_bibentry
  } else {
    warning(
      "The dataset already has a description: ",
      existing_description, "."
    )
  }
  invisible(x)
}

test_that("description() <- assignment works with and without overwrite", {
  x <- orange_df
  description(x, overwrite = TRUE) <- "Description 1"
  expect_equal(description(x), "Description 1")

  # Attempt to overwrite without permission triggers warning and no change
  expect_warning(
    {
      description(x, overwrite = FALSE) <- "Should Not Overwrite"
    },
    regexp = "The dataset already has a description"
  )
  expect_equal(description(x), "Description 1")

  # Explicit overwrite works
  description(x, overwrite = TRUE) <- "Description 2"
  expect_equal(description(x), "Description 2")
})

test_that("description() <- NULL sets to ':unas'", {
  x <- orange_df
  description(x, overwrite = TRUE) <- NULL
  expect_equal(description(x), ":unas")
})

test_that("description() <- works when no initial description exists", {
  # Strip description from copy of orange_df
  x <- orange_df
  attr(x, "dataset_bibentry")$description <- NULL
  expect_equal(length(description(x)), 0)

  # Assign a new one without error
  description(x, overwrite = TRUE) <- "Fresh Description"
  expect_equal(description(x), "Fresh Description")
})

test_that("description() fails on non-dataset_df", {
  expect_error(description(mtcars), "x must be a dataset object")
  expect_error(description(mtcars, overwrite = TRUE) <- "Bad", "x must be a dataset object")
})

#' @title Dataset to triples (three columns or N-Triples)
#' @description Converts a dataset to RDF-style triples with subject, predicate,
#'   and object columns. Supports semantic expansion via variable metadata.
#' @param x A `dataset_df` or `data.frame`.
#' @param idcol Name or index of the subject column. If NULL, defaults to
#'   `"rowid"` or rownames.
#' @param expand_uri Logical; if TRUE, expands URIs using namespaces and
#'   definitions.
#' @param format Output format: `"data.frame"` (default) or `"nt"` for
#'   N-Triples.
#' @return Either a `data.frame` with columns `s`, `p`, and `o`, or a character
#'   vector of N-Triple lines.
#' @export
#' @details
#' For publishing examples, a minimal serverless scaffold is provided at
#' <https://github.com/dataobservatory-eu/dataset-template>, which shows how
#' to host CSV + RDF serialisations on GitHub Pages without any server setup.
#'
#' @note A simple, serverless scaffolding for publishing `dataset_df` objects
#'   on the web (with HTML + RDF exports) is available at
#'   <https://github.com/dataobservatory-eu/dataset-template>.
#'
#' @examples
#' # A minimal example with just rowid and geo
#' data("gdp", package = "dataset")
#' small_geo <- dataset_df(
#'   geo = defined(
#'     gdp$geo[1:3],
#'     label = "Geopolitical entity",
#'     concept = "http://example.com/prop/geo",
#'     namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/geo/$1"
#'   )
#' )
#'
#' # View as triple table
#' dataset_to_triples(small_geo)
#'
#' # View as N-Triples
#' dataset_to_triples(small_geo, format = "nt")
dataset_to_triples <- function(x,
                               idcol = NULL,
                               expand_uri = TRUE,
                               format = "data.frame") {
  is_dataset <- inherits(x, "dataset_df")
  if (!is_dataset) {
    stop("Works only with dataset_df objects.")
  }

  # Determine subject (s)
  if (is.null(idcol)) {
    if ("rowid" %in% names(x)) {
      idcol <- "rowid"
    } else {
      x <- cbind(rowid = row.names(x))
      idcol <- "rowid"
    }
  }

  idcol_pos <- idcol_find(x, idcol)
  data_cols <- setdiff(seq_along(x), idcol_pos)

  # Expand subject URIs using namespace
  s_vec <- as.character(x[[idcol]])
  ns <- tryCatch(var_namespace(x[[idcol]]), error = function(e) NULL)

  if (expand_uri && !is.null(ns) && nzchar(ns)) {
    if (grepl("\\$1", ns)) {
      s_vec <- vapply(s_vec, function(id) sub("\\$1", id, ns), character(1))
    } else {
      s_vec <- paste0(ns, s_vec)
    }
  }


  # Construct triple list
  triple_list <- lapply(data_cols, function(i) {
    triples_column_generate(s_vec, x[[i]], names(x)[i])
  })

  out <- do.call(rbind, triple_list)

  # Add metadata if available
  if (is_dataset) {
    attr(out, "bibentry") <- get_bibentry(x)
    attr(out, "title") <- paste0(dataset_title(x), " [triple form]")
    attr(out, "subject") <- subject(x)
  }

  row.names(out) <- NULL

  # Return either data.frame or N-Triples
  if (format %in% c("nt", "ntriples", "N-Triples")) {
    triples_to_ntriples(out)
  } else {
    out
  }
}

#' @title Internal: Expand multi-valued DC fields to RDF triples
#' @description Converts scalar or vector fields into RDF triples.
#' @param dataset_id The subject URI
#' @param predicate_uri The RDF predicate URI
#' @param values A scalar, character vector, or list (e.g., person objects)
#' @return A character vector of RDF triples
#' @keywords internal
expand_triples <- function(dataset_id, predicate_uri, values) {
  if (is.null(values)) {
    return(character(0))
  }

  # Ensure list for consistency
  if (!is.list(values)) values <- as.list(values)

  vapply(values, function(val) {
    if (inherits(val, "person")) {
      val <- format(val)
    }
    n_triple(dataset_id, predicate_uri, val)
  }, character(1))
}

#' @title Internal: Generate RDF triples for a single column
#' @description Create subject-predicate-object triples from one column of a dataset
#' @param s_vec A character vector of subject URIs (length = number of rows)
#' @param col The column vector (e.g., \code{x[[i]]})
#' @param colname The name of the column (used as fallback for predicate)
#' @return A data.frame with columns s, p, o
#' @keywords internal
triples_column_generate <- function(s_vec, col, colname) {
  def <- tryCatch(var_concept(col), error = function(e) NULL)
  ns <- tryCatch(var_namespace(col), error = function(e) NULL)

  # predicate: use definition or fallback
  pred_uri <- if (!is.null(def) && nzchar(def)) {
    def
  } else {
    paste0("http://example.com/prop/", colname)
  }

  # object: URI from namespace or typed literal via xsd_convert
  if (!is.null(ns) && nzchar(ns)) {
    o_val <- vapply(as.character(col), function(val) {
      if (grepl("\\$1", ns)) {
        sub("\\$1", val, ns)
      } else {
        paste0(ns, val)
      }
    }, character(1))
  } else {
    o_val <- xsd_convert(col)
  }

  data.frame(
    s = s_vec,
    p = rep(pred_uri, length(o_val)),
    o = o_val,
    stringsAsFactors = FALSE
  )
}


#' @title Internal: Convert triple data.frame to N-Triples format
#' @description Turns a data.frame with `s`, `p`, `o` columns into N-Triples strings.
#' @param df A data.frame with columns `s`, `p`, and `o`.
#' @return A character vector of N-Triple lines.
#' @keywords internal
triples_to_ntriples <- function(df) {
  stopifnot(all(c("s", "p", "o") %in% names(df)))
  vapply(seq_len(nrow(df)), function(i) {
    n_triple(df$s[i], df$p[i], df$o[i])
  }, character(1))
}

test_that("dataset_to_triples expands subject URIs correctly", {
  data("gdp", package = "dataset")

  small_country_dataset <- dataset_df(
    geo = defined(
      gdp$geo,
      label = "Country name",
      concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
      namespace = "https://www.geonames.org/countries/$1/"
    ),
    year = defined(
      gdp$year,
      label = "Reference Period (Year)",
      concept = "http://purl.org/linked-data/sdmx/2009/dimension#refPeriod"
    ),
    gdp = defined(
      gdp$gdp,
      label = "Gross Domestic Product",
      unit = "CP_MEUR",
      concept = "http://data.europa.eu/83i/aa/GDP"
    ),
    unit = gdp$unit,
    freq = defined(
      gdp$freq,
      label = "Frequency",
      concept = "http://purl.org/linked-data/sdmx/2009/code"
    ),
    dataset_bibentry = dublincore(
      title = "Small Country Dataset",
      creator = person("Jane", "Doe"),
      publisher = "Example Inc.",
      datasource = "https://doi.org/10.2908/NAIDA_10_GDP",
      rights = "CC-BY",
      coverage = "Andorra, Lichtenstein and San Marino"
    )
  )

  triples <- dataset_to_triples(small_country_dataset)

  # Extract actual s values
  s_values <- unique(triples$s)
  # Expect all s values to begin with the namespace prefix
  expect_true(all(grepl("^http://example\\.com/dataset#", s_values)))

  # Check that s is URI-expanded
  expect_false(any(s_values %in% gdp$geo)) # none should be raw codes like "AD"
})


test_that("triples_column_generate generates correct RDF triples", {
  s <- c("http://example.com/obs/1", "http://example.com/obs/2")

  # Case 1: defined with namespace and concept
  x <- defined(
    c("AD", "FR"),
    label = "Country",
    concept = "http://example.com/prop/geo",
    namespace = "https://eurostat.europa.eu/geo/$1"
  )
  triples <- triples_column_generate(s, x, "geo")

  expect_equal(triples$p[1], "http://example.com/prop/geo")
  expect_equal(triples$o[2], "https://eurostat.europa.eu/geo/FR")

  # Case 2: numeric without namespace
  x <- c(1.23, 4.56)
  triples <- triples_column_generate(s, x, "value")
  expect_true(all(grepl('^".+"\\^\\^<xsd:decimal>$', triples$o)))
})

test_that("dataset_to_triples generates valid N-Triples", {
  data("gdp", package = "dataset")

  small_country_dataset <- dataset_df(
    geo = defined(
      gdp$geo,
      label = "Country name",
      concept = "http://example.com/prop/geo",
      namespace = "https://www.geonames.org/countries/$1/"
    ),
    year = defined(
      gdp$year,
      label = "Reference Period (Year)",
      concept = "http://purl.org/linked-data/sdmx/2009/dimension#refPeriod"
    ),
    gdp = defined(
      gdp$gdp,
      label = "Gross Domestic Product",
      unit = "CP_MEUR",
      concept = "http://data.europa.eu/83i/aa/GDP"
    ),
    unit = defined(
      gdp$unit,
      label = "Unit of measure",
      concept = "http://purl.org/linked-data/sdmx/2009/attribute#unitMeasure",
      namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/unit/$1"
    ),
    freq = defined(
      gdp$freq,
      label = "Frequency",
      concept = "http://purl.org/linked-data/sdmx/2009/code#freq",
      namespace = "http://purl.org/linked-data/sdmx/2009/code#freq-"
    ),
    dataset_bibentry = dublincore(
      title = "Small Country Dataset",
      creator = person("Jane", "Doe"),
      publisher = "Example Inc.",
      datasource = "https://doi.org/10.2908/NAIDA_10_GDP",
      rights = "CC-BY",
      coverage = "Andorra, Lichtenstein and San Marino"
    )
  )

  # Get expanded triple table and N-Triples
  triples_df <- dataset_to_triples(small_country_dataset)
  ntriples <- dataset_to_triples(small_country_dataset, format = "nt")

  # Validate structure
  expect_true(all(c("s", "p", "o") %in% names(triples_df)))
  expect_true(all(grepl("^<.*> <.*> .+ \\.$", ntriples))) # basic N-Triples format

  # Spot-check GDP literal
  gdp_triples <- subset(triples_df, grepl("GDP", p))
  expect_true(all(grepl("\\^\\^<xsd:decimal>$", gdp_triples$o)))

  # Spot-check geo URIs
  geo_triples <- subset(triples_df, grepl("prop/geo", p))
  expect_true(all(grepl("^https://www\\.geonames\\.org/countries/", geo_triples$o)))

  # Spot-check unit URIs
  unit_triples <- subset(triples_df, grepl("unitMeasure", p))
  expect_true(all(grepl("^https://dd\\.eionet\\.europa\\.eu/vocabulary/eurostat/unit/", unit_triples$o)))

  # Check frequency URIs
  freq_triples <- subset(triples_df, grepl("code#freq", p))
  expect_true(all(grepl("^http://purl\\.org/linked-data/sdmx/2009/code#freq-", freq_triples$o)))

  # Ensure N-Triples length matches data frame
  expect_equal(length(ntriples), nrow(triples_df))
})

#' Add or Retrieve Dublin Core Metadata
#'
#' Adds or retrieves metadata conforming to the
#' [Dublin Core Metadata Terms](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/)
#' standard, enabling consistent and structured citation and retrieval of R
#' dataset objects.
#'
#' @details
#' The Dublin Core Metadata Element Set (DCMES) is a standardized vocabulary for
#' describing digital and physical resources. It includes 15 core fields and is
#' formally standardized as ISO 15836, IETF RFC 5013, and ANSI/NISO Z39.85.
#'
#' This function constructs a [utils::bibentry()] object extended with DCMI
#' terms and is compatible with [dataset_df()] objects. The resulting metadata
#' can be used for semantic documentation and machine-readable citation.
#'
#' For compatibility with [utils::bibentry()], the `dataset_date` parameter is
#' automatically used to derive both `publication_date` and `year` fields.
#'
#' @param x An object to annotate. Typically a `data.frame`, `tibble`,
#'   or named list; can be added with [`set_bibentry<-`] to a semantically
#'   rich `dataset_df`.
#' @param title A name given to the resource. See [dataset_title()].
#' @param creator One or more [utils::person()] objects representing the
#'   creator(s). See [creator()].
#' @param contributor Additional contributors ([utils::person()]) with optional
#'   roles. See [contributor()].
#' @param publisher A character or [utils::person()] indicating the publishing
#'   entity. See [publisher()].
#' @param dataset_date A publication or release date (`Date`, `POSIXct`, or
#'   character in `YYYY`, `YYYY-MM-DD`, or ISO format).
#' @param year An explicit publication year. If omitted, inferred from
#'   `dataset_date`.
#' @param identifier A unique persistent identifier (e.g., DOI). See [identifier()].
#' @param subject A keyword or controlled vocabulary term. See [subject()] and
#'   [subject_create()].
#' @param description A free-text summary of the dataset. See [description()].
#' @param language ISO 639-1 language code. See [language()].
#' @param rights A string describing intellectual property or usage rights.
#' Use a URI like `"https://creativecommons.org/public-domain/cc0/"`.
#' @param dataset_format The technical format of the dataset (e.g., MIME type).
#' See [dataset_format()].
#' @param relation A related resource (e.g., version, paper, or parent dataset).
#'   Currently only supports an URI, for example,
#'   `"https:://doi.org/10.32614/CRAN.package.dataset"`.
#' @param datasource A URL or label for the original source of the dataset.
#' @param coverage Geographic or temporal extent (spatial/temporal coverage).
#' @param type The resource type. For datasets, use `"Dataset"`. See
#'   [DCMI Type Vocabulary](https://www.dublincore.org/specifications/dublin-core/dcmi-type-vocabulary/).
#' @param ... Additional metadata fields.
#'
#' @return
#' A `bibentry` object extended with class `"bibrecord"`, storing structured
#' Dublin Core metadata. Use [as_dublincore()] to extract the metadata in list,
#' tabular, or RDF form.
#'
#' @source
#' - [DCMI Metadata Terms](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/)
#'
#' @importFrom utils person bibentry
#'
#' @examples
#' orange_bibentry <- dublincore(
#'   title = "Growth of Orange Trees",
#'   creator = c(
#'     person(
#'       given = "N.R.",
#'       family = "Draper",
#'       role = "cre",
#'       comment = c(VIAF = "http://viaf.org/viaf/84585260")
#'     ),
#'     person(given = "H", family = "Smith", role = "cre")
#'   ),
#'   contributor = person(given = "Antal", family = "Daniel", role = "dtm"),
#'   publisher = "Wiley",
#'   datasource = "https://isbnsearch.org/isbn/9780471170822",
#'   dataset_date = 1998,
#'   identifier = "https://doi.org/10.5281/zenodo.14917851",
#'   language = "en",
#'   description = "The Orange data frame has 35 rows and 3 columns of records
#'                  of the growth of orange trees."
#' )
#'
#' # To inspect structured metadata from a dataset_df object:
#' as_dublincore(orange_df, type = "list")
#'
#' @export
#' @seealso
#' Learn more in the vignette:
#' [`bibrecord`](https://dataset.dataobservatory.eu/articles/bibrecord.html)
#' @family bibrecord functions

dublincore <- function(
  title,
  creator,
  contributor = NULL,
  year = NULL,
  publisher = NULL,
  identifier = NULL,
  subject = NULL,
  type = "DCMITYPE:Dataset",
  dataset_date = NULL,
  language = NULL,
  relation = NULL,
  dataset_format = "application/r-rds",
  rights = NULL,
  datasource = NULL,
  description = NULL,
  coverage = NULL
) {
  if (missing(creator) || is.null(creator)) {
    stop("dublincore(): A valid `creator` (as person or list of person) is required.")
  }

  if (inherits(creator, "person")) {
    creator <- list(creator)
  }

  if (length(title) != 1) {
    stop("dublincore(): title must be a single character string.")
  }

  creators <- normalize_roles(creator, default_role = "cre")

  dataset_date <- ifelse(is.null(dataset_date), ":tba", as.character(dataset_date))
  identifier <- ifelse(is.null(identifier), ":tba", as.character(identifier))
  dataset_format <- ifelse(is.null(dataset_format), "application/r-rds", as.character(dataset_format))
  if (is.null(relation)) {
    relation <- ":unas"
  }
  rights <- ifelse(is.null(rights), ":tba", as.character(rights))
  coverage <- ifelse(is.null(coverage), ":unas", as.character(coverage))
  datasource <- ifelse(is.null(datasource), ":unas", as.character(datasource))
  publishers <- if (is.null(publisher)) ":unas" else publisher
  contributor <- if (is.null(contributor)) NULL else contributor
  creators <- if (is.null(creator)) creators <- ":tba" else creators <- creator
  year <- if (!is.null(dataset_date)) substr(as.character(dataset_date), 1, 4) else NULL

  publisher <- fix_publisher(publishers = publishers)

  new_dublincore(
    title = title,
    creator = creators,
    identifier = identifier,
    publisher = publisher,
    subject = subject,
    type = type,
    contributor = contributor,
    dataset_date = dataset_date,
    year = year,
    language = language,
    relation = relation,
    dataset_format = dataset_format,
    rights = rights,
    datasource = datasource,
    description = description,
    coverage = coverage
  )
}


#' @keywords internal
new_dublincore <- function(title,
                           creator,
                           identifier = NULL,
                           publisher = NULL,
                           subject = NULL,
                           type = "DCMITYPE:Dataset",
                           contributor = NULL,
                           dataset_date = NULL,
                           year = NULL,
                           language = NULL,
                           relation = NULL,
                           dataset_format = NULL,
                           rights = NULL,
                           datasource = NULL,
                           description = NULL,
                           coverage = NULL) {
  year <- if (!is.null(dataset_date)) {
    substr(as.character(dataset_date), 1, 4)
  } else {
    NULL
  }

  # flatten relation for bibrecord
  relation_flat <- if (is.related(relation)) {
    relation$relatedIdentifier
  } else {
    relation
  }

  dublincore_object <- bibrecord(
    title       = title,
    author      = creator,
    identifier  = identifier,
    publisher   = publisher,
    subject     = if (is.subject(subject)) subject$term else as.character(subject),
    type        = type,
    contributor = contributor,
    date        = dataset_date,
    year        = year,
    language    = language,
    relation    = relation_flat,
    format      = dataset_format,
    rights      = rights,
    datasource  = datasource,
    description = description,
    coverage    = coverage
  )

  # preserve structured subject
  if (!is.null(subject) && is.subject(subject)) {
    attr(dublincore_object, "subject") <- subject
  }

  # preserve structured relation
  if (!is.null(relation) && is.related(relation)) {
    attr(dublincore_object, "relation") <- relation
  }

  if (!is.null(contributor)) {
    attr(dublincore_object, "contributor") <- contributor
  }

  class(dublincore_object) <- c("dublincore", class(dublincore_object))
  dublincore_object
}


#' @rdname dublincore
#'
#' @description
#' `is.dublincore()` checks whether an object inherits from the `"dublincore"`
#' class.
#'
#' @param x An object to test.
#'
#' @return
#' A logical value: `TRUE` if `x` is a Dublin Core metadata record (i.e.,
#' inherits from `"dublincore"`), otherwise `FALSE`.
#'
#' @export
is.dublincore <- function(x) {
  inherits(x, "dublincore")
}

#' @rdname dublincore
#' @exportS3Method
print.dublincore <- function(x, ...) {
  cat("Dublin Core Metadata Record\n")
  cat("--------------------------\n")

  pr <- function(label, value) {
    if (!is.null(value) && length(value) > 0 && any(nzchar(value))) {
      cat(sprintf("%-12s %s\n", paste0(label, ":"), paste(value, collapse = "; ")))
    }
  }

  pr("Title", x$title)
  pr("Creator(s)", paste(format(x$author), collapse = "; "))

  contributor <- attr(x, "contributor")
  if (!is.null(contributor)) {
    pr("Contributor(s)", fix_contributor(contributor))
  }

  subj <- attr(x, "subject")
  if (!is.null(subj) && is.subject(subj)) {
    subj_val <- subj$term
    if (!is.null(subj$schemeURI) && nzchar(subj$schemeURI)) {
      subj_val <- paste0(subj_val, " [", subj$schemeURI, "]")
    }
    pr("Subject(s)", subj_val)
  } else if (!is.null(x$subject)) {
    pr("Subject(s)", x$subject)
  }

  pr("Publisher", x$publisher)
  pr("Year", x$year)
  pr("Language", x$language)
  pr("Description", x$description)

  invisible(x)
}

test_that("creates valid dublincore object", {
  dc <- dublincore(
    title = "Sample Dataset",
    creator = person("Jane", "Doe")
  )
  expect_s3_class(dc, c("dublincore", "bibrecord", "bibentry"))
  expect_equal(dc$title, "Sample Dataset")
  expect_true(inherits(dc$author[[1]], "person"))
})

test_that("dublincore() errors when title has length > 1", {
  expect_error(
    dublincore(
      title = c("Too", "Many", "Titles"),
      creator = person("Jane", "Doe")
    ),
    "title must be a single character string"
  )
})


test_that("missing creator fails", {
  expect_error(
    dublincore(title = "Missing Creator"),
    "creator.*required"
  )
})

test_that("default values are set", {
  dc <- dublincore(
    title = "Defaults Example",
    creator = person("Jane", "Doe")
  )
  expect_equal(dc$rights, ":tba")
  expect_equal(dc$format, "application/r-rds")
  expect_equal(dc$relation, ":unas")
})

test_that("custom contributor is set", {
  contrib <- person(given = "Antal", family = "Daniel", role = "dtm")
  dc <- dublincore(
    title = "Contrib Example",
    creator = person("Jane", "Doe"),
    contributor = contrib
  )
  expect_equal(attr(dc, "contributor")[[1]]$given, "Antal")
  expect_equal(attr(dc, "contributor")[[1]]$role, "dtm")
})

test_that("dataset_date sets year correctly", {
  dc <- dublincore(
    title = "Date Example",
    creator = person("Jane", "Doe"),
    dataset_date = as.Date("2022-06-01")
  )
  expect_equal(dc$year, "2022")
})

test_that("identifier, language, and format are preserved", {
  dc <- dublincore(
    title = "Complete Example",
    creator = person("Jane", "Doe"),
    identifier = "https://doi.org/10.1234/test",
    language = "en",
    dataset_format = "application/json"
  )
  expect_equal(dc$identifier, "https://doi.org/10.1234/test")
  expect_equal(dc$language, "en")
  expect_equal(dc$format, "application/json")
})

test_that("print.dublincore prints key fields", {
  s1 <- subject_create("Climate Change", schemeURI = "http://id.loc.gov/subjects", subjectScheme = "LCSH")

  dc <- dublincore(
    title = "Climate Data",
    creator = person("Eve", "Rivera", role = "cre"),
    publisher = "Climate Org",
    subject = s1,
    description = "A dataset on climate change indicators."
  )

  expect_output(print(dc), "Dublin Core Metadata Record")

  out <- capture_output(print(dc))

  # Header
  expect_match(out, "Dublin Core Metadata Record")
  expect_match(out, "--------------------------")

  # Fields
  expect_match(out, "Title:\\s+Climate Data")
  expect_match(out, "Creator\\(s\\):\\s+Eve Rivera \\[cre\\]")
  expect_match(out, "Subject\\(s\\):\\s+Climate Change")
  expect_match(out, "Publisher:\\s+Climate Org")
  expect_match(out, "Year:\\s+:tba")
  expect_match(out, "Description:\\s+A dataset on climate change indicators")

  # Alignment: all labels should have the colon at same position
  lines <- strsplit(out, "\n")[[1]]
  meta_lines <- grep(":", lines, value = TRUE)

  # find where values start (first non-space after colon)
  value_start <- vapply(meta_lines, function(l) {
    m <- regexpr(":", l)
    rest <- substr(l, m + 1, nchar(l))
    m + regexpr("[^ ]", rest)
  }, integer(1))

  expect_true(length(unique(value_start)) == 1)
})

test_that("is.dublincore returns TRUE for dublincore object", {
  dc <- dublincore(
    title = "Dublin Check",
    creator = person("Jane", "Doe")
  )
  expect_true(is.dublincore(dc))
})

test_that("is.dublincore returns FALSE for non-dublincore object", {
  df <- data.frame(x = 1:3)
  expect_false(is.dublincore(df))
})

test_that("dublincore stores structured relation as attribute and
          flat relation in slot", {
  rel <- related_create("https://doi.org/10.5678/def", "References", "DOI")
  dc <- dublincore(
    title = "X",
    creator = person("A", "B", role = "cre"),
    relation = rel
  )
  expect_equal(dc$relation, "https://doi.org/10.5678/def")
  expect_s3_class(attr(dc, "relation"), "related", exact = FALSE)
})

#' @keywords internal
datacite_to_triples <- function(dc_list,
                                dataset_id = "http://example.com/dataset") {
  if (is.null(dc_list$title) || nchar(dc_list$title) == 0) {
    stop("datacite_to_triples(): title is required")
  }

  base <- "http://datacite.org/schema/kernel-4/"
  triples <- character()

  triples <- c(triples, n_triple(
    dataset_id,
    paste0(base, "title"), dc_list$title
  ))

  if (!is.null(dc_list$author)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "creator"),
      dc_list$author
    ))
  }

  if (!is.null(dc_list$contributor)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "contributor"),
      dc_list$contributor
    ))
  }

  if (!is.null(dc_list$identifier)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "identifier"),
      dc_list$identifier
    ))
  }

  if (!is.null(dc_list$publisher)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "publisher"),
      dc_list$publisher
    ))
  }

  if (!is.null(dc_list$publicationyear)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "publicationYear"),
      dc_list$publicationyear
    ))
  }

  if (!is.null(dc_list$language)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "language"),
      dc_list$language
    ))
  }

  if (!is.null(dc_list$rights)) {
    triples <- c(triples, n_triple(dataset_id, paste0(base, "rights"), dc_list$rights))
  }

  if (!is.null(dc_list$description)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "descriptions"),
      dc_list$description
    ))
  }

  if (!is.null(dc_list$subject)) {
    subj_value <- dc_list$subject

    if (is.subject(subj_value)) {
      subj_value <- subj_value$term
    } else if (is.list(subj_value)) {
      subj_value <- vapply(subj_value, function(s) {
        if (is.subject(s)) {
          s$term
        } else {
          as.character(s)
        }
      }, character(1))
    } else {
      subj_value <- as.character(subj_value)
    }

    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "subjects"),
      subj_value
    ))
  }


  if (!is.null(dc_list$format)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "formats"),
      dc_list$format
    ))
  }

  if (!is.null(dc_list$version)) {
    triples <- c(triples, n_triple(
      dataset_id,
      paste0(base, "version"),
      dc_list$version
    ))
  }

  # --- Related identifiers ---
  rel_attr <- attr(dc_list, "relation", exact = TRUE)
  if (!is.null(rel_attr)) {
    # normalize: single object → list
    if (is.related(rel_attr)) rel_attr <- list(rel_attr)

    if (is.list(rel_attr) && all(vapply(rel_attr, is.related, logical(1)))) {
      for (ri in rel_attr) {
        triples <- c(
          triples,
          n_triple(
            dataset_id,
            paste0(base, "relatedIdentifier"),
            ri$relatedIdentifier
          ),
          n_triple(
            dataset_id,
            paste0(base, "relationType"),
            ri$relationType
          ),
          n_triple(
            dataset_id,
            paste0(base, "relatedIdentifierType"),
            ri$relatedIdentifierType
          )
        )
        if (!is.null(ri$resourceTypeGeneral) && nzchar(ri$resourceTypeGeneral)) {
          triples <- c(
            triples,
            n_triple(
              dataset_id,
              paste0(base, "resourceTypeGeneral"),
              ri$resourceTypeGeneral
            )
          )
        }
      }
    }
  } else if (!is.null(dc_list$relatedidentifier) &&
    dc_list$relatedidentifier != ":unas") {
    triples <- c(
      triples,
      n_triple(
        dataset_id,
        paste0(base, "relatedIdentifier"),
        dc_list$relatedidentifier
      )
    )
  }


  n_triples(triples)
}

#' Coerce a defined vector to character
#'
#' `as_character()` is the recommended method to convert a
#' [`defined()`][defined] vector into a character vector. It is metadata-aware
#' and provides explicit control over whether semantic attributes are preserved.
#'
#' If `preserve_attributes = TRUE`, the returned character vector retains
#' metadata attributes (`unit`, `concept`, `namespace`, `label`). The
#' `"defined"` class is always removed.
#'
#' If `preserve_attributes = FALSE` (default), a plain character vector is
#' returned with *all* metadata stripped.
#'
#' @details
#' Use \code{strip_attributes = TRUE} when flattening or preparing data for
#' external pipelines, but keep the default when working with defined
#' vectors directly.
#'
#' Base R's `as.character()` always drops all attributes and returns plain
#' character values. It is equivalent to:
#' `as_character(x, strip_attributes = TRUE)`.
#'
#' @param x A vector created with [defined()].
#' @param strip_attributes Logical; should semantic metadata attributes
#'   (such as \code{label}, \code{unit}, \code{definition}, and
#'   \code{namespace}) be removed from the returned vector?
#'   Defaults to \code{FALSE}.
#'
#' @param ... Reserved for potential future use.
#'
#' @return A character vector (plain or with attributes).
#'
#' @examples
#' x <- defined(c("apple", "banana"), label = "Fruit", unit = "kg")
#'
#' # Recommended:
#' as_character(x)
#'
#' # Preserve metadata:
#' as_character(x, strip_attributes = FALSE)
#'
#' # Base R:
#' as.character(x)
#'
#' @export
as_character <- function(x, ...) {
  UseMethod("as_character", x)
}

#' @rdname as_character
#' @export
as_character.haven_labelled_defined <- function(
  x,
  strip_attributes = TRUE,
  ...
) {
  base <- vctrs::vec_data(x)
  out <- as.character(base)

  if (!strip_attributes) {
    attr(out, "label") <- attr(x, "label")
    attr(out, "unit") <- attr(x, "unit")
    attr(out, "concept") <- attr(x, "concept")
    attr(out, "namespace") <- attr(x, "namespace")
  }

  out
}

#' @rdname as_character
#' @description
#' Base R `as.character()` always strips the class and metadata.
#'
#' @export
as.character.haven_labelled_defined <- function(x, ...) {
  as.character(vctrs::vec_data(x))
}

#' @title Get or Set the Publication Year of a Dataset Object
#'
#' @description
#' Access or assign the optional `publication_year` attribute to a semantically
#' rich dataset object.
#'
#' @details
#' The `publication_year` represents the year when the dataset was or will be
#' made publicly available, in `YYYY` format. For additional context, see
#' [DataCite: Publication Year-Additional Guidance](https://support.datacite.org/docs/datacite-metadata-schema-v44-mandatory-properties#publicationyearadditional-guidance).
#'
#' @param x A dataset object created by [dataset_df()] or
#'   `dataset::as_dataset_df()`.
#' @param value A character string specifying the publication year.
#' @param overwrite Logical. If `TRUE` (default), the existing
#'   `publication_year` attribute is replaced with `value`. If `FALSE`, the
#'   function returns a message and does not overwrite the existing value.
#'
#' @return The `publication_year` attribute as a character string.
#'
#' @examples
#' publication_year(orange_df)
#' publication_year(orange_df) <- "1998"
#'
#' @family bibliographic helper functions
#' @export

publication_year <- function(x) {
  assert_that(is.dataset_df(x),
    msg = "publication_year(x): x must be a dataset object created with dataset() or as_dataset()."
  )

  ds_bibentry <- get_bibentry(x)
  as.character(ds_bibentry$date)
}

#' @rdname publication_year
#' @export
`publication_year<-` <- function(x, overwrite = TRUE, value) {
  assert_that(is.dataset_df(x),
    msg = "publication_year(x) <- value: x must be a dataset object created with dataset_df() or as_dataset_df()."
  )

  ds_bibentry <- get_bibentry(x)
  publication_year <- ds_bibentry$date

  if (is.null(value)) {
    value <- ":unas"
  }

  if (overwrite) {
    ds_bibentry$date <- as.character(value)
    attr(x, "dataset_bibentry") <- ds_bibentry
  } else {
    warning("The dataset has already an publication_year: ", ds_bibentry$date, ".")
  }
  invisible(x)
}

#' @title Get/set the Creator of the object.
#' @description Add the optional `Creator` property as an attribute to a
#'   dataset object.
#' @details The `Creator` corresponds to
#'   [dct:creator](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/creator/)
#'   in Dublin Core and Creator in DataCite. The name of the entity that holds,
#'   archives, publishes prints, distributes, releases, issues, or produces the
#'   dataset. This property will be used to formulate the citation, so consider
#'   the prominence of the role.
#' @param x A semantically rich data frame object created by
#'   [dataset_df()] or `dataset::\link{as_dataset_df}`.
#' @param value The `Creator` as a
#'   [utils::person()] object.
#' @param overwrite If the attributes should be overwritten. In case it is set
#'   to `FALSE`,it gives a message with the current `Creator` property
#'   instead of overwriting it. Defaults to `TRUE` when the attribute is
#'   set to `value` regardless of previous setting.
#' @return The Creator attribute as a character of length one is added to
#'   `x`.
#' @importFrom utils person
#' @importFrom assertthat assert_that
#' @family bibliographic helper functions
#' @examples
#' creator(orange_df)
#' # To change author:
#' creator(orange_df) <- person("Jane", "Doe")
#' # To add author:
#' creator(orange_df, overwrite = FALSE) <- person("John", "Doe")
#' @export
creator <- function(x) {
  assert_that(is.dataset_df(x),
    msg = "creator(x): x must be a dataset object created with dataset() or as_dataset_df()."
  )

  ds_bibentry <- get_bibentry(x)
  ds_bibentry$author
}

#' @rdname creator
#' @export
`creator<-` <- function(x, overwrite = TRUE, value) {
  assert_that(is.dataset_df(x),
    msg = "creator(x) <- value: x must be a dataset object created with dataset_df() or as_dataset_df()."
  )

  if (is.null(value)) {
    return(x)
  }

  if (!inherits(value, "person")) {
    stop("creator(x) <- value: `value` must be a utils::person object.")
  }

  ds_creator <- get_bibentry(x)$author

  if (overwrite) {
    ds_creator <- value
  } else {
    ds_creator <- c(ds_creator, value)
  }

  databibentry <- attr(x, "dataset_bibentry")
  databibentry$author <- ds_creator
  attr(x, "dataset_bibentry") <- databibentry
  invisible(x)
}

#' @keywords internal
compare_creators <- function(x, y) {
  creators_x <- creator(x)
  creators_y <- creator(y)
  all_creators <- c(creators_x, creators_y)

  identity_keys <- vapply(all_creators, function(p) {
    paste(
      p$given %||% "",
      p$family %||% "",
      p$email %||% "",
      p$comment %||% "",
      sep = "|"
    )
  }, character(1))

  unique_keys <- unique(identity_keys)
  merged <- vector("list", length(unique_keys))

  for (i in seq_along(unique_keys)) {
    key <- unique_keys[i]
    matches <- all_creators[identity_keys == key]
    roles <- unique(unlist(lapply(matches, function(p) p$role)))
    base <- matches[[1]]

    merged[[i]] <- person(
      given = base$given,
      family = base$family,
      email = base$email,
      comment = base$comment,
      role = roles
    )
  }

  do.call(c, merged)
}

#' @title Get or set contributors
#'
#' @description `contributor()` is a lightweight wrapper around [creator()] that
#'   works only with contributors. It retrieves or updates only the contributor
#'   entries in the dataset's bibliographic metadata.
#'
#' @details All people are stored in the `author` slot of the underlying
#'   [`utils::bibentry`]. This helper preserves primary creators and filters or
#'   updates only those entries that represent contributors.
#'
#'   A *contributor* is defined as:
#'
#' * a person with `role == "ctb"`, **or**
#' * a person with a `comment[["contributorType"]]`.
#'
#'   Primary creators (authors) typically have `role %in% c("aut", "cre")`.
#'
#'   Contributors can be further annotated with metadata in `comment`, for
#'   example:
#'
#' ```r
#' comment = c(contributorType = "hostingInstitution", ORCID = "0000-0000-0000-0000")
#' ```
#'
#' @param x A dataset object created with [dataset_df()] or [as_dataset_df()].
#' @param value A [`utils::person`] object representing a single contributor. If
#'   the `role` field is missing, it will be set to `"ctb"`. If `NULL`, the
#'   dataset is returned unchanged.
#' @param overwrite Logical. If `TRUE`, replace all existing contributors with
#'   `value`. If `FALSE`, append `value` to the existing contributors. Defaults
#'   to `FALSE`.
#'
#' @return
#' * `contributor()` returns a [`utils::person`] or a list of such objects
#' corresponding to contributors.
#' * `contributor<-()` returns the updated dataset (invisibly).
#'
#' @examples
#' df <- dataset_df(data.frame(x = 1))
#' creator(df) <- person("Jane", "Doe", role = "aut")
#'
#' # Add a contributor
#' contributor(df, overwrite = FALSE) <-
#'   person("GitHub",
#'     role = "ctb",
#'     comment = c(contributorType = "hostingInstitution")
#'   )
#'
#' # Replace all contributors
#' contributor(df) <- person("Support", "Team", role = "ctb")
#'
#' # Inspect only contributors
#' contributor(df)
#'
#' @family bibliographic helper functions
#' @importFrom utils person
#' @export
contributor <- function(x) {
  assertthat::assert_that(
    is.dataset_df(x),
    msg = "contributor(x): x must be a dataset created with dataset_df() or as_dataset_df()."
  )
  auth <- get_bibentry(x)$author
  if (is.null(auth)) {
    return(auth)
  }

  as_list <- as.list(auth)
  is_ctb <- function(p) {
    inherits(p, "person") &&
      (identical(p$role, "ctb") ||
        (!is.null(p$comment) && !is.null(p$comment[["contributorType"]])))
  }
  keep <- vapply(as_list, is_ctb, logical(1))
  if (!any(keep)) {
    return(auth[0])
  }
  utils::as.person(as_list[keep])
}


#' @rdname contributor
#' @export
`contributor<-` <- function(x, overwrite = FALSE, value) {
  assertthat::assert_that(
    is.dataset_df(x),
    msg = "contributor(x) <- value: x must be a dataset_df object."
  )
  if (is.null(value)) {
    return(x)
  }
  if (!inherits(value, "person")) {
    stop("contributor(x) <- value: `value` must be a utils::person object.")
  }
  if (is.null(value$role)) value$role <- "ctb"
  if (!identical(value$role, "ctb")) {
    stop("contributor(x) <- value: role must be 'ctb'.")
  }

  all_people <- get_bibentry(x)$author
  non_ctb <- Filter(function(p) !identical(p$role, "ctb"), as.list(all_people))

  new_auth <- if (overwrite) {
    c(non_ctb, value)
  } else {
    c(all_people, value)
  }

  databibentry <- attr(x, "dataset_bibentry")
  databibentry$author <- utils::as.person(new_auth)
  attr(x, "dataset_bibentry") <- databibentry
  invisible(x)
}

---
title: "An Introduction to the dataset Package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{An Introduction to the dataset Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

if (!requireNamespace("rdflib",
  quietly = TRUE
)) {
  stop("Please install 'rdflib' to run this vignette.")
}
```

## Overview

The `dataset` package extends tidy data with semantic metadata, provenance, and machine-readable definitions.

It supports a gradual workflow from provisional semantic harmonisation with `prelabel()` to formally defined variables with `defined()` and fully described datasets with `dataset_df()`.

This makes datasets easier to exchange, reuse, publish, and serialize to RDF and other FAIR-compliant formats.

This vignette provides a high-level introduction. For details on key components, see:

- `vignette("prelabelled", package = "dataset")`: Handling Semantic Ambiguity with `prelabelled` Vectors.
- `vignette("defined", package = "dataset")`: Semantic vectors with `defined()`
- `vignette("dataset_df", package = "dataset")`: Structuring and metadata with `dataset_df()`
- `vignette("rdf", package = "dataset")`: Exporting to RDF and Linked Data
- `vignette("bibrecord", package = "dataset")`: Creating rich citation metadata using `bibrecord()`

## Why extend tidy data?

Hadley Wickham (2014) defines [tidy data](https://vita.had.co.nz/papers/tidy-data.pdf) with three principles:

- Each variable forms a column
- Each observation forms a row
- Each observational unit forms a table

This structure is ideal for analysis because it links the structure of a dataset with its meaning. A variable represents an underlying attribute, and an observation represents measurements collected on the same unit.

In practice, however, analysts rarely begin with perfectly harmonized data. During data cleaning, transformation, and integration, they make many semantic decisions: resolving inconsistent coding schemes, standardizing categories, selecting units of measurement, or deciding how concepts from different sources correspond to one another. By the time a dataset is ready for analysis, these assumptions are usually clear to the analyst who created it.

The problem arises when the dataset leaves its original context. Other analysts may use different terminology, apply different coding conventions, or simply lack knowledge of the decisions that were made during data preparation. Even the original analyst may find these assumptions difficult to reconstruct months or years later.

The `dataset` package extends tidy data by making such semantic assumptions explicit and preserving them alongside the data. Rather than treating semantic harmonisation and data provenance as undocumented steps in a workflow, it allows them to be recorded incrementally as the dataset evolves.

The goal is not to burden analysts with complex semantic technologies. Instead, the package provides lightweight tools for gradually recording the information needed to review, reuse, audit, publish, and correctly combine datasets across projects, organisations, and time.

## Example: gradual semantic stabilisation

Many data integration problems begin with values that refer to the same concept but use different coding conventions.

```{r predefine}
library(dataset)

country <- prelabel(
  c("AD", "Andorra", "AND", "LI", "Liechtenstein"),
  labels = c(
    Andorra = "AD",
    AND = "AD",
    Liechtenstein = "LI"
  )
)

country
```

The `prelabelled` class records provisional semantic assumptions without requiring a formal semantic definition. In this example, "AD", "Andorra", and "AND" are treated as equivalent representations of the same geopolitical entity.

The current mappings can be inspected directly:

```{r predefine-attribute}
attr(country, "prelabel")
```

This approach is useful during data cleaning and integration, where semantic assumptions may still evolve. Once these assumptions become sufficiently stable, they can be formalized with `defined()`.

For further information, see `vignette("prelabelled", package = "dataset")`: Handling Semantic Ambiguity with `prelabelled` Vectors.

## Example: defining semantically rich vectors

After values have been harmonized, variables can be formally defined with machine-readable semantic metadata.

Semantically rich vectors are vectors in a data.frame that contain richer semantics than a simple column name; a long-form human-readable title; a machine- and human-readable variable definition; and if needed, an external resource that contains the codebook.

```{r definegdpdataset}
library(dataset)

gdp <- defined(
  c(2355, 2592, 2884),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo <- defined(
  rep("AD", 3),
  label = "Geopolitical Entity",
  concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
  namespace = "https://www.geonames.org/countries/$1/"
)

gdp
geo
```

In this case, we define `geo` as the geopolitical entity <http://purl.org/linked-data/sdmx/2009/dimension#refArea>, and we know that the `AD` value can resolve to Andorra: <https://www.geonames.org/countries/AD/>. These vectors now carry metadata you can inspect directly — including their label, unit, and concept URI — which will be preserved even after transformation or storage.

For further information, see vignette("defined", package = "dataset")`: Semantic vectors with`defined()\`.

## Example: creating a dataset from a metadata-enriched data frame

```{r smalldatasetexample}
small_dataset <- dataset_df(
  geo = geo,
  gdp = gdp,
  identifier = c(gdp = "http://example.com/dataset#gdp"),
  dataset_bibentry = dublincore(
    title = "Small GDP Dataset",
    creator = person("Jane", "Doe", role = "aut"),
    publisher = "Small Repository",
    subject = "Gross Domestic Product"
  )
)

small_dataset
```

For further information see `vignette("dataset_df", package = "dataset")`: Structuring and metadata with `dataset_df()`.

This dataset not only stores the variables and values, but also includes embedded metadata that supports precise interpretation and repository-level publication.

```{r dublincoremetadata}
as_dublincore(small_dataset)
```

For further information see`vignette("bibrecord", package = "dataset")`: Creating rich citation metadata using `bibrecord()`

## Exporting to RDF

As Carl Boettinger has shown in the vignettes accompanying the R-binding to the popular Python library [rdflib](https://CRAN.R-project.org/package=rdflib), (see: [A tidyverse lover's intro to RDF](https://docs.ropensci.org/rdflib/articles/rdf_intro.html)), tidy datasets can be retrofitted with rich metadata if they are pivoted to a strictly three-column long format.

Our packages tries to lower the burden of such retrofitting with early binding and sensible defaults to serialise the dataset's contents and the dataset's bibliographic data to this format for those who are not familiar with RDF.

You can convert any `dataset_df` object into a tidy 3-column representation (subject–predicate–object) using `dataset_to_triples()`:

```{r triplesexample}
triples <- dataset_to_triples(small_dataset,
  format = "nt"
)
triples
```

This 3-column format (subject–predicate–object) is compatible with semantic web tools such as SPARQL, `rdflib`, and triple stores.

```{r ntexample}
mycon <- tempfile("my_dataset",
  fileext = "nt"
)
my_description <- describe(
  x = small_dataset,
  con = mycon
)

# Only three statements are shown:
readLines(mycon)[c(4, 8, 12)]
```

```{r provenancexample}
## Show two lines of provenance:
provenance(small_dataset)[c(6, 7)]
```

For further information, see `vignette("rdf", package = "dataset")`: Exporting to RDF and Linked Data.

## Coercing back

There may be use cases when your richer dataset needs to be simplified to as base R `data.frame` or a `tbf_df`.

We offer two coercion forms:

```{r smalldf}
small_df <- as.data.frame(small_dataset,
  strip_attributes = FALSE
)

attr(small_dataset, "subject")
```

Using the `strip_attributes = FALSE` the rich attributes remain in the base R data.frame. In most pipelines the attributes play no role, and you can retain it, and perhaps later load it back to a richer form.

You can also strip all these attributes, and choose `tbl_df` (if you have `tibble`) installed":

```{r smalltbl}
small_tbl <- as_tibble(
  small_dataset,
  strip_attributes = TRUE
)

small_tbl
```

## Summary

The *dataset* package enriches tidy data by attaching metadata from the start of the workflow. It helps avoid semantic mismatches, supports RDF publication, and meets interoperability standards like SDMX, DataCite, and Dublin Core. Use it when you need:

- Meaningful variable descriptions and URIs

- Dataset-level metadata embedded directly in .rds or .rda files

- Easy export to RDF and semantic web formats.

---
title: "Handling Semantic Ambiguity with prelabelled Vectors"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Handling Semantic Ambiguity with prelabelled Vectors}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

You are developing a family of R packages that extend tidy data workflows with richer semantic and provenance-aware capabilities. The work began from practical experience building tidyverse-based data pipelines and repeatedly encountering the same limitation: while tidy datasets are highly efficient and semantically clear within a given workflow, much of their meaning remains implicit and dependent on the contextual knowledge of their creator. Once exported, serialized, or transferred across environments, this contextual information is often lost. :contentReference[oaicite:0]{index="0"}

```{r setup}
library(dataset)
```

The `dataset` package introduces semantically enriched vectors and data frames that preserve explicit metadata throughout the workflow lifecycle. However, fully formal semantic annotation is verbose and cognitively demanding. Constructing semantically complete RDF-compatible objects is appropriate only for mature stages of a workflow.

In practice, semantic stabilization is usually incremental. Observational data often arrive with partially inconsistent, incomplete, or ambiguous labels. Before a variable can mature into a formally defined vector created with `labelled::labelled()` or `dataset::defined()`, analysts typically perform several rounds of semantic harmonisation.

The `prelabelled` class supports this intermediate stage.

Unlike formally defined semantic vectors, `prelabelled` vectors tolerate:

- incomplete semantic mappings;
- unresolved observational values;
- mixed coding conventions;
- gradual semantic stabilization.

This vignette demonstrates how provisional semantic assertions can be incrementally stabilised while preserving the original observational evidence.

## A small ambiguous dataset

We begin with a small dataset containing country observations. The dataset is intentionally inconsistent: some observations use full country names, while others already use ISO 3166 alpha-2 country codes.

Such ambiguity is extremely common in operational analytical workflows, particularly when datasets are merged from multiple sources or manually curated over time.

```{r countrydata1}
country_data_1 <- data.frame(
  country = c("Andorra", "LI", "San Marino", "AD", "Liechtenstein"),
  time = c(2020, 2020, 2020, 2021, 2021),
  value = c(1.2, 2.4, 3.1, 1.3, 2.5)
  )
```

### Creating provisional semantic assertions

We now create a lightweight semantic mapping.

The goal is not yet to create a formally closed semantic vocabulary. Instead, we begin stabilising the semantics incrementally by mapping some observational values to candidate semantic assertions.

Values that are not explicitly mapped remain self-describing.

```{r countrymap1}
country_map <- c(
  "Andorra" = "AD",
  "Liechtenstein" = "LI",
  "San Marino" = "SM"
)

country_data_1$country <-
  prelabel(
    country_data_1$country,
    labels = country_map
  )
```

### Inspecting the prelabelled vector

The resulting vector preserves the original observational values while attaching a provisional semantic vocabulary in the `"prelabel"` attribute.

```{r printcountrydata1}
print(country_data_1$country)
```

This separation between:

- observational evidence;
- semantic interpretation;

is a central design principle of the `prelabelled` class.

The observational values remain unchanged, while semantic operationalisation may evolve iteratively over time.

### Semantic operationalisation

Using `as.character()` operationalises the semantic assertions into a semantically stabilised character vector.

```{r countrydata2}
country_data_2 <- data.frame(
  country = as.character(country_data_1$country),
  time = country_data_1$time,
  value = country_data_1$value
)

country_data_2
```

Mapped observations are converted into their candidate semantic assertions, while unmatched values remain self-describing.

This allows analysts to gradually reduce semantic ambiguity without destroying the original observational evidence.

## A more ambiguous dataset

The next dataset contains a more difficult form of semantic ambiguity.

Some observations use ISO 3166 alpha-2 country codes, while others use ISO 3166 alpha-3 codes or full country names. Although the observations are semantically related, they do not yet form a stable closed vocabulary.

```{r countrydata3}
country_data_3 <- data.frame(
  country = c(
    "AD", "AND", "LI", "LIE", "SMR", "San Marino"
  ),
  time = c(2020, 2020, 2020, 2021, 2021, 2021),
  value = c(1, 2, 3, 4, 5, 6)
)
```

## Incremental semantic stabilization

The `prelabelled` workflow does not require complete semantic resolution from the outset.

Instead, semantic stabilization can proceed incrementally:

- observational ambiguities become explicit;
- partial semantic mappings accumulate gradually;
- unresolved values remain operationally usable;
- semantic assertions become progressively more stable.

```{r countrymap3}
country_map_3 <- c(
  "Andorra" = "AD",
  "Andorra" = "AND",
  "Liechtenstein" = "LI",
  "San Marino" = "SM",
  "San Marino" = "SMR"
)

prelabelled_country <- prelabel(
  country_data_3$country,
  labels = country_map_3
)
```

This approach is particularly useful in exploratory analytical workflows, archival reconstruction, metadata harmonisation, and cross-dataset integration tasks.

```{r}
prelabelled_country
```

### Semantic workspaces

While `as.character()` provides lightweight semantic coercion, which may be more useful after semantic stabilisation.

```{r}
as.character(prelabelled_country)
```

The `as_character()` method creates a provenance-preserving semantic workspace.

```{r}
as_character(prelabelled_country)
```

The resulting vector retains:

- the original observational values;
- the provisional semantic vocabulary;
- additional semantic attributes.

This allows analysts to continue semantic refinement workflows while preserving reversibility and provenance awareness.

### From provisional semantics to formally defined semantics

The goal of `prelabelled` vectors is not to replace formally defined semantic vectors.

Instead, they provide a lightweight preparatory stage for incremental semantic stabilization.

Once semantic ambiguity has been sufficiently reduced, `prelabelled` vectors may mature into formally defined semantic vectors created with `labelled::labelled()` or `dataset::defined()`. For further information, see `vignette("defined", package = "dataset")`- Working with semantic vectors: Semantic vectors with `defined()`.

In this sense, semantic enrichment becomes an iterative analytical workflow rather than a single terminal annotation step.

---
title: "Example Dataset Definitions"
output:
  rmarkdown::html_vignette:
    md_extensions: -autolink_bare_uris
vignette: >
  %\VignetteIndexEntry{Example Dataset Definitions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<!-- Machine-readable alternate -->
<link rel="alternate" type="text/turtle" href="https://dataset.dataobservatory.eu/examples/dataset.ttl" />

This page documents the illustrative IRIs used in the `dataset` package
vignettes. They are **examples only** and do not point to real datasets. 

## Purpose

- To illustrate how IRIs can uniquely identify dataset elements.  
- To provide dereferenceable links that pass CRAN checks.  
- To show Linked Data best practice: one namespace doc, with fragments. 

# Example Dataset Definitions For RDF

The base namespace in `vignette("rdf", package = "dataset")` is:

<https://dataset.dataobservatory.eu/examples/dataset.html#>

Although these example IRIs use `.html#` in the base namespace, the `.html` part
does not imply that the resource is literally an HTML file. In RDF, any absolute
IRI is valid. We use this form only for illustration, because the R package
documentation infrastructure is not designed to support full content
negotiation between human-readable HTML and machine-readable Turtle files.  
In real publishing scenarios, a bare namespace ending with `/dataset#` is more
conventional.

---

<a id="obs:1"></a>

## obs:1

Example observation 1 (not real data).  


<a id="obs:2"></a>

## obs:2  

Example observation 2 (not real data).  

<a id="obs:3"></a>

## obs:3  
Example observation 3 (not real data).  

---

In real publishing scenarios, you would replace these with persistent URIs
that identify actual datasets and their observations. For example, a DOI-based identifier such as:

`https://doi.org/10.5281/zenodo.14917851#obs:1`

could be used to refer to the first observation in a dataset archived at Zenodo. 

## Machine-readable form

A Turtle serialization of the same example definitions is also available: 


```
@prefix ex:   <https://dataset.dataobservatory.eu/examples/dataset#> .
@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
@prefix dcat: <http://www.w3.org/ns/dcat#> .

ex:obs1 a dcat:Observation ;
    rdfs:label "Example observation 1 (not real data)" .

ex:obs2 a dcat:Observation ;
    rdfs:label "Example observation 2 (not real data)" .

ex:obs3 a dcat:Observation ;
    rdfs:label "Example observation 3 (not real data)" .
```

In fact, semantic applications know that they should not even read this HTML page, but instead go directly to the `dataset.ttl` version, as it is a World Wide Web standard to disseminate data and metadata.

👉 [Download dataset.ttl](https://dataset.dataobservatory.eu/examples/dataset.ttl)  

---
title: "dataset_df: Create Datasets that are Easy to Share Exchange and Extend"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{dataset_df: Create Datasets that are Easy to Share Exchange and Extend}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setupvignette, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The `dataset` package extends R’s native data structures with machine-readable metadata. It follows a *semantic early-binding* approach: metadata is embedded as soon as the data is created, making datasets suitable for long-term reuse, FAIR-compliant publishing, and integration into semantic web systems.

In R, a `data.frame` is defined as a tightly coupled collection of variables that share many of the properties of matrices and lists, and it serves as the fundamental data structure for most of R’s modeling software. Users of the R ecosystem often use the term *data frame* interchangeably with *dataset*. However, the standards used in libraries, repositories, and statistical systems for publishing, exchanging, and reusing datasets require metadata that even “tidy” data frames do not provide.

This vignette introduces the `dataset_df` class and the `dataset_df()` constructor, which extend tidy data frames with a semantic layer. For details on semantically enriched vectors, see `vignette("defined", package = "dataset")`. Readers interested in the underlying ISO and W3C definitions of *dataset* will find them discussed in `vignette("design", package = "dataset")`.

## Purpose

The `dataset_df()` function helps you create **semantically rich datasets** that meet the interoperability, exchange, and reuse requirements of libraries, repositories, and statistical systems. It defines a new S3 class, inherited from the modernised data frame of `tibble::tibble()`, that retains compatibility with existing workflows but is easier to:

-   understand by humans,\
-   validate and process by machines,\
-   deposit, exchange, and publish,\
-   share across tools, teams, and domains.

This vignette walks you through creating such a dataset using a subset of the *GDP and main aggregates – international data cooperation annual data* dataset from Eurostat\
(DOI: [https://doi.org/10.2908/NAIDA_10_GDP).](https://doi.org/10.2908/NAIDA_10_GDP).)

## Load example data

```{r loaddata}
library(dataset)
data("gdp")
```

```{r printgdp}
print(gdp)
```

This example dataset is already in tidy format: each row represents a single observation for a country and year, and each column is a variable. `dataset_df` builds on this structure by adding semantic information to the variables and the dataset itself, ensuring that both the shape and the meaning of the data are preserved and unambiguous.

While the raw dataset represented in the `gdp` data.frame is valid and tidy, it can be hard to interpret without external documentation. For example:

-   Countries are encoded in the `geo` variable.

-   Reporting frequency (e.g., `A` for annual) is stored in `freq`.

## Add metadata to your dataset

The `dataset_df()` constructor enables two levels of semantic annotation for a `tbl_df` object:

-   **Variable-level metadata** — label, unit, definition, namespace.

-   **Dataset-level metadata** — title, author, license, description.

-   Let’s create a smaller dataset and enrich it with metadata.

Let’s create a semantically enriched subset:

```{r createdataasetdf}
small_country_dataset <- dataset_df(
  geo = defined(
    gdp$geo,
    label = "Country name",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/geo/$1"
  ),
  year = defined(
    gdp$year,
    label = "Reference Period (Year)",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refPeriod"
  ),
  gdp = defined(
    gdp$gdp,
    label = "Gross Domestic Product",
    unit = "CP_MEUR",
    concept = "http://data.europa.eu/83i/aa/GDP"
  ),
  unit = defined(
    gdp$unit,
    label = "Unit of Measure",
    concept = "http://purl.org/linked-data/sdmx/2009/attribute#unitMeasure",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/unit/$1"
  ),
  freq = defined(
    gdp$freq,
    label = "Frequency",
    concept = "http://purl.org/linked-data/sdmx/2009/code"
  ),
  dataset_bibentry = dublincore(
    title = "Small Country Dataset",
    creator = person("Jane", "Doe"),
    publisher = "Example Inc.",
    datasource = "https://doi.org/10.2908/NAIDA_10_GDP",
    rights = "CC-BY",
    coverage = "Andorra, Liechtenstein, San Marino and the Feroe Islands"
  )
)
```

## Inspecting variable-level metadata

Columns created with the `defined` class store semantic information such as the label, the concept’s definition link, and the unit of measure.

Check the variable label:

```{r varlabel}
var_label(small_country_dataset$gdp)
```

And the measure of unit:

```{r varunit}
var_unit(small_country_dataset$gdp)
```

## Adding dataset-level metadata

A `dataset_df()` object can also store metadata describing the dataset as a whole. This metadata follows widely adopted standards:

-   Dublin Core Terms (`dublincore()`), used in libraries and data repositories.
-   DataCite (`datacite()`), commonly used in research data repositories.

Each metadata field can be accessed or modified using simple assignment functions. For example, you can set the dataset language.

```{r language}
language(small_country_dataset) <- "en"
```

## Reviewing dataset-level metadata

To see the complete dataset description, you can print it as a BibTeX-style entry, which is suitable for citation or export.

```{r bibentry}
print(get_bibentry(small_country_dataset), "bibtex")
```

This prints a complete BibTeX-style entry, suitable for citation or export.

## Joining datasets

The previous dataset contains observations for three data subjects — Andorra, Liechtenstein, and San Marino — but does not include the Feroe Islands.

```{r feroedf}
feroe_df <- data.frame(
  geo = rep("FO", 3),
  year = 2020:2022,
  gdp = c(2523.6, 2725.8, 3013.2),
  unit = rep("CP_MEUR", 3),
  freq = rep("A", 3)
)
```

The `dataset_df` class does not allow binding two datasets directly unless their concept definitions, units of measure, and URI namespaces match.

```{r notevaluatedrbind, eval=FALSE}
rbind(small_country_dataset, feroe_df)
```

```         
Error in rbind(deparse.level, ...) :
numbers of columns of arguments do not match
```

While this constraint can feel restrictive during an analysis workflow, it ensures semantic consistency when the data is later published or exchanged.

This is similar in spirit to tidy data principles: when combining datasets, both structure and meaning must align. In `dataset_df`, the tidy data rule that “variables are columns” is complemented by the requirement that variables with the same name also share the same definition, units, and concept references.

To add the missing Feroe Islands data, first create a compatible dataset using the same definitions, country coding, and units of measure as the original.

```{r fereodataset}
feroe_dataset <- dataset_df(
  geo = defined(
    feroe_df$geo,
    label = "Country name",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/geo/$1"
  ),
  year = defined(
    feroe_df$year,
    label = "Reference Period (Year)",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refPeriod"
  ),
  gdp = defined(
    feroe_df$gdp,
    label = "Gross Domestic Product",
    unit = "CP_MEUR",
    concept = "http://data.europa.eu/83i/aa/GDP"
  ),
  unit = defined(
    feroe_df$unit,
    label = "Unit of Measure",
    concept = "http://purl.org/linked-data/sdmx/2009/attribute#unitMeasure",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/unit/$1"
  ),
  freq = defined(
    feroe_df$freq,
    label = "Frequency",
    concept = "http://purl.org/linked-data/sdmx/2009/code"
  )
)
```

Once the new dataset is defined in this way, you can combine it with the existing one using `bind_defined_rows()`.

```{r binddefinedrows}
joined_dataset <- bind_defined_rows(small_country_dataset, feroe_dataset)
joined_dataset
```

The combined dataset behaves like a regular tibble but retains its metadata. If you convert it to a base R `data.frame`, you will lose the helper methods and built-in checks, but the metadata will remain in the object’s attributes.

```{r backwardcompatibility}
attributes(as.data.frame(joined_dataset))
```

### Coercion inside `dataset_df` conversion

When converting a `dataset_df` to a base `data.frame`:

```{r coercion}
as.data.frame(small_country_dataset)
```

the following rules apply:

-   **By default**, all semantic attributes are stripped (`label`, `unit`, `concept`, `namespace`).

-   Categorical variables (with labels) become **character vectors**.

-   Numeric `defined()` vectors become **numeric**.

-   Date / POSIXct vectors keep their underlying date/time values.

-   The `haven_labelled_defined` class is always removed.

-   Dataset-level metadata (`dataset_bibentry`, `prov`, `subject`) is preserved.

-   You can retain semantic attributes via:

```{r richdataframe}
as.data.frame(small_country_dataset,
  strip_attributes = FALSE
)
```

`as_tibble()` behaves the same way as `as.data.frame()` but returns a tibble:

```{r astibble}
as_tibble(orange_df)
```


## Conclusion

With `dataset_df()` your datasets are:

-   **Self-descriptive** — variables carry labels, units, and definitions.
-   **Machine-readable** — linked vocabularies and standard identifiers are embedded.
-   **Ready to publish and share** — compliant with metadata standards like Dublin Core and DataCite.

This approach supports the FAIR data principles (Findable, Accessible, Interoperable, Reusable) and makes your data easier to reuse, interpret, and validate. By maintaining metadata from creation through publication, `dataset_df` helps preserve meaning across the entire data lifecycle.

The package is designed to work seamlessly with the rOpenSci [rdflib](https://github.com/ropensci/rdflib) package and complements tidyverse workflows while enabling exports to semantic web formats like RDF.

---
title: "Design Principles & Future Work Semantically Enriched, Standards-Aligned Datasets in R"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{Design Principles & Future Work Semantically Enriched, Standards-Aligned Datasets in R}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 80
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Abstract

```{r setup}
library(dataset)
```

> “A dataset is an identifiable collection of data available for access or
> download in one or more formats.” — ISO/IEC 20546

The `dataset` package enriches R data objects with machine-readable metadata by
embedding semantic definitions and provenance at both the variable and dataset
levels. It follows a semantic **early-binding** design: metadata is attached at
creation time, not retrofitted post hoc. This ensures that meaning and context
are preserved throughout the data lifecycle — from exploration to publication —
and enables partial automation of documentation.

This article outlines the design philosophy behind the `dataset` package,
including its theoretical foundations, structure, relationship to other R tools,
and example workflows. It serves as a long-form complement to the package
vignettes.

## Introduction and Motivation

> “The principles of tidy data provide a standard way to organise data values
> within a dataset.”\
> — Wickham (2014)

The `dataset` package extends R’s native data structures by embedding
machine-readable semantics and provenance directly in tidy data objects. It
builds on tidy data principles (Wickham, 2014) but introduces a **semantic
early-binding** approach: metadata is attached when the dataset is created,
ensuring that context and meaning are preserved through all stages of the
workflow — including transformation, validation, serialization, and reuse.

While tidyverse tools enforce structural clarity, they are generally agnostic
about semantics. Variables may be misinterpreted, joined incorrectly, or
published without context. `dataset` addresses this gap by aligning with
international metadata standards, supporting RDF export, and providing an
interface to the W3C Data Cube model.

A tidy dataset, per Wickham’s definition, adheres to three core rules:

- Each variable forms a column\
- Each observation forms a row\
- Each value forms a cell

![Figure 12.1 in R for Data Science: tidy data structure.](images/tidy-1.png)

However, this tidy structure — typically implemented as a `data.frame` or
`tibble` — is not semantically self-describing. In practical workflows, users
often conflate the in-memory structure with the abstract concept of a dataset,
which in metadata terms refers not just to structure but also to definitions,
units, provenance, and contributors.

Several ISO and W3C standards define what constitutes a dataset. According to
ISO/IEC 20546, a dataset is an identifiable collection of data available for
access or download in one or more formats. The Dublin Core DCMI Metadata Terms
define a
[dataset](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/dcmitype/Dataset/)
as “data encoded in a defined structure.” The W3C’s [Data Cube
Vocabulary](https://www.w3.org/TR/vocab-data-cube/#cubes-model-datasets), widely
used in official statistics, describes a dataset as a “collection of statistical
data that corresponds to a defined structure.” That structure includes
observations, metadata about their organisation, *structural metadata* (e.g.,
units of measure), and *reference metadata* (e.g., creator, publisher).

This differs from R’s `data.frame` object, which is defined as “tightly coupled
collections of variables which share many of the properties of matrices and of
lists, used as the fundamental data structure by most of R's modeling software.”
In practice, R users often use the terms *data frame* (or tibble) and *dataset*
interchangeably. However, even a tidy data frame is underspecified for use in
scientific repositories, statistical data exchanges, or many database
applications. A `data.frame` exists only in the memory of an R session, limiting
its interoperability and reusability. While R can already serialise data frames
to formats like `.rds`, `.rda`, or `.csv`, these serialisations by default lack
rich, standardised metadata. The `dataset` package bridges that gap by aligning
with established metadata standards, producing serialisations that are easier to
reuse and interpret.

The `dataset` package extends R's native data structures with machine-readable
metadata. It follows a *semantic early-binding* approach: metadata is embedded
as soon as the data is created, making datasets suitable for long-term reuse,
FAIR-compliant publishing, and integration into semantic web systems.

The central innovation of the package is an extended data-frame-like object: a
`tibble::tibble()` enhanced with R’s `attributes()` system to store standard
metadata from ISO and W3C vocabularies. This `dataset_df` class integrates
naturally with tidy data principles (Wickham, 2014), where each variable is a
column, each observation is a row, and each type of observational unit forms a
table. On top of this tidy structure, `dataset_df` adds a semantic layer so that
the meaning of variables and datasets is explicit and machine-readable. This new
class is introduced in \`vignette("dataset_df", package = "dataset").

In research or institutional contexts, a dataset is a form of digital resource,
often archived, cited, or published. Such resources are typically described with
metadata using the Resource Description Framework (RDF), enabling
machine-actionable, language-independent, schema-neutral representation. Our aim
is to facilitate the translation or annotation of a tidy R data.frame into such
a resource.

RDF also enables description at the level of **elementary statements** — that
is, per-cell metadata combining variable (column) and observation (row). This
allows for fine-grained semantic annotation, supporting full data traceability
and interoperability.

The original tidy workflow was designed for solo, interactive analysis where
analysts had full context. But in collaborative, institutional, or
public-sharing contexts, assumptions must be replaced with formal semantics. Not
only structure, but also clear definitions — of units, classifications, codes,
and contributors — become essential.

Moreover, many statistical data providers follow the **data cube model**, which
resembles tidy data but supports higher dimensionality and more formal metadata.
Examples include SDMX and the W3C Data Cube vocabulary.

Tidy data assumes that column names and structure are sufficient for clarity. However, ambiguity arises quickly when combining datasets from heterogeneous sources. A column named `geo` might contain ISO codes in one dataset and
Eurostat codes in another. GDP figures may differ in currency or base year. These inconsistencies often go unnoticed until late-stage analytical errors.

Real analytical workflows rarely begin with fully stabilized
semantics. Datasets may contain inconsistent labels,
abbreviations, or partially harmonised coding systems.

The `prelabel()` constructor supports lightweight semantic
stabilization by attaching provisional semantic mappings while
preserving original observational values. Once semantic
assumptions become sufficiently stable, variables may be
formalized with `defined()` and assembled into `dataset_df()`
objects.

For example:

```{r dataframe}
data.frame(
  geo = c(
    "LI", "Liechtenstein", "SM", "SMR"
  ),
  year = c(2024, 2025, 2024, 2025),
  CPI = c(
    "0.8", "0.8", "0.9", "0.9"
  ),
  GNI = c(
    "8976", "8976", "9672", "9672"
  )
)
```
This dataset is tidy, but not semantically stable. The `geo`
column mixes country names with multiple coding systems:
ISO-3166 alpha-2 (`LI`, `SM`) and ISO-3166 alpha-3 (`SMR`).

Real analytical workflows rarely begin with fully harmonised
semantics. Datasets may contain inconsistent labels,
abbreviations, or partially reconciled coding systems.

The `prelabel()` constructor supports lightweight semantic
stabilization by attaching provisional semantic mappings while
preserving the original observational values:

```{r}
geo <- prelabel(
  c("LI", "Liechtenstein", "SM", "SMR"),
  labels = c(
    Liechtenstein = "LI", SMR = "SM"
  )
)

as.character(geo)
```
Once semantic assumptions become sufficiently stable, vectors
may be formalized with `defined()` and assembled into
`dataset_df()` objects with explicit units, namespaces,
provenance, and machine-readable metadata.

This approach extends tidy data workflows with semantic and
provenance-aware structures while remaining compatible with
ordinary tidyverse operations. Because metadata is attached
workflows.

## Related Work

Several R packages have offered tools to improve the metadata management of
datasets within the tidyverse ecosystem or its surrounding statistical
traditions.

The `labelled` class in the `labelled` and `haven` packages supports long-form
variable labels and improved handling of value label sets compared to base R’s
`factor` class. This is particularly helpful for variables collected in survey
instruments — a major source of microdata in statistical workflows. However,
real statistical production, as standardized by GSIM (Generic Statistical
Information Model) and DDI (Data Documentation Initiative), involves a far more
complex metadata model. Our contribution builds on these efforts by enabling
users to attach standardized, cross-domain codebook references to such
variables, ensuring that labelling follows recognized metadata vocabularies.

The `dataspice` package allows users to create auxiliary metadata datasets for
publication. Its strength lies in its simplicity: it uses human-readable CSV
files to capture key metadata fields. However, this simplicity introduces
fragility: the metadata may become detached, outdated, or unsynchronised from
the data file, especially in collaborative or iterative settings.

The `rdflib` package, a high-level interface to the RDF library of the same name
in Python, supports RDF serialization and querying. It allows tidy data to be
mapped to RDF triples and serialized into N-Triples, Turtle, RDF/XML, or
JSON-LD. However, it assumes that metadata is retrofitted — applied after the
analytical workflow is complete. While `rdflib` is essential for
interoperability, it requires users to leave the tidyverse workflow or gain RDF
modelling expertise.

Our goal with the `dataset` package is to bridge the semantic and methodological
gap between the `tidyverse` and `rdflib`: to make semantically annotated,
publication-ready datasets part of the R-native workflow from the start.

Another important initiative is the **Frictionless Data** project, which
provides lightweight standards for describing datasets (e.g., via
`datapackage.json`). It enables platform-independent validation and metadata
exchange. In R, the `frictionless` package supports reading, writing, and
validating data packages. However, this system relies heavily on external JSON
schemas and does not integrate metadata into the objects used during analysis —
meaning users must juggle separate metadata files and validation steps,
potentially losing semantic continuity during transformation.

Tierney and Cook (2020), in their paper *Expanding Tidy Data Principles to
Facilitate Missing Data Exploration, Visualization and Assessment of
Imputations*, demonstrate how tidy data can be extended through consistent
metadata structures and function design. Their concept of “nabular data” —
datasets with shadow columns representing missingness — shows how tidy workflows
can accommodate new dimensions of metadata. While their focus is on handling
missing data, their methodological framing is closely aligned with ours:
extending tidy conventions by designing new object classes and verbs, rather
than retrofitting external metadata after the fact.

Collectively, these tools highlight a shared recognition: tidy data principles
offer a strong foundation, but do not, by themselves, guarantee semantic
clarity, interoperability, or reuse. The `dataset` package responds to this need
by embedding standardized metadata directly in R objects, enabling datasets to
remain semantically intact throughout transformation, validation, and
publication.

## Design Principles

- **Early binding of semantics**: Metadata is attached at the point of dataset
  creation, not after the fact.
- **Attribute-based, not schema-based**: Metadata lives inside the R object
  itself, not in external schemas or files.
- **Minimal friction with tidyverse workflows**: Compatible with `dplyr`,
  `tidyr`, `vctrs`, and coercible to `tibble` or `data.frame`.
- **Persistence across save/load cycles**: Metadata survives R serialization
  (`.rds`, `.rda`).
- **Tidyverse-like grammar**: Core verbs include `defined()`, `dataset_df()`,
  `provenance()`, `describe()`, `datacite()`, and `dublincore()`.
- **Full interoperability outside R**: Datasets can be exported as RDF using
  `dataset_to_triples()` and ingested into triple stores.

### Semantic Early Binding

The `dataset` package introduces several new S3 classes that remain fully
compatible with tidyverse idioms and largely interoperable with base R. These
classes rely on R's native attribute system to embed metadata directly within
vectors and tibbles. This enables metadata such as labels, concept URIs,
namespaces, and provenance details to persist during filtering, joining, or
transformation.

The attribute system in R is underused, and most user-friendly packages offer
little support or interface for working directly with object attributes. This
leads to redundancy — with metadata often duplicated within the dataset content
itself.

The `defined()` constructor builds on `labelled::labelled` (originally from
`haven`) and provides a more expressive way to annotate vectors with:

- A human-readable label (e.g., `"Gross Domestic Product"`)
- A unit or measurement system (e.g., `"CP_MEUR"`), accessible via `var_unit()`
  and set with `var_unit() <-`
- A concept URI that uniquely identifies the variable or dimension, handled via
  `var_concept()` and assignment
- A namespace URI pattern for resolving coded values (e.g., ISO or Eurostat
  country codes), via `var_namespace()`

The `dataset_df()` class extends `tibble` and supports combining enriched
vectors with dataset-level metadata. This includes Dublin Core and DataCite
elements such as title, creator, publisher, subject, and contributors, along
with provenance metadata like creation time or software agent.

### Attribute-Based, Not Schema-Based

The `dataset` package adopts an attribute-based design rather than a
schema-based approach. Metadata is stored directly in R objects using native
attributes, ensuring semantic annotations remain tightly coupled with the data
throughout transformation, saving, and reuse.

This approach eliminates the need for separate schema definitions or JSON
metadata files — lowering the barrier to semantic data publishing within R
workflows.

In R, most objects (especially vectors and data frames) can carry attributes
such as:

- `names`
- `class`
- `label`
- `unit`
- `concept`
- `namespace`

These are lightweight, internal, and flexible. For example:

```{r attributes}
x <- 2457
attr(x, "unit") <- "CP_MEUR"
attr(x, "concept") <- "http://data.europa.eu/83i/aa/GDP"
```

In the dataset package, this metadata is preserved in defined and dataset_df
objects and moves with the data — whether it's saved, joined, subsetted, or
filtered.

By contrast, many CRAN or rOpenSci packages are schema-based: they require
external metadata definitions that describe expected columns, data types, and
semantic rules. While these can support more complex use cases — such as SDMX
structural metadata or JSON Schema validation — they introduce additional
overhead, increase complexity, and risk desynchronisation between data and
metadata.

Schema-based solutions may be more appropriate when data analysts work in teams
alongside research data managers or other documentation specialists. In
contrast, the `dataset` package is designed for individual researchers or small
teams who want to avoid semantic errors when ingesting new data from external
sources — while also enabling standards-compliant data exchange and publication
with minimal additional tooling.

## Persistence Across Save/Load Cycles

Because all metadata is stored as object attributes, it remains intact when
datasets are saved using native R serialization formats like .rds or .rda. These
attributes can be queried, extracted, or exported — but they do not interfere
with regular data manipulation or analysis.

Metadata is added at the time of object creation, in contrast to workflows where
metadata is generated after analysis or stored in sidecar files (e.g., JSON-LD).
This design reduces the risk of metadata being detached, outdated, or
incomplete.

## Base Examples: Using the `dataset` Grammar

This section demonstrates the core grammar of the `dataset` package using
minimal, synthetic examples. These illustrate how to define semantically
enriched vectors, assemble them into annotated datasets, and prepare them for
RDF export or validation.

### Creating Defined Vectors

The `defined()` constructor creates semantically enriched vectors. It extends
`labelled::labelled` with additional attributes such as `unit`, `concept`, and
`namespace`.

```{r}
library(dataset)

gdp <- defined(
  c(2355, 2592, 2884),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo <- defined(
  rep("AD", 3),
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)
```

These vectors behave like regular R vectors but carry internal metadata. This
metadata can be retrieved or reassigned using the accessor and setter functions
provided by the package:

```{r}
var_concept(gdp)
var_unit(gdp)
var_namespace(geo)
```

These attributes are preserved across most data transformations, and persist
when saving to `.rds` or `.rda`.

### Assembling a Dataset with Metadata

Use `dataset_df()` to combine defined vectors into a tibble-like object that
includes dataset-level metadata, such as bibliographic information, identifiers,
and provenance.

```{r smalldataset}
small_dataset <- dataset_df(
  geo = geo,
  gdp = gdp,
  identifier = c(gdp = "http://example.com/dataset#gdp"),
  dataset_bibentry = dublincore(
    title = "Small GDP Dataset",
    creator = person("Jane", "Doe", role = "aut"),
    publisher = "Small Repository",
    subject = "Gross Domestic Product"
  )
)
```

Behind the scenes, the package uses a custom bibrecord class that extends
`utils::bibentry()` to accommodate all metadata fields defined by Dublin Core
and DataCite — two major standards used in repositories, library systems, and
FAIR data infrastructures.

You can review the dataset-level metadata in both formats:

```{r bibrecord}
as_dublincore(small_dataset)
as_datacite(small_dataset)
```

Since these metadata models do not fully overlap, using `dublincore()` will
leave some DataCite-specific fields empty.

### Provenance Tracking

One benefit of early metadata binding is that basic provenance is automatically
tracked. The `provenance()` function returns metadata about when and how the
dataset was created — including the system time and, optionally, the software
environment.

```{r provenance}
provenance(small_dataset)
```

This provenance is also included in the machine-readable metadata that can be
exported using `describe()`, which generates an RDF description of the dataset.

```{r description}
description_nt <- tempfile(pattern = "small_dataset", fileext = ".nt")
describe(small_dataset, description_nt)

# Only a few lines shown:
readLines(description_nt)[5:8]
```

The dataset grammar provides a lightweight but standards-compliant way to attach
metadata during the creation of R objects. Unlike retrofitted metadata tools, it
keeps semantic annotations inside the object throughout filtering, saving, and
publishing. In the next section, we apply this grammar to a real-world scenario
involving statistical datasets with conflicting semantics.

## Applied Example: Joining Data with Semantic Constraints

This example demonstrates how the `dataset` package helps avoid semantic errors
when combining data from heterogeneous sources. We create a small GDP dataset
for three European microstates, measured in millions of euros (CP_MEUR), and
then attempt to append an observation from Tuvalu, measured in US dollars (USD).
The semantic mismatch triggers an error.

### Step 1: Create a Eurostat-Compatible GDP Dataset

```{r eurogpd}
euro_gdp <- defined(
  c(2355, 2592),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo_europe <- defined(
  c("AD", "LI"),
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

euros_dataset <- dataset_df(
  geo = geo_europe,
  gdp = euro_gdp,
  dataset_bibentry = dublincore(
    title = "European Microstates GDP",
    creator = person("Statistical Unit", role = "aut"),
    publisher = "Eurostat",
    subject = "Gross Domestic Product"
  )
)
```

### Step 2: Create a Dollar-based GDP Dataset

```{r usdgdp}
usd_gdp <- defined(
  56,
  label = "Gross Domestic Product",
  unit = "USD_MILLIONS",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo_tuvalu <- defined(
  "TV",
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

tuvalu_dataset <- dataset_df(
  geo = geo_tuvalu,
  gdp = usd_gdp,
  dataset_bibentry = dublincore(
    title = "Tuvalu GDP (USD)",
    creator = person("Island", "Bureau", role = "aut"),
    publisher = "PacificStats",
    subject = "Gross Domestic Product"
  )
)
```

The tidy workflow is based around five operational actions: - Data reshaping
goes from long to wide formats; - sorting arranges rows in a specific order; -
filtering removes rows based on a condition; - transforming, changes existing
variables or adds new ones; - aggregating creates a single value from many
values, say, for example, in computing the minimum, maximum, and mean.

Ideally, each of these steps should be recorded in the metadata. We will only
show data reshaping and transforming, because aggregation can be well described
with defining the new aggregate with `defined()`, and sorting and filtering are
trivial in a format where each observation is uniquely identified.

```{r}
binded <- try(bind_defined_rows(euros_dataset, tuvalu_dataset), silent = TRUE)
```

This will raise an error or warning because the gdp column has inconsistent
units (CP_MEUR vs USD_MILLIONS). The semantic definitions attached to each
vector allow dataset to detect and prevent accidental joins across incompatible
measurement systems.

### Step 3: Transform the Data and Document the Change

```{r mutate}
exchange_rate <- 1.02
eur_tuv_gdp <- defined(
  56 * exchange_rate,
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

tuvalu_dataset <- dataset_df(
  geo = geo_tuvalu,
  gdp = eur_tuv_gdp,
  dataset_bibentry = dublincore(
    title = "Tuvalu GDP (USD)",
    creator = person("Island", "Bureau", role = "aut"),
    publisher = "PacificStats",
    subject = "Gross Domestic Product"
  )
)
```

In a larger dataset, the user will likely use the tidyverse grammar (or the
grammar of data.table), with mutating the dollar values into euro values. In
this case, the transformation or the mutation should be recorded in the change
of the unit. If you would add population data to the GDP dataset, and compute
GDP/capita, you would also want to add a new long-form variable label, perhaps
change the unit from millions of euros to euros.

```{r}
var_unit(eur_tuv_gdp) <- "M_EUR"
```

The joined dataset needs a new title, and it can be attributed to a new author
and publisher. The vocabulary of the Dublin Core and DataCite metadata standards
used by most repositories and exchanges are covered with convenient helper
functions that retrieve or set the descriptive metadata value. Some of them,
like the title, are protected with explicit overwrite permissions.

```{r metadatachanges}
global_dataset <- bind_defined_rows(euros_dataset, tuvalu_dataset)
dataset_title(global_dataset, overwrite = TRUE) <- "Global Microstates GDP"
publisher(global_dataset) <- "My Research Institute"
creator(global_dataset) <- person("Jane Doe", role = "aut")
language(global_dataset) <- "en"
description(global_dataset) <- "A dataset created from various sources about the GDP of very small states."
global_dataset
```

You can review the descriptive metadata of the dataset with `as_dublincore()` or
`[as_datacite()]` in various formats.

```{r}
as_dublincore(global_dataset)
```

A tidy dataset can be serialised to RDF with dataset_to_triples, which performs
the data reshaping goes from wide to long formats. You can read a lot more in
the vignette-articles of the high-level R-binding to the Python RDFLib library,
rdflib, particularly the *A tidyverse lover's introduction to R* on how to
normalise the data to a format that it can be serialised to a flat RDF file or a
graph database.

```{r}
dataset_to_triples(global_dataset)
```

```{r}
dataset_to_triples(global_dataset, format = "nt")
```

## Full Interoperability

In the semantic web, datasets are often represented as collections of triples:
subject, predicate, and object. The `dataset_to_triples()` function enables this
by converting any dataset_df into a long-form representation where each row
represents a semantically annotated cell.

Unlike tidy datasets that require column-wise joins and reshape operations,
RDF-based datasets eliminate structural joins by relying on identity, context,
and concept URIs. Repeated values are normalized at the semantic level. This
makes triple-based data more flexible for publishing, integration, and querying
across domains.

This design choice affects how we implemented joins and bindings. The package
avoids implementing column-wise joins or wide-format merging because
semantically rich datasets can be recombined or queried directly via SPARQL or
other RDF tooling. Instead, row-wise binding via bind_defined_rows() is
supported, allowing users to append consistent datasets without losing
semantics.

This reflects a deliberate philosophy: rather than duplicate tidyverse
behaviours, dataset encourages upstream semantic modelling and downstream
interoperability.

The dataset_to_triples() function exports a tidy dataset to RDF-style triplets:

triples \<- dataset_to_triples(small_dataset) head(triples)

Each row becomes a triple (subject, predicate, object), typed with XSD and
optionally resolved via URIs. Export is supported through rdflib.

This example illustrates the core design goal of the `dataset` package: to make
semantic metadata first-class citizens of the R data workflow. By embedding
units, concept URIs, and provenance directly in data objects, the package
supports not only reproducible research but also semantically interoperable
publication — all without departing from familiar tidyverse idioms.

The dataset created in this example could be easily validated, documented, and
exported as linked data using standard RDF tooling. This forms the basis for
reproducible, standards-aligned workflows that extend beyond the analyst’s
desktop — into repositories, triple stores, or domain-specific data services.

Yet, the applied example also reveals current limitations and areas for growth
in the `dataset` package, which we now turn to.

## Export and Interoperability

The `dataset` package is designed with FAIR principles in mind, particularly the
goal of enabling machine-actionable data publishing. To support semantic web
compatibility and downstream interoperability, it provides functions that allow
users to convert annotated datasets into RDF-compatible formats.

The key function in this process is:

- `dataset_to_triples()`: Converts a `dataset_df` into a three-column long-form
  structure—subject, predicate, object—representing each cell as an RDF-style
  triple. These can be exported to tabular or text-based formats, or directly
  ingested by triple stores.

This structure aligns with the W3C's RDF and Data Cube vocabularies, where: -
The **subject** typically encodes an observation or observation unit - The
**predicate** is derived from a concept URI associated with the variable - The
**object** is the value, typed using XML Schema Definitions (e.g.,
`xsd:integer`, `xsd:string`)

These outputs are fully compatible with the
[`rdflib`](https://docs.ropensci.org/rdflib/) package, which can serialize RDF
datasets into: - Turtle (`.ttl`) - RDF/XML (`.rdf`) - N-Triples (`.nt`) -
JSON-LD (`.jsonld`)

This enables dataset publication to:

- SPARQL endpoints

- FAIR data repositories

- Wikibase instances (via planned extensions)

- Semantic web catalogues

Triple-based export promotes structural normalization, eliminates redundancy,
and facilitates data integration across domains and systems.

## Limitations and Future Work

The `dataset` package prioritizes ease of use and integration with existing
tidyverse workflows. It intentionally implements a practical subset of features
drawn from more formal metadata and ontology systems used in statistical
domains, such as SDMX, DDI, and DCAT.

Some features have been deliberately left out to keep the package lightweight
and analyst-friendly:

- No native support for **data cube slicing** (e.g., filtering all observations
  for a specific dimension level)
- No column-wise binding (e.g., `bind_cols()`) with semantic integrity checks
- No built-in **validation against controlled vocabularies** or semantic
  registries
- Limited UI or interactive support for defining or editing metadata.

One key limitation is the lack of experience with `dataset` in large-scale,
multi-institutional ingestion, exchange, or publication workflows. For example,
it remains unclear whether column-wise binding is necessary in practice, given
that many users will serialize data to RDF triples — where redundancy is
automatically filtered out by triple stores.

Some features could be better developed as stand-alone packages.

- The
  [`bibrecord()`](https://dataset.dataobservatory.eu/articles/bibrecord.html) s3
  class with its constructor was created out of necessity, because the
  `utils::bibentry` class and the `utils::person()` do not handle well modern
  library and repository metadata. Most of the work carried out with the
  `bibentry` class to use the `dublincore()` and `datacite()` constructors could
  be easily adopted in the utils package of R, because it does not raise
  backward compatibility problems.

- The
  [`provenance()`](https://dataset.dataobservatory.eu/reference/provenance.html)
  function could safely be developed into a package of its own, because there
  are countless ways to improve the granularity a dataset provenance
  description.

Several downstream features and companion packages are under development:

- **`wbdataset`**: export to the Wikibase data model for collaborative metadata
  curation
- **Observation status flags**: support for tagging cells as "estimated",
  "provisional", or "forecasted"
- **Validation helpers**: checks for missing concept URIs, unit mismatches, or
  inconsistent namespaces
- **External thesauri support**: integration with vocabularies such as EuroVoc
  or GEMET
- **FAIR alignment**: better coverage of 5-star and 8-star FAIR metadata
  criteria

We expect the need for tailored adaptations in specific domains — including
environmental statistics, cultural heritage, and social sciences — where
existing metadata models often deviate from general-purpose ontologies.

The `dataset` package does not aim to replace enterprise-scale metadata
infrastructure (e.g., SDMX registries), but rather to empower individual
researchers and small teams to produce semantically valid, publication-ready
datasets without high setup costs.

## Limitations and Future Work

The `dataset` package prioritizes ease of use and integration with existing
tidyverse workflows. It implements a practical subset of features inspired by
more formal metadata and ontology systems used in statistical domains, such as
SDMX, DDI, and DCAT.

Several features have been deliberately left out to keep the package lightweight
and analyst-friendly:

- No native support for **data cube slicing** (e.g., filtering all observations
  for a specific dimension level)
- No column-wise binding (e.g., `bind_cols()`) with semantic integrity checks
- No built-in validation against **controlled vocabularies** or semantic
  registries
- Limited UI or interactive tooling for defining or editing metadata

A key limitation is the limited experience with `dataset` in large-scale,
multi-institutional ingestion, exchange, or publication workflows. For example,
it is still unclear whether column-wise semantic binding is necessary in
practice — given that many users export to RDF triples, where redundancy is
naturally eliminated by triple stores.

Some internal components could be better developed as stand-alone packages:

- The
  [`bibrecord()`](https://dataset.dataobservatory.eu/articles/bibrecord.html) S3
  class was introduced out of necessity. Base R’s `utils::bibentry` and
  `utils::person()` do not adequately support modern library and repository
  metadata. Much of the work done in the `dublincore()` and `datacite()`
  constructors could be ported upstream to the `utils` package without
  introducing backward compatibility issues.

- The
  [`provenance()`](https://dataset.dataobservatory.eu/reference/provenance.html)
  function could reasonably be split into a separate package, as there are many
  opportunities to increase the granularity and expressiveness of dataset
  provenance descriptions.

Several downstream features and companion packages are under development:

- **`wbdataset`**: export to the Wikibase data model for collaborative metadata
  curation
- **Observation status flags**: tagging of individual cells as "estimated",
  "provisional", or "forecasted"
- **Validation helpers**: checks for missing concept URIs, unit mismatches, or
  inconsistent namespaces
- **External thesauri support**: integration with vocabularies such as EuroVoc
  or GEMET
- **FAIR alignment**: support for 5-star and 8-star FAIR metadata compliance

We anticipate the need for tailored extensions in domain-specific contexts —
including environmental statistics, cultural heritage, and social sciences —
where metadata conventions often deviate from general-purpose ontologies.

The `dataset` package is not intended to replace enterprise-scale metadata
infrastructure (e.g., SDMX registries), but rather to empower individual
researchers and small teams to produce semantically valid, publication-ready
datasets with minimal overhead.

---
title: "From R to RDF"
output:
  rmarkdown::html_vignette:
    md_extensions: -autolink_bare_uris
vignette: >
  %\VignetteIndexEntry{From R to RDF}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{css, echo=FALSE}
.smaller .table {
  font-size: 11px;
}

.smaller pre,
.smaller code {
  font-size: 11px;
  line-height: 1.2;
}
```

## From tidy data to RDF triples

This vignette demonstrates how to convert tidy R datasets into semantically enriched RDF triple structures, using the `dataset` and `rdflib` packages. These packages help you annotate variables with machine-readable concepts, units, and links to controlled vocabularies.

We’ll start with a small example of a tidy dataset representing countries (`geo`) with unique identifiers (`rowid`) and then show how to transform the dataset into RDF triples using standard vocabularies.

```{r setup}
library(dataset)
library(rdflib)
data("gdp")
```

## Creating a minimal semantically defined dataset

```{r minimaldf}
small_geo <- dataset_df(
  geo = defined(
    gdp$geo[1:3],
    label = "Geopolitical entity",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refArea",
    namespace = "https://www.geonames.org/countries/$1/"
  ),
  identifier = c(
    obs = "https://dataset.dataobservatory.eu/examples/dataset.html#"
  )
)
```

The dataset has no creator or author, but the rows have identifiers that can be resolved with <https://dataset.dataobservatory.eu/examples/dataset.html#>. In real publishing
scenarios, you would replace these with persistent URIs
that identify actual datasets and their observations. For example, a DOI-based identifier such as:

`https://doi.org/10.5281/zenodo.14917851#obs:1`

So let's see how this minimal dataset prints in R:

```{r printsmallgeodf}
print(small_geo)
```

A tidy dataset can always be pivotted to a three-column long (tidy) format, which can define every cell value in the tabular dataset with a subject-predicate-object triple. 

```{r triplesdf, eval=FALSE}
triples_df <- dataset_to_triples(small_geo)
knitr::kable(triples_df)
```

::: smaller
```{r triplesdfprintsmall, echo=FALSE}
triples_df <- dataset_to_triples(small_geo)
knitr::kable(triples_df)
```
::: 

This produces triples like:

```{r createntriples}
ntriples <- dataset_to_triples(small_geo, format = "nt")
```
```{r pritriples, eval=FALSE}
cat(ntriples, sep = "\n")
```

::: smaller
```{r printsmaller}
cat(ntriples, sep = "\n")
```
:::

Each row of your dataset becomes a **subject**, each variable a **predicate**, and each value either a **URI** or a typed literal (like a date or number) — depending on how it's defined. The first statement in the example defines the intersection of the first row (observation, identified by the `rowid`) `dataset#eg:1` and the column [reference area](http://purl.org/linked-data/sdmx/2009/dimension#refArea) defined by the URI as [Andorra](https://www.geonames.org/countries/AD/).The advantage of this approach is that the row and column definitions as well as coded cell values have a permanent metadata definition.

### RDF triples enable interoperability

The Resource Description Framework (RDF) represents data as subject–predicate–object triples. This allows your dataset to be machine-readable, linkable to external vocabularies, and to be ready for queries via SPARQL.

### RDF triples enable interoperability

The Resource Description Framework (RDF) represents data as subject–predicate–object triples. This allows your dataset to be machine-readable, linkable to external vocabularies, and queryable via SPARQL.

```{r ntripleexample}
n_triple(
  s = "https://dataset.dataobservatory.eu/examples/dataset.html#obs1",
  p = "http://purl.org/dc/terms/title",
  o = "Small Country Dataset"
)
```


```{r readrdf}
# We write to a temporary file our Ntriples created earlier
temp_file <- tempfile(fileext = ".nt")
writeLines(ntriples, con = temp_file)

rdf_graph <- rdf()
rdf_parse(rdf_graph, doc = temp_file, format = "ntriples")
rdf_graph
```

A simple, serverless scaffolding for publishing `dataset_df` objects on the web (with HTML + RDF exports) is available at    <https://github.com/dataobservatory-eu/dataset-template> with the example
of this vignette tutorial.

## Clean up

It is a good practice to close connections, or clean up larger objects living in the memory:

```{r cleanup}
# Clean up: delete file and clear RDF graph
unlink(temp_file)
rm(rdf_graph)
gc()
```

## Scale up

We build a slightly bigger graph, save it, and reload it. 

```{r scaleup}
small_country_dataset <- dataset_df(
  geo = defined(
    gdp$geo,
    label = "Country name",
    concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
    namespace = "https://www.geonames.org/countries/$1/"
  ),
  year = defined(
    gdp$year,
    label = "Reference Period (Year)",
    concept = "http://purl.org/linked-data/sdmx/2009/dimension#refPeriod"
  ),
  gdp = defined(
    gdp$gdp,
    label = "Gross Domestic Product",
    unit = "https://dd.eionet.europa.eu/vocabularyconcept/eurostat/unit/CP_MEUR",
    concept = "http://data.europa.eu/83i/aa/GDP"
  ),
  unit = gdp$unit,
  freq = defined(
    gdp$freq,
    label = "Frequency",
    concept = "http://purl.org/linked-data/sdmx/2009/code"
  ),
  identifier = c(
    obs = "https://dataset.dataobservatory.eu/examples/dataset.html#"
  ),
  dataset_bibentry = dublincore(
    title = "Small Country Dataset",
    creator = person("Jane", "Doe"),
    publisher = "Example Inc.",
    datasource = "https://doi.org/10.2908/NAIDA_10_GDP",
    rights = "CC-BY",
    coverage = "Andorra, Lichtenstein and San Marino"
  )
)
```

```{r smallcountrydfnt}
small_country_df_nt <- dataset_to_triples(
  small_country_dataset,
  format = "nt"
)
```


The following lines read as:

-   [1] `Observation #1` is a geopolitical entity, `Andorra`.
-   [11] `Observation #1` has a reference time period of `2020`.
-   [21] `Observation #1` has a decimal GDP value of `2354.8`
-   [31] `Observation #1` has a unit of `million euros, current prices`.
-   [41] `Observation #1` has a measurement frequency that is `annual`.

:::: smaller
```{r smallcountrydfntsample}
## See rows 1,11,21
small_country_df_nt[c(1, 11, 21, 31, 41)]
```
::::

he statements about `Observation 1`, i.e. Andorra's national economy in 2020, is not serialised consecutively in the text file. This is not necessary, because each cell is precisely connected to the *row* (first part of the triple) and *column* (second part of the triple). We could say that the entire map to the original dataset is embedded into the flat text file, therefore it can be easily imported into a database.

*Note: The `.html#` in these example IRIs does not mean the resource is an HTML file.  
Any absolute IRI is valid in RDF. This form is used here only for illustration;  
in practice, a bare namespace such as `/dataset#` is more conventional.*


```{r readrdf2}
# We write to a temporary file our Ntriples created earlier
temp_file <- tempfile(fileext = ".nt")
writeLines(small_country_df_nt,
  con = temp_file
)

rdf_graph <- rdf()
rdf_parse(rdf_graph, doc = temp_file, format = "ntriples")
```
```{r readrdf2print, eval = FALSE}
rdf_graph
```
:::: smaller
```{r readrdf2printsmaller}
rdf_graph
```
:::: 

Your dataset is now ready to be exported to meet the true FAIR standards, because they are:

- **self-descriptive**: variables carry labels, units, and definitions.
- **machine-readable**: linked vocabularies and standard identifiers.
- **ready to publish and share**: they carry the metadata of each variable, potentially each observation unit, and through metadata standards like Dublin Core and DataCite the information about the whole dataset, too.

```{r readjsonld}
# Create temporary JSON-LD output file
jsonld_file <- tempfile(fileext = ".jsonld")

# Serialize (export) the entire graph to JSON-LD format
rdf_serialize(rdf_graph, doc = jsonld_file, format = "jsonld")
```

Read it back to R for display (only first 30 lines are shown):

:::: smaller
```{r readjsonldprint}
cat(readLines(jsonld_file)[1:30], sep = "\n")
```
::::

```{r clenup2, echo=FALSE, message=FALSE}
unlink(temp_file)
rm(rdf_graph)
gc()
```

---
title: "Design Principles & Future Work Semantically Enriched, Standards-Aligned Datasets in R"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{Design Principles & Future Work Semantically Enriched, Standards-Aligned Datasets in R}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 80
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Abstract

```{r setup}
library(dataset)
```

> “A dataset is an identifiable collection of data available for access or
> download in one or more formats.” — ISO/IEC 20546

The `dataset` package enriches R data objects with machine-readable metadata by
embedding semantic definitions and provenance at both the variable and dataset
levels. It follows a semantic **early-binding** design: metadata is attached at
creation time, not retrofitted post hoc. This ensures that meaning and context
are preserved throughout the data lifecycle — from exploration to publication —
and enables partial automation of documentation.

This article outlines the design philosophy behind the `dataset` package,
including its theoretical foundations, structure, relationship to other R tools,
and example workflows. It serves as a long-form complement to the package
vignettes.

## Introduction and Motivation

> “The principles of tidy data provide a standard way to organise data values
> within a dataset.”\
> — Wickham (2014)

The `dataset` package extends R’s native data structures by embedding
machine-readable semantics and provenance directly in tidy data objects. It
builds on tidy data principles (Wickham, 2014) but introduces a **semantic
early-binding** approach: metadata is attached when the dataset is created,
ensuring that context and meaning are preserved through all stages of the
workflow — including transformation, validation, serialization, and reuse.

While tidyverse tools enforce structural clarity, they are generally agnostic
about semantics. Variables may be misinterpreted, joined incorrectly, or
published without context. `dataset` addresses this gap by aligning with
international metadata standards, supporting RDF export, and providing an
interface to the W3C Data Cube model.

A tidy dataset, per Wickham’s definition, adheres to three core rules:

-   Each variable forms a column\
-   Each observation forms a row\
-   Each value forms a cell

![Figure 12.1 in R for Data Science: tidy data structure.](images/tidy-1.png)

However, this tidy structure — typically implemented as a `data.frame` or
`tibble` — is not semantically self-describing. In practical workflows, users
often conflate the in-memory structure with the abstract concept of a dataset,
which in metadata terms refers not just to structure but also to definitions,
units, provenance, and contributors.


Several ISO and W3C standards define what constitutes a dataset. According to ISO/IEC 20546, a dataset is an identifiable collection of data available for access or download in one or more formats. The Dublin Core DCMI Metadata Terms define a [dataset](https://www.dublincore.org/specifications/dublin-core/dcmi-terms/dcmitype/Dataset/) as “data encoded in a defined structure.” The W3C’s [Data Cube Vocabulary](https://www.w3.org/TR/vocab-data-cube/#cubes-model-datasets), widely used in official statistics, describes a dataset as a “collection of statistical data that corresponds to a defined structure.” That structure includes observations, metadata about their organisation, *structural metadata* (e.g., units of measure), and *reference metadata* (e.g., creator, publisher).  

This differs from R’s `data.frame` object, which is defined as “tightly coupled collections of variables which share many of the properties of matrices and of lists, used as the fundamental data structure by most of R's modeling software.” In practice, R users often use the terms *data frame* (or tibble) and *dataset* interchangeably. However, even a tidy data frame is underspecified for use in scientific repositories, statistical data exchanges, or many database applications. A `data.frame` exists only in the memory of an R session, limiting its interoperability and reusability. While R can already serialise data frames to formats like `.rds`, `.rda`, or `.csv`, these serialisations by default lack rich, standardised metadata. The `dataset` package bridges that gap by aligning with established metadata standards, producing serialisations that are easier to reuse and interpret.  

The `dataset` package extends R's native data structures with machine-readable metadata. It follows a *semantic early-binding* approach: metadata is embedded as soon as the data is created, making datasets suitable for long-term reuse, FAIR-compliant publishing, and integration into semantic web systems.  

The central innovation of the package is an extended data-frame-like object: a `tibble::tibble()` enhanced with R’s `attributes()` system to store standard metadata from ISO and W3C vocabularies. This `dataset_df` class integrates naturally with tidy data principles (Wickham, 2014), where each variable is a column, each observation is a row, and each type of observational unit forms a table. On top of this tidy structure, `dataset_df` adds a semantic layer so that the meaning of variables and datasets is explicit and machine-readable.  This new class is introduced in  `vignette("dataset_df", package = "dataset").

In research or institutional contexts, a dataset is a form of digital resource,
often archived, cited, or published. Such resources are typically described with
metadata using the Resource Description Framework (RDF), enabling
machine-actionable, language-independent, schema-neutral representation. Our aim is to facilitate the translation or annotation of a tidy R data.frame into such a resource.

RDF also enables description at the level of **elementary statements** — that
is, per-cell metadata combining variable (column) and observation (row). This
allows for fine-grained semantic annotation, supporting full data traceability
and interoperability.

The original tidy workflow was designed for solo, interactive analysis where
analysts had full context. But in collaborative, institutional, or
public-sharing contexts, assumptions must be replaced with formal semantics. Not
only structure, but also clear definitions — of units, classifications, codes,
and contributors — become essential.

Moreover, many statistical data providers follow the **data cube model**, which
resembles tidy data but supports higher dimensionality and more formal metadata.
Examples include SDMX and the W3C Data Cube vocabulary.

Tidy data assumes that column names and structure are sufficient for clarity.
However, ambiguity arises quickly when combining datasets from heterogeneous
sources. A column named `geo` might contain ISO codes in one dataset and
Eurostat codes in another. GDP figures may differ in currency or base year.
These inconsistencies often go unnoticed until late-stage analytical errors.

For example:

```{r dataframe}
data.frame(
  geo = c("LI", "SM"),
  CPI = c("0.8", "0.9"),
  GNI = c("8976", "9672")
)
```

This dataset is tidy, but not self-describing. Is geo using ISO 3166 or Eurostat
codes? Is GNI measured in euros, dollars, or PPP-adjusted values?

The dataset package addresses these challenges by introducing structures for
semantically rich vectors (`defined()`) and annotated tibbles (`dataset_df()`).
It integrates machine-readable metadata directly into R objects and ensures that
labels, units, concept URIs, and provenance are preserved from creation to
publication.

This approach bridges the gap between tidy data and RDF, making formal semantics
part of the tidyverse workflow — without requiring users to leave R or manually
manage external metadata schemas.

## Related Work

Several R packages have offered tools to improve the metadata management of
datasets within the tidyverse ecosystem or its surrounding statistical
traditions.

The `labelled` class in the `labelled` and `haven` packages supports long-form
variable labels and improved handling of value label sets compared to base R’s
`factor` class. This is particularly helpful for variables collected in survey
instruments — a major source of microdata in statistical workflows. However,
real statistical production, as standardized by GSIM (Generic Statistical
Information Model) and DDI (Data Documentation Initiative), involves a far more
complex metadata model. Our contribution builds on these efforts by enabling
users to attach standardized, cross-domain codebook references to such
variables, ensuring that labelling follows recognized metadata vocabularies.

The `dataspice` package allows users to create auxiliary metadata datasets for
publication. Its strength lies in its simplicity: it uses human-readable CSV
files to capture key metadata fields. However, this simplicity introduces
fragility: the metadata may become detached, outdated, or unsynchronised from
the data file, especially in collaborative or iterative settings.

The `rdflib` package, a high-level interface to the RDF library of the same name
in Python, supports RDF serialization and querying. It allows tidy data to be
mapped to RDF triples and serialized into N-Triples, Turtle, RDF/XML, or
JSON-LD. However, it assumes that metadata is retrofitted — applied after the
analytical workflow is complete. While `rdflib` is essential for
interoperability, it requires users to leave the tidyverse workflow or gain RDF
modelling expertise.

Our goal with the `dataset` package is to bridge the semantic and methodological
gap between the `tidyverse` and `rdflib`: to make semantically annotated,
publication-ready datasets part of the R-native workflow from the start.

Another important initiative is the **Frictionless Data** project, which
provides lightweight standards for describing datasets (e.g., via
`datapackage.json`). It enables platform-independent validation and metadata
exchange. In R, the `frictionless` package supports reading, writing, and
validating data packages. However, this system relies heavily on external JSON
schemas and does not integrate metadata into the objects used during analysis —
meaning users must juggle separate metadata files and validation steps,
potentially losing semantic continuity during transformation.

Tierney and Cook (2020), in their paper *Expanding Tidy Data Principles to
Facilitate Missing Data Exploration, Visualization and Assessment of
Imputations*, demonstrate how tidy data can be extended through consistent
metadata structures and function design. Their concept of “nabular data” —
datasets with shadow columns representing missingness — shows how tidy workflows
can accommodate new dimensions of metadata. While their focus is on handling
missing data, their methodological framing is closely aligned with ours:
extending tidy conventions by designing new object classes and verbs, rather
than retrofitting external metadata after the fact.

Collectively, these tools highlight a shared recognition: tidy data principles
offer a strong foundation, but do not, by themselves, guarantee semantic
clarity, interoperability, or reuse. The `dataset` package responds to this need
by embedding standardized metadata directly in R objects, enabling datasets to
remain semantically intact throughout transformation, validation, and
publication.

## Design Principles

-   **Early binding of semantics**: Metadata is attached at the point of dataset
    creation, not after the fact.
-   **Attribute-based, not schema-based**: Metadata lives inside the R object
    itself, not in external schemas or files.
-   **Minimal friction with tidyverse workflows**: Compatible with `dplyr`,
    `tidyr`, `vctrs`, and coercible to `tibble` or `data.frame`.
-   **Persistence across save/load cycles**: Metadata survives R serialization
    (`.rds`, `.rda`).
-   **Tidyverse-like grammar**: Core verbs include `defined()`, `dataset_df()`,
    `provenance()`, `describe()`, `datacite()`, and `dublincore()`.
-   **Full interoperability outside R**: Datasets can be exported as RDF using
    `dataset_to_triples()` and ingested into triple stores.

### Semantic Early Binding

The `dataset` package introduces several new S3 classes that remain fully
compatible with tidyverse idioms and largely interoperable with base R. These
classes rely on R's native attribute system to embed metadata directly within
vectors and tibbles. This enables metadata such as labels, concept URIs,
namespaces, and provenance details to persist during filtering, joining, or
transformation.

The attribute system in R is underused, and most user-friendly packages offer
little support or interface for working directly with object attributes. This
leads to redundancy — with metadata often duplicated within the dataset content
itself.

The `defined()` constructor builds on `labelled::labelled` (originally from
`haven`) and provides a more expressive way to annotate vectors with:

-   A human-readable label (e.g., `"Gross Domestic Product"`)
-   A unit or measurement system (e.g., `"CP_MEUR"`), accessible via
    `var_unit()` and set with `var_unit() <-`
-   A concept URI that uniquely identifies the variable or dimension, handled
    via `var_concept()` and assignment
-   A namespace URI pattern for resolving coded values (e.g., ISO or Eurostat
    country codes), via `var_namespace()`

The `dataset_df()` class extends `tibble` and supports combining enriched
vectors with dataset-level metadata. This includes Dublin Core and DataCite
elements such as title, creator, publisher, subject, and contributors, along
with provenance metadata like creation time or software agent.

### Attribute-Based, Not Schema-Based

The `dataset` package adopts an attribute-based design rather than a
schema-based approach. Metadata is stored directly in R objects using native
attributes, ensuring semantic annotations remain tightly coupled with the data
throughout transformation, saving, and reuse.

This approach eliminates the need for separate schema definitions or JSON
metadata files — lowering the barrier to semantic data publishing within R
workflows.

In R, most objects (especially vectors and data frames) can carry attributes
such as:

-   `names`
-   `class`
-   `label`
-   `unit`
-   `concept`
-   `namespace`

These are lightweight, internal, and flexible. For example:

```{r attributes}
x <- 2457
attr(x, "unit") <- "CP_MEUR"
attr(x, "concept") <- "http://data.europa.eu/83i/aa/GDP"
```

In the dataset package, this metadata is preserved in defined and dataset_df
objects and moves with the data — whether it's saved, joined, subsetted, or
filtered.

By contrast, many CRAN or rOpenSci packages are schema-based: they require
external metadata definitions that describe expected columns, data types, and
semantic rules. While these can support more complex use cases — such as SDMX
structural metadata or JSON Schema validation — they introduce additional
overhead, increase complexity, and risk desynchronisation between data and
metadata.

Schema-based solutions may be more appropriate when data analysts work in teams
alongside research data managers or other documentation specialists. In
contrast, the `dataset` package is designed for individual researchers or small
teams who want to avoid semantic errors when ingesting new data from external
sources — while also enabling standards-compliant data exchange and publication
with minimal additional tooling.

## Persistence Across Save/Load Cycles

Because all metadata is stored as object attributes, it remains intact when
datasets are saved using native R serialization formats like .rds or .rda. These
attributes can be queried, extracted, or exported — but they do not interfere
with regular data manipulation or analysis.

Metadata is added at the time of object creation, in contrast to workflows where
metadata is generated after analysis or stored in sidecar files (e.g., JSON-LD).
This design reduces the risk of metadata being detached, outdated, or
incomplete.

## Base Examples: Using the `dataset` Grammar

This section demonstrates the core grammar of the `dataset` package using
minimal, synthetic examples. These illustrate how to define semantically
enriched vectors, assemble them into annotated datasets, and prepare them for
RDF export or validation.

### Creating Defined Vectors

The `defined()` constructor creates semantically enriched vectors. It extends
`labelled::labelled` with additional attributes such as `unit`, `concept`, and
`namespace`.

```{r}
library(dataset)

gdp <- defined(
  c(2355, 2592, 2884),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo <- defined(
  rep("AD", 3),
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)
```

These vectors behave like regular R vectors but carry internal metadata. This
metadata can be retrieved or reassigned using the accessor and setter functions
provided by the package:

```{r}
var_concept(gdp)
var_unit(gdp)
var_namespace(geo)
```

These attributes are preserved across most data transformations, and persist
when saving to `.rds` or `.rda`.

### Assembling a Dataset with Metadata

Use `dataset_df()` to combine defined vectors into a tibble-like object that
includes dataset-level metadata, such as bibliographic information, identifiers,
and provenance.

```{r smalldataset}
small_dataset <- dataset_df(
  geo = geo,
  gdp = gdp,
  identifier = c(gdp = "http://example.com/dataset#gdp"),
  dataset_bibentry = dublincore(
    title = "Small GDP Dataset",
    creator = person("Jane", "Doe", role = "aut"),
    publisher = "Small Repository",
    subject = "Gross Domestic Product"
  )
)
```

Behind the scenes, the package uses a custom bibrecord class that extends
`utils::bibentry()` to accommodate all metadata fields defined by Dublin Core
and DataCite — two major standards used in repositories, library systems, and
FAIR data infrastructures.

You can review the dataset-level metadata in both formats:

```{r bibrecord}
as_dublincore(small_dataset)
as_datacite(small_dataset)
```

Since these metadata models do not fully overlap, using `dublincore()` will
leave some DataCite-specific fields empty.

### Provenance Tracking

One benefit of early metadata binding is that basic provenance is automatically
tracked. The `provenance()` function returns metadata about when and how the
dataset was created — including the system time and, optionally, the software
environment.

```{r provenance}
provenance(small_dataset)
```

This provenance is also included in the machine-readable metadata that can be
exported using `describe()`, which generates an RDF description of the dataset.

```{r description}
description_nt <- tempfile(pattern = "small_dataset", fileext = ".nt")
describe(small_dataset, description_nt)

# Only a few lines shown:
readLines(description_nt)[5:8]
```

The dataset grammar provides a lightweight but standards-compliant way to attach
metadata during the creation of R objects. Unlike retrofitted metadata tools, it
keeps semantic annotations inside the object throughout filtering, saving, and
publishing. In the next section, we apply this grammar to a real-world scenario
involving statistical datasets with conflicting semantics.

## Applied Example: Joining Data with Semantic Constraints

This example demonstrates how the `dataset` package helps avoid semantic errors
when combining data from heterogeneous sources. We create a small GDP dataset
for three European microstates, measured in millions of euros (CP_MEUR), and
then attempt to append an observation from Tuvalu, measured in US dollars (USD).
The semantic mismatch triggers an error.

### Step 1: Create a Eurostat-Compatible GDP Dataset

```{r eurogpd}
euro_gdp <- defined(
  c(2355, 2592),
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo_europe <- defined(
  c("AD", "LI"),
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

euros_dataset <- dataset_df(
  geo = geo_europe,
  gdp = euro_gdp,
  dataset_bibentry = dublincore(
    title = "European Microstates GDP",
    creator = person("Statistical Unit", role = "aut"),
    publisher = "Eurostat",
    subject = "Gross Domestic Product"
  )
)
```

### Step 2: Create a Dollar-based GDP Dataset

```{r usdgdp}
usd_gdp <- defined(
  56,
  label = "Gross Domestic Product",
  unit = "USD_MILLIONS",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

geo_tuvalu <- defined(
  "TV",
  label = "Geopolitical Entity",
  concept = "http://dd.eionet.europa.eu/vocabulary/eurostat/geo/",
  namespace = "https://www.geonames.org/countries/$1/"
)

tuvalu_dataset <- dataset_df(
  geo = geo_tuvalu,
  gdp = usd_gdp,
  dataset_bibentry = dublincore(
    title = "Tuvalu GDP (USD)",
    creator = person("Island", "Bureau", role = "aut"),
    publisher = "PacificStats",
    subject = "Gross Domestic Product"
  )
)
```

The tidy workflow is based around five operational actions: - Data reshaping
goes from long to wide formats; - sorting arranges rows in a specific order; -
filtering removes rows based on a condition; - transforming, changes existing
variables or adds new ones; - aggregating creates a single value from many
values, say, for example, in computing the minimum, maximum, and mean.

Ideally, each of these steps should be recorded in the metadata. We will only
show data reshaping and transforming, because aggregation can be well described
with defining the new aggregate with `defined()`, and sorting and filtering are
trivial in a format where each observation is uniquely identified.

```{r}
binded <- try(bind_defined_rows(euros_dataset, tuvalu_dataset), silent = TRUE)
```

This will raise an error or warning because the gdp column has inconsistent
units (CP_MEUR vs USD_MILLIONS). The semantic definitions attached to each
vector allow dataset to detect and prevent accidental joins across incompatible
measurement systems.

### Step 3: Transform the Data and Document the Change

```{r mutate}
exchange_rate <- 1.02
eur_tuv_gdp <- defined(
  56 * exchange_rate,
  label = "Gross Domestic Product",
  unit = "CP_MEUR",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

tuvalu_dataset <- dataset_df(
  geo = geo_tuvalu,
  gdp = eur_tuv_gdp,
  dataset_bibentry = dublincore(
    title = "Tuvalu GDP (USD)",
    creator = person("Island", "Bureau", role = "aut"),
    publisher = "PacificStats",
    subject = "Gross Domestic Product"
  )
)
```

In a larger dataset, the user will likely use the tidyverse grammar (or the
grammar of data.table), with mutating the dollar values into euro values. In
this case, the transformation or the mutation should be recorded in the change
of the unit. If you would add population data to the GDP dataset, and compute
GDP/capita, you would also want to add a new long-form variable label, perhaps
change the unit from millions of euros to euros.

```{r}
var_unit(eur_tuv_gdp) <- "M_EUR"
```

The joined dataset needs a new title, and it can be attributed to a new author
and publisher. The vocabulary of the Dublin Core and DataCite metadata standards
used by most repositories and exchanges are covered with convenient helper
functions that retrieve or set the descriptive metadata value. Some of them,
like the title, are protected with explicit overwrite permissions.

```{r metadatachanges}
global_dataset <- bind_defined_rows(euros_dataset, tuvalu_dataset)
dataset_title(global_dataset, overwrite = TRUE) <- "Global Microstates GDP"
publisher(global_dataset) <- "My Research Institute"
creator(global_dataset) <- person("Jane Doe", role = "aut")
language(global_dataset) <- "en"
description(global_dataset) <- "A dataset created from various sources about the GDP of very small states."
global_dataset
```

You can review the descriptive metadata of the dataset with `as_dublincore()` or
`[as_datacite()]` in various formats.

```{r}
as_dublincore(global_dataset)
```

A tidy dataset can be serialised to RDF with dataset_to_triples, which performs
the data reshaping goes from wide to long formats. You can read a lot more in
the vignette-articles of the high-level R-binding to the Python RDFLib library,
rdflib, particularly the *A tidyverse lover's introduction to R* on how to
normalise the data to a format that it can be serialised to a flat RDF file or a
graph database.

```{r}
dataset_to_triples(global_dataset)
```

```{r}
dataset_to_triples(global_dataset, format = "nt")
```

## Full Interoperability

In the semantic web, datasets are often represented as collections of triples:
subject, predicate, and object. The `dataset_to_triples()` function enables this
by converting any dataset_df into a long-form representation where each row
represents a semantically annotated cell.

Unlike tidy datasets that require column-wise joins and reshape operations,
RDF-based datasets eliminate structural joins by relying on identity, context,
and concept URIs. Repeated values are normalized at the semantic level. This
makes triple-based data more flexible for publishing, integration, and querying
across domains.

This design choice affects how we implemented joins and bindings. The package
avoids implementing column-wise joins or wide-format merging because
semantically rich datasets can be recombined or queried directly via SPARQL or
other RDF tooling. Instead, row-wise binding via bind_defined_rows() is
supported, allowing users to append consistent datasets without losing
semantics.

This reflects a deliberate philosophy: rather than duplicate tidyverse
behaviours, dataset encourages upstream semantic modelling and downstream
interoperability.

The dataset_to_triples() function exports a tidy dataset to RDF-style triplets:

triples \<- dataset_to_triples(small_dataset) head(triples)

Each row becomes a triple (subject, predicate, object), typed with XSD and
optionally resolved via URIs. Export is supported through rdflib.

This example illustrates the core design goal of the `dataset` package: to make
semantic metadata first-class citizens of the R data workflow. By embedding
units, concept URIs, and provenance directly in data objects, the package
supports not only reproducible research but also semantically interoperable
publication — all without departing from familiar tidyverse idioms.

The dataset created in this example could be easily validated, documented, and
exported as linked data using standard RDF tooling. This forms the basis for
reproducible, standards-aligned workflows that extend beyond the analyst’s
desktop — into repositories, triple stores, or domain-specific data services.

Yet, the applied example also reveals current limitations and areas for growth
in the `dataset` package, which we now turn to.

## Export and Interoperability

The `dataset` package is designed with FAIR principles in mind, particularly the
goal of enabling machine-actionable data publishing. To support semantic web
compatibility and downstream interoperability, it provides functions that allow
users to convert annotated datasets into RDF-compatible formats.

The key function in this process is:

-   `dataset_to_triples()`: Converts a `dataset_df` into a three-column
    long-form structure—subject, predicate, object—representing each cell as an
    RDF-style triple. These can be exported to tabular or text-based formats, or
    directly ingested by triple stores.

This structure aligns with the W3C's RDF and Data Cube vocabularies, where: -
The **subject** typically encodes an observation or observation unit - The
**predicate** is derived from a concept URI associated with the variable - The
**object** is the value, typed using XML Schema Definitions (e.g.,
`xsd:integer`, `xsd:string`)

These outputs are fully compatible with the
[`rdflib`](https://docs.ropensci.org/rdflib/) package, which can serialize RDF
datasets into: - Turtle (`.ttl`) - RDF/XML (`.rdf`) - N-Triples (`.nt`) -
JSON-LD (`.jsonld`)

This enables dataset publication to: 

- SPARQL endpoints 

- FAIR data repositories

- Wikibase instances (via planned extensions) 

- Semantic web catalogues

Triple-based export promotes structural normalization, eliminates redundancy,
and facilitates data integration across domains and systems.

## Limitations and Future Work

The `dataset` package prioritizes ease of use and integration with existing
tidyverse workflows. It intentionally implements a practical subset of features
drawn from more formal metadata and ontology systems used in statistical
domains, such as SDMX, DDI, and DCAT.

Some features have been deliberately left out to keep the package lightweight
and analyst-friendly:

-   No native support for **data cube slicing** (e.g., filtering all
    observations for a specific dimension level)
-   No column-wise binding (e.g., `bind_cols()`) with semantic integrity checks
-   No built-in **validation against controlled vocabularies** or semantic
    registries
-   Limited UI or interactive support for defining or editing metadata.

One key limitation is the lack of experience with `dataset` in large-scale,
multi-institutional ingestion, exchange, or publication workflows. For example,
it remains unclear whether column-wise binding is necessary in practice, given
that many users will serialize data to RDF triples — where redundancy is
automatically filtered out by triple stores.

Some features could be better developed as stand-alone packages.

-   The
    [`bibrecord()`](https://dataset.dataobservatory.eu/articles/bibrecord.html)
    s3 class with its constructor was created out of necessity, because the
    `utils::bibentry` class and the `utils::person()` do not handle well modern
    library and repository metadata. Most of the work carried out with the
    `bibentry` class to use the `dublincore()` and `datacite()` constructors
    could be easily adopted in the utils package of R, because it does not raise
    backward compatibility problems.

-   The
    [`provenance()`](https://dataset.dataobservatory.eu/reference/provenance.html)
    function could safely be developed into a package of its own, because there
    are countless ways to improve the granularity a dataset provenance
    description.

Several downstream features and companion packages are under development:

-   **`wbdataset`**: export to the Wikibase data model for collaborative
    metadata curation
-   **Observation status flags**: support for tagging cells as "estimated",
    "provisional", or "forecasted"
-   **Validation helpers**: checks for missing concept URIs, unit mismatches, or
    inconsistent namespaces
-   **External thesauri support**: integration with vocabularies such as EuroVoc
    or GEMET
-   **FAIR alignment**: better coverage of 5-star and 8-star FAIR metadata
    criteria

We expect the need for tailored adaptations in specific domains — including
environmental statistics, cultural heritage, and social sciences — where
existing metadata models often deviate from general-purpose ontologies.

The `dataset` package does not aim to replace enterprise-scale metadata
infrastructure (e.g., SDMX registries), but rather to empower individual
researchers and small teams to produce semantically valid, publication-ready
datasets without high setup costs.

## Limitations and Future Work

The `dataset` package prioritizes ease of use and integration with existing
tidyverse workflows. It implements a practical subset of features inspired by
more formal metadata and ontology systems used in statistical domains, such as
SDMX, DDI, and DCAT.

Several features have been deliberately left out to keep the package lightweight
and analyst-friendly:

-   No native support for **data cube slicing** (e.g., filtering all
    observations for a specific dimension level)
-   No column-wise binding (e.g., `bind_cols()`) with semantic integrity checks
-   No built-in validation against **controlled vocabularies** or semantic
    registries
-   Limited UI or interactive tooling for defining or editing metadata

A key limitation is the limited experience with `dataset` in large-scale,
multi-institutional ingestion, exchange, or publication workflows. For example,
it is still unclear whether column-wise semantic binding is necessary in
practice — given that many users export to RDF triples, where redundancy is
naturally eliminated by triple stores.

Some internal components could be better developed as stand-alone packages:

-   The
    [`bibrecord()`](https://dataset.dataobservatory.eu/articles/bibrecord.html)
    S3 class was introduced out of necessity. Base R’s `utils::bibentry` and
    `utils::person()` do not adequately support modern library and repository
    metadata. Much of the work done in the `dublincore()` and `datacite()`
    constructors could be ported upstream to the `utils` package without
    introducing backward compatibility issues.

-   The
    [`provenance()`](https://dataset.dataobservatory.eu/reference/provenance.html)
    function could reasonably be split into a separate package, as there are
    many opportunities to increase the granularity and expressiveness of dataset
    provenance descriptions.

Several downstream features and companion packages are under development:

-   **`wbdataset`**: export to the Wikibase data model for collaborative
    metadata curation
-   **Observation status flags**: tagging of individual cells as "estimated",
    "provisional", or "forecasted"
-   **Validation helpers**: checks for missing concept URIs, unit mismatches, or
    inconsistent namespaces
-   **External thesauri support**: integration with vocabularies such as EuroVoc
    or GEMET
-   **FAIR alignment**: support for 5-star and 8-star FAIR metadata compliance

We anticipate the need for tailored extensions in domain-specific contexts —
including environmental statistics, cultural heritage, and social sciences —
where metadata conventions often deviate from general-purpose ontologies.

The `dataset` package is not intended to replace enterprise-scale metadata
infrastructure (e.g., SDMX registries), but rather to empower individual
researchers and small teams to produce semantically valid, publication-ready
datasets with minimal overhead.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_labels.R
\name{var_labels}
\alias{var_labels}
\alias{var_labels<-}
\title{Get or set all variable labels on a dataset}
\usage{
var_labels(
  x,
  unlist = FALSE,
  null_action = c("keep", "fill", "skip", "na", "empty")
)

var_labels(x) <- value
}
\arguments{
\item{x}{A \code{data.frame} or \code{\link{dataset_df}} object.}

\item{unlist}{Logical; if \code{TRUE}, return a named character vector instead
of a list. Defaults to \code{FALSE}.}

\item{null_action}{How to handle columns without labels. One of:
\itemize{
\item \code{"keep"} (default): keep \code{NULL} values for unlabeled columns.
\item \code{"fill"}: use the column name as a fallback label.
\item \code{"skip"}: exclude unlabeled columns from the result.
\item \code{"na"}: use \code{NA_character_} for unlabeled columns.
\item \code{"empty"}: use an empty string \code{""} for unlabeled columns.
}}

\item{value}{\itemize{
\item For setting: a named list or named character vector of labels.
Names must match column names in \code{x}.
Unnamed elements are ignored.
\item For getting: ignored.
}}
}
\value{
\itemize{
\item Getter: a named list (or vector if \code{unlist = TRUE}) of variable labels.
\item Setter: the modified \code{x} with updated labels, returned invisibly.
}
}
\description{
Retrieve or assign labels for all variables (columns) in a dataset.
}
\details{
This is the dataset-level equivalent of \code{\link[=var_label]{var_label()}}.
It works with any \code{data.frame}-like object, including \code{\link[=dataset_df]{dataset_df()}}, and
returns/sets the \code{"label"} attribute of each column.

Labels are useful for storing human-readable descriptions of variables that
may have short or cryptic column names.

For internal purposes, this function uses the \code{"var_labels"} dataset
attribute and delegates to \code{\link[=var_label]{var_label()}} and
\code{\link[=var_label<-]{var_label<-()}} on individual columns.
}
\examples{
df <- dataset_df(
  id = defined(1:3, label = "Observation ID"),
  temp = defined(c(22.5, 23.0, 21.8), label = "Temperature (°C)"),
  site = defined(c("A", "B", "A"))
)

# Get all variable labels
var_labels(df)

# Set multiple labels at once
var_labels(df) <- list(site = "Site code")

# Return as a named vector with empty string for unlabeled vars
var_labels(df, unlist = TRUE, null_action = "empty")

}
\seealso{
\code{\link[=var_label]{var_label()}}

Other defined metadata methods and functions:
\code{\link{var_label}},
\code{\link[=var_namespace]{var_namespace()}},
\code{\link[=var_unit]{var_unit()}}
}
\concept{defined metadata methods and functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_concept.R
\name{get_variable_concepts}
\alias{get_variable_concepts}
\title{Get concepts for all variables in a dataset_df}
\usage{
get_variable_concepts(x)
}
\arguments{
\item{x}{A \code{dataset_df} object.}
}
\value{
A named list of concept URIs for each variable.
}
\description{
Returns a named list of concept URIs (or NULLs) for all
variables.
}
\examples{
get_variable_concepts(orange_df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contributor.R
\name{contributor}
\alias{contributor}
\alias{contributor<-}
\title{Get or set contributors}
\usage{
contributor(x)

contributor(x, overwrite = FALSE) <- value
}
\arguments{
\item{x}{A dataset object created with \code{\link[=dataset_df]{dataset_df()}} or \code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{overwrite}{Logical. If \code{TRUE}, replace all existing contributors with
\code{value}. If \code{FALSE}, append \code{value} to the existing contributors. Defaults
to \code{FALSE}.}

\item{value}{A \code{\link[utils:person]{utils::person}} object representing a single contributor. If
the \code{role} field is missing, it will be set to \code{"ctb"}. If \code{NULL}, the
dataset is returned unchanged.}
}
\value{
\itemize{
\item \code{contributor()} returns a \code{\link[utils:person]{utils::person}} or a list of such objects
corresponding to contributors.
\item \verb{contributor<-()} returns the updated dataset (invisibly).
}
}
\description{
\code{contributor()} is a lightweight wrapper around \code{\link[=creator]{creator()}} that
works only with contributors. It retrieves or updates only the contributor
entries in the dataset's bibliographic metadata.
}
\details{
All people are stored in the \code{author} slot of the underlying
\code{\link[utils:bibentry]{utils::bibentry}}. This helper preserves primary creators and filters or
updates only those entries that represent contributors.

A \emph{contributor} is defined as:
\itemize{
\item a person with \code{role == "ctb"}, \strong{or}
\item a person with a \code{comment[["contributorType"]]}.

Primary creators (authors) typically have \code{role \%in\% c("aut", "cre")}.

Contributors can be further annotated with metadata in \code{comment}, for
example:
}

\if{html}{\out{<div class="sourceCode r">}}\preformatted{comment = c(contributorType = "hostingInstitution", ORCID = "0000-0000-0000-0000")
}\if{html}{\out{</div>}}
}
\examples{
df <- dataset_df(data.frame(x = 1))
creator(df) <- person("Jane", "Doe", role = "aut")

# Add a contributor
contributor(df, overwrite = FALSE) <-
  person("GitHub",
    role = "ctb",
    comment = c(contributorType = "hostingInstitution")
  )

# Replace all contributors
contributor(df) <- person("Support", "Team", role = "ctb")

# Inspect only contributors
contributor(df)

}
\seealso{
Other bibliographic helper functions:
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_title.R
\name{dataset_title}
\alias{dataset_title}
\alias{dataset_title<-}
\title{Get or Set the Title of a Dataset}
\usage{
dataset_title(x)

dataset_title(x, overwrite = FALSE) <- value
}
\arguments{
\item{x}{A dataset object created by \code{\link[=dataset_df]{dataset_df()}} or \code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{overwrite}{Logical. If \code{TRUE}, the existing title is replaced. If
\code{FALSE} (default) and a title is already present, a warning is issued and
the title is not changed.}

\item{value}{A character string representing the new title. If \code{NULL}, a
placeholder value \code{":tba"} is assigned. If \code{value} is a character vector of
length > 1, an error is raised.}
}
\value{
\code{dataset_title()} returns the current dataset title as a character
string. \verb{dataset_title<-()} returns the updated dataset object (invisible).
}
\description{
Retrieve or assign the main title of a dataset, typically used
as the primary label in metadata exports (e.g., DataCite or Dublin Core).
}
\details{
According to the \href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/title/}{Dublin Core specification for \code{title}},
the title represents the name by which the resource is formally known.

The DataCite metadata schema supports multiple titles (e.g., translated,
alternative), but this function currently supports only a single main title.
}
\examples{
dataset_title(orange_df)

# Set a new title with overwrite = TRUE
dataset_title(orange_df, overwrite = TRUE) <- "The Growth of Orange Trees"
dataset_title(orange_df)

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/creator.R
\name{creator}
\alias{creator}
\alias{creator<-}
\title{Get/set the Creator of the object.}
\usage{
creator(x)

creator(x, overwrite = TRUE) <- value
}
\arguments{
\item{x}{A semantically rich data frame object created by
\code{\link[=dataset_df]{dataset_df()}} or \verb{dataset::\link{as_dataset_df}}.}

\item{overwrite}{If the attributes should be overwritten. In case it is set
to \code{FALSE},it gives a message with the current \code{Creator} property
instead of overwriting it. Defaults to \code{TRUE} when the attribute is
set to \code{value} regardless of previous setting.}

\item{value}{The \code{Creator} as a
\code{\link[utils:person]{utils::person()}} object.}
}
\value{
The Creator attribute as a character of length one is added to
\code{x}.
}
\description{
Add the optional \code{Creator} property as an attribute to a
dataset object.
}
\details{
The \code{Creator} corresponds to
\href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/creator/}{dct:creator}
in Dublin Core and Creator in DataCite. The name of the entity that holds,
archives, publishes prints, distributes, releases, issues, or produces the
dataset. This property will be used to formulate the citation, so consider
the prominence of the role.
}
\examples{
creator(orange_df)
# To change author:
creator(orange_df) <- person("Jane", "Doe")
# To add author:
creator(orange_df, overwrite = FALSE) <- person("John", "Doe")
}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fix_contributor.R
\name{clean_person_name}
\alias{clean_person_name}
\title{Remove role suffixes from formatted person names}
\usage{
clean_person_name(p)
}
\arguments{
\item{p}{A \code{person} object.}
}
\value{
Character string without role annotations, e.g. \code{"Jane Doe"}.
}
\description{
Remove role suffixes from formatted person names
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_character.R
\name{as_character}
\alias{as_character}
\alias{as_character.haven_labelled_defined}
\alias{as.character.haven_labelled_defined}
\title{Coerce a defined vector to character}
\usage{
as_character(x, ...)

\method{as_character}{haven_labelled_defined}(x, strip_attributes = TRUE, ...)

\method{as.character}{haven_labelled_defined}(x, ...)
}
\arguments{
\item{x}{A vector created with \code{\link[=defined]{defined()}}.}

\item{...}{Reserved for potential future use.}

\item{strip_attributes}{Logical; should semantic metadata attributes
(such as \code{label}, \code{unit}, \code{definition}, and
\code{namespace}) be removed from the returned vector?
Defaults to \code{FALSE}.}
}
\value{
A character vector (plain or with attributes).
}
\description{
\code{as_character()} is the recommended method to convert a
\code{\link[=defined]{defined()}} vector into a character vector. It is metadata-aware
and provides explicit control over whether semantic attributes are preserved.

Base R \code{as.character()} always strips the class and metadata.
}
\details{
If \code{preserve_attributes = TRUE}, the returned character vector retains
metadata attributes (\code{unit}, \code{concept}, \code{namespace}, \code{label}). The
\code{"defined"} class is always removed.

If \code{preserve_attributes = FALSE} (default), a plain character vector is
returned with \emph{all} metadata stripped.

Use \code{strip_attributes = TRUE} when flattening or preparing data for
external pipelines, but keep the default when working with defined
vectors directly.

Base R's \code{as.character()} always drops all attributes and returns plain
character values. It is equivalent to:
\code{as_character(x, strip_attributes = TRUE)}.
}
\examples{
x <- defined(c("apple", "banana"), label = "Fruit", unit = "kg")

# Recommended:
as_character(x)

# Preserve metadata:
as_character(x, strip_attributes = FALSE)

# Base R:
as.character(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_df.R
\name{dataset_df}
\alias{dataset_df}
\alias{as_dataset_df}
\alias{is.dataset_df}
\alias{print.dataset_df}
\alias{is_dataset_df}
\alias{names.dataset_df}
\title{Create a new \code{dataset_df} object}
\usage{
dataset_df(
  ...,
  identifier = c(obs = "http://example.com/dataset#obs"),
  var_labels = NULL,
  units = NULL,
  concepts = NULL,
  dataset_bibentry = NULL,
  dataset_subject = NULL
)

as_dataset_df(
  df,
  identifier = c(obs = "http://example.com/dataset#obs"),
  var_labels = NULL,
  units = NULL,
  concepts = NULL,
  dataset_bibentry = NULL,
  dataset_subject = NULL,
  ...
)

is.dataset_df(x)

\method{print}{dataset_df}(x, ...)

is_dataset_df(x)

\method{names}{dataset_df}(x)
}
\arguments{
\item{...}{Vectors (columns) that should be included in the dataset.}

\item{identifier}{A named vector of one or more URI prefixes for row IDs.
Defaults to \code{c(eg = "http://example.com/dataset#")}. For example, if your
dataset will be published under DOI \verb{https://doi.org/1234}, you may use
\code{c(obs = "https://doi.org/1234#")}, which will generate row URIs such as
\verb{https://doi.org/1234#1}, ..., \verb{#n}.}

\item{var_labels}{A named list of human-readable labels for each variable.}

\item{units}{A named list of measurement units for measured variables.}

\item{concepts}{A named list of linked concepts (URIs) for variables or
dimensions.}

\item{dataset_bibentry}{A bibliographic metadata record for the dataset,
created using \code{\link[=datacite]{datacite()}} or \code{\link[=dublincore]{dublincore()}}.}

\item{dataset_subject}{A subject descriptor created with \code{\link[=subject]{subject()}} or
\code{\link[=subject_create]{subject_create()}}.}

\item{df}{A \code{data.frame} to convert to a \code{dataset_df}.}

\item{x}{A \code{dataset_df} object (used in method dispatch).}
}
\value{
A \code{dataset_df} object: a tibble with attached metadata stored in
attributes.

\code{is.dataset_df} returns a logical value
(if the object is of class \code{dataset_df}.)
}
\description{
The \code{dataset_df()} constructor creates semantically rich modern data frames.
These inherit from \code{\link[tibble:tibble]{tibble::tibble}} and carry structured metadata using
attributes.
}
\details{
Use \code{is.dataset_df()} to check class membership.

S3 methods for \code{dataset_df} include:
\itemize{
\item \code{print()} to display the dataset with metadata
\item \code{summary()} to summarize both data and metadata
\item \code{names()} to retrieve variable names using standard data frame semantics
}

For full details, see \code{vignette("dataset_df", package = "dataset")}.
}
\note{
A simple, serverless scaffolding for publishing \code{dataset_df} objects
on the web (with HTML + RDF exports) is available at
\url{https://github.com/dataobservatory-eu/dataset-template}.
}
\examples{
my_dataset <- dataset_df(
  country_name = defined(
    c("AD", "LI"),
    concept = "http://data.europa.eu/bna/c_6c2bb82d",
    namespace = "https://www.geonames.org/countries/$1/"
  ),
  gdp = defined(
    c(3897, 7365),
    label = "Gross Domestic Product",
    unit = "million dollars",
    concept = "http://data.europa.eu/83i/aa/GDP"
  ),
  identifier = c(
    obs = "https://dataobservatory-eu.github.io/dataset-template#"
  ),
  dataset_bibentry = dublincore(
    title = "GDP of Andorra and Liechtenstein",
    description = "A small but semantically rich dataset example.",
    creator = person("Jane", "Doe", role = "cre"),
    publisher = "Open Data Institute",
    language = "en"
  )
)

# Basic usage
print(my_dataset)
head(my_dataset)
summary(my_dataset)

# Metadata access
as_dublincore(my_dataset)
as_datacite(my_dataset)

# Export description as RDF triples
my_description <- describe(my_dataset, con = tempfile())
my_description

}
\seealso{
\code{\link[=defined]{defined()}}, \code{\link[=dublincore]{dublincore()}}, \code{\link[=datacite]{datacite()}}, \code{\link[=subject]{subject()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/description.R
\name{description}
\alias{description}
\alias{description<-}
\title{Get or set the dataset Description}
\usage{
description(x)

description(x, overwrite = FALSE) <- value
}
\arguments{
\item{x}{A dataset object created with \code{\link[=dataset_df]{dataset_df()}} or
\code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{overwrite}{Logical. If \code{TRUE}, will overwrite any existing description.
If \code{FALSE} (default), will warn and keep the existing description.}

\item{value}{The new description, as a character string.}
}
\value{
The \code{Description} attribute as a character vector of length 1.
}
\description{
Get or set the optional \code{Description} property as an attribute
on a dataset object.
}
\details{
The \code{Description} is recommended for discovery in DataCite. It
captures additional information that does not fit other metadata categories
— such as technical notes or dataset usage. It is a free-text field. See
\href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/description/}{dct:description}.
}
\examples{
description(orange_df)
description(orange_df, overwrite = TRUE) <- "This dataset records orange tree growth."
description(orange_df)

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/provenance.R
\name{default_provenance}
\alias{default_provenance}
\title{Build default provenance bundle}
\usage{
default_provenance(
  dataset_id = "http://example.com/dataset#",
  author = NULL,
  dtm = NULL,
  generated_at_time = NULL
)
}
\arguments{
\item{dataset_id}{Base IRI for the dataset (used as the \code{Entity} subject).}

\item{author}{Optional creator/author agent.}

\item{dtm}{Optional data team/maintainer agent.}

\item{generated_at_time}{Optional POSIXct time; defaults to \code{\link[=Sys.time]{Sys.time()}}.}
}
\value{
A character vector of N‑Triples suitable for the \code{"prov"} attribute.
}
\description{
Construct a small PROV bundle (as N‑Triples) describing the dataset, the
software agent, and an optional creation time.
}
\details{
This helper is used internally to seed provenance metadata. It emits a set of
PROV statements including an \code{Entity} for the dataset, an \code{Activity} for
creation, and \code{SoftwareAgent} entries for the package citation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subject.R
\name{subject}
\alias{subject}
\alias{subject_create}
\alias{subject<-}
\alias{is.subject}
\title{Create, add, or retrieve a subject}
\usage{
subject(x)

subject_create(
  term,
  schemeURI = NULL,
  valueURI = NULL,
  prefix = NULL,
  subjectScheme = NULL,
  classificationCode = NULL
)

subject(x) <- value

is.subject(x)
}
\arguments{
\item{x}{A dataset object created with \code{\link[=dataset_df]{dataset_df()}} or \code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{term}{A subject term, for example \code{"Data sets"}.}

\item{schemeURI}{URI of the subject identifier scheme, for example
\code{"http://id.loc.gov/authorities/subjects"}.}

\item{valueURI}{URI of the subject term, for example
\code{"https://id.loc.gov/authorities/subjects/sh2018002256"}.}

\item{prefix}{Abbreviated prefix for a scheme URI, for example \code{"lcch:"}.
Widely used namespaces (schemes) have conventional abbreviations.}

\item{subjectScheme}{Name of the subject scheme, classification code, or
authority if one is used. This acts as a namespace.}

\item{classificationCode}{Classification code for schemes that do not have
\code{valueURI} entries for each subject term (e.g., ANZSRC).}

\item{value}{A subject object created by \code{\link[=subject_create]{subject_create()}} or a character
string. Used by \verb{subject<-} to replace the subject.}
}
\value{
\itemize{
\item \code{subject(x)} returns:
\itemize{
\item a single \code{"subject"} object if only one is present,
\item a list of \code{"subject"} objects if multiple are present,
\item otherwise falls back to the plain string from the bibentry.
}
\item \code{subject(x) <- value} accepts a character vector, a \code{"subject"} object, or
a list of \code{"subject"} objects, and updates both the bibentry slot and the
\code{"subject"} attribute. Returns the dataset invisibly.
\item \code{subject_create()} returns a structured \code{"subject"} object — or a list of
them if multiple terms are provided.
\item \code{is.subject(x)} returns \code{TRUE} if \code{x} inherits from class \code{"subject"}.
}
}
\description{
Manage the subject metadata of a dataset. The subject can be stored as a
simple character term or as a structured object with subproperties created by
\code{\link[=subject_create]{subject_create()}}.
}
\details{
The subject property records what the dataset is about.
The \href{https://schema.datacite.org/meta/kernel-4/}{DataCite subject property}
allows multiple subproperties, but these cannot be stored directly in a
standard \code{\link[utils:bibentry]{utils::bibentry}} object.
Therefore:
\itemize{
\item If you set a character string as the subject, it is stored in both the
bibentry and the \code{"subject"} attribute.
\item If you set a structured subject (via \code{\link[=subject_create]{subject_create()}}), the \verb{$term} value
is stored in the bibentry, and the full object is stored in the \code{"subject"}
attribute of the \code{\link{dataset_df}} object.
}
}
\examples{
# Set a structured subject
subject(orange_df) <- subject_create(
  term = "Oranges",
  schemeURI = "http://id.loc.gov/authorities/subjects",
  valueURI = "http://id.loc.gov/authorities/subjects/sh85095257",
  subjectScheme = "LCCH",
  prefix = "lcch:"
)

# Retrieve subject with subproperties
subject(orange_df)

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_to_triples.R
\name{dataset_to_triples}
\alias{dataset_to_triples}
\title{Dataset to triples (three columns or N-Triples)}
\usage{
dataset_to_triples(x, idcol = NULL, expand_uri = TRUE, format = "data.frame")
}
\arguments{
\item{x}{A \code{dataset_df} or \code{data.frame}.}

\item{idcol}{Name or index of the subject column. If NULL, defaults to
\code{"rowid"} or rownames.}

\item{expand_uri}{Logical; if TRUE, expands URIs using namespaces and
definitions.}

\item{format}{Output format: \code{"data.frame"} (default) or \code{"nt"} for
N-Triples.}
}
\value{
Either a \code{data.frame} with columns \code{s}, \code{p}, and \code{o}, or a character
vector of N-Triple lines.
}
\description{
Converts a dataset to RDF-style triples with subject, predicate,
and object columns. Supports semantic expansion via variable metadata.
}
\details{
For publishing examples, a minimal serverless scaffold is provided at
\url{https://github.com/dataobservatory-eu/dataset-template}, which shows how
to host CSV + RDF serialisations on GitHub Pages without any server setup.
}
\note{
A simple, serverless scaffolding for publishing \code{dataset_df} objects
on the web (with HTML + RDF exports) is available at
\url{https://github.com/dataobservatory-eu/dataset-template}.
}
\examples{
# A minimal example with just rowid and geo
data("gdp", package = "dataset")
small_geo <- dataset_df(
  geo = defined(
    gdp$geo[1:3],
    label = "Geopolitical entity",
    concept = "http://example.com/prop/geo",
    namespace = "https://dd.eionet.europa.eu/vocabulary/eurostat/geo/$1"
  )
)

# View as triple table
dataset_to_triples(small_geo)

# View as N-Triples
dataset_to_triples(small_geo, format = "nt")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_dublincore.R, R/dublincore.R
\name{as_dublincore}
\alias{as_dublincore}
\alias{dublincore}
\alias{is.dublincore}
\alias{print.dublincore}
\title{Add or Retrieve Dublin Core Metadata}
\source{
\itemize{
\item \href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/}{DCMI Metadata Terms}
}
}
\usage{
as_dublincore(x, type = "bibentry", ...)

dublincore(
  title,
  creator,
  contributor = NULL,
  year = NULL,
  publisher = NULL,
  identifier = NULL,
  subject = NULL,
  type = "DCMITYPE:Dataset",
  dataset_date = NULL,
  language = NULL,
  relation = NULL,
  dataset_format = "application/r-rds",
  rights = NULL,
  datasource = NULL,
  description = NULL,
  coverage = NULL
)

is.dublincore(x)

\method{print}{dublincore}(x, ...)
}
\arguments{
\item{x}{An object to test.}

\item{type}{The resource type. For datasets, use \code{"Dataset"}. See
\href{https://www.dublincore.org/specifications/dublin-core/dcmi-type-vocabulary/}{DCMI Type Vocabulary}.}

\item{...}{Additional metadata fields.}

\item{title}{A name given to the resource. See \code{\link[=dataset_title]{dataset_title()}}.}

\item{creator}{One or more \code{\link[utils:person]{utils::person()}} objects representing the
creator(s). See \code{\link[=creator]{creator()}}.}

\item{contributor}{Additional contributors (\code{\link[utils:person]{utils::person()}}) with optional
roles. See \code{\link[=contributor]{contributor()}}.}

\item{year}{An explicit publication year. If omitted, inferred from
\code{dataset_date}.}

\item{publisher}{A character or \code{\link[utils:person]{utils::person()}} indicating the publishing
entity. See \code{\link[=publisher]{publisher()}}.}

\item{identifier}{A unique persistent identifier (e.g., DOI). See \code{\link[=identifier]{identifier()}}.}

\item{subject}{A keyword or controlled vocabulary term. See \code{\link[=subject]{subject()}} and
\code{\link[=subject_create]{subject_create()}}.}

\item{dataset_date}{A publication or release date (\code{Date}, \code{POSIXct}, or
character in \code{YYYY}, \code{YYYY-MM-DD}, or ISO format).}

\item{language}{ISO 639-1 language code. See \code{\link[=language]{language()}}.}

\item{relation}{A related resource (e.g., version, paper, or parent dataset).
Currently only supports an URI, for example,
\code{"https:://doi.org/10.32614/CRAN.package.dataset"}.}

\item{dataset_format}{The technical format of the dataset (e.g., MIME type).
See \code{\link[=dataset_format]{dataset_format()}}.}

\item{rights}{A string describing intellectual property or usage rights.
Use a URI like \code{"https://creativecommons.org/public-domain/cc0/"}.}

\item{datasource}{A URL or label for the original source of the dataset.}

\item{description}{A free-text summary of the dataset. See \code{\link[=description]{description()}}.}

\item{coverage}{Geographic or temporal extent (spatial/temporal coverage).}
}
\value{
A \code{bibentry} object extended with class \code{"bibrecord"}, storing structured
Dublin Core metadata. Use \code{\link[=as_dublincore]{as_dublincore()}} to extract the metadata in list,
tabular, or RDF form.

A logical value: \code{TRUE} if \code{x} is a Dublin Core metadata record (i.e.,
inherits from \code{"dublincore"}), otherwise \code{FALSE}.
}
\description{
Adds or retrieves metadata conforming to the
\href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/}{Dublin Core Metadata Terms}
standard, enabling consistent and structured citation and retrieval of R
dataset objects.

\code{is.dublincore()} checks whether an object inherits from the \code{"dublincore"}
class.
}
\details{
The Dublin Core Metadata Element Set (DCMES) is a standardized vocabulary for
describing digital and physical resources. It includes 15 core fields and is
formally standardized as ISO 15836, IETF RFC 5013, and ANSI/NISO Z39.85.

This function constructs a \code{\link[utils:bibentry]{utils::bibentry()}} object extended with DCMI
terms and is compatible with \code{\link[=dataset_df]{dataset_df()}} objects. The resulting metadata
can be used for semantic documentation and machine-readable citation.

For compatibility with \code{\link[utils:bibentry]{utils::bibentry()}}, the \code{dataset_date} parameter is
automatically used to derive both \code{publication_date} and \code{year} fields.
}
\examples{
orange_bibentry <- dublincore(
  title = "Growth of Orange Trees",
  creator = c(
    person(
      given = "N.R.",
      family = "Draper",
      role = "cre",
      comment = c(VIAF = "http://viaf.org/viaf/84585260")
    ),
    person(given = "H", family = "Smith", role = "cre")
  ),
  contributor = person(given = "Antal", family = "Daniel", role = "dtm"),
  publisher = "Wiley",
  datasource = "https://isbnsearch.org/isbn/9780471170822",
  dataset_date = 1998,
  identifier = "https://doi.org/10.5281/zenodo.14917851",
  language = "en",
  description = "The Orange data frame has 35 rows and 3 columns of records
                 of the growth of orange trees."
)

# To inspect structured metadata from a dataset_df object:
as_dublincore(orange_df, type = "list")

}
\seealso{
Learn more in the vignette:
\href{https://dataset.dataobservatory.eu/articles/bibrecord.html}{\code{bibrecord}}

Other bibrecord functions:
\code{\link[=as_datacite]{as_datacite()}},
\code{\link[=bibrecord]{bibrecord()}}
}
\concept{bibrecord functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defined.R
\name{strip_defined}
\alias{strip_defined}
\title{Strip the class from a defined vector}
\usage{
strip_defined(x)
}
\arguments{
\item{x}{A \code{defined} vector.}
}
\value{
A base R vector with attributes (\code{label}, \code{unit}, etc.) intact.
}
\description{
Converts a \code{defined} vector to a base R numeric or character,
retaining metadata as passive attributes.
}
\examples{
gdp <- defined(c(3897L, 7365L), label = "GDP", unit = "million dollars")
strip_defined(gdp)

fruits <- defined(c("apple", "avocado", "kiwi"),
  label = "Fruit", unit = "kg"
)

strip_defined(fruits)
}
\seealso{
\code{\link[=as_numeric]{as_numeric()}}, \code{\link[=as_character]{as_character()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relation.R
\name{relation}
\alias{relation}
\alias{relation<-}
\alias{related_create}
\alias{is.related}
\alias{related_item}
\alias{related_item<-}
\title{Add or retrieve related items (DataCite/Dublin Core)}
\usage{
relation(x)

relation(x) <- value

related_create(
  relatedIdentifier,
  relationType,
  relatedIdentifierType,
  resourceTypeGeneral = NULL
)

is.related(x)

related_item(x)

related_item(x) <- value
}
\arguments{
\item{x}{A dataset object created with \code{\link[=dataset_df]{dataset_df()}} or \code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{value}{A \code{related} object from \code{\link[=related_create]{related_create()}} or a character.
Vectors of characters are also supported and will be converted to a list of
\code{"related"} objects.}

\item{relatedIdentifier}{A string with the identifier of the related
resource.}

\item{relationType}{A string naming the relation type (per DataCite
vocabulary).}

\item{relatedIdentifierType}{A string naming the identifier type (\code{"DOI"},
\code{"URL"}, etc.).}

\item{resourceTypeGeneral}{Optional: a string naming the general type of the
related resource.}
}
\value{
\itemize{
\item \code{relation(x)} returns:
\itemize{
\item a single structured \code{"related"} object (from \code{\link[=related_create]{related_create()}}) if only
one relation is present,
\item a list of \code{"related"} objects if multiple relations are present,
\item otherwise it falls back to the bibentry field (\code{relatedidentifier} for
DataCite or \code{relation} for Dublin Core).
}
\item \code{relation(x) <- value} sets the \code{"relation"} attribute (structured object
or list of objects) and the bibentry string fields (\code{relatedidentifier} and
\code{relation}), and returns the dataset invisibly.
\item \code{related_create()} constructs a structured \code{"related"} object.
\item \code{is.related(x)} returns \code{TRUE} if \code{x} inherits from class \code{"related"}.
}
}
\description{
Manage related resources for a dataset using a unified accessor.
\itemize{
\item For \strong{DataCite 4.x}, this maps to \code{relatedIdentifier} (+ type & relation).
\item For \strong{Dublin Core}, this maps to \code{dct:relation} (string).
}
}
\details{
To remain compatible with \code{\link[utils:bibentry]{utils::bibentry()}}, the bibentry stores
only the
\strong{string identifier} (e.g., DOI/URL). The full structured object created by
\code{\link[=related_create]{related_create()}} is preserved in the \code{"relation"} attribute.

A \code{"related"} object is a small S3 list with the following elements:
\itemize{
\item \code{relatedIdentifier}: the related resource identifier (DOI, URL, etc.)
\item \code{relationType}: the DataCite relation type (e.g., \code{"IsPartOf"}, \code{"References"})
\item \code{relatedIdentifierType}: the type of identifier (\code{"DOI"}, \code{"URL"}, etc.)
\item \code{resourceTypeGeneral}: optional, the general type of the related resource (e.g., \code{"Text"}, \code{"Dataset"})
}
}
\examples{
df <- dataset_df(data.frame(x = 1))
relation(df) <- related_create(
  relatedIdentifier = "10.1234/example",
  relationType = "IsPartOf",
  relatedIdentifierType = "DOI"
)
relation(df) # structured object
get_bibentry(df)$relation # "10.1234/example"
get_bibentry(df)$relatedidentifier # "10.1234/example"

# Character input is normalized to a DOI/URL with default types
relation(df) <- "https://doi.org/10.5678/xyz"
relation(df) # structured object (relationType/Type filled with defaults)

# Create related object directly
rel <- related_create("https://doi.org/10.5678/xyz", "References", "DOI")
is.related(rel) # TRUE

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defined.R
\name{print.haven_labelled_defined}
\alias{print.haven_labelled_defined}
\title{Print a defined (haven_labelled_defined) vector}
\usage{
\method{print}{haven_labelled_defined}(x, ...)
}
\arguments{
\item{x}{A \code{haven_labelled_defined} vector.}

\item{...}{Passed on to \code{\link[base:print]{base::print()}}.}
}
\value{
\code{x}, invisibly.
}
\description{
Custom print method for \code{\link{haven_labelled_defined}} vectors created with
\code{\link[=defined]{defined()}}. It prints the variable name, label, and a short semantic
summary before the underlying values.
}
\examples{
sex <- defined(
  c(0, 1, 1, 0),
  label  = "Sex",
  labels = c("Female" = 0, "Male" = 1)
)

print(sex)

}
\seealso{
\code{\link[=defined]{defined()}}, \code{\link[=summary.haven_labelled_defined]{summary.haven_labelled_defined()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.POSIXct.R
\name{as.POSIXct.haven_labelled_defined}
\alias{as.POSIXct.haven_labelled_defined}
\title{Coerce a defined POSIXct vector to a base R POSIXct}
\usage{
\method{as.POSIXct}{haven_labelled_defined}(x, tz = "", strip_attributes = TRUE, ...)
}
\arguments{
\item{x}{A vector created with \code{\link[=defined]{defined()}} with underlying type
\code{POSIXct}.}

\item{tz}{a character string. The time zone specification to be used for
the conversion, if one is required. System-specific timezones
(see \code{\link[base:timezones]{base::timezones()}},
but "" is the current time zone, and "GMT" is UTC (Universal Time,
Coordinated). Invalid values are most commonly treated as UTC, on
some platforms with a warning.}

\item{strip_attributes}{Logical; should semantic metadata attributes
(label, unit, definition, namespace) be removed? Defaults to
\code{FALSE}.}

\item{...}{Additional arguments passed to \code{\link[base:as.POSIXct]{base::as.POSIXct()}}.}
}
\value{
A \code{POSIXct} vector with timestamp values preserved.
}
\description{
Coerces a \code{\link{haven_labelled_defined}} vector whose underlying type is
\code{\link{POSIXct}} into a base R \code{POSIXct} time vector.

This method preserves both the timestamp values and the original time zone.
By default, semantic metadata is also retained.
}
\details{
Use \code{strip_attributes = TRUE} when flattening or preparing data for
external pipelines, but keep the default when working with defined
vectors directly.\cr
Base R's \code{\link[=as.POSIXct]{as.POSIXct()}} also works, as it dispatches to this method via
S3. Using this method directly is preferred when metadata preservation
matters.
}
\examples{
p <- defined(
  as.POSIXct("2024-01-01 12:00:00", tz = "UTC"),
  label = "Timestamp"
)

# Recommended usage
as.POSIXct(p)

# Explicit attribute stripping
as.POSIXct(p, strip_attributes = TRUE)

}
\seealso{
\code{\link[=as.Date]{as.Date()}},
\code{\link[=as_numeric]{as_numeric()}}, \code{\link[=as_character]{as_character()}}, \code{\link[=as_logical]{as_logical()}},
\code{\link[=defined]{defined()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_to_triples.R
\name{expand_triples}
\alias{expand_triples}
\title{Internal: Expand multi-valued DC fields to RDF triples}
\usage{
expand_triples(dataset_id, predicate_uri, values)
}
\arguments{
\item{dataset_id}{The subject URI}

\item{predicate_uri}{The RDF predicate URI}

\item{values}{A scalar, character vector, or list (e.g., person objects)}
}
\value{
A character vector of RDF triples
}
\description{
Converts scalar or vector fields into RDF triples.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/haven_labelled_defined.R
\name{haven_labelled_defined}
\alias{haven_labelled_defined}
\title{Semantic labelled vector class}
\value{
No value is returned. This documentation page exists so that
the class can be referenced in help topics.
}
\description{
The class \code{haven_labelled_defined} represents a semantically
enriched labelled vector created by \code{\link{defined}}.

Objects of this class inherit from:

\itemize{
\item \code{\link[haven:labelled]{haven::labelled}} for numeric,
character, and factor data;
\item \code{Date} for temporal dates;
\item \code{POSIXct} and \code{POSIXt} for timestamps;
\item \code{logical} for boolean data.
}

The class tag enables S3 method dispatch for:

\itemize{
\item \code{\link{as_character}}
\item \code{\link{as_numeric}}
\item \code{\link{as_logical}}
\item \code{as.Date}
\item \code{as.POSIXct}
\item \code{\link{print.haven_labelled_defined}}
\item \code{\link{summary.haven_labelled_defined}}
}

Users normally do not construct this class directly; it is returned
as the result of calling \code{\link{defined}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rights.R
\name{rights}
\alias{rights}
\alias{rights<-}
\title{Get or Set the Rights of a Dataset Object}
\usage{
rights(x)

rights(x, overwrite = FALSE) <- value
}
\arguments{
\item{x}{A semantically rich data frame created with \code{\link[=dataset_df]{dataset_df()}} or
\code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{overwrite}{Logical. Should the existing value be replaced? If \code{FALSE}
and a value already exists, the function emits a message instead of
overwriting. Defaults to \code{FALSE}.}

\item{value}{A character string specifying the rights (e.g., \code{"CC-BY-4.0"}).}
}
\value{
The \code{"rights"} attribute of the dataset as a character string (length 1).
When assigning, the updated object \code{x} is returned invisibly.
}
\description{
Adds or retrieves the optional \code{"rights"} attribute of a dataset object.
This field contains information about intellectual property or usage rights.
}
\details{
The \code{"rights"} field corresponds to
\href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/rights/}{dct:rights}
from Dublin Core, and to \code{rights} in \href{https://schema.datacite.org/}{DataCite}.

Rights information typically includes statements about legal ownership,
licensing, or usage conditions. It helps ensure that users understand how
a dataset may be reused, cited, or shared.
}
\examples{
rights(orange_df) <- "CC-BY-SA"
rights(orange_df)

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/publication_year.R
\name{publication_year}
\alias{publication_year}
\alias{publication_year<-}
\title{Get or Set the Publication Year of a Dataset Object}
\usage{
publication_year(x)

publication_year(x, overwrite = TRUE) <- value
}
\arguments{
\item{x}{A dataset object created by \code{\link[=dataset_df]{dataset_df()}} or
\code{dataset::as_dataset_df()}.}

\item{overwrite}{Logical. If \code{TRUE} (default), the existing
\code{publication_year} attribute is replaced with \code{value}. If \code{FALSE}, the
function returns a message and does not overwrite the existing value.}

\item{value}{A character string specifying the publication year.}
}
\value{
The \code{publication_year} attribute as a character string.
}
\description{
Access or assign the optional \code{publication_year} attribute to a semantically
rich dataset object.
}
\details{
The \code{publication_year} represents the year when the dataset was or will be
made publicly available, in \code{YYYY} format. For additional context, see
\href{https://support.datacite.org/docs/datacite-metadata-schema-v44-mandatory-properties#publicationyearadditional-guidance}{DataCite: Publication Year-Additional Guidance}.
}
\examples{
publication_year(orange_df)
publication_year(orange_df) <- "1998"

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_logical.R
\name{as_logical}
\alias{as_logical}
\alias{as_logical.haven_labelled_defined}
\title{Coerce a defined vector to logical}
\usage{
as_logical(x, ...)

\method{as_logical}{haven_labelled_defined}(x, strip_attributes = TRUE, ...)
}
\arguments{
\item{x}{A vector created with \code{\link[=defined]{defined()}}.}

\item{...}{Additional arguments (currently unused).}

\item{strip_attributes}{Logical; should semantic metadata attributes
(such as \code{label}, \code{unit}, \code{definition}, and
\code{namespace}) be removed from the returned vector?
Defaults to \code{FALSE}.}
}
\value{
A logical vector. If \code{strip_attributes = FALSE}, any semantic
metadata attached to \code{x} will be carried over to the returned vector.
}
\description{
Coerces a \code{\link{haven_labelled_defined}} vector created with \code{\link[=defined]{defined()}}
into a base R logical vector.

This function is the recommended, semantic-aware interface for converting
defined logical vectors. It preserves the underlying logical values and,
unless otherwise requested, retains the semantic metadata attached to the
variable.
}
\details{
Use \code{strip_attributes = TRUE} when flattening or preparing data for
external pipelines, but keep the default when working with defined
vectors directly.

Users may also call base R's \code{\link[=as.logical]{as.logical()}}, which will dispatch to this
method automatically via S3. However, \code{as_logical()} is preferred because
it makes the semantics explicit and supports the \code{strip_attributes}
argument.
}
\examples{
# Basic usage
flg <- defined(c(TRUE, FALSE, TRUE), label = "Flag")
as_logical(flg)

# Metadata preserved by default
attr(as_logical(flg), "label")

# Stripping metadata
as_logical(flg, strip_attributes = TRUE)

# Base R coercion also works (via S3 method dispatch)
as.logical(flg)

}
\seealso{
\code{\link[=as_numeric]{as_numeric()}}, \code{\link[=as_character]{as_character()}},
\code{\link[=as.Date]{as.Date()}}, \code{\link[=as.POSIXct]{as.POSIXct()}},
\code{\link[=defined]{defined()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_label.R
\name{var_label}
\alias{var_label}
\alias{var_label.defined}
\alias{label_attribute}
\alias{var_label<-}
\alias{var_label<-.haven_labelled_defined}
\alias{var_label.dataset_df}
\title{Get or Set a Variable Label}
\usage{
\method{var_label}{defined}(x, ...)

label_attribute(x)

var_label(x) <- value

\method{var_label}{haven_labelled_defined}(x) <- value

\method{var_label}{dataset_df}(
  x,
  unlist = FALSE,
  null_action = c("keep", "fill", "skip", "na", "empty"),
  recurse = FALSE,
  ...
)
}
\arguments{
\item{x}{A vector or data frame.}

\item{...}{Further arguments passed to or used by methods.}

\item{value}{A character string to assign as the label, or \code{NULL} to remove it.}

\item{unlist}{For data frames, return a named vector instead of a list.}

\item{null_action}{For data frames, controls how to handle columns without a variable label.
Options are:
\itemize{
\item \code{"keep"} (default): keep \code{NULL} for unlabeled columns
\item \code{"fill"}: use the column name as a fallback
\item \code{"skip"}: exclude columns with no label from the result
\item \code{"na"}: use \code{NA_character_}
\item \code{"empty"}: use an empty string \code{""}
}}

\item{recurse}{If \code{TRUE}, applies \code{var_label()} recursively on packed columns
(as created by \code{\link[tidyr:pack]{tidyr::pack()}}) to retrieve sub-column labels. If \code{FALSE},
only the outer (grouped) column label is returned.}
}
\value{
\itemize{
\item \code{var_label(x)} returns the \code{"label"} attribute of \code{x} as a character string.
\item \code{var_label(x) <- value} sets, removes, or replaces the label attribute of \code{x},
returning the updated object invisibly.
}

A character string if the \code{"label"} attribute exists, or \code{NULL} if not present.

The modified object \code{x}, returned invisibly with the updated \code{"label"} attribute.
}
\description{
Adds or retrieves a human-readable label as a metadata attribute for a
variable or vector. This label is useful for making variables easier to
understand than their programmatic names (e.g., column names).

\code{label_attribute()} is a low-level helper that retrieves the \code{"label"} attribute
of an object without any fallback or printing logic. It is primarily used internally.

The \verb{var_label<-} assignment method sets or removes the \code{"label"} attribute
of a vector or data frame column. This allows attaching human-readable
descriptions to variables for interpretability and downstream metadata use.
}
\details{
This interface builds on \code{\link[labelled:var_label]{labelled::var_label()}} and is compatible with
the \code{defined()} infrastructure for semantic metadata (labels, namespaces,
units, and variable identifiers).

See \code{\link[labelled:var_label]{labelled::var_label()}} for low-level usage. For a comprehensive
guide to working with variable labels and semantic metadata, see:
\code{vignette("defined", package = "dataset")}.
}
\examples{
# Retrieve the label attribute
var_label(orange_df$circumference)

# Set or update the label attribute
var_label(orange_df$circumference) <- "circumference (breast height)"

# Example: Retrieve variable labels from a dataset_df
df <- dataset_df(
  id = defined(1:3, label = "Observation ID"),
  temp = defined(c(22.5, 23.0, 21.8), label = "Temperature (°C)"),
  site = defined(c("A", "B", "A"))
)

# List form (default)
var_label(df)

# Character vector form
var_label(df, unlist = TRUE, null_action = "empty")

# Exclude variables without labels
var_label(df, null_action = "skip")

# Replace missing labels with column names
var_label(df, null_action = "fill")
}
\seealso{
\code{\link[labelled:var_label]{labelled::var_label()}}, \code{\link[=var_labels]{var_labels()}}, \code{\link[=defined]{defined()}}

Other defined metadata methods and functions:
\code{\link[=var_labels]{var_labels()}},
\code{\link[=var_namespace]{var_namespace()}},
\code{\link[=var_unit]{var_unit()}}
}
\concept{defined metadata methods and functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.R
\name{as_tibble.dataset_df}
\alias{as_tibble.dataset_df}
\alias{as_tibble}
\alias{as.tibble.dataset_df}
\title{Coerce a \code{dataset_df} to a tibble}
\usage{
as_tibble(x, ...)

\method{as_tibble}{dataset_df}(x, ..., strip_attributes = TRUE, .name_repair = "check_unique")

as.tibble.dataset_df(
  x,
  ...,
  strip_attributes = TRUE,
  .name_repair = "check_unique"
)
}
\arguments{
\item{x}{A \code{dataset_df} object.}

\item{...}{Additional arguments passed to \code{\link[=as.data.frame.dataset_df]{as.data.frame.dataset_df()}}.}

\item{strip_attributes}{Logical: should column-level semantic metadata
be stripped? Default: \code{TRUE}.}

\item{.name_repair}{Treatment of problematic column names:
\itemize{
\item \code{"minimal"}: No name checks.
\item \code{"unique"}: Enforce uniqueness.
\item \code{"check_unique"}: (default) require unique names.
\item \code{"universal"}: Make unique and syntactic.
\item A function or purrr-style anonymous function, see \code{\link[rlang:as_function]{rlang::as_function()}}.
}}
}
\value{
A \code{\link[tibble:tibble]{tibble}} containing the data (and optionally some
attributes) of the \code{dataset_df}.
}
\description{
\code{\link[=as_tibble]{as_tibble()}} method for \code{dataset_df} objects.

Converts a \code{dataset_df} into a tibble. By default, additional metadata
attached to the object is removed unless \code{strip_attributes = FALSE} is
specified.
}
\section{Metadata handling}{

The \code{strip_attributes} argument controls which attributes of the \code{dataset_df}
object are preserved. When \code{strip_attributes = TRUE} (the default), only
column-level information is kept.
}

\section{About column names}{

The \code{dataset_df} class may internally use reserved names for indexing,
identifiers, or metadata. These are never exposed in the resulting tibble.
Column names in the output tibble may be repaired according to
\code{.name_repair}.
}

\examples{
# Convert a dataset_df to a tibble
x <- dataset_df(orange_df)
as_tibble(x)

# Keep attributes
as_tibble(x, strip_attributes = FALSE)

}
\seealso{
\itemize{
\item \code{\link[tibble:as_tibble]{tibble::as_tibble()}}
\item \code{\link[=as.data.frame.dataset_df]{as.data.frame.dataset_df()}}
}
}
\concept{tibble.methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/identifier.R
\name{identifier}
\alias{identifier}
\alias{identifier<-}
\title{Get or Set the Identifier of a Dataset or Metadata Record}
\usage{
identifier(x)

identifier(x, overwrite = TRUE) <- value
}
\arguments{
\item{x}{A \code{\link[=dataset_df]{dataset_df()}} object or a \code{\link[utils:bibentry]{utils::bibentry}} object (including
\code{\link[=dublincore]{dublincore()}} or \code{\link[=datacite]{datacite()}} records).}

\item{overwrite}{Logical. If \code{TRUE} (default), any existing identifier is
replaced. If \code{FALSE}, an existing identifier is preserved unless it is
\code{":unas"} or \code{":tba"}.}

\item{value}{A character string giving the identifier. Can be named (e.g.,
\code{c(doi = "...")}) or unnamed. Numeric values are coerced to character.}
}
\value{
For \code{identifier()}, the current identifier as a character string. For
\verb{identifier<-()}, the updated object (invisible).
}
\description{
Retrieve or assign the \code{identifier} attribute of a dataset or
bibliographic metadata object.
}
\details{
An \emph{identifier} provides an unambiguous reference to a resource.
Recommended practice is to supply a persistent identifier string, such as a
DOI, ISBN, or URN, that conforms to a recognized identification system.

Both \href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/#identifier}{Dublin Core}
and \href{https://support.datacite.org/docs/datacite-metadata-schema-v44-mandatory-properties#1-identifier}{DataCite 4.4}
define \code{identifier} as a core property. If the identifier is a DOI, it will
also be stored in the \code{doi} field of the metadata record.

Although \code{identifier} is not part of the minimal Dublin Core term set, it is
always included in \code{dataset} metadata for compatibility with publishing and
indexing systems. You may omit it if working under a strict DC profile.

For best practice in choosing identifier schemes, see the \href{https://www.iana.org/assignments/uri-schemes/uri-schemes.xhtml}{IANA-registered URI schemes}.
}
\examples{
orange_copy <- orange_df

# Get the current identifier
identifier(orange_copy)

# Set a new identifier (e.g., a DOI)
identifier(orange_copy) <- "https://doi.org/10.9999/example.doi"

# Prevent accidental overwrite
identifier(orange_copy, overwrite = FALSE) <- "https://example.org/id"

# Use numeric and NULL values
identifier(orange_copy) <- 12345
identifier(orange_copy) <- NULL # Sets ":unas"

}
\concept{Reference metadata functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset-orange_df.R
\docType{data}
\name{orange_df}
\alias{orange_df}
\title{Growth of Orange Trees}
\format{
A data frame with 35 rows and 4 variables:
\itemize{
\item \code{rowid}: A unique identifier for each row (character)
\item \code{tree}: Tree identifier (ordered factor)
\item \code{age}: Age of the tree in days (numeric)
\item \code{circumference}: Trunk circumference in mm (numeric)
}
}
\usage{
orange_df
}
\description{
A dataset recording the growth of orange trees, replicated from the classic
\href{https://stat.ethz.ch/R-manual/R-devel/library/datasets/html/Orange.html}{\code{datasets::Orange}}
dataset and implemented as a \code{dataset_df} S3 class with enhanced semantic metadata.
}
\details{
This is a semantically enriched version of the classic Orange dataset,
constructed using the \code{dataset_df()} and \code{dublincore()} constructors.
Each column includes semantic metadata such as units, labels, concepts,
or namespace identifiers. The dataset also embeds a machine-readable citation
for reproducibility and provenance tracking.
\subsection{Constructor Example}{

\if{html}{\out{<div class="sourceCode r">}}\preformatted{orange_bibentry <- dublincore(
  title = "Growth of Orange Trees",
  creator = c(
    person(
      given = "N.R.",
      family = "Draper",
      role = "cre",
      comment = c(VIAF = "http://viaf.org/viaf/84585260")
    ),
    person(
      given = "H",
      family = "Smith",
      role = "cre"
    )
  ),
  contributor = person(
    given = "Antal",
    family = "Daniel",
    role = "dtm"
  ),
  publisher = "Wiley",
  datasource = "https://isbnsearch.org/isbn/9780471170822",
  dataset_date = 1998,
  identifier = "https://doi.org/10.5281/zenodo.14917851",
  language = "en",
  description = "The Orange data frame has 35 rows and 3 columns of records of the growth of orange trees."
)

orange_df <- dataset_df(
  rowid = defined(paste0("orange:", row.names(Orange)),
    label = "ID in the Orange dataset",
    namespace = c("orange" = "datasets::Orange")
  ),
  tree = defined(Orange$Tree,
    label = "The number of the tree"
  ),
  age = defined(Orange$age,
    label = "The age of the tree",
    unit = "days since 1968/12/31"
  ),
  circumference = defined(Orange$circumference,
    label = "circumference at breast height",
    unit = "milimeter",
    concept = "https://www.wikidata.org/wiki/Property:P2043"
  ),
  dataset_bibentry = orange_bibentry
)

orange_df$rowid <- defined(orange_df$rowid,
  namespace = "https://doi.org/10.5281/zenodo.14917851"
)
}\if{html}{\out{</div>}}
}
}
\examples{
# Print with semantic citation and data preview
print(orange_df)

# Access semantic metadata associated with variables
print(orange_df$age)

# Retrieve the embedded bibliographic record
as_dublincore(orange_df)

}
\references{
\itemize{
\item Draper, N. R. & Smith, H. (1998). \emph{Applied Regression Analysis} (3rd ed.). Wiley.
\item Pinheiro, J. C. & Bates, D. M. (2000). \emph{Mixed-effects Models in S and S-PLUS}. Springer.
\item Becker, R. A., Chambers, J. M. & Wilks, A. R. (1988). \emph{The New S Language}. Wadsworth & Brooks/Cole.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geolocation.R
\name{geolocation}
\alias{geolocation}
\alias{geolocation<-}
\title{Get or Set the Geolocation of a Dataset Object}
\usage{
geolocation(x)

geolocation(x, overwrite = TRUE) <- value
}
\arguments{
\item{x}{A dataset object created by \code{\link[=dataset_df]{dataset_df()}} or
\code{dataset::as_dataset_df()}.}

\item{overwrite}{Logical. If \code{TRUE} (default), the existing \code{geolocation}
attribute is replaced with \code{value}. If \code{FALSE}, the function returns a
message and does not overwrite the existing value.}

\item{value}{A character string specifying the \code{geolocation}.}
}
\value{
A character string of length 1, representing the \code{geolocation}
attribute attached to \code{x}.
}
\description{
Access or assign the optional \code{geolocation} attribute to a semantically rich
dataset object.
}
\details{
The \code{geolocation} field describes the spatial region or named place where
the data was collected or that the dataset is about. This field is
recommended for data discovery in DataCite Metadata Schema 4.4.

See: \href{https://support.datacite.org/docs/datacite-metadata-schema-v44-recommended-and-optional-properties#18-geolocation}{DataCite: Geolocation Guidance}
}
\examples{
orange_dataset <- orange_df
geolocation(orange_df) <- "US"
geolocation(orange_df)

geolocation(orange_df, overwrite = FALSE) <- "GB"

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_unit.R
\name{var_unit}
\alias{var_unit}
\alias{var_unit<-}
\alias{var_unit<-.default}
\alias{get_variable_units}
\alias{unit_attribute}
\alias{get_unit_attribute}
\alias{set_unit_attribute}
\alias{unit_attribute<-}
\title{Get or Set a Unit of Measure}
\usage{
var_unit(x, ...)

var_unit(x) <- value

\method{var_unit}{default}(x) <- value

get_variable_units(x, ...)

unit_attribute(x)

get_unit_attribute(x)

set_unit_attribute(x, value)

unit_attribute(x) <- value
}
\arguments{
\item{x}{A vector.}

\item{...}{Further arguments for method extensions.}

\item{value}{A single character string or \code{NULL}. If not of length one, an
error is thrown.}
}
\value{
\itemize{
\item \code{var_unit(x)} returns the \code{"unit"} attribute as a character string.
\item \code{var_unit(x) <- value} sets, updates, or removes the unit and returns
the modified vector invisibly.
}

The modified object \code{x}, returned invisibly with the updated \code{"unit"}
attribute.

The \code{"unit"} attribute of the object \code{x}, or \code{NULL} if not set.

The object \code{x} with updated \code{"unit"} attribute.
}
\description{
Adds or retrieves a unit of measure (UoM) attribute to a vector. Units
provide semantic meaning for numeric or character data — such as currency,
weight, or time — helping prevent incorrect operations like merging values
measured in incompatible units.

The \verb{var_unit<-} assignment method sets, updates, or removes the \code{"unit"}
attribute of a vector. This can be used with \code{defined()} vectors or base
vectors to ensure consistent semantic annotation.

\code{unit_attribute()} is a low-level helper to directly access the \code{"unit"}
attribute of a vector, without applying fallback logic. It is mainly used
internally.

\code{get_unit_attribute()} is an alias for \code{unit_attribute()}, included for naming
consistency in codebases that distinguish getter/setter patterns.

\code{set_unit_attribute()} is the low-level assignment function that sets or
removes the \code{"unit"} attribute of an object. Used internally by
\verb{unit_attribute<-}.
}
\details{
The \code{"unit"} attribute stores a machine-readable representation of a
unit of measure (e.g., \code{"kg"}, \code{"USD"}, \code{"days"}). This is useful when
working with linked open data or when combining data from multiple sources
where silent mismatches in units could cause errors.

For full integration with semantic metadata (e.g., labels, concepts,
namespaces), use \code{\link[=defined]{defined()}} vectors or \code{\link[=dataset_df]{dataset_df()}} objects.

\code{get_variable_units()} is an alias for \code{var_unit()}.

See \code{vignette("defined", package = "dataset")} for end-to-end examples
involving semantic enrichment.
}
\examples{
# Retrieve the unit of measure (if defined)
var_unit(orange_df$circumference)

# Regular data.frame columns have no unit by default
var_unit(mtcars$wt)

# Add a unit to a column
var_unit(mtcars$wt) <- "1000 lbs"

# Remove the unit
var_unit(mtcars$wt) <- NULL

}
\seealso{
Other defined metadata methods and functions:
\code{\link{var_label}},
\code{\link[=var_labels]{var_labels()}},
\code{\link[=var_namespace]{var_namespace()}}
}
\concept{defined metadata methods and functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defined.R
\name{c.haven_labelled_defined}
\alias{c.haven_labelled_defined}
\title{Combine defined vectors with metadata checks}
\usage{
\method{c}{haven_labelled_defined}(...)
}
\arguments{
\item{...}{One or more vectors created with \code{\link[=defined]{defined()}}.}
}
\value{
A single \code{defined} vector with concatenated values and retained
metadata.
}
\description{
The \code{c()} method for \code{defined} vectors ensures that all semantic metadata
(label, unit, concept, namespace, and value labels) match exactly. This
prevents accidental loss or mixing of incompatible definitions during
concatenation.
}
\details{
All input vectors must:
\itemize{
\item Have identical \code{label} attributes
\item Have identical \code{unit}, \code{concept}, and \code{namespace}
\item Have identical value labels (or none)
}
}
\examples{
a <- defined(1:3, label = "Length", unit = "meter")
b <- defined(4:6, label = "Length", unit = "meter")
c(a, b)

}
\seealso{
\code{\link[=defined]{defined()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defined.R
\name{vec_cast_named}
\alias{vec_cast_named}
\title{From haven}
\usage{
vec_cast_named(x, to, ...)
}
\description{
From haven
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_to_triples.R
\name{triples_column_generate}
\alias{triples_column_generate}
\title{Internal: Generate RDF triples for a single column}
\usage{
triples_column_generate(s_vec, col, colname)
}
\arguments{
\item{s_vec}{A character vector of subject URIs (length = number of rows)}

\item{col}{The column vector (e.g., \code{x[[i]]})}

\item{colname}{The name of the column (used as fallback for predicate)}
}
\value{
A data.frame with columns s, p, o
}
\description{
Create subject-predicate-object triples from one column of a dataset
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-gdp.R
\docType{data}
\name{gdp}
\alias{gdp}
\title{A Small GDP Dataset}
\format{
A data frame with 10 rows and 5 variables:
\itemize{
\item \code{geo}: Country name (character)
\item \code{year}: Reference year (integer)
\item \code{gdp}: Gross Domestic Product value (numeric)
\item \code{unit}: Unit of measurement, e.g., "Million EUR" (character)
\item \code{freq}: Observation frequency, e.g., "Annual" (character)
}
}
\source{
Eurostat (2021). GDP and main aggregates - international data
cooperation (annual data). \doi{10.2908/NAIDA_10_GDP}
}
\usage{
gdp
}
\description{
A compact sample of GDP and main aggregates from Eurostat's annual
international cooperation dataset. This data subset contains illustrative
records for select countries and time periods.
}
\details{
This dataset is intended for examples, tests, and demonstration purposes. It
reflects simplified GDP data as published by Eurostat. The actual Eurostat
dataset includes more countries, breakdowns, and metadata.
}
\examples{
head(gdp)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/n_triple.R
\name{n_triple}
\alias{n_triple}
\title{Create an N-Triple}
\source{
\href{https://www.w3.org/TR/n-triples/}{RDF 1.1 N-Triples}
}
\usage{
n_triple(s, p, o)
}
\arguments{
\item{s}{The subject of a triplet.}

\item{p}{The predicate of a triplet.}

\item{o}{The object of a triplet.}
}
\value{
A character vector containing one N-Triple string.
}
\description{
Create a single N-Triple triple.
}
\details{
N-Triples is an easy to parse line-based subset of Turtle to serialize
RDF. An N-Triple triple is a sequence of RDF terms representing the subject,
predicate and object of an RDF Triple. Use \code{\link[=n_triples]{n_triples()}} to serialize
multiple statements.
}
\examples{
s <- "http://example.org/show/218"
p <- "http://www.w3.org/2000/01/rdf-schema#label"
o <- "That Seventies Show"
n_triple(s, p, o)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/publisher.R
\name{publisher}
\alias{publisher}
\alias{publisher<-}
\title{Get or Set the Publisher of a Dataset Object}
\usage{
publisher(x)

publisher(x, overwrite = TRUE) <- value
}
\arguments{
\item{x}{A dataset object created with \code{\link[=dataset_df]{dataset_df()}} or
\code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{overwrite}{Logical. Should existing publisher metadata be overwritten?
Defaults to \code{FALSE}. If \code{FALSE} and the field exists, a warning is issued.}

\item{value}{A character string specifying the publisher.}
}
\value{
A character string of length one containing the \code{"publisher"} attribute.
When assigning, the updated object \code{x} is returned invisibly.
}
\description{
The publisher is the entity responsible for holding, archiving, releasing,
or distributing the resource. It is typically included in dataset citation
metadata.

For software, this might refer to a code repository (e.g., GitHub). If both
a hosting platform and a producing institution are involved, use the
publisher for the institution and \code{\link[=creator]{creator()}} with
\code{contributorType = "hostingInstitution"} for the platform.
}
\details{
Adds or retrieves the optional \code{"publisher"} attribute for a dataset object.
This property aligns with \code{dct:publisher} (Dublin Core) and \code{publisher}
(DataCite).
}
\examples{
publisher(orange_df) <- "Wiley"
publisher(orange_df)

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fix_contributor.R
\name{fix_contributor}
\alias{fix_contributor}
\title{Format contributors into a citation string}
\usage{
fix_contributor(contributors = NULL)
}
\arguments{
\item{contributors}{A vector or list of \code{person} objects, or \code{NULL}, or the
character string \code{":unas"}.}
}
\value{
A single character string, e.g. \code{"{Jane Doe [dtm, ctb]} and {John Smith [ctb]}"}.
}
\description{
Format a list of \code{utils::person} objects into a compact string, merging roles
per person and normalizing names. Contributors without explicit roles are
assigned \code{"ctb"}. If \code{NULL} or \code{":unas"} is supplied, returns \code{":unas"}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/provenance.R
\name{provenance}
\alias{provenance}
\alias{provenance<-}
\title{Get or update provenance information}
\usage{
provenance(x)

provenance(x) <- value
}
\arguments{
\item{x}{A dataset created with \code{\link[=dataset_df]{dataset_df()}}.}

\item{value}{Character vector of N‑Triples created by \code{\link[=n_triple]{n_triple()}} or
\code{\link[=n_triples]{n_triples()}} to append to existing provenance.}
}
\value{
\itemize{
\item \code{provenance(x)} returns the contents of the \code{"prov"} attribute (character
vector of N‑Triples), or \code{NULL} if none is set.
\item \code{provenance(x) <- value} appends \code{value} to the \code{"prov"} attribute and
returns the modified dataset invisibly.
}
}
\description{
Retrieve or append provenance statements (in N‑Triples form) stored on a
\code{\link[=dataset_df]{dataset_df()}} object.
}
\details{
Provenance is stored in the \code{"prov"} attribute as N‑Triples text. Use
\code{\link[=n_triple]{n_triple()}} or \code{\link[=n_triples]{n_triples()}} to construct valid statements that follow
PROV‑O (e.g., \code{prov:wasGeneratedBy}, \code{prov:wasInformedBy}).
}
\examples{
provenance(orange_df)

# Add a provenance statement:
provenance(orange_df) <- n_triple(
  "https://doi.org/10.5281/zenodo.10396807",
  "http://www.w3.org/ns/prov#wasInformedBy",
  "http://example.com/source#1"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bind_defined_rows.R
\name{bind_defined_rows}
\alias{bind_defined_rows}
\title{Bind strictly defined rows}
\usage{
bind_defined_rows(x, y, ..., strict = FALSE)
}
\arguments{
\item{x}{A \code{dataset_df} object.}

\item{y}{A \code{dataset_df} object to bind to \code{x}.}

\item{...}{Optional dataset-level attributes such as \code{title} or
\code{creator} to override.}

\item{strict}{Logical. If \code{TRUE} (default), require full semantic
compatibility, including rowid.}
}
\value{
A new \code{dataset_df} object with rows from \code{x} and \code{y}, combined
semantically.
}
\description{
Add rows of dataset \code{y} to dataset \code{x}, validating all
semantic metadata. Metadata (labels, units, concept definitions,
namespaces) must match exactly. Additional dataset-level metadata such as
title and creator can be overridden using \code{...}.
}
\details{
This function combines two semantically enriched datasets created
with \code{dataset_df()}. All variable-level attributes — including labels,
units, concept definitions, and namespaces — must match. If \code{strict =
TRUE} (the default), the row identifier namespace (used in the \code{rowid}
column) must also match exactly.

If \code{strict = FALSE}, row identifiers from \code{y} may differ and will
be ignored; the output will inherit \code{x}'s row identifier scheme.
}
\examples{
A <- dataset_df(
  length = defined(c(10, 15),
    label = "Length",
    unit = "cm", namespace = "http://example.org"
  ),
  identifier = c(id = "http://example.org/dataset#"),
  dataset_bibentry = dublincore(
    title = "Dataset A",
    creator = person("Alice", "Smith")
  )
)

B <- dataset_df(
  length = defined(c(20, 25),
    label = "Length",
    unit = "cm", namespace = "http://example.org"
  ),
  identifier = c(id = "http://example.org/dataset#")
)

bind_defined_rows(A, B) # succeeds

C <- dataset_df(
  length = defined(c(30, 35),
    label = "Length",
    unit = "cm", namespace = "http://example.org"
  ),
  identifier = c(id = "http://another.org/dataset#")
)

\dontrun{
bind_defined_rows(A, C, strict = TRUE) # fails: mismatched rowid
}

bind_defined_rows(A, C, strict = FALSE) # succeeds: rowid inherited
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_factor.R
\name{as_factor}
\alias{as_factor}
\alias{as_factor.haven_labelled_defined}
\title{Coerce a defined vector to a factor}
\usage{
as_factor(x, ...)

\method{as_factor}{haven_labelled_defined}(x, strip_attributes = TRUE, ...)
}
\arguments{
\item{x}{A vector created with \code{\link[=defined]{defined()}}.}

\item{...}{Reserved for future extensions.}

\item{strip_attributes}{Logical; should semantic metadata attributes
(such as \code{label}, \code{unit}, \code{definition}, and
\code{namespace}) be removed from the returned vector?
Defaults to \code{TRUE}.}
}
\value{
A factor vector.
}
\description{
\code{as_factor()} converts a \code{\link[=defined]{defined()}} vector into a standard R
factor. If value labels are present, they are turned into factor levels
via \code{\link[haven:as_factor]{haven::as_factor()}}. Otherwise, the underlying values are converted
with \code{\link[base:factor]{base::factor()}}.
}
\details{
Use \code{strip_attributes = TRUE} when flattening or preparing data for
external pipelines, but keep the default when working with defined
vectors directly.
}
\examples{
sex <- defined(
  c(0, 1, 1, 0),
  label  = "Sex",
  labels = c("Female" = 0, "Male" = 1)
)

as_factor(sex)
as_factor(sex, strip_attributes = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/describe.R
\name{describe}
\alias{describe}
\title{Describe a dataset in N-Triples format}
\usage{
describe(x, con)
}
\arguments{
\item{x}{A \code{dataset_df} object.}

\item{con}{A connection or a character string path (e.g. from \code{tempfile()}).}
}
\value{
Writes N-Triples to \code{con} and invisibly returns \code{x}.
}
\description{
Writes provenance and Dublin Core metadata of a dataset to a
file or connection in N-Triples format.
}
\examples{
test_ds <- dataset_df(
  rowid = defined(c("eg:1", "eg:2"),
    namespace = "http://example.com/dataset#"
  ),
  geo = defined(
    gdp$geo[1:2],
    label = "Country",
    concept = "http://example.com/prop/geo",
    namespace = "https://eionet.europa.eu/geo/$1"
  ),
  dataset_bibentry = dublincore(
    title = "Example Dataset",
    creator = person("John", "Doe")
  )
)

# returns invisibly the contents of the text file serialisation:
testdescription <- describe(test_ds, con = tempfile())
testdescription
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_bibentry.R
\name{get_bibentry}
\alias{get_bibentry}
\alias{set_bibentry<-}
\title{Get or set the bibentry}
\usage{
get_bibentry(dataset)

set_bibentry(dataset) <- value
}
\arguments{
\item{dataset}{A dataset created with \code{\link[=dataset_df]{dataset_df()}}.}

\item{value}{A \code{\link[utils:bibentry]{utils::bibentry}} to store on the dataset. If \code{NULL}, a
minimal default entry is created.}
}
\value{
\itemize{
\item \code{get_bibentry(dataset)} returns the \code{\link[utils:bibentry]{utils::bibentry}} stored in
\code{dataset}'s attributes.
\item \code{set_bibentry(dataset) <- value} sets the attribute and returns the
modified dataset invisibly.
}
}
\description{
Retrieve or replace the bibliographic entry stored in a dataset's attributes.
The entry is a \code{\link[utils:bibentry]{utils::bibentry}} used to hold citation metadata for
\code{\link[=dataset_df]{dataset_df()}} objects.
}
\details{
New datasets are initialized with reasonable defaults. To build a new
bibentry with sensible defaults and field names, use \code{\link[=datacite]{datacite()}} (DataCite)
or \code{\link[=dublincore]{dublincore()}} (Dublin Core), then assign it with
\code{set_bibentry(dataset) <- value}.

See the vignette for more background:
\code{vignette("bibentry", package = "dataset")}.
}
\examples{
# Get the bibentry of a dataset_df object:
be <- get_bibentry(orange_df)

# Create a well-formed bibentry (DataCite-style):
be2 <- datacite(
  Creator = person("Jane", "Doe"),
  Title = "The Orange Trees Dataset",
  Publisher = "MyOrg"
)

# Assign the new bibentry:
set_bibentry(orange_df) <- be2

# Inspect in different notations:
as_datacite(orange_df, type = "list")
as_dublincore(orange_df, type = "list")

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defined.R
\name{defined}
\alias{defined}
\alias{is.defined}
\alias{summary.haven_labelled_defined}
\title{Create a semantically enriched vector with variable-level metadata}
\usage{
defined(
  x,
  labels = NULL,
  label = NULL,
  unit = NULL,
  concept = NULL,
  namespace = NULL,
  ...
)

is.defined(x)

\method{summary}{haven_labelled_defined}(object, ...)
}
\arguments{
\item{x}{A vector to annotate.}

\item{labels}{Optional named vector of value labels. Only supported for
numeric or character vectors (not for logical).}

\item{label}{A short human-readable variable label (character of length 1).}

\item{unit}{Unit of measurement (character length 1) or \code{NULL}.}

\item{concept}{A URI or identifier describing the meaning or definition
of the variable. This replaces the deprecated \code{definition} argument.}

\item{namespace}{Optional string or named character vector used to generate
value-level URIs via substitution (\verb{$1} macro).}

\item{...}{For backward compatibility; the deprecated \code{definition}
argument is still accepted and mapped to \code{concept}.}

\item{object}{An R object to be summarised.}
}
\value{
A vector of class \code{"haven_labelled_defined"} or \code{"datetime_defined"},
depending on the input type.
}
\description{
\code{defined()} constructs a vector that behaves like a base R vector but carries
semantic metadata used for documentation, validation, and interoperability.
The resulting object inherits from \code{\link[haven:labelled]{haven::labelled()}} (for numeric,
character, and factor data) or from base date/time classes, and adds:
}
\details{
\itemize{
\item a human-readable variable label,
\item an optional unit of measurement,
\item a \strong{concept URI} identifying the meaning of the variable
(formerly called \emph{definition}),
\item an optional namespace used for value-level URI expansion,
\item optional labelled values (where supported).
}

The \code{concept} attribute is a general semantic anchor and may refer to:
a measure or dimension concept (SDMX-style), a property IRI (e.g. Wikibase),
or any URI that defines or describes the variable.

\code{defined()} vectors preserve their metadata during subsetting, printing,
summarizing, comparisons, and many tidyverse operations. They integrate
smoothly with \code{\link[=dataset_df]{dataset_df()}} objects and can be safely flattened via
\code{\link[=as.data.frame]{as.data.frame()}}, \code{\link[=as_tibble]{as_tibble()}}, or coercion helpers such as
\code{\link[=as_numeric]{as_numeric()}} and \code{\link[=as_character]{as_character()}}.
}
\section{Supported input types}{

\itemize{
\item numeric (integer or double)
\item character
\item factor (converted via \code{\link[labelled:to_labelled]{labelled::to_labelled()}})
\item \code{\link{Date}}
\item \code{\link{POSIXct}}
\item \code{\link[haven:labelled]{haven::labelled()}}
\item logical (with restrictions: logical vectors \strong{cannot} have value labels)
}
}

\examples{
gdp_vector <- defined(
  c(3897, 7365, 6753),
  label = "Gross Domestic Product",
  unit = "million dollars",
  concept = "http://data.europa.eu/83i/aa/GDP"
)

is.defined(gdp_vector)
print(gdp_vector)
summary(gdp_vector)
gdp_vector[1:2]

}
\seealso{
\code{\link[=is.defined]{is.defined()}},
\code{\link[=as_numeric]{as_numeric()}},
\code{\link[=as_character]{as_character()}},
\code{\link[=as_logical]{as_logical()}},
\code{\link[=strip_defined]{strip_defined()}},
\code{\link[=dataset_df]{dataset_df()}}
\code{\link[=print.haven_labelled_defined]{print.haven_labelled_defined()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.Date.R
\name{as.Date.haven_labelled_defined}
\alias{as.Date.haven_labelled_defined}
\title{Coerce a defined Date vector to a base R Date}
\usage{
\method{as.Date}{haven_labelled_defined}(x, strip_attributes = FALSE, ...)
}
\arguments{
\item{x}{A vector created with \code{\link[=defined]{defined()}} with underlying type
\code{Date}.}

\item{strip_attributes}{Logical; should the semantic metadata attributes
(label, unit, definition, namespace) be removed from the returned
vector? Defaults to \code{FALSE}.}

\item{...}{Additional arguments passed to \code{\link[base:as.Date]{base::as.Date()}}.}
}
\value{
A \code{Date} vector, optionally carrying semantic metadata.
}
\description{
Coerces a \code{\link{haven_labelled_defined}} vector whose underlying type is
\code{\link{Date}} into a base R \code{Date} vector.

This method preserves the underlying date values and, by default,
also retains any semantic metadata attached to the variable.
}
\details{
Use \code{strip_attributes = TRUE} when flattening or preparing data for
external pipelines, but keep the default when working with defined
vectors directly.

Base R's \code{\link[=as.Date]{as.Date()}} also works, as it dispatches to this method via
S3. However, using \code{as.Date()} on defined vectors is considered safe
because this method ensures metadata is handled predictably.
}
\examples{
d <- defined(Sys.Date() + 0:2, label = "Observation date")

# Recommended usage
as.Date(d)

# Stripping metadata
as.Date(d, strip_attributes = TRUE)

}
\seealso{
\code{\link[=as.POSIXct]{as.POSIXct()}},
\code{\link[=as_numeric]{as_numeric()}}, \code{\link[=as_character]{as_character()}}, \code{\link[=as_logical]{as_logical()}},
\code{\link[=defined]{defined()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/id_to_column.R
\name{id_to_column}
\alias{id_to_column}
\title{Add Identifier to First Column of a Dataset}
\usage{
id_to_column(x, prefix = "eg:", ids = NULL)
}
\arguments{
\item{x}{A dataset created with \code{\link[=dataset_df]{dataset_df()}}, or a regular data frame.}

\item{prefix}{A character string used as the prefix for row identifiers.
Defaults to \code{"eg:"} (referring to \href{https://example.com}{example.com}).}

\item{ids}{Optional. A character vector of custom IDs to use instead of row names.}
}
\value{
A dataset of the same class as \code{x}, with the first column updated to include
unique prefixed identifiers.
}
\description{
Adds a prefixed identifier (e.g., \verb{eg:}) to the first column of a dataset,
useful for generating semantic row IDs (e.g., for RDF serialization).
}
\examples{
# Example with a dataset_df object:
id_to_column(orange_df)

# Example with a regular data.frame:
id_to_column(Orange, prefix = "orange:")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.R
\name{as.data.frame.dataset_df}
\alias{as.data.frame.dataset_df}
\title{Convert a \code{dataset_df} to a base \code{data.frame}}
\usage{
\method{as.data.frame}{dataset_df}(
  x,
  ...,
  strip_attributes = TRUE,
  optional = FALSE,
  stringsAsFactors = FALSE
)
}
\arguments{
\item{x}{A \code{\link{dataset_df}}.}

\item{...}{Passed to \code{base::as.data.frame()}.}

\item{strip_attributes}{Logical: should column-level semantic metadata
be stripped? Default: \code{TRUE}.}

\item{optional}{logical. If \code{TRUE}, setting row names and
    converting column names (to syntactic names: see
    \code{\link[base]{make.names}}) is optional.  Note that all of \R's
    \pkg{base} package \code{as.data.frame()} methods use
    \code{optional} only for column names treatment, basically with the
    meaning of \code{\link[base]{data.frame}(*, check.names = !optional)}.
    See also the \code{make.names} argument of the \code{matrix} method.}

\item{stringsAsFactors}{logical: should the character vector be converted
    to a factor?}
}
\value{
A base R \code{data.frame} without the \code{dataset_df} class.
}
\description{
Converts a \code{\link{dataset_df}} into a plain \code{data.frame}. By default this
strips semantic metadata (label, unit, concept/definition, namespace) from
each column, but this can be controlled via the
\code{strip_attributes} argument.

Dataset-level metadata remains attached as inert attributes.
}
\examples{
data(orange_df)
as.data.frame(orange_df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_namespace.R
\name{var_namespace}
\alias{var_namespace}
\alias{var_namespace<-}
\alias{get_variable_namespaces}
\alias{namespace_attribute}
\alias{get_namespace_attribute}
\alias{set_namespace_attribute}
\alias{namespace_attribute<-}
\title{Get or Set the Namespace of a Variable}
\usage{
var_namespace(x, ...)

var_namespace(x) <- value

get_variable_namespaces(x, ...)

namespace_attribute(x)

get_namespace_attribute(x)

set_namespace_attribute(x, value)

namespace_attribute(x) <- value
}
\arguments{
\item{x}{A vector.}

\item{...}{Additional arguments for method compatibility with other classes.}

\item{value}{A character string specifying the namespace, or \code{NULL} to
remove it.}
}
\value{
A character string representing the namespace attribute of a vector
constructed with \code{\link[=defined]{defined()}}. Returns the updated object (in setter forms).
}
\description{
Retrieve or assign the namespace part of a permanent, global variable
identifier, independent of the current R session or instance.
}
\details{
The \code{namespace} attribute is useful when working with remote, linked, or
open data sources. Variable identifiers in such datasets are often qualified
with a common namespace prefix. When combined, the prefix and namespace form
a persistent URI or IRI for the variable.

Retaining the namespace ensures the identifiers remain valid and resolvable
during validation, merging, or future updates of the vector (such as when it
is used as a column in a dataset).

\code{get_variable_namespaces()} is an alias for \code{var_namespace()}.
\code{namespace_attribute()} and \code{set_namespace_attribute()} are internal helpers.

For full usage, see:
\code{vignette("defined", package = "dataset")} <U+2014> demonstrating integration of
variable labels, namespaces, units of measure, and machine-independent
identifiers.
}
\examples{
# Define a vector with a namespace
x <- defined("Q42", namespace = c(wd = "https://www.wikidata.org/wiki/"))

# Get the namespace
var_namespace(x)
get_variable_namespaces(x)

# Set the namespace
var_namespace(x) <- "https://example.org/ns/"

# Remove the namespace
var_namespace(x) <- NULL

# Use lower-level helpers (not typically used directly)
namespace_attribute(x)
namespace_attribute(x) <- "https://example.org/custom/"

}
\seealso{
Other defined metadata methods and functions:
\code{\link{var_label}},
\code{\link[=var_labels]{var_labels()}},
\code{\link[=var_unit]{var_unit()}}
}
\concept{defined metadata methods and functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xsd_convert.R
\name{xsd_convert}
\alias{xsd_convert}
\alias{xsd_convert.haven_labelled_defined}
\alias{xsd_convert.data.frame}
\alias{xsd_convert.dataset_df}
\alias{xsd_convert.tbl_df}
\alias{xsd_convert.character}
\alias{xsd_convert.numeric}
\alias{xsd_convert.integer}
\alias{xsd_convert.logical}
\alias{xsd_convert.factor}
\alias{xsd_convert.POSIXct}
\alias{xsd_convert.Date}
\alias{xsd_convert.difftime}
\title{Convert to XML Schema Definition (XSD) Types}
\usage{
xsd_convert(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{haven_labelled_defined}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{data.frame}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{dataset_df}(x, idcol = "rowid", shortform = TRUE, ...)

\method{xsd_convert}{tbl_df}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{character}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{numeric}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{integer}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{logical}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{factor}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{POSIXct}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{Date}(x, idcol = NULL, shortform = TRUE, ...)

\method{xsd_convert}{difftime}(x, idcol = NULL, shortform = TRUE, ...)
}
\arguments{
\item{x}{An object (vector, data frame, tibble, or \code{dataset_df}).}

\item{idcol}{Column name or position to use as row (observation) identifier.
If \code{NULL}, row names are used.}

\item{shortform}{Logical. If \code{TRUE} (default), datatypes are abbreviated with
the \verb{xsd:} prefix (e.g. \verb{"42"^^<xsd:integer>}). If \code{FALSE}, datatypes are
expanded to full URIs (e.g.
\verb{"42"^^<http://www.w3.org/2001/XMLSchema#integer>}).}

\item{...}{Additional arguments passed to methods.}
}
\value{
A character vector or data frame with values serialized as
XSD-compatible RDF literals.
}
\description{
Converts R vectors, data frames, and \code{dataset_df} objects to
\href{https://www.w3.org/TR/xmlschema11-2/}{XML Schema Definition (XSD)}
compatible string representations such as \code{xsd:decimal}, \code{xsd:boolean},
\code{xsd:date}, and \code{xsd:dateTime}.
}
\details{
This is primarily used for generating RDF-compatible typed literals.
\itemize{
\item For \strong{vectors}, returns a character vector of typed literals.
\item For \strong{data frames} or tibbles, returns a data frame with the same
structure but with all values converted to XSD strings.
\item For \code{dataset_df} objects, behaves like the data frame method but
preserves dataset-level attributes.
}
}
\section{Class-specific examples}{


\if{html}{\out{<div class="sourceCode r">}}\preformatted{xsd_convert(42L)                   # integer -> xsd:integer
xsd_convert(c(TRUE, FALSE, NA))    # logical -> xsd:boolean
xsd_convert(Sys.Date())            # Date -> xsd:date
xsd_convert(Sys.time())            # POSIXct -> xsd:dateTime
xsd_convert(factor("apple"))       # factor -> xsd:string
xsd_convert(c("apple", "banana"))  # character -> xsd:string
}\if{html}{\out{</div>}}
}

\examples{
# Simple data frame with mixed types
df <- data.frame(
  id     = 1:2,
  value  = c(3.14, 2.71),
  active = c(TRUE, FALSE),
  date   = as.Date(c("2020-01-01", "2020-12-31"))
)

# Short vs long-form URI:
xsd_convert(120L, shortform = TRUE)
xsd_convert(121L, shortform = FALSE)

}
\concept{RDF and linked data helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_format.R
\name{dataset_format}
\alias{dataset_format}
\alias{dataset_format<-}
\title{Get or set the technical format of a dataset}
\usage{
dataset_format(x)

dataset_format(x, overwrite = FALSE) <- value
}
\arguments{
\item{x}{A semantically rich data frame created with \code{\link[=dataset_df]{dataset_df()}} or
\code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{overwrite}{Logical. Replace an existing non‑default value? If \code{FALSE}
and a non‑default value already exists, a message is emitted and the value
is kept. Defaults to \code{FALSE}.}

\item{value}{A length‑one character string specifying the format
(e.g., \code{"text/csv"}). Use \code{NULL} to reset to the default.}
}
\value{
The \code{"format"} (technical format) as a character string (length 1).
When assigning, the updated object \code{x} is returned invisibly.
}
\description{
Adds or retrieves the optional \code{"format"} field of a dataset's bibentry.
This field is the dataset's technical/media type (e.g., a MIME type).
}
\details{
The format field corresponds to
\href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/format/}{dct:format}
in Dublin Core and to \code{format} in
\href{https://schema.datacite.org/}{DataCite}.
It is useful for indicating serialization such as \code{"text/csv"},
\code{"application/parquet"}, or \code{"application/r-rds"}.

If no format is set, this helper uses the package default
\code{"application/r-rds"}.
}
\examples{
dataset_format(orange_df) <- "text/csv"
dataset_format(orange_df)

# Reset to the package default
dataset_format(orange_df) <- NULL

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=language]{language()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_to_triples.R
\name{triples_to_ntriples}
\alias{triples_to_ntriples}
\title{Internal: Convert triple data.frame to N-Triples format}
\usage{
triples_to_ntriples(df)
}
\arguments{
\item{df}{A data.frame with columns \code{s}, \code{p}, and \code{o}.}
}
\value{
A character vector of N-Triple lines.
}
\description{
Turns a data.frame with \code{s}, \code{p}, \code{o} columns into N-Triples strings.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/var_concept.R
\name{var_concept}
\alias{var_concept}
\alias{var_concept<-}
\alias{var_concept<-.default}
\title{Get / set a concept definition for a vector or a dataset}
\usage{
var_concept(x, ...)

var_concept(x) <- value

\method{var_concept}{default}(x) <- value
}
\arguments{
\item{x}{A vector to which the concept URI will be assigned.}

\item{...}{Further parameters for inheritance, not in use.}

\item{value}{A character string with a concept URI or NULL to remove the concept.}
}
\value{
The (linked) concept of the meaning of the data contained by a
vector constructed with\code{\link[=defined]{defined()}}.

The modified vector with updated \code{concept} metadata.
}
\description{
Assigns a concept URI to a vector created with \code{defined()}. This
method updates the \code{concept} attribute and validates that the input is a single
character string or NULL.
}
\details{
\code{get_variable_concepts()} is identical to \code{var_concept()}.
}
\examples{
small_country_dataset <- dataset_df(
  country_name = defined(c("Andorra", "Lichtenstein"), label = "Country"),
  gdp = defined(c(3897, 7365),
    label = "Gross Domestic Product",
    unit = "million dollars"
  )
)
var_concept(small_country_dataset$country_name) <- "http://data.europa.eu/bna/c_6c2bb82d"
var_concept(small_country_dataset$country_name)
# To remove a concept definition of variable
var_concept(small_country_dataset$country_name) <- NULL
x <- defined(c(1, 2, 3), label = "Example Variable")
var_concept(x) <- "http://example.org/concept/XYZ"
var_concept(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_datacite.R, R/datacite.R
\name{as_datacite}
\alias{as_datacite}
\alias{datacite}
\alias{is.datacite}
\alias{is.datacite.datacite}
\alias{print.datacite}
\title{Create a Bibentry Object with DataCite Metadata Fields}
\source{
\itemize{
\item \href{https://support.datacite.org/docs/schema-mandatory-properties-v43}{DataCite 4.3 Mandatory Properties}
\item \href{https://support.datacite.org/docs/schema-optional-properties-v43}{DataCite 4.3 Optional Properties}
}
}
\usage{
as_datacite(x, type = "bibentry", ...)

datacite(
  Title,
  Creator,
  Identifier = NULL,
  Publisher = NULL,
  PublicationYear = NULL,
  Subject = subject_create(term = "data sets", subjectScheme =
    "Library of Congress Subject Headings (LCSH)", schemeURI =
    "https://id.loc.gov/authorities/subjects.html", valueURI =
    "http://id.loc.gov/authorities/subjects/sh2018002256"),
  Type = "Dataset",
  Contributor = NULL,
  Date = ":tba",
  DateList = NULL,
  Language = NULL,
  AlternateIdentifier = ":unas",
  RelatedIdentifier = ":unas",
  Format = ":tba",
  Version = "0.1.0",
  Rights = ":tba",
  Description = ":tba",
  Geolocation = ":unas",
  FundingReference = ":unas"
)

is.datacite(x)

\method{is.datacite}{datacite}(x)

\method{print}{datacite}(x, ...)
}
\arguments{
\item{x}{An object that is tested if it has a class "datacite".}

\item{type}{A DataCite 4.4 metadata can be returned as:
\code{"list"}, \code{"dataset_df"}, \code{"bibentry"} (default), or \code{"ntriples"}.}

\item{...}{Optional parameters to add to a \code{datacite} object.
For example, \code{author = person("Jane", "Doe")} adds an author if
\code{type = "dataset_df"}.}

\item{Title}{The name(s) by which the resource is known. Similar to
\href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/title/}{dct:title}.}

\item{Creator}{One or more \code{\link[utils:person]{utils::person()}} objects describing the main
authors or contributors responsible for creating the resource.}

\item{Identifier}{A persistent identifier (e.g., DOI or URI). May refer to a
specific version or all versions of the resource.}

\item{Publisher}{The name of the organization that holds, publishes, or
distributes the resource. Required by DataCite. See \code{\link[=publisher]{publisher()}}.}

\item{PublicationYear}{The year of public availability (in \code{YYYY} format).
See \code{\link[=publication_year]{publication_year()}}.}

\item{Subject}{A topic, keyword, or classification term. See \code{\link[=subject]{subject()}} and
\code{\link[=subject_create]{subject_create()}} for structured vocabularies.}

\item{Type}{The resource type. Defaults to \code{"Dataset"} for general use. See
\href{https://www.dublincore.org/specifications/dublin-core/dcmi-terms/elements11/type/}{dcm:type}.}

\item{Contributor}{An individual or institution that contributed to the
development, distribution, or curation of the resource.}

\item{Date}{A date in \code{"YYYY"}, \code{"YYYY-MM-DD"} or ISO datetime format.
Can also be a \link{Date} or \link{POSIXct} object.}

\item{DateList}{A list of multiple dates. Currently not supported.}

\item{Language}{Language code as per IETF BCP 47 / ISO 639-1. See \code{\link[=language]{language()}}.}

\item{AlternateIdentifier}{Optional local or secondary identifier. Defaults
to \code{":unas"}.}

\item{RelatedIdentifier}{Related resources (e.g., prior versions, papers).
Defaults to \code{":unas"}.}

\item{Format}{A technical format (e.g., \code{"application/pdf"}, \code{"text/csv"}).}

\item{Version}{A free-text version string (e.g., \code{"1.0.0"}). Defaults to
\code{"0.1.0"}. See \code{\link[=version]{version()}}.}

\item{Rights}{Licensing or usage restrictions for the resource. Defaults to
\code{":tba"}. See \code{\link[=rights]{rights()}}.}

\item{Description}{Free-text summary or additional information. Defaults to
\code{":tba"}.}

\item{Geolocation}{Geographic location covered or referenced by the resource.
See \code{\link[=geolocation]{geolocation()}}.}

\item{FundingReference}{Information about funding or financial support.
Defaults to \code{":unas"}. Structured funding metadata not yet implemented.}
}
\value{
\code{as_datacite(x, type)} returns the DataCite bibliographical metadata
of \code{x} either as a list, a bibentry object, an N-Triples text serialisation
or a dataset_df object.

A \code{\link[utils:bibentry]{utils::bibentry()}} object with DataCite-compliant fields. Use
\code{\link[=as_datacite]{as_datacite()}} to extract the metadata as a list or bibentry object.

\code{is.datacite(x)} returns a logical values (if the object
\code{x} is of class \code{datacite}).
}
\description{
Constructs a bibliographic metadata record conforming to the
\href{https://schema.datacite.org/}{DataCite Metadata Schema}. The resulting
object is stored as a modified \code{\link[utils:bibentry]{utils::bibentry()}} enriched with structured
Dublin Core and DataCite-compliant metadata.
}
\details{
DataCite is a leading non-profit organization that provides persistent
identifiers (DOIs) for research data and other research outputs. Members of
the research community use DataCite to register datasets with globally
resolvable metadata for citation and discovery.

This function sets \code{"Dataset"} as the default resource type. The \code{Size}
attribute (e.g., bytes, pages, etc.) is automatically added if available.
}
\examples{
datacite(
  Title = "Growth of Orange Trees",
  Creator = c(
    person(
      given = "N.R.",
      family = "Draper",
      role = "cre",
      comment = c(VIAF = "http://viaf.org/viaf/84585260")
    ),
    person(
      given = "H",
      family = "Smith",
      role = "cre"
    )
  ),
  Publisher = "Wiley",
  Date = 1998,
  Language = "en"
)

# Extract bibliographic metadata
as_datacite(orange_df)

# As a list
as_datacite(orange_df, "list")

}
\seealso{
Learn more in the vignette:
\href{https://dataset.dataobservatory.eu/articles/bibrecord.html}{\code{bibrecord}}

Other bibrecord functions:
\code{\link[=as_dublincore]{as_dublincore()}},
\code{\link[=bibrecord]{bibrecord()}}
}
\concept{bibrecord functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_numeric.R
\name{as_numeric}
\alias{as_numeric}
\alias{as_numeric.haven_labelled_defined}
\title{Coerce a defined vector to numeric}
\usage{
as_numeric(x, ...)

\method{as_numeric}{haven_labelled_defined}(x, strip_attributes = TRUE, ...)
}
\arguments{
\item{x}{A vector created with \code{\link[=defined]{defined()}}.}

\item{...}{Reserved for future use.}

\item{strip_attributes}{Logical; whether to remove semantic metadata
(\code{label}, \code{unit}, \code{concept}, \code{namespace}). Defaults to \code{TRUE}.}
}
\value{
A numeric vector with or without preserved attributes.
}
\description{
\code{as_numeric()} converts a \code{\link[=defined]{defined()}} vector to a numeric vector.
It validates that the underlying data are numeric, and optionally preserves
or strips semantic metadata.
}
\details{
Use \code{strip_attributes = TRUE} when flattening or preparing data for
external pipelines, but keep the default when working with defined
vectors directly.

\code{as.numeric()} drops all metadata and returns only the numeric values in
a vector.
}
\examples{
x <- defined(
  1:3,
  label = "Count",
  unit = "n",
  concept = "http://example.org/count",
  namespace = "http://example.org/ns"
)

as_numeric(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fix_contributor.R
\name{map_role_to_schema}
\alias{map_role_to_schema}
\title{Map R person roles to schema.org-style roles}
\usage{
map_role_to_schema(role)
}
\arguments{
\item{role}{A character vector of roles (e.g. \code{"cre"}, \code{"ctb"}).}
}
\value{
A character vector with schema.org-style roles.
}
\description{
Map R person roles to schema.org-style roles
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/n_triple.R
\name{n_triples}
\alias{n_triples}
\title{Create N-Triples}
\usage{
n_triples(triples)
}
\arguments{
\item{triples}{A character vector of concatenated N-Triples, created with
\code{\link[=n_triple]{n_triple()}}.}
}
\value{
A character vector of unique N-Triple strings.
}
\description{
Create RDF triple statements to annotate your dataset with standard,
interoperable metadata.
}
\details{
N-Triples is a line-based serialization format for RDF. It is easy to parse
and widely supported. For details, see the
\href{https://www.w3.org/TR/rdf12-n-triples/}{W3C RDF 1.2 N-Triples specification}.
}
\examples{
triple_1 <- n_triple(
  "http://example.org/show/218",
  "http://www.w3.org/2000/01/rdf-schema#label",
  "That Seventies Show"
)

triple_2 <- n_triple(
  "http://example.org/show/218",
  "http://example.org/show/localName",
  '"Cette Série des Années Septante"@fr-be'
)

n_triples(c(triple_1, triple_2, triple_1))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/language.R
\name{language}
\alias{language}
\alias{language<-}
\title{Set the Primary Language of a Dataset}
\usage{
language(x)

language(x, iso_639_code = "639-3") <- value

language(x, iso_639_code = "639-3") <- value
}
\arguments{
\item{x}{A dataset object created by \code{\link[=dataset_df]{dataset_df()}} or \code{\link[=as_dataset_df]{as_dataset_df()}}.}

\item{iso_639_code}{A character string indicating the desired return format:
either \code{"639-3"} (default; terminologic) or \code{"639-1"} (2-letter code).}

\item{value}{A 2-letter or 3-letter language code (ISO 639-1 or ISO 639-2),
or a full language name (case-insensitive).}
}
\value{
The dataset with an updated \code{language} attribute, typically an ISO
639-2/T code (\code{Alpha_3_T}) such as \code{"fra"}, \code{"eng"}, \code{"spa"}, etc.
}
\description{
Assign the primary language of a semantically rich dataset object using an
ISO 639 language code or full language name. This sets the \code{language}
attribute in the dataset's metadata.
}
\details{
This function supports recognition of:
\itemize{
\item 2-letter codes (ISO 639-1, e.g., \code{"en"}, \code{"fr"})
\item 3-letter codes from both:
\itemize{
\item \code{Alpha_3_B} (bibliographic, e.g., \code{"fre"})
\item \code{Alpha_3_T} (terminologic, e.g., \code{"fra"})
}
\item Full language names (e.g., \code{"English"}, \code{"French"})
}

For compatibility with open science repositories and modern metadata
standards, this function \strong{returns the terminologic code} (\code{Alpha_3_T})
when available. If \code{Alpha_3_T} is missing for a language, the legacy
bibliographic code (\code{Alpha_3_B}) is used as a fallback.

Full language names (e.g., \code{"English"}, \code{"Spanish"}) are matched
case-insensitively against the ISO 639-2 Name field. Exact matches are
attempted first; if none are found, a prefix match is used. For example:
\itemize{
\item \code{"English"} returns \code{"eng"}
\item \code{"English, Old"} returns \code{"ang"}
}

This means that:
\itemize{
\item Both \code{"fra"} (terminologic) and \code{"fre"} (bibliographic) will be accepted
as valid input for French
\item The resulting value stored and returned will be \code{"fra"}
}

This behaviour aligns with:
\itemize{
\item \href{https://support.datacite.org/docs/datacite-metadata-schema-v44-recommended-and-optional-properties#9-language}{DataCite Metadata Schema 4.4}
\item \href{https://schema.org/inLanguage}{schema.org}
\item Common repository practices (Zenodo, OSF, Figshare)
}

If \code{value} is \code{NULL}, the language is marked as \code{":unas"} (unspecified).

In some cases<U+2014>especially for historical or moribund languages<U+2014>multiple
similar names may exist. In such cases, it is safer to use a specific
language code (e.g., \code{"ang"} instead of \code{"English, Old"} and \code{"enm"}
for \code{"English, Middle (1100-1500)"}). You can also
refer directly to the definitions in \code{\link[ISOcodes:ISO_639_2]{ISOcodes::ISO_639_2}}
for clarity.
}
\examples{
df <- dataset_df(data.frame(x = 1:3))

language(df) <- "English" # Returns "eng"
language(df) <- "fre" # Legacy code; returns "fra"
language(df) <- "fra" # Returns "fra"
language(df, iso_639_code = "639-1") <- "fra" # Returns "fr"

language(df) <- NULL # Sets ":unas"

}
\seealso{
Other bibliographic helper functions:
\code{\link[=contributor]{contributor()}},
\code{\link[=creator]{creator()}},
\code{\link[=dataset_format]{dataset_format()}},
\code{\link[=dataset_title]{dataset_title()}},
\code{\link[=description]{description()}},
\code{\link[=geolocation]{geolocation()}},
\code{\link[=get_bibentry]{get_bibentry()}},
\code{\link[=publication_year]{publication_year()}},
\code{\link[=publisher]{publisher()}},
\code{\link[=relation]{relation()}},
\code{\link[=rights]{rights()}},
\code{\link[=subject]{subject()}}
}
\concept{bibliographic helper functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibrecord.R
\name{bibrecord}
\alias{bibrecord}
\title{Create a Modern Metadata Object Compatible with bibentry}
\usage{
bibrecord(
  title,
  author,
  contributor = NULL,
  publisher = NULL,
  year = NULL,
  date = Sys.Date(),
  identifier = NULL,
  subject = NULL,
  ...
)
}
\arguments{
\item{title}{A character string specifying the dataset title.}

\item{author}{A \code{\link[utils:person]{utils::person()}} or list/vector of person objects. Mapped to
\code{creator} in DataCite and DCMI.}

\item{contributor}{Optional list or vector of \code{\link[utils:person]{utils::person()}} objects.
Contributor roles are merged if duplicated.}

\item{publisher}{A character string or \code{\link[utils:person]{utils::person()}} representing the
publishing entity.}

\item{year}{Publication year. Automatically derived from \code{date} if not
provided explicitly.}

\item{date}{A \link{Date} object or character string in ISO format.}

\item{identifier}{A persistent identifier (e.g., DOI or URL).}

\item{subject}{Optional keyword, tag, or controlled vocabulary term.}

\item{...}{Additional fields such as \code{language}, \code{format}, \code{rights}, or
\code{description}.}
}
\value{
An object of class \code{"bibrecord"} and \code{"bibentry"}, suitable for citation and
embedding in metadata-aware structures such as \code{\link[=dataset_df]{dataset_df()}}.
}
\description{
Constructs a \code{\link[utils:bibentry]{utils::bibentry()}} object extended with Dublin Core and
DataCite-compatible fields. This unified structure supports use with
functions such as \code{\link[=dublincore]{dublincore()}} and \code{\link[=datacite]{datacite()}}, and is the internal
format for storing rich metadata with datasets.
}
\examples{
bibrecord(
  title = "Gross domestic product, volumes",
  author = person("Eurosat"),
  publisher = person("Eurostat"),
  identifier = "https://doi.org/10.2908/TEINA011",
  date = as.Date("2025-05-20")
)

}
\seealso{
Learn more in the vignette:
\href{https://dataset.dataobservatory.eu/articles/bibrecord.html}{\code{bibrecord}}

Other bibrecord functions:
\code{\link[=as_datacite]{as_datacite()}},
\code{\link[=as_dublincore]{as_dublincore()}}
}
\concept{bibrecord functions}
